Type: Package
Package: Rage
Title: Life History Metrics from Matrix Population Models
Version: 1.9.0
Authors@R: c(
    person("Patrick", "Barks", , "barks@biology.sdu.dk", role = "aut",
           comment = c(ORCID = "0000-0002-5947-8151")),
    person("Danny", "Buss", , "dlb50@cam.ac.uk", role = "ctb"),
    person("Pol", "Capdevila", , "pcapdevila.pc@gmail.com", role = "aut",
           comment = c(ORCID = "0000-0002-2842-4302")),
    person("Hal", "Caswell", , "h.caswell@uva.nl", role = "aut",
           comment = c(ORCID = "0000-0003-4394-6894")),
    person(c("Judy", "P."), "Che-Castaldo", , "jchecastaldo@lpzoo.org", role = "aut",
           comment = c(ORCID = "0000-0002-9118-9202")),
    person(c("Richard", "A."), "Hinrichsen", , "rich@hinrichsenenvironmental.com", role = "aut",
           comment = c(ORCID = "0000-0003-0761-3005")),
    person("John", "Jackson", , "jjackson0308@gmail.com", role = "aut",
           comment = c(ORCID = "0000-0002-4563-2840")),
    person("Tamora", "James", , "tamoradjames@protonmail.com", role = "aut",
           comment = c(ORCID = "0000-0003-1363-4742")),
    person("Owen", "Jones", , "jones@biology.sdu.dk", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0001-5720-4686")),
    person("Sam", "Levin", , "levisc8@gmail.com", role = "aut",
           comment = c(ORCID = "0000-0002-3289-9925")),
    person("William K.", "Petry", , "wpetry@ncsu.edu", role = "aut",
           comment = c(ORCID = "0000-0002-5230-5987")),
    person("Roberto", "Salguero-Gomez", , "rob.salguero@zoo.ox.ac.uk", role = "aut",
           comment = c(ORCID = "0000-0002-6085-4433")),
    person("Caroline", "Schuette", , "cschuette17@gmail.com", role = "ctb",
           comment = c(ORCID = "0000-0002-2063-8736")),
    person("Iain", "Stott", , "stott@biology.sdu.dk", role = "aut",
           comment = c(ORCID = "0000-0003-2724-7436")),
    person(c("Chelsea", "C."), "Thomas", , "esochels@gmail.com", role = "aut",
           comment = c(ORCID = "0000-0002-8155-9353")),
    person(c("Christina", "M."), "Hernández", , "cmh352@cornell.edu", role = "aut",
           comment = c(ORCID = "0000-0002-7188-8217")),
    person("Lotte", "de Vries", , "c.devries@uva.nl", role = "aut",
           comment = c(ORCID = "0000-0001-8955-0479")),
    person("Stefano", "Giaimo", , "giaimo@evolbio.mpg.de", role = "aut",
           comment = c(ORCID = "0000-0003-0421-3065"))
  )
Description: Functions for calculating life history metrics using matrix
    population models ('MPMs'). Described in Jones et al. (2021)
    <doi:10.1101/2021.04.26.441330>.
License: GPL-3
URL: https://github.com/jonesor/Rage, https://jonesor.github.io/Rage/
BugReports: https://github.com/jonesor/Rage/issues
Depends: R (>= 3.5.0)
Imports: DiagrammeR, expm, MASS, popdemo, stats, utils
Suggests: DiagrammeRsvg, ggplot2, knitr, Rcompadre, rmarkdown, rsvg,
        spelling, testthat (>= 3.0.0)
VignetteBuilder: knitr
Config/testthat/edition: 3
Config/testthat/parallel: false
Encoding: UTF-8
Language: en-GB
LazyData: true
RoxygenNote: 7.3.2
NeedsCompilation: no
Packaged: 2026-04-23 15:01:34 UTC; jones
Author: Patrick Barks [aut] (ORCID: <https://orcid.org/0000-0002-5947-8151>),
  Danny Buss [ctb],
  Pol Capdevila [aut] (ORCID: <https://orcid.org/0000-0002-2842-4302>),
  Hal Caswell [aut] (ORCID: <https://orcid.org/0000-0003-4394-6894>),
  Judy P. Che-Castaldo [aut] (ORCID:
    <https://orcid.org/0000-0002-9118-9202>),
  Richard A. Hinrichsen [aut] (ORCID:
    <https://orcid.org/0000-0003-0761-3005>),
  John Jackson [aut] (ORCID: <https://orcid.org/0000-0002-4563-2840>),
  Tamora James [aut] (ORCID: <https://orcid.org/0000-0003-1363-4742>),
  Owen Jones [aut, cre] (ORCID: <https://orcid.org/0000-0001-5720-4686>),
  Sam Levin [aut] (ORCID: <https://orcid.org/0000-0002-3289-9925>),
  William K. Petry [aut] (ORCID: <https://orcid.org/0000-0002-5230-5987>),
  Roberto Salguero-Gomez [aut] (ORCID:
    <https://orcid.org/0000-0002-6085-4433>),
  Caroline Schuette [ctb] (ORCID:
    <https://orcid.org/0000-0002-2063-8736>),
  Iain Stott [aut] (ORCID: <https://orcid.org/0000-0003-2724-7436>),
  Chelsea C. Thomas [aut] (ORCID:
    <https://orcid.org/0000-0002-8155-9353>),
  Christina M. Hernández [aut] (ORCID:
    <https://orcid.org/0000-0002-7188-8217>),
  Lotte de Vries [aut] (ORCID: <https://orcid.org/0000-0001-8955-0479>),
  Stefano Giaimo [aut] (ORCID: <https://orcid.org/0000-0003-0421-3065>)
Maintainer: Owen Jones <jones@biology.sdu.dk>
Repository: CRAN
Date/Publication: 2026-04-23 15:30:02 UTC
Built: R 4.6.0; ; 2026-04-29 22:21:46 UTC; unix

# Generated by roxygen2: do not edit by hand

export(entropy_d)
export(entropy_k)
export(entropy_k_age)
export(entropy_k_stage)
export(evenness_shannon)
export(gen_time)
export(hx_to_lx)
export(hx_to_px)
export(is_leslie_matrix)
export(leslie_collapse)
export(life_elas)
export(life_expect_mean)
export(life_expect_var)
export(longevity)
export(lx_to_hx)
export(lx_to_px)
export(mature_age)
export(mature_distrib)
export(mature_prob)
export(mpm_collapse)
export(mpm_rearrange)
export(mpm_split)
export(mpm_standardise)
export(mpm_standardize)
export(mpm_to_hx)
export(mpm_to_lx)
export(mpm_to_mx)
export(mpm_to_px)
export(mpm_to_table)
export(name_stages)
export(net_repro_rate)
export(perturb_matrix)
export(perturb_stochastic)
export(perturb_trans)
export(perturb_vr)
export(plot_life_cycle)
export(pop_vectors)
export(px_to_hx)
export(px_to_lx)
export(qsd_converge)
export(repro_stages)
export(scale_mpm_to_lambda1)
export(shape_rep)
export(shape_surv)
export(stage_at_death_dist)
export(standard_stages)
export(vital_rates)
export(vr_dorm_enter)
export(vr_dorm_exit)
export(vr_fecundity)
export(vr_growth)
export(vr_mat_R)
export(vr_mat_U)
export(vr_shrinkage)
export(vr_stasis)
export(vr_survival)
export(vr_vec_dorm_enter)
export(vr_vec_dorm_exit)
export(vr_vec_growth)
export(vr_vec_reproduction)
export(vr_vec_shrinkage)
export(vr_vec_stasis)
export(vr_vec_survival)
importFrom(DiagrammeR,grViz)
importFrom(MASS,ginv)
importFrom(expm,"%^%")
importFrom(popdemo,isErgodic)
importFrom(popdemo,project)
importFrom(stats,na.omit)
importFrom(stats,runif)
importFrom(utils,head)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# Rage <img src="man/figures/logo_rage.png" align="right" height="160" style="float:right; height:160px;">

| Project                                                                                                                                                                                                | Main branch                                                                                                                             | Devel branch                                                                                                                           |
|--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|:----------------------------------------------------------------------------------------------------------------------------------------|:---------------------------------------------------------------------------------------------------------------------------------------|
| [![License: GPL v3](https://img.shields.io/badge/License-GPL%20v3-blue.svg)](https://www.gnu.org/licenses/gpl-3.0.html)                                                                                | [![R-CMD-check-main](https://github.com/jonesor/Rage/workflows/R-CMD-check-main/badge.svg)](https://github.com/jonesor/Rage/actions)    | [![R-CMD-check-devel](https://github.com/jonesor/Rage/workflows/R-CMD-check-devel/badge.svg)](https://github.com/jonesor/Rage/actions) |
| [![Project Status: Active – The project has reached a stable, usable state and is being actively developed.](https://www.repostatus.org/badges/latest/active.svg)](https://www.repostatus.org/#active) | [![codecov](https://codecov.io/gh/jonesor/Rage/branch/master/graph/badge.svg?token=jGoxqT78T0)](https://app.codecov.io/gh/jonesor/Rage) |                                                                                                                                        |
| ![](http://cranlogs.r-pkg.org/badges/grand-total/Rage)                                                                                                                                                 | [![CRAN_Status_Badge](https://www.r-pkg.org/badges/version/Rage)](https://cran.r-project.org/package=Rage)                              |                                                                                                                                        |
| ![](http://cranlogs.r-pkg.org/badges/Rage)                                                                                                                                                             |                                                                                                                                         |                                                                                                                                        |

------------------------------------------------------------------------

Functions for calculating life history metrics from matrix population
models (MPMs).

Includes functions for:

- deriving life history traits
- deriving life tables or life table components
- deriving vital rates
- perturbation analyses
- manipulating and transforming MPMs

## Installation

Install the stable release package from CRAN with:

``` r
install.packages("Rage")
```

Install from GitHub with:

``` r
# install.packages("remotes")
remotes::install_github("jonesor/Rage")
```

### Usage

``` r
library(Rage)
```

### Loading an example MPM

The functions in Rage work on MPMs (or components of MPMs), so we’ll
start by loading one of the example MPMs included in the Rage package
(`mpm1`).

``` r
library(Rage) # load Rage
data(mpm1) # load data object 'mpm1'

mpm1
#> $matU
#>         seed small medium large dormant
#> seed    0.10  0.00   0.00  0.00    0.00
#> small   0.05  0.12   0.10  0.00    0.00
#> medium  0.00  0.35   0.12  0.23    0.12
#> large   0.00  0.03   0.28  0.52    0.10
#> dormant 0.00  0.00   0.16  0.11    0.17
#> 
#> $matF
#>         seed small medium large dormant
#> seed       0     0   17.9  45.6       0
#> small      0     0    0.0   0.0       0
#> medium     0     0    0.0   0.0       0
#> large      0     0    0.0   0.0       0
#> dormant    0     0    0.0   0.0       0
```

The object `mpm1` is a list containing two elements: the growth/survival
component of the MPM (the **U** matrix), and the sexual reproduction
component (the **F** matrix). We can obtain the full MPM by adding the
two components together (**A** = **U** + **F**).

### Deriving life history traits from an MPM

One of the most common arguments among functions in Rage is `start`,
which is used to specify the stage class that represents the ‘beginning
of life’ for the purposes of calculation. Because the first stage class
in `mpm1` is a ‘seed’ stage, which we might consider
functionally-distinct from the ‘above-ground’ stages, we’ll specify
`start = 2` to set our starting stage class of interest to the ‘small’
stage.

``` r
life_expect_mean(mpm1$matU, start = 2) # life expectancy
#> [1] 2.509116
longevity(mpm1$matU, start = 2, lx_crit = 0.05) # longevity (age at lx = 0.05)
#> [1] 7
mature_age(mpm1$matU, mpm1$matF, start = 2) # mean age at first reproduction
#>    small 
#> 2.136364
mature_prob(mpm1$matU, mpm1$matF, start = 2) # prob survival to first repro
#> [1] 0.4318182
```

Some life history traits are independent of the starting stage class, in
which case we don’t need to specify `start`.

``` r
net_repro_rate(mpm1$matU, mpm1$matF) # net reproductive rate
#> [1] 1.852091
gen_time(mpm1$matU, mpm1$matF) # generation time
#> [1] 5.394253
```

Some life history traits are calculated from a life table rather than an
MPM. For example, traits like entropy (`entropy_k_stage`) and shape
measures (`shape_surv`, `shape_rep`) etc. In these cases, the
calculation of age trajectories (lx and or mx trajectories) is handled
internally. Nevertheless, it can still be useful to obtain the
trajectories directly, in which case we can first use the `mpm_to_`
group of functions.

``` r
# first derive age-trajectories of survivorship (lx) and fecundity (mx)
lx <- mpm_to_lx(mpm1$matU, start = 2)
mx <- mpm_to_mx(mpm1$matU, mpm1$matF, start = 2)
```

### Life tables and the quasi-stationary distribution

Some MPMs are parameterized with a stasis loop at the maximum stage
class, which can lead to apparent plateaus in mortality or fertility
trajectories derived using age-from-stage methods. The function
`qsd_converge()` can be used to identify the time it takes for a cohort
to reach the quasi-stationary distribution (QSD). This quantity can then
be used to subset age trajectories of mortality or fertility to periods
earlier than the QSD, so as to avoid artefactual plateaus in mortality
or fertility.

``` r
# derive life table from MPM
lt <- mpm_to_table(mpm1$matU, start = 2)

# calculate time to QSD
(q <- qsd_converge(mpm1$matU, start = 2))
#> [1] 6

# plot mortality trajectory w/ vertical line at time to QSD
par(mar = c(4.5, 4.5, 1, 1))
plot(qx ~ x, data = lt, type = "l", ylim = c(0, 0.65))
abline(v = q, lty = 2)
```

![](man/figures/unnamed-chunk-9-1.png)<!-- -->

From the life table derived from `mpm1`, we can see a plateau in the
mortality rate (qx) beginning around age 5. However, this plateau
corresponds to the QSD and is therefore probably an artefact of the
stasis loop rather than a biological reality for the population
represented by `mpm1`.

One approach to accounting for this artefactual plateau in subsequent
life history calculations is to limit our life table to the period prior
to the QSD.

``` r
# calculate the shape of the survival/mortality trajectory
shape_surv(lt$lx) # based on full lx trajectory
#> [1] -0.04681254
shape_surv(lt$lx[1:q]) # based on lx trajectory prior to the QSD
#> [1] -0.06573764
```

### Standardized vital rates

The transition rates that make up MPMs generally reflect products of two
or more vital rates (sometimes called ‘lower-level vital rates’).
Assuming a post-breeding census design, we can retroactively break apart
each transition rate into at least two vital rate components: survival,
and ‘something’ conditional on survival. That ‘something’ might be
growth, shrinkage, stasis, dormancy, fecundity, or clonality.

##### Stage-specific vital rates (vector)

To summarize vital rates *within* stage classes, we can use the
`vr_vec_` group of functions. We’ll use the `exclude` argument here to
exclude certain stage classes (‘seed’ and ‘dormant’) from the
calculation of certain vital rates (e.g. we don’t consider the
large-to-dormant transition to actually represent ‘growth’).

``` r
vr_vec_survival(mpm1$matU)
#>    seed   small  medium   large dormant 
#>    0.15    0.50    0.66    0.86    0.39
vr_vec_growth(mpm1$matU, exclude = c(1, 5))
#>      seed     small    medium     large   dormant 
#>        NA 0.7600000 0.4242424        NA        NA
vr_vec_shrinkage(mpm1$matU, exclude = 5)
#>      seed     small    medium     large   dormant 
#>        NA        NA 0.1515152 0.2674419        NA
vr_vec_stasis(mpm1$matU)
#>      seed     small    medium     large   dormant 
#> 0.6666667 0.2400000 0.1818182 0.6046512 0.4358974
vr_vec_dorm_enter(mpm1$matU, dorm_stages = 5)
#>      seed     small    medium     large   dormant 
#>        NA        NA 0.2424242 0.1279070        NA
vr_vec_dorm_exit(mpm1$matU, dorm_stages = 5)
#>      seed     small    medium     large   dormant 
#>        NA        NA        NA        NA 0.5641026
vr_vec_reproduction(mpm1$matU, mpm1$matF)
#>     seed    small   medium    large  dormant 
#>       NA       NA 27.12121 53.02326       NA
```

##### MPM-specific vital rates (scalar)

To summarize vital rates *across* stage classes, we can use the `vr_`
group of functions. By default these functions take a simple average of
the stage-specific vital rates produced by the corresponding `vr_vec_`
function. However, here we’ll demonstrate how to specify a *weighted*
average across stages, based on the stable stage distribution at
equilibrium (*w*).

``` r
# derive full MPM (matA)
mpm1$matA <- mpm1$matU + mpm1$matF

# calculate stable stage distribution at equilibrium using popdemo::eigs
library(popdemo)
#> Welcome to popdemo! This is version 1.3-0
#> Use ?popdemo for an intro, or browseVignettes('popdemo') for vignettes
#> Citation for popdemo is here: doi.org/10.1111/j.2041-210X.2012.00222.x
#> Development and legacy versions are here: github.com/iainmstott/popdemo
w <- popdemo::eigs(mpm1$matA, what = "ss")

# calculate MPM-specific vital rates
vr_survival(mpm1$matU, exclude_col = c(1, 5), weights_col = w)
#> [1] 0.5963649
vr_growth(mpm1$matU, exclude = c(1, 5), weights_col = w)
#> [1] 0.6602975
vr_shrinkage(mpm1$matU, exclude = c(1, 5), weights_col = w)
#> [1] 0.1960601
vr_stasis(mpm1$matU, exclude = c(1, 5), weights_col = w)
#> [1] 0.2824323
vr_dorm_enter(mpm1$matU, dorm_stages = 5, weights_col = w)
#> [1] 0.1984209
vr_dorm_exit(mpm1$matU, dorm_stages = 5, weights_col = w)
#> [1] 0.5641026
vr_fecundity(mpm1$matU, mpm1$matF, weights_col = w)
#> [1] 37.07409
```

Note how we’ve chosen to exclude the ‘seed’ and ‘dormant’ stage classes
from our vital rate summaries, because we consider these to be special
classes (e.g. ‘growth’ from the ‘seed’ stage is really ‘germination’,
which we may think of as separate from somatic growth from ‘small’ to
‘medium’, or ‘medium’ to ‘large’).

### Perturbation analyses

The `perturb_matrix()` function measures the response of a demographic
statistic to perturbation of individual matrix elements
(i.e. sensitivities and elasticities). The `perturb_vr()` and
`perturb_trans()` functions implement perturbation analyses by vital
rate type (survival, growth, etc.) and transition type (stasis,
retrogression, etc.), respectively.

``` r
# matrix element perturbation
perturb_matrix(mpm1$matA, type = "sensitivity")
#>               seed      small      medium       large     dormant
#> seed     0.2173031 0.01133203 0.004786308 0.002986833 0.001150703
#> small    4.4374613 0.23140857 0.097739870 0.060993320 0.023498191
#> medium  10.8654599 0.56661979 0.239323184 0.149346516 0.057537001
#> large   21.3053309 1.11104269 0.469270739 0.292842885 0.112820081
#> dormant  3.6111989 0.18831947 0.079540419 0.049636195 0.019122779

# vital rate perturbation
# (we use as.data.frame here for prettier printing)
as.data.frame(perturb_vr(mpm1$matU, mpm1$matF, type = "sensitivity"))
#>   survival   growth  shrinkage  fecundity clonality
#> 1 2.986054 1.077597 -0.1653284 0.00572764         0

# transition type perturbation
as.data.frame(perturb_trans(mpm1$matU, mpm1$matF, type = "sensitivity"))
#>     stasis     retro    progr   fecundity clonality
#> 1 1.000001 0.4174435 6.713571 0.007773141        NA
```

### Transforming MPMs

Rage includes a variety of functions that can be used to manipulate or
transform MPMs. For example, we can collapse an MPM to a smaller number
of stage classes using `mpm_collapse()`.

``` r
# collapse 'small', 'medium', and 'large' stages into single stage class
col1 <- mpm_collapse(mpm1$matU, mpm1$matF, collapse = list(1, 2:4, 5))
col1$matA
#>      [,1]        [,2] [,3]
#> [1,] 0.10 11.61331815 0.00
#> [2,] 0.05  0.53908409 0.22
#> [3,] 0.00  0.05728085 0.17
```

The transition rates in the collapsed matrix are a weighted average of
the transition rates from the relevant stages of the original matrix,
weighted by the stable distribution at equilibrium. This process
guarantees that the collapsed MPM will retain the same population growth
rate as the original. However, other demographic and life history
characteristics will not necessarily be preserved.

``` r
# compare population growth rate of original and collapsed MPM (preserved)
popdemo::eigs(mpm1$matA, what = "lambda")
#> [1] 1.121037
popdemo::eigs(col1$matA, what = "lambda")
#> [1] 1.121037

# compare net reproductive rate of original and collapsed MPM (not preserved)
net_repro_rate(mpm1$matU, mpm1$matF)
#> [1] 1.852091
net_repro_rate(col1$matU, col1$matF)
#> [1] 1.447468
```

For a complete list of functions see the package
[Reference](https://jonesor.github.io/Rage/reference/index.html) page.

## Previous releases

Specific earlier releases of this package can be installed using the
appropriate `@` tag.

For example to install version 1.0.0:

``` r
remotes::install_github("jonesor/Rage@v1.0.0")
```

See the Changelog for more details.

## Citation

Jones, O. R., Barks, P., Stott, I., James, T. D., Levin, S., Petry, W.
K., Capdevila, P., Che-Castaldo, J., Jackson, J., Römer, G., Schuette,
C., Thomas, C. C., & Salguero-Gómez, R. (2022). `Rcompadre` and
`Rage`—Two R packages to facilitate the use of the COMPADRE and COMADRE
databases and calculation of life-history traits from matrix population
models. *Methods in Ecology and Evolution*, 13, 770–781.

## Contributions

All contributions are welcome. Please note that this project is released
with a [Contributor Code of
Conduct](https://contributor-covenant.org/version/2/0/CODE_OF_CONDUCT.html).
By participating in this project you agree to abide by its terms.

There are numerous ways of contributing.

1.  You can submit bug reports, suggestions etc. by [opening an
    issue](https://github.com/jonesor/Rage/issues).

2.  You can copy or fork the repository, make your own code edits and
    then send us a pull request. [Here’s how to do
    that](https://jarv.is/notes/how-to-pull-request-fork-github/).

3.  You can get to know us and join as a collaborator on the main
    repository.

4.  You are also welcome to email us.

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----echo=FALSE,fig.align='center',fig.height=2,fig.width=6-------------------
# hidden code to produce figures
library(DiagrammeR)
matA <- rbind(
  c(0.0, 0.0, 3.2),
  c(0.5, 0.3, 0.8),
  c(0.0, 0.4, 0.9)
)
stages <- c("seedling", "rosette", "flowering")
title <- NULL
graph <- expand.grid(to = stages, from = stages)
graph$trans <- round(c(matA), 3)
graph <- graph[graph$trans > 0, ]
nodes <- paste(paste0("'", stages, "'"), collapse = "; ")
graph$min_len <- (as.numeric(graph$to) - as.numeric(graph$from)) * 3
graph$col <- c(
  "PaleGreen4", "PaleGreen4", "PaleGreen4", "Goldenrod1",
  "MediumOrchid4", "PaleGreen4"
)
edges <- paste0("'", graph$from, "'", " -> ", "'", graph$to, "'",
  "[minlen=", graph$min_len,
  ",fontsize=", 10,
  ",color=", graph$col,
  ",xlabel=", paste("\"", graph$trans),
  "\"]\n",
  collapse = ""
)
grViz(
  paste(
    "
digraph {
  {
    graph[overlap=false];
    rank=same;
    node [shape=", "egg", ", fontsize=", 12, "];",
    nodes, "
  }",
    "ordering=out
  x [style=invis]
  x -> {", nodes, "} [style=invis]", edges,
    "labelloc=\"t\";
  label=\"", title, "\"
}"
  )
)

## ----echo=FALSE,fig.align='center',fig.height=4,fig.width=4-------------------
library(ggplot2)
ggdat <- merge(graph,
  expand.grid(to = stages, from = stages),
  by = c("to", "from"),
  all.x = TRUE, all.y = TRUE
)
ggdat$trans[is.na(ggdat$trans)] <- 0
ggdat$col[is.na(ggdat$col)] <- "transparent"
ggdat$to <- factor(ggdat$to, levels = c("flowering", "rosette", "seedling"))
ggdat$from <- factor(ggdat$from, levels = c("seedling", "rosette", "flowering"))
ggplot(ggdat, aes(x = from, y = to, label = trans)) +
  geom_tile(color = "black", fill = "white", linewidth = 0.25, show.legend = FALSE) +
  geom_text(size = 6) +
  scale_x_discrete(position = "top") +
  labs(x = "current life stage", y = "life stage at time t+1") +
  coord_equal(expand = FALSE) +
  theme_bw(base_size = 18) +
  theme(panel.border = element_blank())

## ----echo=FALSE,fig.align='center',fig.height=4,fig.width=9,out.width='100%'----
blankdat <- expand.grid(to = stages, from = stages, trans = 0)
blankdat$to <- factor(blankdat$to, levels = c(
  "flowering", "rosette",
  "seedling"
))
blankdat$from <- factor(blankdat$from,
  levels = c("seedling", "rosette", "flowering")
)
ggdat$col <- factor(ggdat$col,
  levels = c(
    "PaleGreen4", "Goldenrod1",
    "MediumOrchid4", "transparent"
  ),
  labels = c("U", "F", "C", "t")
)
ggplot(
  ggdat[ggdat$col != "t", ],
  aes(x = from, y = to, fill = col, label = trans)
) +
  geom_tile(
    data = blankdat,
    aes(fill = NULL), fill = "white", color = "black", linewidth = 0.25
  ) +
  geom_text(data = blankdat, aes(fill = NULL), size = 6) +
  geom_tile(color = "black", linewidth = 0.25, show.legend = FALSE) +
  geom_text(size = 6) +
  scale_x_discrete(position = "top") +
  scale_fill_manual(values = c(
    "F" = "goldenrod1",
    "C" = "mediumorchid4",
    "U" = "palegreen4",
    "t" = "white"
  )) +
  labs(x = "current life stage", y = "life stage at time t+1") +
  coord_equal(expand = FALSE) +
  facet_wrap(~col, nrow = 1) +
  theme_bw(base_size = 18) +
  theme(
    panel.border = element_blank(),
    strip.text = element_text(face = "bold"),
    strip.placement = "outside"
  )

## ----message=FALSE------------------------------------------------------------
library(popdemo)

# define the transition matrix, A
A <- rbind(
  c(0.0, 0.0, 3.2),
  c(0.5, 0.3, 0.8),
  c(0.0, 0.4, 0.9)
)

# lambda: equilibrium per-capita population growth rate
popdemo::eigs(A = A, what = "lambda")

# w: stable stage distribution (relative frequencies)
popdemo::eigs(A = A, what = "ss")

## -----------------------------------------------------------------------------
library(Rage) # load Rage
data(mpm1) # load data object 'mpm1'
mpm1 # display the contents

## ----echo=FALSE, message=FALSE, warnings=FALSE, results='asis'----------------
tabl <- "
| Function category                       | Stand-alone vignette             |
|-----------------------------------------|----------------------------------|
| 1. [Vital rates](#vitalrates)  | [VitalRates](a02_VitalRates.html)     |
| 2. [Life tables](#lifetable) | [AgeFromStage](a04_AgeFromStage.html) |
| 3. [Perturbation analysis](#perturb)    | n/a |
| 4. [Deriving life history traits](#lifehist) | [LifeHistoryTraits](a03_LifeHistoryTraits.html) |
| 5. [Transformation of matrices](#maniptransform) | n/a                                   |
"
cat(tabl)

## -----------------------------------------------------------------------------
vr_vec_survival(matU = mpm1$matU)
vr_vec_stasis(matU = mpm1$matU)

## -----------------------------------------------------------------------------
# product of Pr(survival) and Pr(stasis) yields Pr(stasis|survived)
vr_vec_survival(matU = mpm1$matU) * vr_vec_stasis(matU = mpm1$matU)
diag(mpm1$matU) # equivalent to the diagonal of U matrix

## -----------------------------------------------------------------------------
vr_survival(matU = mpm1$matU, exclude_col = 1) # exclude 'seed' stage
mean(vr_vec_survival(mpm1$matU)[-1]) # equivalent to the mean without 'seed'

## -----------------------------------------------------------------------------
lt <- mpm_to_table(matU = mpm1$matU, matF = mpm1$matF) # full life table
lt
lx <- mpm_to_lx(matU = mpm1$matU) # survivorship to start of each age class
lx

## -----------------------------------------------------------------------------
lx_to_px(lx = lx) # survivorship to survival probability
lx_to_hx(lx = lx) # survivorship to mortality hazard

## ----warning=FALSE, message=FALSE, fig.align='center', fig.height=5, fig.width=6----
# project a germinated cohort through the U matrix
cohort <- popdemo::project(A = mpm1$matU, vector = c(0, 1, 0, 0, 0), time = 10)
popStructure <- vec(cohort) / rowSums(vec(cohort))

matplot(popStructure,
  type = "l", xlab = "time",
  ylab = "proportion in stage class"
)

## -----------------------------------------------------------------------------
# calculate time to QSD from the U matrix of an MPM
(q <- qsd_converge(mat = mpm1$matU, start = "small"))

# subset the life table rows to ages prior to the QSD
lt_preQSD <- lt[1:q, ]

# plot mortality trajectory from the life table subset (blue),
# showing plateau effect if the trajectory (grey) was allowed to continue to the
# QSD (dashed vertical line) and beyond
plot(qx ~ x,
  data = lt, type = "l", col = "darkgrey", ylim = c(0, 1),
  xlab = "age"
)
lines(qx ~ x, data = lt_preQSD, type = "l", col = "blue", lwd = 4)
abline(v = q, lty = "dashed")

## -----------------------------------------------------------------------------
# construct the transition matrix A = U + F (+ C when present)
mpm1$matA <- with(mpm1, matU + matF)

# sensitivity of lambda to...
# ...matrix element perturbations
perturb_matrix(
  matA = mpm1$matA,
  type = "sensitivity", demog_stat = "lambda"
)
# ...vital rate perturbations
perturb_vr(
  matU = mpm1$matU, matF = mpm1$matF,
  type = "sensitivity", demog_stat = "lambda"
)
# ...transition type perturbations
perturb_trans(
  matU = mpm1$matU, matF = mpm1$matF,
  type = "sensitivity", demog_stat = "lambda"
)

## -----------------------------------------------------------------------------
# post-germination time steps until post-germination survivorship falls below 5%
longevity(matU = mpm1$matU, start = "small", lx_crit = 0.05)
# expected lifetime production of 'small' offspring by a 'small' individual
net_repro_rate(
  matU = mpm1$matU, matR = mpm1$matF, start = "seed",
  method = "start"
)

## -----------------------------------------------------------------------------
# derive post-germination survivorship trajectory from U matrix
lx <- mpm_to_lx(matU = mpm1$matU, start = "small")

## -----------------------------------------------------------------------------
# collapse 'small', 'medium', and 'large' stages into single stage class
col1 <- mpm_collapse(
  matU = mpm1$matU, matF = mpm1$matF,
  collapse = list(1, 2:4, 5)
)
col1$matA

## -----------------------------------------------------------------------------
# automated stage naming
(col1_auto <- name_stages(mat = col1, prefix = "class_"))

# overwrite with custom stages
(col1_cust <- name_stages(
  mat = col1, names = c("seed", "active", "dormant"),
  prefix = NULL
))

## -----------------------------------------------------------------------------
# compare population growth rate of original and collapsed MPM (preserved)
popdemo::eigs(A = mpm1$matA, what = "lambda")
popdemo::eigs(A = col1_cust$matA, what = "lambda")

# compare net reproductive rate of original and collapsed MPM (not preserved)
net_repro_rate(matU = mpm1$matU, matR = mpm1$matF)
net_repro_rate(matU = col1_cust$matU, matR = col1_cust$matF)

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## -----------------------------------------------------------------------------
library(Rage) # load Rage
data(mpm1) # load data object 'mpm1'
mpm1 # display the contents

## -----------------------------------------------------------------------------
dimnames(mpm1$matU)

# extract U and F matrices
mat_U <- mpm1$matU
mat_F <- mpm1$matF

# calculate lx
lx <- mpm_to_lx(mat_U, start = 1, xmax = 30)

# calculate mx
mx <- mpm_to_mx(mat_U, mat_F, start = 1, xmax = 30)

## -----------------------------------------------------------------------------
# calculate lx
lx <- mpm_to_lx(mat_U, start = 1, xmax = 30)

# calculate mx
mx <- mpm_to_mx(mat_U, mat_F, start = 1, xmax = 30)

## ----fig.width = 6, fig.height = 4--------------------------------------------
plot(lx, ylim = c(0, 1), type = "l", xlab = "Age")
plot(mx, type = "l", xlab = "Age")

## -----------------------------------------------------------------------------
library(Rcompadre)
data(Compadre)

# In older versions of Com(p)adre the ProjectionInterval column was called
# AnnualPeriodicity.
if ("AnnualPeriodicity" %in% names(Compadre)) {
  Compadre$ProjectionInterval <- Compadre$AnnualPeriodicity
}

comp_flag <- cdb_flag(Compadre, "check_NA_U")

comp_use <- subset(comp_flag, OrganismType == "Tree" &
  check_NA_U == FALSE &
  ProjectionInterval == 1)

## -----------------------------------------------------------------------------
CompadreData(comp_use)[, c(
  "SpeciesAccepted", "MatrixPopulation",
  "MatrixTreatment"
)]

## -----------------------------------------------------------------------------
# add column ID-ing matrices with same MatrixClassAuthor vector
comp_use$stage_id <- cdb_id_stages(comp_use)

# collapse database to single matrix per species * MatrixClassAuthor
comp_collapse <- cdb_collapse(comp_use, "stage_id")

# check species/populations again
CompadreData(comp_collapse)[, c(
  "SpeciesAccepted", "MatrixPopulation",
  "MatrixTreatment"
)]

## -----------------------------------------------------------------------------
MatrixClassOrganized(comp_collapse)

## -----------------------------------------------------------------------------
comp_collapse$start_life <- mpm_first_active(comp_collapse)

## ----fig.width = 6, fig.height = 4--------------------------------------------
lx_list <- lapply(seq_len(nrow(comp_collapse)),
  function(x, comp_collapse) {
    U <- matU(comp_collapse$mat[[x]])

    rownames(U) <- colnames(U) # ensure row and col names are present

    mpm_to_lx(
      matU = U,
      start = comp_collapse$start_life[x],
      xmax = 40
    )
  },
  comp_collapse = comp_collapse
)

lx_array <- do.call(cbind, lx_list)

matplot(lx_array,
  type = "l", lty = 1, log = "y", ylim = c(0.0001, 1),
  lwd = 1.5, xlab = "Age (years)", ylab = "lx"
)

#' Perturbation analysis of transition types within a matrix population model
#'
#' @description Calculates the summed sensitivities or elasticities for various
#' transition types within a matrix population model (MPM), including stasis,
#' retrogression, progression, fecundity, and clonality.
#'
#' Sensitivities or elasticities are calculated by perturbing elements of the
#' MPM and measuring the response of the per-capita population growth rate at
#' equilibrium (\eqn{\lambda}), or, with a user-supplied function, any other
#' demographic statistic.
#'
#' @param matU The survival component submatrix of a MPM (i.e., a square
#'   projection matrix reflecting survival-related transitions; e.g.,
#'   progression, stasis, and retrogression).
#' @param matF The sexual component submatrix of a MPM (i.e., a square
#'   projection matrix reflecting transitions due to sexual reproduction).
#' @param matC The clonal component submatrix of a MPM (i.e., a square
#'   projection matrix reflecting transitions due to clonal reproduction).
#'   Defaults to \code{NULL}, indicating no clonal reproduction possible.
#' @param posU A logical matrix of the same dimension as \code{matU}, with
#'   elements indicating whether a given \code{matU} transition is possible
#'   (\code{TRUE}) or not (\code{FALSE}). Defaults to \code{matU > 0} (see
#'   Details).
#' @param posF A logical matrix of the same dimension as \code{matF}, with
#'   elements indicating whether a given \code{matF} transition is possible
#'   (\code{TRUE}) or not (\code{FALSE}). Defaults to \code{matF > 0} (see
#'   Details).
#' @param posC A logical matrix of the same dimension as \code{matC}, with
#'   elements indicating whether a given \code{matC} transition is possible
#'   (\code{TRUE}) or not (\code{FALSE}). Defaults to \code{matC > 0} (see
#'   Details).
#' @param exclude_row A vector of row indices or stages names indicating stages
#'   for which transitions \emph{to} the stage should be excluded from
#'   perturbation analysis. Alternatively, a logical vector of length
#'   \code{nrow(matU)} indicating which stages to include \code{TRUE} or exclude
#'   \code{FALSE} from the calculation. See section \emph{Excluding stages}.
#' @param exclude_col A vector of column indices or stages names indicating
#'   stages for which transitions \emph{to} the stage should be excluded from
#'   perturbation analysis. Alternatively, a logical vector of length
#'   \code{ncol(matU)} indicating which stages to include \code{TRUE} or exclude
#'   \code{FALSE} from the calculation. See section \emph{Excluding stages}.
#' @param pert The magnitude of the perturbation (defaults to \code{1e-6}).
#' @param type An argument defining whether to return `sensitivity` or
#'   `elasticity` values. Defaults to `sensitivity`.
#' @param demog_stat An argument defining which demographic statistic should be
#'   used, as in "the sensitivity/elasticity of \code{demog_stat} to matrix
#'   element perturbations." Defaults to the per-capita population growth rate
#'   at equilibrium (\eqn{lambda}). Also accepts a user-supplied function that
#'   performs a calculation on a MPM and returns a single numeric value.
#' @param ... Additional arguments passed to the function \code{demog_stat}.
#'
#' @details A transition rate of \code{0} within a matrix population model can
#' either indicate that the transition is not possible in the given life cycle
#' (e.g., tadpoles never revert to eggs), or that the transition is possible but
#' was estimated to be \code{0} in the relevant population and time period.
#' Because transition rates of zero \emph{do} generally yield non-zero
#' sensitivities, it is important to distinguish between structural (i.e.
#' impossible) zeros and sampled zeros when summing multiple sensitivities for a
#' given process (e.g., progression/growth).
#'
#' By default, the \code{perturb_} functions assume that a transition rate of
#' \code{0} indicates an impossible transition, in which case the sensitivity
#' for that transition will not be included in any calculation. Specifically,
#' the arguments \code{posX} are specified by the logical expression \code{(matX
#' > 0)}. If the matrix population model includes transitions that are possible
#' but estimated to be \code{0}, users should specify the \code{posX}
#' argument(s) manually.
#'
#' If there are no possible transitions for a given process (e.g., clonality, in
#' many species), the value of sensitivity or elasticity returned for that
#' process will be \code{NA}.
#'
#' @return A list with 5 elements: \item{stasis}{The sensitivity or elasticity
#'   of \code{demog_stat} to stasis.} \item{retrogression}{The sensitivity or
#'   elasticity of \code{demog_stat} to retrogression.} \item{progression}{The
#'   sensitivity or elasticity of \code{demog_stat} to progression.}
#'   \item{fecundity}{The sensitivity or elasticity of \code{demog_stat} to
#'   sexual fecundity.} \item{clonality}{The sensitivity or elasticity of
#'   \code{demog_stat} to clonality.}
#'
#' @section Excluding stages: It may be desirable to exclude one or more stages
#'   from the calculation. For instance, we might not believe that 'progression'
#'   to a dormant stage class truly reflects progression. In this case we could
#'   exclude transitions \emph{to} the dormant stage class using the argument
#'   \code{exclude_row}. We may or may not want to ignore progression
#'   transitions \emph{from} the dormant stage class, which can be done in a
#'   similar way using the argument \code{exclude_col}. The \code{exclude_}
#'   arguments simply set the relevant row or column of the \code{posX}
#'   arguments to \code{FALSE}, to prevent those transitions from being used in
#'   subsequent calculations.
#'
#' @author Rob Salguero-Gómez <rob.salguero@@zoo.ox.ac.uk>
#' @author Patrick Barks <patrick.barks@@gmail.com>
#'
#' @family perturbation analysis
#'
#' @examples
#' matU <- rbind(
#'   c(0.1, 0, 0, 0),
#'   c(0.5, 0.2, 0.1, 0),
#'   c(0, 0.3, 0.3, 0.1),
#'   c(0, 0, 0.5, 0.6)
#' )
#'
#' matF <- rbind(
#'   c(0, 0, 1.1, 1.6),
#'   c(0, 0, 0.8, 0.4),
#'   c(0, 0, 0, 0),
#'   c(0, 0, 0, 0)
#' )
#'
#'
#' perturb_trans(matU, matF)
#'
#' # Use a larger perturbation than the default of 1e-6.
#' perturb_trans(matU, matF, pert = 0.01)
#'
#' # Calculate the sensitivity/elasticity of the damping ratio to perturbations.
#' # First, define function for damping ratio:
#' damping <- function(matA) {
#'   eig <- eigen(matA)$values
#'   dm <- rle(Mod(eig))$values
#'   return(dm[1] / dm[2])
#' }
#'
#' # Second, run the perturbation analysis using demog_stat = "damping".
#' perturb_trans(matU, matF, demog_stat = "damping")
#'
#' @export perturb_trans
perturb_trans <- function(matU, matF, matC = NULL,
                          posU = matU > 0, posF = matF > 0, posC = matC > 0,
                          exclude_row = NULL, exclude_col = NULL,
                          pert = 1e-6, type = "sensitivity",
                          demog_stat = "lambda", ...) {
  # Validate arguments
  checkValidMat(matU)
  checkValidMat(matF)

  if (!is.null(matC)) {
    checkValidMat(matC, warn_all_zero = FALSE)
  }
  checkValidStages(matU, exclude_row)
  checkValidStages(matU, exclude_col)
  type <- match.arg(type, c("sensitivity", "elasticity"))

  # Get statfun
  if (is.character(demog_stat) && demog_stat == "lambda") {
    statfun <- lambda
  } else {
    statfun <- try(match.fun(demog_stat), silent = TRUE)
    if (inherits(statfun, "try-error")) {
      stop(strwrap(
        prefix = " ", initial = "",
        "`demog_stat` must be `lambda` or the name
      of a function that returns a single numeric value.\n"
      ), call. = FALSE)
    }
  }

  # Matrix dimension
  m <- nrow(matU)

  # If matC null, convert to zeros
  if (is.null(matC)) {
    matC <- matrix(0, m, m)
    posC <- matrix(FALSE, m, m)

    if (!is.null(dimnames(matU))) {
      dimnames(posC) <- dimnames(matC) <- dimnames(matU)
    }
  }

  # Excluded stage classes
  posU[exclude_row, ] <- FALSE
  posU[, exclude_col] <- FALSE
  posF[exclude_row, ] <- FALSE
  posF[, exclude_col] <- FALSE
  posC[exclude_row, ] <- FALSE
  posC[, exclude_col] <- FALSE

  # Combine components into matA
  matA <- matU + matF + matC

  # Lower and upper triangles (reflecting growth and retrogression)
  lwr <- upr <- matrix(FALSE, nrow = m, ncol = m)
  lwr[lower.tri(lwr)] <- TRUE
  upr[upper.tri(upr)] <- TRUE

  posStasi <- posU & diag(m)
  posRetro <- posU & upr
  posProgr <- posU & lwr

  # Get sensitivity or elasticity
  pertMat <- perturb_matrix(
    matA = matA, pert = pert, type = type,
    demog_stat = statfun, ...
  )

  if (type == "sensitivity") {
    stasis <- ifelse(!any(posStasi), NA_real_, sum(pertMat[posStasi]))
    retro <- ifelse(!any(posRetro), NA_real_, sum(pertMat[posRetro]))
    progr <- ifelse(!any(posProgr), NA_real_, sum(pertMat[posProgr]))
    fecund <- ifelse(!any(posF), NA_real_, sum(pertMat[posF]))
    clonal <- ifelse(!any(posC), NA_real_, sum(pertMat[posC]))
  } else {
    propU <- matU / matA
    propU[!posU] <- NA_real_
    propU[matA == 0 & posU] <- 1

    propProgr <- propRetro <- propU
    propProgr[upper.tri(propU, diag = TRUE)] <- NA
    propRetro[lower.tri(propU, diag = TRUE)] <- NA

    propStasi <- matrix(NA_real_, nrow = m, ncol = m)
    diag(propStasi) <- diag(propU)

    propF <- matF / matA
    propF[!posF] <- NA_real_
    propF[matA == 0 & posF] <- 1

    propC <- matC / matA
    propC[!posC] <- NA_real_
    propC[matA == 0 & posC] <- 1

    stasis <- sum_elast(pertMat, posStasi, propStasi)
    retro <- sum_elast(pertMat, posRetro, propRetro)
    progr <- sum_elast(pertMat, posProgr, propProgr)
    fecund <- sum_elast(pertMat, posF, propF)
    clonal <- sum_elast(pertMat, posC, propC)
  }

  return(list(
    stasis = stasis, retro = retro, progr = progr,
    fecundity = fecund, clonality = clonal
  ))
}


#  A convenience function to sum elasticities given the perturbation matrix, the
#  matrix of possible transitions, and the matrix reflecting the proportional
#  contribution of the given process to the given element.
sum_elast <- function(pert_mat, pos_mat, prop_mat) {
  ifelse(!any(pos_mat), NA_real_, sum(pert_mat * prop_mat, na.rm = TRUE))
}

test_that("perturb_trans works correctly", {
  x_sens <- perturb_trans(mat_u, mat_f)
  expect_type(x_sens, "list")
  expect_identical(length(x_sens), 5L)
  expect_true(is.na(x_sens$clonality))

  x_elas <- perturb_trans(mat_u, mat_f, type = "elasticity")
  expect_type(x_elas, "list")
  expect_identical(length(x_elas), 5L)
  expect_true(is.na(x_elas$clonality))

  x_f_zero <- suppressWarnings(
    perturb_trans(mat_u, matF = mat_f_zero, matC = mat_c)
  )
  expect_true(is.na(x_f_zero$fecundity))

  # check no stasis/retrogression in age-only model
  x_age <- perturb_trans(mat_u_age, mat_f_age)
  expect_true(is.na(x_age$stasis))
  expect_true(is.na(x_age$retro))

  # check works with custom demog_stat function
  fn_custom <- function(x) {
    return(1)
  }
  x_cust <- perturb_trans(mat_u, mat_f, mat_c, demog_stat = "fn_custom")
  expect_true(all(x_cust == 0))
})


test_that("perturb_trans warns and fails gracefully", {
  notfn <- "rtpsqwpaclfkhamw"
  expect_error(perturb_trans(mat_u, mat_f, demog_stat = notfn))
})

#' Shannon Evenness (Pielou's J)
#'
#' Computes the general Shannon evenness index (aka Pielou's J), defined as the
#' ratio of the observed Shannon entropy to the maximum possible entropy given
#' the number of categories. This function is included here because Shannon
#' evenness can be applied to stage-at-death distributions to quantify how
#' evenly deaths are distributed across stages.
#'
#' @param p A numeric vector of counts or probabilities. If counts are
#'   provided, they are normalised to sum to 1 internally.
#'
#' @return A numeric value between 0 and 1 representing Shannon evenness. A
#'   value of 1 indicates an even distribution, while low values indicate uneven
#'   distributions. A value of 0 indicates that all deaths are lumped into one
#'   stage.
#'
#' @details Zero entries in \code{p} contribute nothing to entropy
#'   but do count towards the total number of categories when computing
#'   the maximum entropy (\eqn{H_{\max} = \log(K)}).
#'
#' @examples
#' # Even
#' evenness_shannon(c(0.2, 0.2, 0.2, 0.2, 0.2))
#'
#' # Skewed
#' evenness_shannon(c(0.8, 0.1, 0.05, 0.03, 0.02))
#'
#' evenness_shannon(c(1, 0, 0, 0, 0))
#'
#' # From a matrix model's stage at death distribution
#' data(mpm1)
#' matA <- mpm1$matU + mpm1$matF
#' ssd <- popdemo::eigs(matA, "ss")
#' x <- stage_at_death_dist(mpm1$matU, start = ssd)
#' evenness_shannon(x)
#'
#' @export
evenness_shannon <- function(p) {
  # 1) Normalise
  p <- p / sum(p)

  # 2) Compute per-category contributions, zeros handled explicitly
  nz <- p > 0
  contrib <- numeric(length(p)) # default 0 for p == 0
  contrib[nz] <- p[nz] * log(p[nz]) # compute only where p > 0

  # 3) Entropy and evenness
  H <- -sum(contrib)
  J <- H / log(length(p))
  return(J)
}

#' Calculate age-specific traits from a matrix population model
#'
#' These functions use age-from-stage decomposition methods to calculate
#' age-specific survivorship (\code{lx}), survival probability (\code{px}),
#' mortality hazard (\code{hx}), or reproduction (\code{mx}) from a matrix
#' population model (MPM). A detailed description of these methods can be found
#' in sections 5.3.1 and 5.3.2 of Caswell (2001). A separate function
#' \code{\link{mpm_to_table}} uses the same methods to calculate a full life
#' table.
#'
#' @param matU The survival component of a MPM (i.e., a
#'   square projection matrix reflecting survival-related transitions; e.g.,
#'   progression, stasis, and retrogression). Optionally with named rows and
#'   columns indicating the corresponding life stage names.
#' @param matR The reproductive component of a matrix population model (i.e., a
#'   square projection matrix only reflecting transitions due to reproduction;
#'   either sexual, clonal, or both). If \code{matR} is supplied, \code{matF}
#'   and \code{matC} are ignored. Otherwise, \code{matR} is constructed by
#'   summing \code{matF} and \code{matC}.
#' @param matF The matrix reflecting sexual reproduction. If provided
#'   without \code{matC}, \code{matC} is assumed to be a zero matrix. If
#'   \code{matR} is provided, this argument is ignored.
#' @param matC The matrix reflecting clonal (asexual) reproduction.
#'   If provided without \code{matF}, \code{matF} is assumed to be a zero
#'   matrix. If \code{matR} is provided, this argument is ignored.
#' @param start The index (or stage name) of the first stage at which the author
#'   considers the beginning of life. Defaults to \code{1}. Alternately, a
#'   numeric vector giving the starting population vector (in which case
#'   \code{length(start)} must match \code{ncol(matU))}. See section
#'   \emph{Starting from multiple stages}.
#' @param xmax Maximum age to which age-specific traits will be calculated
#'   (defaults to \code{1000}).
#' @param lx_crit Minimum value of \code{lx} to which age-specific traits will
#'   be calculated (defaults to \code{0.01}).
#' @param tol To account for floating point errors that occasionally lead to
#'   values of \code{lx} slightly greater than 1, values of \code{lx} within the
#'   open interval (\code{1}, \code{1 + tol}) are coerced to 1. Defaults to
#'   \code{0.0001}. To prevent coercion, set \code{tol} to \code{0}.
#'
#' @return A vector
#'
#' @note Note that the units of time for the returned vectors (i.e., \code{x})
#'   are the same as the projection interval (\code{ProjectionInterval}) of the
#'   MPM.
#'
#'
#' @section Starting from multiple stages:
#' Rather than specifying argument \code{start} as a single stage class from
#' which all individuals start life, it may sometimes be desirable to allow for
#' multiple starting stage classes. For example, if users want to start their
#' calculation of age-specific traits from reproductive maturity (i.e., first
#' reproduction), they should account for the possibility that there may be
#' multiple stage classes in which an individual could first reproduce.
#'
#' To specify multiple starting stage classes, users should specify argument
#' \code{start} as the desired starting population vector (\strong{n1}), giving
#' the proportion of individuals starting in each stage class (the length of
#' \code{start} should match the number of columns in the relevant MPM).
#'
#' See function \code{\link{mature_distrib}} for calculating the proportion of
#' individuals achieving reproductive maturity in each stage class.
#'
#' @note The output vector is calculated recursively until the age class
#'   (\code{x}) reaches \code{xmax} or survivorship (\code{lx}) falls below
#'   \code{lx_crit}, whichever comes first. To force calculation to \code{xmax},
#'   set \code{lx_crit} to \code{0}. Conversely, to force calculation to
#'   \code{lx_crit}, set \code{xmax} to \code{Inf}.
#'
#' @note For \code{mpm_to_mx()}, the output reflects reproductive output as
#'   represented at the census boundary. As a result, \code{mx} trajectories
#'   from pre-breeding and post-breeding models are not necessarily directly
#'   comparable. See Kendall et al. (2019).
#'
#' @note Note that the units of time in returned values (i.e., \code{x}) are the
#'   same as the projection interval (`ProjectionInterval`) of the MPM.
#'
#' @author Owen R. Jones <jones@@biology.sdu.dk>
#' @author Roberto Salguero-Gómez <rob.salguero@@zoo.ox.ac.uk>
#' @author Hal Caswell <h.caswell@@uva.nl>
#' @author Patrick Barks <patrick.barks@@gmail.com>
#'
#' @family life tables
#'
#' @seealso \code{\link{lifetable_convert}}
#'
#' @references
#' Caswell, H. 2001. Matrix Population Models: Construction, Analysis, and
#' Interpretation. Sinauer Associates; 2nd edition. ISBN: 978-0878930968
#'
#' Jones O. R. 2021. Life tables: Construction and interpretation In:
#' Demographic Methods Across the Tree of Life. Edited by Salguero-Gomez R &
#' Gamelon M. Oxford University Press. Oxford, UK. ISBN: 9780198838609
#'
#' Preston, S., Heuveline, P., & Guillot, M. 2000. Demography: Measuring and
#' Modeling Population Processes. Wiley. ISBN: 9781557864512
#'
#' @examples
#' data(mpm1)
#'
#' # age-specific survivorship
#' mpm_to_lx(mpm1$matU)
#' mpm_to_lx(mpm1$matU, start = 2) # starting from stage 2
#' mpm_to_lx(mpm1$matU, start = "small") # equivalent using named life stages
#' mpm_to_lx(mpm1$matU, xmax = 10) # to a maximum age of 10
#' mpm_to_lx(mpm1$matU, lx_crit = 0.05) # to a minimum lx of 0.05
#'
#' # age-specific survival probability
#' mpm_to_px(mpm1$matU)
#'
#' # age-specific mortality hazard
#' mpm_to_hx(mpm1$matU)
#'
#' # age-specific fecundity
#' mpm_to_mx(mpm1$matU, mpm1$matF)
#'
#'
#' ### starting from first reproduction
#' repstages <- repro_stages(mpm1$matF)
#' n1 <- mature_distrib(mpm1$matU, start = 2, repro_stages = repstages)
#'
#' mpm_to_lx(mpm1$matU, start = n1)
#' mpm_to_px(mpm1$matU, start = n1)
#' mpm_to_hx(mpm1$matU, start = n1)
#' mpm_to_mx(mpm1$matU, mpm1$matF, start = n1)
#'
#' # specifying matrices explictly
#' mpm_to_mx(matU = mpm1$matU, matF = mpm1$matF, start = n1)
#' mpm_to_mx(matU = mpm1$matU, matR = mpm1$matF, start = n1)
#' mpm_to_mx(matU = mpm1$matU, matC = mpm1$matF, start = n1)
#'
#' @name age_from_stage
NULL


#' @rdname age_from_stage
#' @export mpm_to_mx
mpm_to_mx <- function(matU, matR = NULL, matF = NULL, matC = NULL,
                      start = 1L, xmax = 1000, lx_crit = 0.01,
                      tol = 1e-4) {
  # Call the helper function to construct matR if not provided
  matR <- process_fertility_inputs(matR = matR, matF = matF, matC = matC)

  # validate arguments (leave rest to mpm_to_lx)
  checkValidMat(matR)

  N <- length(mpm_to_lx(matU = matU, start, xmax, lx_crit, tol))

  if (length(start) == 1) {
    start_vec <- rep(0.0, nrow(matU))
    if (!is.null(dimnames(matU))) {
      checkMatchingStageNames(matU)
      names(start_vec) <- colnames(matU)
    }
    start_vec[start] <- 1.0
  } else {
    start_vec <- start
  }

  n <- start_vec
  mx <- numeric(N)

  for (i in 1:N) {
    n <- n / sum(n)
    phi <- matR %*% n
    mx[i] <- sum(phi)
    n <- matU %*% n
  }

  mx[is.nan(mx)] <- 0
  return(mx)
}


#' @rdname age_from_stage
#' @export mpm_to_lx
mpm_to_lx <- function(matU, start = 1L, xmax = 1000, lx_crit = 0.01,
                      tol = 1e-4) {
  # validate arguments
  checkValidMat(matU, warn_surv_issue = TRUE)
  checkValidStartLife(start, matU, start_vec = TRUE)

  t <- 0L
  lx <- 1.0
  lx_vec <- lx

  if (length(start) == 1) {
    start_vec <- rep(0.0, nrow(matU))
    if (!is.null(dimnames(matU))) {
      checkMatchingStageNames(matU)
      names(start_vec) <- colnames(matU)
    }
    start_vec[start] <- 1.0
  } else {
    start_vec <- start
  }

  n <- start_vec

  while (lx > lx_crit && t < xmax) {
    n <- matU %*% n
    lx <- sum(n)
    t <- t + 1L
    lx_vec[t] <- lx
  }

  # fix floating point errors resulting in lx > 1.0
  if (tol > 0) {
    lx_vec[lx_vec > 1.0 & lx_vec < (1.0 + tol)] <- 1.0
  }

  return(c(1.0, lx_vec))
}


#' @rdname age_from_stage
#' @export mpm_to_px
mpm_to_px <- function(matU, start = 1L, xmax = 1000, lx_crit = 0.01,
                      tol = 1e-4) {
  # leave argument validation to mpm_to_lx
  lx <- mpm_to_lx(matU, start, xmax, lx_crit, tol)
  return(lx_to_px(lx))
}


#' @rdname age_from_stage
#' @export mpm_to_hx
mpm_to_hx <- function(matU, start = 1L, xmax = 1000, lx_crit = 0.01,
                      tol = 1e-4) {
  # leave argument validation to mpm_to_lx
  lx <- mpm_to_lx(matU, start, xmax, lx_crit, tol)
  return(lx_to_hx(lx))
}

test_that("mpm_to_ functions work correctly", {
  xmax <- 20

  # mpm_to_lx
  lx <- mpm_to_lx(mat_u, start = 1, xmax = xmax, lx_crit = 0)
  expect_length(lx, xmax + 1)
  expect_true(all(lx >= 0))
  expect_true(all(lx <= 1))
  expect_true(all(lx == cummin(lx))) # monotonic declining

  lx_zero <- suppressWarnings(mpm_to_lx(mat_u_zero, start = 1, xmax = xmax))
  expect_identical(lx_zero[1], 1)
  expect_true(all(lx_zero[-1] == 0))

  # specify start as integer vs. vector
  lx_a <- mpm_to_lx(mat_u, start = 2)
  lx_b <- mpm_to_lx(mat_u, start = c(0, 1, 0, 0))
  expect_identical(lx_a, lx_b)

  # mpm_to_px
  px <- mpm_to_px(mat_u, start = 1, xmax = xmax, lx_crit = 0)
  expect_length(px, length(lx))

  # mpm_to_hx
  hx <- mpm_to_hx(mat_u, start = 1, xmax = xmax, lx_crit = 0)
  expect_length(hx, length(lx))

  # mpm_to_mx
  mx <- mpm_to_mx(mat_u, mat_f, start = 1, xmax = xmax, lx_crit = 0)
  mx_r <- mpm_to_mx(mat_u, matR = mat_f, start = 1, xmax = xmax, lx_crit = 0)
  mx_fc <- mpm_to_mx(mat_u, matF = mat_f, matC = mat_c, start = 1, xmax = xmax, lx_crit = 0)
  mx_rfc <- mpm_to_mx(mat_u, matR = mat_f + mat_c, start = 1, xmax = xmax, lx_crit = 0)
  mx_precedence <- mpm_to_mx(mat_u,
    matR = mat_f, matF = mat_c, matC = mat_c,
    start = 1, xmax = xmax, lx_crit = 0
  )
  expect_length(mx, length(lx))
  expect_true(all(mx >= 0))
  expect_identical(mx, mx_r)
  expect_identical(mx_fc, mx_rfc)
  expect_identical(mx, mx_precedence)

  mx_zero <- suppressWarnings(mpm_to_mx(mat_u_zero, mat_f,
    start = 1, xmax = xmax
  ))
  expect_true(all(mx_zero == 0))

  # using named life stages
  lx_named <- mpm_to_lx(mat_u_named, start = "sm", xmax = xmax, lx_crit = 0)
  px_named <- mpm_to_px(mat_u_named, start = 1, xmax = xmax, lx_crit = 0)
  hx_named <- mpm_to_hx(mat_u_named, start = 1, xmax = xmax, lx_crit = 0)
  mx_named <- mpm_to_mx(mat_u_named, mat_f_named,
    start = 1, xmax = xmax,
    lx_crit = 0
  )
  expect_identical(lx, lx_named)
  expect_identical(px, px_named)
  expect_identical(hx, hx_named)
  expect_identical(mx, mx_named)
})


test_that("mpm_to_ functions warn and fail gracefully", {
  # mpm_to_lx
  expect_error(mpm_to_lx(mat_u, start = 10))
  expect_error(mpm_to_lx(mat_u_na))
  expect_error(mpm_to_lx(mat_u, start = "stage name"))

  # mpm_to_px
  expect_error(mpm_to_px(mat_u, start = 10))
  expect_error(mpm_to_px(mat_u_na))
  expect_error(mpm_to_px(mat_u, start = "stage name"))

  # mpm_to_hx
  expect_error(mpm_to_hx(mat_u, start = 10))
  expect_error(mpm_to_hx(mat_u_na))
  expect_error(mpm_to_hx(mat_u, start = "stage name"))

  # mpm_to_mx
  expect_error(mpm_to_mx(mat_u, mat_f, start = 10))
  expect_error(mpm_to_mx(mat_u_na, mat_f))
  expect_error(mpm_to_mx(mat_u, mat_f_na))
  expect_error(mpm_to_mx(mat_u, mat_f, start = "stage name"))
  expect_error(mpm_to_mx(mat_u, matR = mat_f_na))
})

#' Transform a matrix population model to a standardized form
#'
#' Transform a matrix population model to a standardized set of stage classes
#' (e.g., propagule, pre-reproductive, reproductive, and post-reproductive). The
#' transition rates in the standardized matrix are a weighted mean of the
#' transition rates and per-capita reproductive values from the relevant stages
#' of the original matrix, weighted by the relative proportion of each stage
#' class expected at the stable distribution.
#'
#' @param matU The survival component of a matrix population model (i.e., a
#'   square projection matrix reflecting survival-related transitions; e.g.
#'   progression, stasis, and retrogression).
#' @param matF The sexual component of a matrix population model (i.e., a square
#'   projection matrix reflecting transitions due to sexual reproduction).
#' @param matC The clonal component of a matrix population model (i.e., a square
#'   projection matrix reflecting transitions due to clonal reproduction).
#'   Defaults to \code{NULL}, indicating no clonal reproduction (i.e.,
#'   \code{matC} is a matrix of zeros).
#' @param repro_stages Logical vector of length \code{ncol(matU)} indicating
#'   which stages are reproductive. Alternatively, a vector of stage indices or
#'   stage names of the reproductive classes.
#' @param matrix_stages Character vector of matrix stage types (e.g.,
#'   "propagule", "active", or "dormant").
#' @return A list with four elements reflecting the standardized matrix and
#'   its components:
#'   \item{matA}{Standardized projection matrix}
#'   \item{matU}{Survival component of the standardized projection matrix}
#'   \item{matF}{Sexual reproduction component of the standardized projection
#'   matrix}
#'   \item{matC}{Clonal reproduction component of the standardized projection
#'   matrix}
#'
#' @section Missing Stages:
#' The returned standardized matrix will always be of dimension \code{4}, even
#' if one or more standardized stages is missing from the original matrix
#' population model. If a standardized stage is missing, the corresponding
#' row/column of the standardized matrix will be coerced to \code{NA}.
#'
#' @details This function is a wrapper for the functions
#'   \code{\link{mpm_rearrange}}, \code{\link{standard_stages}} and
#'   \code{\link{mpm_collapse}}, which it calls in sequence.
#'
#' @note The method used by this function to collapse a matrix population model
#'   preserves the equilibrium population growth rate (\eqn{\lambda}) and
#'   relative stable distribution, but is not expected to preserve other
#'   demographic characteristics such as relative reproductive value,
#'   sensitivities, net reproductive rate, life expectancy, etc.
#'
#' @author Rob Salguero-Gomez <rob.salguero@@zoo.ox.ac.uk>
#'
#' @family transformation
#'
#' @examples
#' matU <- rbind(
#'   c(0.1, 0, 0, 0, 0),
#'   c(0.5, 0.2, 0.1, 0, 0),
#'   c(0, 0.3, 0.3, 0.1, 0),
#'   c(0, 0, 0.4, 0.4, 0.1),
#'   c(0, 0, 0, 0.1, 0.4)
#' )
#'
#' matF <- rbind(
#'   c(0, 1.1, 0, 1.6, 0),
#'   c(0, 0.8, 0, 0.4, 0),
#'   c(0, 0, 0, 0, 0),
#'   c(0, 0, 0, 0, 0),
#'   c(0, 0, 0, 0, 0)
#' )
#'
#' matC <- rbind(
#'   c(0, 0.6, 0, 0.5, 0),
#'   c(0, 0.1, 0, 0.3, 0),
#'   c(0, 0, 0, 0, 0),
#'   c(0, 0, 0, 0, 0),
#'   c(0, 0, 0, 0, 0)
#' )
#'
#' repro_stages <- c(2, 4)
#' matrix_stages <- c("prop", "active", "active", "active", "active")
#'
#' mpm_standardize(matU, matF, matC, repro_stages, matrix_stages)
#'
#' @export mpm_standardize
mpm_standardize <- function(matU, matF, matC = NULL, repro_stages,
                            matrix_stages) {
  checkValidMat(matU)
  checkValidMat(matF)
  checkMatchingStageNames(matU, matF)
  if (!is.null(matC)) {
    checkValidMat(matC, warn_all_zero = FALSE)
    checkMatchingStageNames(matU, matC)
  }
  checkValidStages(matU, repro_stages)

  # note argument validation done by component functions

  # populate matC NULL, populate with zeros
  if (is.null(matC)) {
    matC <- matrix(0, nrow = ncol(matF), ncol = ncol(matF))
  }

  # put non-reproductive stages at the end of the matrix
  rearr <- mpm_rearrange(matU, matF, matC, repro_stages, matrix_stages)

  # defines which columns need to be collapsed for each of the four stages
  collapse <- standard_stages(
    rearr$matF,
    rearr$repro_stages,
    rearr$matrix_stages
  )

  # collapse
  out <- mpm_collapse(matU, matF, matC, collapse = collapse)

  return(out)
}

#' @rdname mpm_standardize
#' @export mpm_standardise
mpm_standardise <- mpm_standardize

test_that("mpm_standardize works correctly", {
  # mpm with inter-reproductive stage
  repro_stages <- apply(mat_f_inter, 2, function(x) any(x > 0))
  matrix_stages <- c("prop", "active", "active", "active", "active")

  x1 <- mpm_standardize(mat_u_inter, mat_f_inter,
    repro_stages = repro_stages,
    matrix_stages = matrix_stages
  )

  expect_type(x1, "list")
  expect_length(x1, 4)
  expect_identical(ncol(x1$matA), 4L)

  x2 <- mpm_standardise(mat_u_inter, mat_f_inter,
    repro_stages = repro_stages,
    matrix_stages = matrix_stages
  )
  expect_identical(x1, x2) # check aliasing to GB spelling
})

#' Aggregate a Leslie matrix
#'
#' Takes a Leslie matrix and aggregates it to a desired dimension m using least
#' squares weights equal to the stable age distribution. The output includes the
#' aggregated matrix, the weight matrix, the original (or expanded) Leslie
#' matrix raised to the k power, the partitioning matrix, the size of the
#' original (or expanded) Leslie matrix, the size of the aggregated matrix, the
#' quotient of the original size divided by the aggregated size, and the
#' effectiveness of aggregation.
#'
#' @param A a Leslie matrix
#' @param m the dimensionality of the desired aggregated matrix
#'
#' @return a list including the following elements:
#' \item{B}{The aggregated matrix}
#' \item{W}{The weight matrix}
#' \item{Ak}{The original (or expanded) Leslie matrix raised to the k power}
#' \item{S}{The partitioning matrix}
#' \item{n}{The size of the original (or expanded) Leslie matrix}
#' \item{m}{The size of the aggregated matrix}
#' \item{k}{The quotient of the original size divided by the aggregated size}
#' \item{EFF}{The effectiveness of aggregation}
#'
#' @examples
#' data(leslie_mpm1)
#' A <- leslie_mpm1$matU + leslie_mpm1$matF
#' leslie_collapse(A, 4)
#'
#' @author Richard A. Hinrichsen <rich@hinrichsenenvironmental.com>
#' @references Hinrichsen, R. A. (2023). Aggregation of Leslie matrix models
#'   with application to ten diverse animal species. Population Ecology, 1–21.
#'   <doi:10.1002/1438-390X.12149>
#' @export leslie_collapse
#' @family transformation
#' @importFrom expm %^%
leslie_collapse <- function(A, m) {
  # data input validation
  if (!m %% 1 == 0 || m < 1) {
    stop("m must be a positive integer")
  }
  if (nrow(A) != ncol(A)) {
    stop("A must be a square matrix")
  }

  # first check whether the matrix is a Leslie matrix
  if (!is_leslie_matrix(A)) {
    stop("A must be a Leslie matrix")
  }
  n <- dim(A)[1]
  if (n %% m != 0) {
    A <- leslie_expand(A, m)
  } # if
  n <- dim(A)[1]
  k <- n / m
  eigA <- eigen(A)
  iii <- (Re(eigA$values) == max(Re(eigA$values)))
  v <- abs(Re(eigA$vectors[, iii]))
  w <- v # weights are equal to the stable age distribution
  Ak <- A %^% k
  e <- rep(1, k)
  Id <- diag(1, m)
  S <- kronecker(Id, t(e))
  W <- diag(w)
  TT <- W %*% t(S) %*% solve(S %*% W %*% t(S))
  B <- S %*% Ak %*% TT
  EFF <- norm(B %*% S %*% sqrt(W), type = "F")^2 / norm(S %*% Ak %*% sqrt(W),
    type = "F"
  )^2
  return(list(B = B, W = W, Ak = Ak, S = S, n = n, m = m, k = k, EFF = EFF))
}

#' Expand a Leslie matrix
#'
#' Expands a Leslie matrix to a desired dimensionality m by adding subdiagonal
#' elements of 1 and diagonal elements from the original matrix. The output is
#' the expanded matrix.
#'
#' @param A a Leslie matrix
#' @param m the desired dimensionality of the expanded matrix
#'
#' @return the expanded matrix
#'
#' @examples
#' A <- matrix(c(
#'   3.8930214, 8.3389426, 5.651921,
#'   0.5670728, 0.0000000, 0.000000,
#'   0.0000000, 0.2194218, 0.000000
#' ), byrow = TRUE, ncol = 3)
#' leslie_expand(A, 2)
#' @author Richard A. Hinrichsen <rich@hinrichsenenvironmental.com>
#' @references Hinrichsen, R. A. (2023). Aggregation of Leslie matrix models
#'   with application to ten diverse animal species. Population Ecology, 1–21.
#'   https://doi.org/10.1002/1438-390X.12149
#' @noRd
leslie_expand <- function(A, m) {
  n <- dim(A)[1]
  A.expanded <- matrix(0, nrow = n * m, ncol = n * m)
  # fill in first row of A.expanded
  for (ii in 1:n) {
    A.expanded[1, ii * m] <- A[1, ii]
  }
  # next fill in the subdiagonal
  for (ii in 2:(n * m)) {
    A.expanded[ii, ii - 1] <- 1
  }
  for (ii in 1:(n - 1)) {
    A.expanded[m * ii + 1, m * ii] <- A[ii + 1, ii]
  }
  # carry plus-group survival into the bottom-right corner of the expanded matrix
  A.expanded[n * m, n * m] <- A[n, n]
  return(A.expanded)
}

data(leslie_mpm1)
A <- leslie_mpm1$matU + leslie_mpm1$matF

testthat::expect_error(
  leslie_collapse(cbind(A, A), -4)
)

testthat::expect_true(
  inherits(leslie_collapse(A, 3), "list")
)

# plus-group survival (A[n,n] > 0) must be preserved after expansion
result <- leslie_collapse(A, 3)
testthat::expect_true(result$Ak[nrow(result$Ak), ncol(result$Ak)] > 0)

#' Determine if a matrix is a Leslie matrix population model
#'
#' Checks if a given matrix is a Leslie matrix.
#'
#' @details
#' A formal Leslie matrix satisfies the following conditions:
#' * All elements are non-negative.
#' * The subdiagonal elements (survivals), excluding the last column, are all
#'   between 0 and 1.
#' * The sum of the elements in the first row (fecundities) is positive (unless
#'   `includes_mat_F = FALSE`).
#' * The upper triangle, excluding the first row, contains only 0s.
#' * The diagonal, excluding the top-left corner (`A[1,1]`), contains only 0s.
#' * The lower triangle, excluding the subdiagonal, contains only 0s.
#'
#' **Plus-group extension.** In some models the oldest age class is an open-
#' ended *plus group* representing individuals that have survived beyond the
#' maximum explicitly modelled age. These individuals remain in the last class
#' with probability `A[n,n] > 0`. This function accepts such matrices by also
#' exempting the bottom-right corner (`A[n,n]`) from the diagonal-zero
#' requirement. When a plus-group matrix is passed to `leslie_collapse`, the
#' `A[n,n]` term is correctly carried through the internal `leslie_expand` step.
#'
#' @param A Matrix to be tested
#' @param includes_mat_F A logical argument (default `TRUE`) indicating whether
#'   A is expected to include fecundity. The idea here is that A may not include
#'   fertility, but could still be a valid Leslie matrix if fertility was truly
#'   measured to be 0, or if fertility was not measured at all. Thus, this
#'   argument relaxes the test for the first row of A summing to a positive
#'   value.
#'
#' @return A logical value indicating whether the matrix is a Leslie matrix or
#' not
#'
#' @examples
#' # Standard Leslie matrix
#' A <- matrix(c(
#'   0.1, 1.2, 1.1,
#'   0.1, 0.0, 0.0,
#'   0.0, 0.2, 0.0
#' ), nrow = 3, byrow = TRUE)
#' is_leslie_matrix(A) # true
#'
#' # Plus-group Leslie matrix: A[n,n] > 0 is permitted
#' A_plus <- matrix(c(
#'   0.1, 1.2, 1.1,
#'   0.1, 0.0, 0.0,
#'   0.0, 0.2, 0.3
#' ), nrow = 3, byrow = TRUE)
#' is_leslie_matrix(A_plus) # true
#'
#' # Not a Leslie matrix: non-zero off-diagonal elements
#' A <- matrix(c(
#'   0.1, 1.2, 1.1,
#'   0.1, 0.2, 0.1,
#'   0.2, 0.3, 0.3
#' ), nrow = 3, byrow = TRUE)
#' is_leslie_matrix(A) # false
#'
#' # leslie_mpm1 is a plus-group Leslie matrix (A[n,n] > 0)
#' data(leslie_mpm1)
#' A <- leslie_mpm1$matU + leslie_mpm1$matF
#' is_leslie_matrix(A) # true
#'
#' @family transformation
#' @export is_leslie_matrix
#' @author Owen Jones <jones@biology.sdu.dk>
is_leslie_matrix <- function(A, includes_mat_F = TRUE) {
  # Validation of input
  # A must be square matrix and at least dimension 2
  if (!is.matrix(A) || nrow(A) != ncol(A) || nrow(A) < 2) {
    stop("A must be a square matrix with at least dimension 2")
  }

  n <- nrow(A)
  # non-negative elements
  test1 <- !any(A < 0)

  # Subdiagonal - all between 0 and 1
  if (n > 2) {
    sd <- diag(A[-1, -ncol(A)])
  }
  if (n == 2) {
    sd <- A[2, 1]
  }
  test2 <- !any(sd < 0, sd > 1)

  # top row - sum is positive
  if (includes_mat_F) {
    test3 <- sum(A[1, ]) > 0
  } else {
    test3 <- TRUE
  }

  upper_tri <- upper.tri(A)
  upper_tri[1, ] <- FALSE

  # zeros in upper triangle, not including first row
  test4 <- all(A[upper_tri] == 0)

  # zeros in the diagonal, except [1,1] and [n,n] (plus-group corner)
  diagIndicator <- diag(TRUE, n)
  diagIndicator[1, 1] <- FALSE
  diagIndicator[n, n] <- FALSE
  A[diagIndicator]
  test5 <- all(A[diagIndicator] == 0)

  # zeros in lower triangle (not including the survival sub-diagonal)
  lower_tri_1 <- lower.tri(A)
  sd <- data.frame(
    r = 2:nrow(lower_tri_1),
    cc = 1:(nrow(lower_tri_1) - 1)
  )
  for (i in seq_len(nrow(sd))) {
    lower_tri_1[sd$r[i], sd$cc[i]] <- FALSE
  }
  test6 <- all(A[lower_tri_1] == 0)

  return(all(test1, test2, test3, test4, test5, test6))
}

# Standard Leslie matrix
A <- matrix(c(
  0.1, 1.2, 1.1,
  0.1, 0.0, 0.0,
  0.0, 0.2, 0.0
), nrow = 3, byrow = TRUE)

expect_true(is_leslie_matrix(A))

# Plus-group Leslie matrix: A[n,n] > 0 is permitted
A_plus <- matrix(c(
  0.1, 1.2, 1.1,
  0.1, 0.0, 0.0,
  0.0, 0.2, 0.3
), nrow = 3, byrow = TRUE)

expect_true(is_leslie_matrix(A_plus))

# Not a Leslie matrix: non-zero off-diagonal elements
A <- matrix(c(
  0.1, 1.2, 1.1,
  0.1, 0.2, 0.1,
  0.2, 0.3, 0.3
), nrow = 3, byrow = TRUE)

expect_false(is_leslie_matrix(A))

# leslie_mpm1 is a plus-group Leslie matrix (A[n,n] > 0)
data(leslie_mpm1)

A <- leslie_mpm1$matU + leslie_mpm1$matF

expect_true(is_leslie_matrix(A))

#' @title Convert matrix population model into U, F and C matrices
#'
#' @description
#' Splits a matrix population model into three constituent matrices, \bold{U}
#' (growth and survival processes), \bold{F} (sexual reproduction) and \bold{C}
#' (clonal reproduction). \strong{Warning!} The functionality is very basic: it
#' assumes that sexual reproduction is located in the top row of the matrix, and
#' that everything else is growth or survival (i.e. the \bold{U} matrix).
#' Clonality is assumed to be non-existent.
#'
#' @param matA A matrix population model (i.e., a square projection matrix).
#'
#' @return A list of three matrices: \code{matU},\code{matF} and \code{matC}.
#' \code{matC} will always contain only zeros.
#'
#' @author Owen R. Jones <jones@@biology.sdu.dk>
#'
#' @family transformation
#'
#' @examples
#' matA <- rbind(
#'   c(0.1, 0, 5.3, 4.2),
#'   c(0.5, 0.2, 0.1, 0),
#'   c(0, 0.3, 0.3, 0.1),
#'   c(0, 0, 0.5, 0.6)
#' )
#'
#' mpm_split(matA)
#'
#' @export mpm_split
mpm_split <- function(matA) {
  # validate arguments
  checkValidMat(matA,
    fail_all_na = FALSE, fail_any_na = FALSE,
    warn_all_zero = FALSE
  )

  # matrix dimension
  dim <- nrow(matA)

  # matU takes everything from matA except first row
  matU <- matA
  matU[1, ] <- rep(0, dim)

  # matF takes first row of matA
  matF <- matrix(0, nrow = dim, ncol = dim)
  matF[1, ] <- matA[1, ]

  # matC is all zeros
  matC <- matrix(0, nrow = dim, ncol = dim)

  # copy attributes of matA
  attributes(matU) <- attributes(matA)
  attributes(matF) <- attributes(matA)
  attributes(matC) <- attributes(matA)

  return(list(matU = matU, matF = matF, matC = matC))
}

test_that("mpm_split works correctly", {
  matA <- mat_u + mat_f

  x <- mpm_split(matA)

  expect_type(x, "list")
  expect_length(x, 3)
  expect_true(all(x$matC == 0))

  expect_identical(ncol(mat_u), ncol(x$matU))
})

#' Identify which stages in a matrix population model are reproductive
#'
#' Takes a reproductive matrix and returns a vector of logical values
#' (\code{TRUE}/\code{FALSE}) indicating which stages are reproductive (i.e.,
#' exhibit any positive values for reproduction). This function is a preparatory
#' step to collapsing the matrix model into a standardized set of stage classes
#' using the function \code{\link{mpm_standardize}}.
#'
#' @param matR The reproductive component of a matrix population model (i.e., a
#'   square projection matrix reflecting transitions due to reproduction; either
#'   sexual (e.g., \code{matF}), clonal (e.g., \code{matC}), or both).
#' @param na_handling One of \code{"return.na"}, \code{"return.true"}, or
#'   \code{"return.false"}. Determines how values of \code{NA} within
#'   \code{matR} should be handled. See Value for more details.
#' @return A logical vector of length \code{ncol(matR)}, with values of
#'   \code{FALSE} corresponding to non-reproductive stages and values of
#'   \code{TRUE} corresponding to reproductive stages.\cr\cr For a given matrix
#'   stage (i.e., column of \code{matR}), if there are any positive values of
#'   reproduction, the function will return \code{TRUE}. However, for a given
#'   stage, if there are no positive values of reproduction and one or more
#'   values of \code{NA}, the function will return \code{NA} if
#'   \code{na_handling == "return.na"}, \code{TRUE} if \code{na_handling ==
#'   "return.true"}, or \code{FALSE} if \code{na_handling == "return.false"}.
#'
#' @author Rob Salguero-Gomez <rob.salguero@@zoo.ox.ac.uk>
#' @author Patrick Barks <patrick.barks@@gmail.com>
#'
#' @family transformation
#'
#' @examples
#' matR1 <- rbind(
#'   c(0, 0.2, 0, 0.5),
#'   c(0, 0.3, 0, 0.6),
#'   c(0, 0, 0, 0),
#'   c(0, 0, 0, 0)
#' )
#'
#' matR2 <- rbind(
#'   c(NA, NA, NA, 1.1),
#'   c(0, 0, 0.3, 0.7),
#'   c(0, 0, 0, 0),
#'   c(0, 0, 0, 0)
#' )
#'
#' repro_stages(matR1)
#'
#' # compare different methods for handling NA
#' repro_stages(matR2, na_handling = "return.na")
#' repro_stages(matR2, na_handling = "return.true")
#' repro_stages(matR2, na_handling = "return.false")
#' @export repro_stages
repro_stages <- function(matR, na_handling = "return.true") {
  # validate arguments
  checkValidMat(matR, fail_any_na = FALSE)
  if (!na_handling %in% c("return.na", "return.true", "return.false")) {
    stop(strwrap(
      prefix = " ", initial = "",
      "Argument na_handling must be either 'return.na',
    'return.true', or 'return.false'.\n"
    ), call. = FALSE)
  }

  if (!anyNA(matR)) {
    reproStages <- apply(matR, 2, function(x) {
      any(x > 0)
    })
  } else if (na_handling == "return.na") {
    # works because of how function `any` handles NA
    # any(c(0, NA, 0) > 0) will return NA
    # any(c(0, NA, 1) > 0) will return TRUE
    reproStages <- apply(matR, 2, function(x) {
      any(x > 0)
    })
  } else if (na_handling == "return.true") {
    matR[which(is.na(matR))] <- Inf
    reproStages <- apply(matR, 2, function(x) {
      any(x > 0)
    })
  } else if (na_handling == "return.false") {
    matR[which(is.na(matR))] <- 0
    reproStages <- apply(matR, 2, function(x) {
      any(x > 0)
    })
  }

  return(reproStages)
}

test_that("repro_stages works correctly", {
  x <- repro_stages(mat_f)

  expect_type(x, "logical")
  expect_length(x, ncol(mat_f))

  x_na_1 <- repro_stages(mat_f_na, na_handling = "return.true")
  x_na_2 <- repro_stages(mat_f_na, na_handling = "return.na")
  x_na_3 <- repro_stages(mat_f_na, na_handling = "return.false")

  expect_false(anyNA(x_na_1))
  expect_true(anyNA(x_na_2))
  expect_false(anyNA(x_na_3))
})

test_that("repro_stages warns and fails gracefully", {
  m_na <- matrix(NA_real_, nrow = 4, ncol = 4)

  expect_error(repro_stages(m_na))
  expect_error(repro_stages(mat_f, na_handling = "eigen"))
})

test_that("mature_distrib works correctly", {
  repstages <- repro_stages(mat_f)
  x1 <- mature_distrib(mat_u, repro_stages = repstages)
  expect_length(x1, ncol(mat_u))
  expect_identical(sum(x1), 1)
  expect_identical(x1[x1 > 0], 1) # repro maturity in only 1 stage class

  # test using named stages
  x2 <- mature_distrib(mat_u_named, repro_stages = c("lg", "xl"))
  expect_length(x2, ncol(mat_u_named))
  expect_identical(sum(x2), 1)
  expect_identical(x1, unname(x2))

  mat_u2 <- mat_u
  mat_u2[4, 2] <- 0.1
  x2 <- mature_distrib(mat_u2, repro_stages = repstages)
  expect_length(x2[x2 > 0], 2) # repro maturity in 2 stage classes

  # test life cycle with no connection from 'start' to repro stage(s)
  matU <- rbind(
    c(0.0, 0.0, 0.0),
    c(0.1, 0.2, 0.0),
    c(0.0, 0.0, 0.0)
  )

  x3 <- mature_distrib(matU, start = 1, repro_stages = c(FALSE, FALSE, TRUE))
  expect_true(all(x3 == 0))
})

test_that("mature_distrib warns and fails gracefully", {
  repstages <- repro_stages(mat_f)
  expect_error(mature_distrib(mat_u_na, repro_stages = repstages))
  expect_error(mature_distrib(mat_u, start = 10, repro_stages = repstages))
  expect_error(mature_distrib(mat_u_named, repro_stages = c(3, "lg")))
  expect_error(mature_distrib(mat_u_named, repro_stages = 3:6))
  expect_error(mature_distrib(mat_u_named, repro_stages = c("lg", "xl", "xxl")))
})

#' Perturbation analysis of vital rates in a matrix population model
#'
#' @description
#' Perturbs lower-level vital rates within a matrix population model and
#' measures the response (sensitivity or elasticity) of the per-capita
#' population growth rate at equilibrium (\eqn{\lambda}), or, with a
#' user-supplied function, any other demographic statistic.
#'
#' These decompositions assume that all transition rates are products of a
#' stage-specific survival term (column sums of \code{matU}) and a lower level
#' vital rate that is conditional on survival (growth, shrinkage, stasis,
#' dormancy, or reproduction). Reproductive vital rates that are not conditional
#' on survival (i.e., within a stage class from which there is no survival) are
#' also allowed.
#'
#' @param matU The survival component of a matrix population model (i.e., a
#'   square projection matrix reflecting survival-related transitions; e.g.,
#'   progression, stasis, and retrogression).
#' @param matF The sexual component of a matrix population model (i.e., a square
#'   projection matrix reflecting transitions due to sexual reproduction).
#' @param matC The clonal component of a matrix population model (i.e., a square
#'   projection matrix reflecting transitions due to clonal reproduction).
#'   Defaults to \code{NULL}, indicating no clonal reproduction (i.e.,
#'   \code{matC} is a matrix of zeros).
#' @param pert Magnitude of the perturbation. Defaults to \code{1e-6}.
#' @param type Whether to return \code{sensitivity} or \code{elasticity} values.
#'   Defaults to \code{sensitivity}.
#' @param demog_stat The demographic statistic to be used, as in "the
#'   sensitivity/elasticity of \code{demog_stat} to vital rate perturbations."
#'   Defaults to the per-capita population growth rate at equilibrium
#'   (\eqn{\lambda}). Also accepts a user-supplied function that performs a
#'   calculation on a projection matrix and returns a single numeric value.
#' @param ... Additional arguments passed to the function \code{demog_stat}.
#'
#' @return A list with 5 elements:
#' \item{survival}{sensitivity or elasticity of \code{demog_stat} to survival}
#' \item{growth}{sensitivity or elasticity of \code{demog_stat} to growth}
#' \item{shrinkage}{sensitivity or elasticity of \code{demog_stat} to shrinkage}
#' \item{fecundity}{sensitivity or elasticity of \code{demog_stat} to sexual
#' fecundity}
#' \item{clonality}{sensitivity or elasticity of \code{demog_stat} to clonality}
#'
#' @note Fecundity and clonality outputs assume a post-breeding census
#'   interpretation. In pre-breeding models, birth rate and newborn survival are
#'   generally confounded within fecundity transitions. See Kendall et al.
#'   (2019).
#'
#' @author Rob Salguero-Gomez <rob.salguero@@zoo.ox.ac.uk>
#' @author Patrick Barks <patrick.barks@@gmail.com>
#'
#' @family perturbation analysis
#'
#' @examples
#' matU <- rbind(
#'   c(0.1, 0, 0, 0),
#'   c(0.5, 0.2, 0.1, 0),
#'   c(0, 0.3, 0.3, 0.1),
#'   c(0, 0, 0.5, 0.6)
#' )
#'
#' matF <- rbind(
#'   c(0, 0, 1.1, 1.6),
#'   c(0, 0, 0.8, 0.4),
#'   c(0, 0, 0, 0),
#'   c(0, 0, 0, 0)
#' )
#'
#' perturb_vr(matU, matF)
#'
#' # use elasticities rather than sensitivities
#' perturb_vr(matU, matF, type = "elasticity")
#'
#' # use a larger perturbation than the default
#' perturb_vr(matU, matF, pert = 0.01)
#'
#' # calculate the sensitivity/elasticity of the damping ratio to vital rate
#' #  perturbations
#' damping <- function(matA) { # define function for damping ratio
#'   eig <- eigen(matA)$values
#'   dm <- rle(Mod(eig))$values
#'   return(dm[1] / dm[2])
#' }
#'
#' perturb_vr(matU, matF, demog_stat = "damping")
#'
#' @export perturb_vr
perturb_vr <- function(matU, matF, matC = NULL,
                       pert = 1e-6, type = "sensitivity",
                       demog_stat = "lambda", ...) {
  # validate arguments
  checkValidMat(matU)
  checkValidMat(matF)
  if (!is.null(matC)) checkValidMat(matC, warn_all_zero = FALSE)
  type <- match.arg(type, c("sensitivity", "elasticity"))

  # get statfun
  if (is.character(demog_stat) && demog_stat == "lambda") {
    statfun <- lambda
  } else {
    statfun <- try(match.fun(demog_stat), silent = TRUE)
    if (inherits(statfun, "try-error")) {
      stop(strwrap(
        prefix = " ", initial = "",
        "`demog_stat` must be `lambda` or the name
      of a function that returns a single numeric value.\n"
      ), call. = FALSE)
    }
  }

  # matrix dimension
  m <- nrow(matU)

  # if matC null, convert to zeros
  if (is.null(matC)) matC <- matrix(0, nrow = m, ncol = m)

  # combine components into matA
  matA <- matU + matF + matC

  # get sensitivity matrix
  sensA <- perturb_matrix(
    matA = matA, pert = pert, type = "sensitivity",
    demog_stat = statfun, ...
  )

  stat <- statfun(matA, ...)

  # stage-specific survival
  sigma <- colSums(matU)

  # which stages defined only by age (single non-zero element in column of matU)
  stage_agedef <- apply(matU, 2, function(x) length(which(x > 0)) == 1)

  # survival-independent matA (divide columns of matA by corresponding element
  #  of sigma; retain original column of matA if no survival)
  noSurvA <- noSurvA_F <- t(t(matA) / sigma)
  noSurvA[, sigma == 0] <- matA[, sigma == 0]

  # sensitivity of survival
  matSensSurv <- noSurvA * sensA
  sensSurv <- colSums(matSensSurv)
  sensSurv[sigma == 0] <- 0 # zero out stages with no survival

  # lower and upper triangles (reflecting growth and retrogression)
  lwr <- upr <- matrix(0, nrow = m, ncol = m)
  lwr[lower.tri(lwr, diag = TRUE)] <- 1
  upr[upper.tri(upr, diag = TRUE)] <- 1

  # sensitivity of survival-independent U components
  matSensGrowShri <- matrix(NA_real_, m, m)

  for (i in 1:m) {
    matSensGrowShri[, i] <- sensA[i, i] * (-sigma[i]) +
      sensA[, i] * (sigma[i])
  }

  # zero out non-existent U transitions, and age-defined stages
  matSensGrowShri[which(matU == 0)] <- 0
  matSensGrowShri[, stage_agedef == TRUE] <- 0

  # sensitivity to growth
  matSensGrow <- lwr * matSensGrowShri

  # sensitivity to shrinkage
  matSensShri <- upr * matSensGrowShri

  # modify sigma (column-specific survival) for reproduction transitions;
  # convert 0s to 1s, so as not to 'pull out' survival from columns
  #  from which there was no survival
  sigma_rep <- sigma
  sigma_rep[sigma_rep == 0] <- 1

  # sensitivity to fecundity
  matSensFec <- sensA
  matSensFec[which(matF == 0)] <- 0 # zero out non-existent transitions
  matSensFec <- t(t(matSensFec) * sigma_rep) # multiply columns by sigma_rep

  # sensitivity to clonality
  matSensClo <- sensA
  matSensClo[which(matC == 0)] <- 0 # zero out non-existent transitions
  matSensClo <- t(t(matSensClo) * sigma_rep) # multiply columns by sigma_rep

  if (type == "sensitivity") {
    out <- list(
      survival = sum(sensSurv, na.rm = TRUE),
      growth = sum(matSensGrow, na.rm = TRUE),
      shrinkage = sum(matSensShri, na.rm = TRUE),
      fecundity = sum(matSensFec, na.rm = TRUE),
      clonality = sum(matSensClo, na.rm = TRUE)
    )
  } else {
    # elasticity of survival
    elasSurv <- sigma * sensSurv / stat

    # survival-independent U elasticities
    matElasGrowShri <- noSurvA * matSensGrowShri / stat

    # elasticity to growth and shrinkage
    matElasGrow <- lwr * matElasGrowShri
    matElasShri <- upr * matElasGrowShri

    # elasticity to fecundity and clonality
    matElasFec <- noSurvA * matSensFec / stat
    matElasClo <- noSurvA * matSensClo / stat

    out <- list(
      survival = sum(elasSurv, na.rm = TRUE),
      growth = sum(matElasGrow, na.rm = TRUE),
      shrinkage = sum(matElasShri, na.rm = TRUE),
      fecundity = sum(matElasFec, na.rm = TRUE),
      clonality = sum(matElasClo, na.rm = TRUE)
    )
  }

  return(out)
}

test_that("perturb_vr works correctly", {
  x <- perturb_vr(mat_u, mat_f)
  expect_type(x, "list")
  expect_identical(length(x), 5L)
  expect_identical(x$clonality, 0)

  x_elast <- perturb_vr(mat_u, mat_f, type = "elasticity")
  expect_identical(x_elast$clonality, 0)

  x_f_zero <- suppressWarnings(
    perturb_vr(mat_u, matF = mat_f_zero, matC = mat_c)
  )
  expect_identical(x_f_zero$fecundity, 0)

  # check no growth/shrinkage in age-only model
  x_age <- perturb_vr(mat_u_age, mat_f_age)
  expect_identical(x_age$growth, 0)
  expect_identical(x_age$shrinkage, 0)

  # check works with custom demog_stat function
  fn_custom <- function(x) {
    return(1.0)
  }
  x_cust <- perturb_vr(mat_u, mat_f, demog_stat = "fn_custom")
  expect_true(all(x_cust == 0))
})

test_that("perturb_vr warns and fails gracefully", {
  notfn <- "rtpQswpAclFKhAmw"
  expect_error(perturb_vr(mat_u, mat_f, demog_stat = notfn))
})

#' Perturbation analysis of a matrix population model
#'
#' Perturbs elements within a matrix population model and measures the response
#' (sensitivity or elasticity) of the per-capita population growth rate at
#' equilibrium (\eqn{\lambda}), or, with a user-supplied function, any other
#' demographic statistic.
#'
#' @param matA A matrix population model (i.e., a square projection matrix).
#' @param pert Magnitude of the perturbation. Defaults to \code{1e-6}.
#' @param type Whether to return \code{sensitivity} or \code{elasticity} values.
#' @param demog_stat The demographic statistic to be used, as in "the
#'   sensitivity/elasticity of \code{demog_stat} to matrix element
#'   perturbations." Defaults to the per-capita population growth rate at
#'   equilibrium (\eqn{\lambda}). Also accepts a user-supplied function that
#'   performs a calculation on a projection matrix and returns a single numeric
#'   value.
#' @param ... Additional arguments passed to the function \code{demog_stat}
#'
#' @return A sensitivity or elasticity matrix.
#'
#' @author Rob Salguero-Gomez <rob.salguero@@zoo.ox.ac.uk>
#'
#' @family perturbation analysis
#'
#' @references Caswell, H. 2001. Matrix Population Models: Construction,
#'   Analysis, and Interpretation. Sinauer Associates; 2nd edition. ISBN:
#'   978-0878930968
#'
#' @examples
#' matA <- rbind(
#'   c(0.1, 0, 1.5, 4.6),
#'   c(0.5, 0.2, 0.1, 0),
#'   c(0, 0.3, 0.3, 0.1),
#'   c(0, 0, 0.5, 0.6)
#' )
#'
#' perturb_matrix(matA)
#'
#' # use a larger perturbation than the default
#' perturb_matrix(matA, pert = 0.01)
#'
#' # calculate the sensitivity/elasticity of the damping ratio to perturbations
#' damping <- function(matA) { # define function for damping ratio
#'   eig <- eigen(matA)$values
#'   dm <- rle(Mod(eig))$values
#'   return(dm[1] / dm[2])
#' }
#'
#' perturb_matrix(matA, demog_stat = "damping")
#'
#' @export perturb_matrix
perturb_matrix <- function(matA, pert = 1e-6, type = "sensitivity",
                           demog_stat = "lambda", ...) {
  # validate arguments
  checkValidMat(matA)

  # get statfun
  if (is.character(demog_stat) && demog_stat == "lambda") {
    statfun <- lambda
  } else {
    statfun <- try(match.fun(demog_stat), silent = TRUE)
    if (inherits(statfun, "try-error")) {
      stop(strwrap(
        prefix = " ", initial = "",
        "demog_stat must be 'lambda' or the name of a
     function that returns a single numeric value.\n"
      ), call. = FALSE)
    }
  }

  # matrix dimension
  m <- nrow(matA)

  # statfun
  stat <- statfun(matA, ...)

  # initialize sensitivity matrix
  sensA <- matrix(NA, ncol = m, nrow = m)

  # matrix perturbation
  for (i in 1:m) {
    for (j in 1:m) {
      pertA <- matA
      pertA[i, j] <- pertA[i, j] + pert
      statPert <- statfun(pertA, ...)
      sensA[i, j] <- (stat - statPert) / (matA[i, j] - pertA[i, j])
    }
  }

  # keep only real values
  sensA <- Re(sensA)

  # copy attributes from matA
  attributes(sensA) <- attributes(matA)

  # return
  if (type == "sensitivity") {
    return(sensA)
  } else {
    return(sensA * matA / stat)
  }
}

test_that("perturb_matrix works correctly", {
  mat_a <- mat_u + mat_f

  x_sens1 <- perturb_matrix(mat_a)
  expect_true(inherits(x_sens1, "matrix"))
  expect_true(nrow(x_sens1) == nrow(mat_a) & ncol(x_sens1) == ncol(mat_a))

  x_sens2 <- perturb_matrix(mat_a, pert = 0.2)
  expect_true(inherits(x_sens1, "matrix"))
  expect_true(sum(x_sens1) != sum(x_sens2))

  x_elas1 <- perturb_matrix(mat_a, type = "elasticity")
  expect_true(inherits(x_elas1, "matrix"))
  expect_lt(abs(sum(x_elas1) - 1), 1e-6)

  # check works with custom demog_stat function
  fn_custom <- function(x) {
    return(1)
  }
  x_cust <- perturb_matrix(mat_a, demog_stat = "fn_custom")
  expect_true(all(x_cust == 0))


  expect_error(perturb_matrix(mat_a, demog_stat = "blurg"))
})


test_that("perturb_matrix warns and fails gracefully", {
  notfn <- "rtpsqwpaclfkhamw"
  expect_error(perturb_matrix(mat_a, demog_stat = notfn))
})

#' Plot a life cycle diagram from a matrix population model
#'
#' Plots the life cycle diagram illustrated by a matrix population model. This
#' function processes the matrix model and passes the information to the
#' graphViz function in DiagrammeR. See
#' \url{https://rich-iannone.github.io/DiagrammeR/}. Transitions with value
#' \code{0} are assumed to be biologically impossible, and arrows are omitted
#' from the diagram. Transition rates of \code{NA} are shown as arrows, labelled
#' with \code{NA}.
#'
#' @param matA A matrix population model (i.e., a square projection matrix)
#' @param stages Optional vector of stage class labels. If missing, it first
#'   attempts to infer them from \code{dimnames(matA)}. If these are also
#'   \code{NULL}, then reverts to integers \code{1:ncol(A)}.
#' @param title Optional title for the plot. Defaults to \code{NULL}.
#' @param shape The shape to be used for the stages of the diagram. Any node
#'   shape accepted by \code{graphViz} is acceptable.
#' @param fontsize Size of the font used in the diagram.
#' @param nodefontsize Size of the font used in the node part of the diagram.
#' @param edgecol Colour of the arrows in the diagram.
#' @param node_order An optional numeric vector giving the order that the nodes
#'   should be presented in the plot. Default is `NULL` whereby the order is the
#'   same as `stages`, or row/column names, of the matrix.
#'
#' @return An object of class \code{grViz} representing the life cycle diagram
#'
#' @author Owen R. Jones <jones@@biology.sdu.dk>
#'
#' @family visualisation
#'
#' @examples
#' matA <- rbind(
#'   c(0.1, 0, 0, 0, 1.4),
#'   c(0.5, 0.2, 0, 0, 0),
#'   c(0, 0.3, 0.3, 0, 0),
#'   c(0, 0, 0.4, 0.4, 0.1),
#'   c(0, 0, 0, 0.1, 0.4)
#' )
#'
#' plot_life_cycle(matA)
#'
#' # Plot and label unknown transitions as NA
#' matA_na <- matA
#' matA_na[1, 5] <- NA
#' plot_life_cycle(matA_na)
#'
#' # One could save the diagram as a PNG file using a combination of `export_svg`
#' # (from the `DiagrammeRsvg` package) and `rsvg_png` (from the `rsvg` package)
#' # like this:
#' \dontrun{
#' p1 <- plot_life_cycle(matA)
#' p1 %>%
#'   DiagrammeRsvg::export_svg %>%
#'   charToRaw() %>%
#'   rsvg::rsvg_png("my life cycle.png")
#' }
#'
#' # Change the order of the nodes and give them names
#' plot_life_cycle(matA,
#'   stages = c("A", "B", "C", "D", "E"),
#'   node_order = 5:1
#' )
#'
#' @importFrom DiagrammeR grViz
#' @export plot_life_cycle
plot_life_cycle <- function(matA, stages, title = NULL, shape = "egg",
                            fontsize = 10, nodefontsize = 12,
                            edgecol = "grey", node_order = NULL) {
  # Identify stages
  if (missing(stages) && is.null(dimnames(matA))) {
    stages <- seq_len(ncol(matA))
  } else if (missing(stages) && !is.null(dimnames(matA))) {
    stages <- dimnames(matA)[[1]]

    if (!identical(
      dimnames(matA)[[1]],
      dimnames(matA)[[2]]
    )) {
      message(strwrap(
        prefix = " ", initial = "",
        "Dimension names of 'matA' are not identical
      for rows and columns. Using row names."
      ))
    }
  }

  if (length(stages) != nrow(matA)) {
    stop("The length of stages does not equal the dimension of matA")
  }

  if (!is.null(node_order)) {
    if (length(node_order) != nrow(matA)) {
      stop("The length of node_order does not equal the dimension of matA")
    }
  }
  # Construct a "from" -> "to" graph dataset (edges)
  graph <- expand.grid(to = stages, from = stages)
  graph$trans <- c(matA)
  graph$label <- ifelse(is.na(graph$trans), "NA", format(round(graph$trans, 3)))

  # Subset to include positive transitions and unknown transitions
  graph <- graph[is.na(graph$trans) | graph$trans > 0, ]

  # Create vector of node names (add semicolon for use by graphViz)
  if (is.null(node_order)) {
    nodes <- paste(paste0("'", stages, "'"), collapse = " -> ")
  } else {
    stages <- stages[order(node_order)]
    nodes <- paste(paste0("'", stages, "'"), collapse = " -> ")
  }
  nodes <- paste0(nodes, " [style=invis]")

  # Manipulate minimim length of edge to make the plot pretty (experimental!)
  graph$min_len <- (as.numeric(graph$to) - as.numeric(graph$from)) * 3
  # Create the edges argument for graphviz by pasting commands together
  edges <- paste0(
    "'", graph$from, "'", " -> ", "'", graph$to, "'",
    "[minlen=", graph$min_len,
    ",fontsize=", fontsize,
    ",color=", edgecol,
    ",xlabel=", paste("\"", graph$label),
    "\"]\n",
    collapse = ""
  )

  graph_label <- if (is.null(title)) {
    ""
  } else {
    paste0("  labelloc=\"t\";\n  label=\"", title, "\"\n")
  }

  # The graphviz argument, pasted together
  grViz(
    paste(
      "
digraph {
  {
    graph[overlap=false, margin=0, pad=0];
    rank=same;
    node [shape=", shape, ", fontsize=", nodefontsize, "];",
      nodes,
      "
  }",
      "
",
      edges,
      graph_label,
      "}"
    )
  )
}

test_that("plot_life_cycle works correctly", {
  mat_a <- mat_u + mat_f
  stages <- letters[seq_len(nrow(mat_a))]
  mat_a_na <- mat_a
  mat_a_na[1, 3] <- NA

  p1 <- plot_life_cycle(mat_a)
  p2 <- plot_life_cycle(mat_a, stages = stages)
  p3 <- plot_life_cycle(mat_a, stages = stages, title = "my_title")
  p4 <- plot_life_cycle(mat_a, node_order = 1:4)
  p5 <- plot_life_cycle(mat_a_na)

  expect_s3_class(p1, "grViz")
  expect_s3_class(p2, "grViz")
  expect_s3_class(p3, "grViz")
  expect_s3_class(p4, "grViz")
  expect_s3_class(p5, "grViz")
  expect_match(p1$x$diagram, "graph\\[overlap=false, margin=0, pad=0\\]")
  expect_false(grepl("x \\[style=invis\\]", p1$x$diagram))
  expect_false(grepl("labelloc=\"t\"", p1$x$diagram, fixed = TRUE))
  expect_true(grepl("labelloc=\"t\"", p3$x$diagram, fixed = TRUE))
  expect_true(grepl("\n  label=\"my_title\"", p3$x$diagram, fixed = TRUE))
})


test_that("plot_life_cycle warns and fails gracefully", {
  mat_a <- mat_u + mat_f
  colnames(mat_a) <- paste0("A", seq_len(nrow(mat_a)))
  rownames(mat_a) <- paste0("B", seq_len(nrow(mat_a)))
  # make rownames different from colnames
  stages <- dimnames(mat_a)[[1]]

  expect_message(plot_life_cycle(mat_a))
})

# Test failure with incorrect dimensions.
mat_a <- mat_u + mat_f

# wrong number of stages
expect_error(plot_life_cycle(mat_a, stages = c("A", "B", "C")))

# wrong number of entries in node_order
expect_error(plot_life_cycle(mat_a, node_order = 1:3))

#' Identify stages corresponding to different parts of the reproductive life
#' cycle
#'
#' Identify the stages of a matrix population model that correspond to
#' different parts of the reproductive life cycle, namely propagule,
#' pre-reproductive, reproductive and post-reproductive. These classifications
#' are used to standardise matrices to allow comparisons across species with
#' different life cycle structures, see \code{\link{mpm_standardize}}.
#'
#' Assumes that fecundity and mean fecundity matrices have been
#' rearranged so that non-reproductive stages are in the final rows/columns.
#' Output indicates groupings to be used when collapsing the matrix model.
#'
#' @param matF The sexual component of a matrix population model (i.e., a square
#'   projection matrix reflecting transitions only due to \emph{sexual}
#'   reproduction). It assumes that it has been rearranged so that
#'   non-reproductive stages are in the final rows/columns.
#' @param repro_stages Logical vector identifying which stages are reproductive.
#' @param matrix_stages (character) vector of stages, values are \code{prop}
#' (propagule), \code{active}, and \code{dorm} (dormant).
#'
#' @return A list with four elements:
#'   \item{propStages}{Position of the propagule stages}
#'   \item{preRepStages}{Position of the pre-reproductive stages}
#'   \item{repStages}{Position of the reproductive stages}
#'   \item{postRepStages}{Position of the post-reproductive stages}
#'
#' @author Rob Salguero-Gomez <rob.salguero@@zoo.ox.ac.uk>
#'
#' @family transformation
#'
#' @note Dormant stages are not currently handled.
#' @seealso \code{\link{mpm_standardize}}
#'
#' @examples
#' matU <- rbind(
#'   c(0.1, 0, 0, 0, 0),
#'   c(0.5, 0.2, 0.1, 0, 0),
#'   c(0, 0.3, 0.3, 0.1, 0),
#'   c(0, 0, 0.4, 0.4, 0.1),
#'   c(0, 0, 0, 0.1, 0.4)
#' )
#'
#' matF <- rbind(
#'   c(0, 1.1, 0, 1.6, 0),
#'   c(0, 0.8, 0, 0.4, 0),
#'   c(0, 0, 0, 0, 0),
#'   c(0, 0, 0, 0, 0),
#'   c(0, 0, 0, 0, 0)
#' )
#'
#' repro_stages <- c(FALSE, TRUE, FALSE, TRUE, FALSE)
#' matrix_stages <- c("prop", "active", "active", "active", "active")
#'
#' r <- mpm_rearrange(matU, matF,
#'   repro_stages = repro_stages,
#'   matrix_stages = matrix_stages
#' )
#'
#' standard_stages(r$matF, r$repro_stages, r$matrix_stages)
#'
#' @export standard_stages
#'
standard_stages <- function(matF, repro_stages, matrix_stages) {
  # validate arguments
  checkValidMat(matF, warn_all_zero = FALSE)
  if (ncol(matF) != length(repro_stages) ||
    length(repro_stages) != length(matrix_stages)) {
    stop("Arguments do not correspond to MPM of the same dimension.\n",
      call. = FALSE
    )
  }
  if (!any(repro_stages == TRUE)) {
    stop(strwrap(prefix = " ", initial = "", "Cannot identify standardised
    stages because no stages are reproductive (i.e., at least one element of
                 repro_stages must be TRUE).\n"),
      call. = FALSE
    )
  }

  if ("prop" %in% matrix_stages) {
    propStage <- which(matrix_stages == "prop")
  } else {
    propStage <- NA
  }

  # set max reproductive stage
  maxRep <- max(which(repro_stages == TRUE))

  # prerep
  matDim <- nrow(matF)
  Rep <- which(repro_stages == TRUE)

  if (min(Rep) == 1) {
    preRep <- NA
  } else if (!is.na(propStage[1]) && (min(Rep) - max(propStage) == 1)) {
    preRep <- NA
  } else {
    preRep <- min(which(matrix_stages == "active")):(min(Rep) - 1)
  }

  # postrep
  if (maxRep == matDim) {
    postRep <- NA
  } else {
    postRep <- (maxRep + 1):matDim
  }

  return(list(
    propStages = propStage,
    preRepStages = preRep,
    repStages = Rep,
    postRepStages = postRep
  ))
}

test_that("standard_stages works correctly", {
  ## mpm without inter-reproductive stage
  repro_stages <- apply(mat_f, 2, function(x) any(x > 0))
  matrix_stages <- c("active", "active", "active", "active")
  r <- mpm_rearrange(mat_u, mat_f, mat_c, repro_stages, matrix_stages)

  x <- standard_stages(r$matF, r$repro_stages, r$matrix_stages)
  expect_type(x, "list")
  expect_length(x, 4)

  # make sure all stages in original mpm represented in output
  stages <- as.numeric(unlist(x))
  stages <- stages[!is.na(stages)]
  expect_true(all(seq_len(nrow(r$matU)) %in% stages))


  ## mpm with inter-reproductive stage
  repro_stages <- apply(mat_f_inter, 2, function(x) any(x > 0))
  matrix_stages <- c("prop", "active", "active", "active", "active")
  r <- mpm_rearrange(mat_u_inter, mat_f_inter,
    repro_stages = repro_stages,
    matrix_stages = matrix_stages
  )

  x <- standard_stages(r$matF, r$repro_stages, r$matrix_stages)
  expect_type(x, "list")
  expect_length(x, 4)

  # make sure all stages in original mpm represented in output
  stages <- as.numeric(unlist(x))
  stages <- stages[!is.na(stages)]
  expect_true(all(seq_len(nrow(r$matU)) %in% stages))


  ## mpm with all stages reproductive
  repro_stages <- apply(mat_f_allrep, 2, function(x) any(x > 0))
  matrix_stages <- c("active", "active")
  r <- mpm_rearrange(mat_u_allrep, mat_f_allrep,
    repro_stages = repro_stages,
    matrix_stages = matrix_stages
  )

  x <- standard_stages(r$matF, r$repro_stages, r$matrix_stages)
  expect_type(x, "list")
  expect_length(x, 4)
  expect_true(is.na(x$propStages))
  expect_true(is.na(x$preRepStages))
  expect_true(is.na(x$postRepStages))

  # make sure all stages in original mpm represented in output
  expect_true(all(seq_len(nrow(r$matU)) %in% stages))
})


test_that("standard_stages warns and fails gracefully", {
  # arguments of different dimension
  repro_stages <- apply(mat_f, 2, function(x) any(x > 0))
  matrix_stages <- c("prop", "active", "active")
  expect_error(standard_stages(mat_f, repro_stages, matrix_stages))

  # matF contains NA
  repro_stages <- apply(mat_f_na, 2, function(x) any(x > 0))
  matrix_stages <- c("prop", "active", "active", "active")
  expect_error(standard_stages(mat_f_na, repro_stages, matrix_stages))

  # no reproductive stages
  repro_stages <- apply(mat_f_zero, 2, function(x) any(x > 0))
  matrix_stages <- c("active", "active", "active", "active")
  expect_error(standard_stages(mat_f_zero, repro_stages, matrix_stages))
})

#' Calculate stochastic elasticities from a time-series of matrix population
#' models and corresponding population vectors
#'
#' Calculate stochastic elasticities given a time-series of matrix population
#' models and corresponding population vectors, using the method described in
#' Haridas et al. (2009).
#'
#' @param X_t A list of matrix population models
#' @param u_t A list of corresponding population vectors
#'
#' @return A list of three matrices:
#'   \item{E}{matrix of stochastic elasticities}
#'   \item{E_mu}{matrix of stochastic elasticities to mean transition rates}
#'   \item{E_sigma}{matrix of stochastic elasticities to the variance in
#'   transition rates}
#'
#' @author Patrick Barks <patrick.barks@@gmail.com>
#'
#' @family perturbation analysis
#'
#' @references Haridas, C. V., Tuljapurkar, S., & Coulson, T. 2009. Estimating
#'   stochastic elasticities directly from longitudinal data. Ecology Letters,
#'   12, 806-812. <doi:10.1111/j.1461-0248.2009.01330.x>
#'
#' @examples
#' # generate list of random MPMs
#' N <- 20 # number of years
#' s <- 3 # matrix dimension
#' X <- list() # matrix population model at time t
#' u <- list() # population vector at time t
#'
#' for (t in 1:N) {
#'   X[[t]] <- matrix(runif(s^2), nrow = s, ncol = s)
#' }
#'
#' # derive corresponding series of population vectors
#' u <- pop_vectors(X)
#'
#' # calculate stochastic elasticities
#' perturb_stochastic(X, u)
#'
#' @export perturb_stochastic
perturb_stochastic <- function(X_t, u_t) {
  if (length(X_t) != length(u_t)) {
    stop("Arguments X_t and u_t must be of same length.\n", call. = FALSE)
  }
  if (length(unique(vapply(X_t, nrow, 1))) != 1) {
    stop("All elements of X_t must be of same dimension.\n", call. = FALSE)
  }
  if (length(unique(vapply(u_t, length, 1))) != 1) {
    stop("All elements of u_t must be of same length.\n", call. = FALSE)
  }
  checks <- lapply(X_t, checkValidMat)

  N <- length(X_t) # number of time periods
  s <- nrow(X_t[[1]]) # matrix dimension

  # mean and mean-standardized matrices
  X_mu <- replicate(N, meanMat(X_t), simplify = FALSE)
  X_std <- lapply(X_t, function(X) X - X_mu[[1]])

  # calculate observed lambda at each time t
  lambda_t <- unlist(Map(function(X, u) sum(X %*% u), X_t, u_t))

  E <- E_mu <- E_sigma <- matrix(0, s, s)
  I <- diag(s)
  e <- rep(1, s)

  # for each matrix element
  for (i in 1:s) {
    for (j in 1:s) {
      ## build perterbation mats for given matrix element [i, j]
      C <- lapply(X_t, build_pert_mats, i = i, j = j)
      C_mu <- lapply(X_mu, build_pert_mats, i = i, j = j)
      C_sigma <- lapply(X_std, build_pert_mats, i = i, j = j)

      ## calculate e_R
      e_R <- unlist(Map(e_fn, mat = C, vec = u_t, lam = lambda_t))
      e_R_mu <- unlist(Map(e_fn, mat = C_mu, vec = u_t, lam = lambda_t))
      e_R_sigma <- unlist(Map(e_fn, mat = C_sigma, vec = u_t, lam = lambda_t))

      ## calculate e_U
      w_t <- w_t_mu <- w_t_sigma <- list()
      w_t[[1]] <- w_t_mu[[1]] <- w_t_sigma[[1]] <- rep(0, s)

      # recursion to obtain w_t
      for (t in 1:(N - 1)) {
        w_t[[t + 1]] <- 1 / lambda_t[t] * (I - u_t[[t + 1]] %*% t(e)) %*%
          (C[[t]] %*% u_t[[t]] + X_t[[t]] %*% w_t[[t]])

        w_t_mu[[t + 1]] <- 1 / lambda_t[t] * (I - u_t[[t + 1]] %*% t(e)) %*%
          (C_mu[[t]] %*% u_t[[t]] + X_t[[t]] %*% w_t_mu[[t]])

        w_t_sigma[[t + 1]] <- 1 / lambda_t[t] * (I - u_t[[t + 1]] %*% t(e)) %*%
          (C_sigma[[t]] %*% u_t[[t]] + X_t[[t]] %*% w_t_sigma[[t]])
      }

      e_U <- unlist(Map(e_fn, mat = X_t, vec = w_t, lam = lambda_t))
      e_U_mu <- unlist(Map(e_fn, mat = X_t, vec = w_t_mu, lam = lambda_t))
      e_U_sigma <- unlist(Map(e_fn, mat = X_t, vec = w_t_sigma, lam = lambda_t))

      ## calculate stochastic elasticities
      E[i, j] <- mean(e_R + e_U)
      E_mu[i, j] <- mean(e_R_mu + e_U_mu)
      E_sigma[i, j] <- mean(e_R_sigma + e_U_sigma)
    }
  }

  return(list(E = E, E_mu = E_mu, E_sigma = E_sigma))
}


# function to create perturbation matrices (C_ijt)
build_pert_mats <- function(X, i, j) {
  C <- matrix(0, nrow(X), ncol(X))
  C[i, j] <- X[i, j]
  return(C)
}


# function to calculate e_R and e_U
e_fn <- function(mat, vec, lam) {
  sum(mat %*% vec) / lam
}

test_that("perturb_stochastic works correctly", {
  s <- 3 # matrix dimension
  N <- 15 # length of time series

  # generate lists of matrices and population vectors
  X1 <- replicate(N, matrix(runif(s^2), s, s), simplify = FALSE)
  u1 <- replicate(N, runif(s), simplify = FALSE)
  u1 <- lapply(u1, function(x) x / sum(x))

  e1 <- perturb_stochastic(X1, u1)
  expect_type(e1, "list")
  expect_true(inherits(e1[[1]], "matrix"))
  expect_length(e1, 3)
  expect_identical(nrow(e1[[1]]), as.integer(s))


  ### test failure
  X2 <- X1[1:(N - 1)]
  u2 <- u1
  expect_error(perturb_stochastic(X2, u2))

  X3 <- X1
  u3 <- u1
  X3[[1]] <- X3[[1]][1:(s - 1), 1:(s - 1)]
  expect_error(perturb_stochastic(X3, u3))

  X4 <- X1
  u4 <- u1
  u4[[1]] <- u4[[1]][1:(s - 1)]
  expect_error(perturb_stochastic(X4, u4))

  X5 <- X1
  u5 <- u1
  X5[[1]][1, 1] <- NA
  expect_error(perturb_stochastic(X5, u5))
})

#' Derive a hypothetical set of population vectors corresponding to a
#' time-series of matrix population models
#'
#' Derive a hypothetical set of population vectors (i.e. population size
#' distributions across stages) given a time-series of matrix population models
#' (MPMs), by taking the stable stage distribution of the mean matrix as the
#' starting vector (or optionally, a uniform or random starting vector), and
#' deriving subsequent vectors through recursive population projection.
#'
#' This function is useful for providing population vectors as input to the
#' \code{\link{perturb_stochastic}} function which calculates stochastic
#' elasticities given a time-series of matrix population models and
#' corresponding population vectors, using the method described in Haridas et
#' al. (2009).
#'
#' @param A A list of MPMs (i.e., square population projection matrices).
#' @param start Method to derive the first population vector in the series.
#'   Either \code{stable.stage} to use the stable stage distribution of the mean
#'   matrix as the starting vector, \code{uniform} to use a uniform starting
#'   vector (all elements equal), or \code{random} to use a randomly-generated
#'   starting vector. Defaults to the stable stage distribution.
#'
#' @return A list of population vectors
#'
#' @author Patrick Barks <patrick.barks@@gmail.com>
#'
#' @family perturbation analysis
#'
#' @references Haridas, C. V., Tuljapurkar, S., & Coulson, T. 2009. Estimating
#'   stochastic elasticities directly from longitudinal data. Ecology Letters,
#'   12, 806-812. <doi:10.1111/j.1461-0248.2009.01330.x>
#'
#' @examples
#' # generate list of matrices
#' matA_l <- replicate(5, matrix(runif(9), 3, 3), simplify = FALSE)
#'
#' # calculate corresponding population vectors
#' pop_vectors(matA_l)
#' pop_vectors(matA_l, start = "uniform")
#' pop_vectors(matA_l, start = "random")
#'
#' @importFrom stats runif
#' @export pop_vectors
pop_vectors <- function(A, start = "stable.stage") {
  A_mean <- meanMat(A)
  s <- nrow(A_mean)
  w <- list()

  if (start == "stable.stage") {
    w[[1]] <- stable.stage(A_mean)
  } else if (start == "uniform") {
    w[[1]] <- rep(1 / s, s)
  } else if (start == "random") {
    w[[1]] <- runif(s)
    w[[1]] <- w[[1]] / sum(w[[1]])
  } else {
    stop("Argument start must be one of 'stable.stage',
         'uniform', or 'random'.\n",
      .call = FALSE
    )
  }

  for (i in 2:length(A)) {
    w[[i]] <- A[[i - 1]] %*% w[[i - 1]]
    w[[i]] <- as.numeric(w[[i]] / sum(w[[i]]))
  }

  return(w)
}

test_that("pop_vectors works correctly", {
  s <- 3 # matrix dimension
  N <- 7 # length of time series

  # generate lists of matrices
  x1 <- replicate(N, matrix(runif(s^2), s, s), simplify = FALSE)

  test_sum_1 <- function(x) abs(sum(x) - 1) < 0.000001

  v1 <- pop_vectors(x1)
  expect_type(v1, "list")
  expect_type(v1[[1]], "double")
  expect_length(v1, N)
  expect_length(v1[[1]], s)
  expect_true(all(vapply(v1, test_sum_1, logical(1))))

  v2 <- pop_vectors(x1, start = "uniform")
  expect_type(v2, "list")
  expect_type(v2[[1]], "double")
  expect_length(v2, N)
  expect_length(v2[[1]], s)
  expect_true(all(vapply(v2, test_sum_1, logical(1))))

  v3 <- pop_vectors(x1, start = "random")
  expect_type(v3, "list")
  expect_type(v3[[1]], "double")
  expect_length(v3, N)
  expect_length(v3[[1]], s)
  expect_true(all(vapply(v3, test_sum_1, logical(1))))
})


test_that("pop_vectors warns and fails gracefully", {
  s <- 3 # matrix dimension
  N <- 7 # length of time series

  x2 <- replicate(N, matrix(runif(s^2), s, s), simplify = FALSE)
  expect_error(pop_vectors(x2, start = "blagjd"))
})

#' Calculate Keyfitz's entropy from a trajectory of age-specific survivorship
#'
#' Calculate Keyfitz's entropy from a vector of age-specific survivorship
#' (\code{lx}), or from the U submatrix of a matrix population model.
#'
#' @section Warning:
#' Note that this function, which was formerly called `entropy_k` may produce
#' unexpected results if used on partial survivorship trajectories. In addition,
#' it is sensitive to the length of the survivorship vector. We direct users to
#' the function `\code{\link{shape_surv}}` which is relatively robust to these
#' issues.
#'
#' Furthermore, de Vries et al. 2023 have shown that the way this function
#' calculates entropy is problematic for other reasons. We recommend to use
#' `\code{\link{entropy_k_age}}` or ``\code{\link{entropy_k_stage}}` as
#' alternatives, See de Vries et al. 2023 for details.
#'
#' @param lx Either a survivorship trajectory (a vector of
#'   monotonically-declining values in the interval [0,1]), or submatrix U from
#'   a matrix population model.
#' @param trapeze A logical argument indicating whether the composite trapezoid
#'   approximation should be used for approximating the definite integral.
#' @param ... Additional variables passed to `mpm_to_lx` if data are supplied as
#'   a matrix. This could include the `start` argument to select a starting
#'   stage.
#'
#' @return Keyfitz's life table entropy.
#'
#' @author Owen R. Jones <jones@@biology.sdu.dk>
#' @author Roberto Salguero-Gomez <rob.salguero@@zoo.ox.ac.uk>
#'
#' @references Keyfitz, N. 1977. Applied Mathematical Demography. New York:
#'   Wiley.
#'
#'   Demetrius, L., & Gundlach, V. M. 2014. Directionality theory and
#'   the entropic principle of natural selection. Entropy 16: 5428-5522.
#'
#'   de Vries, C., Bernard, C., & Salguero-Gómez, R. 2023. Discretising
#'   Keyfitz' entropy for studies of actuarial senescence and comparative
#'   demography. Methods in Ecology and Evolution, 14, 1312–1319.
#'   <doi:10.1111/2041-210X.14083>
#'
#' @family life history traits
#'
#' @examples
#' data(mpm1)
#'
#' # derive lx trajectory, starting from stage 2
#' lx <- mpm_to_lx(mpm1$matU, start = 2)
#'
#' # calculate Keyfitz' entropy
#' life_elas(lx)
#'
#' # use trapezoid approximation for definite integral
#' life_elas(lx, trapeze = TRUE)
#'
#' # calculate directly from the matrix
#' life_elas(mpm1$matU)
#'
#' @export
life_elas <- function(lx, trapeze = FALSE, ...) {
  # This function is deprecated, use entropy_k2 which has two functions within
  # it suitable for age-based and stage-based matrices, selected with a type
  # argument.

  if (inherits(lx, "numeric")) {
    # validate arguments
    if (any(lx < 0 | lx > 1)) {
      stop("All values of lx must be within the interval [0, 1].\n")
    }
    if (any(diff(lx) > 1e-7)) {
      stop("Values of lx must be monotonically declining.\n")
    }
  }

  if (inherits(lx, "matrix")) {
    lx <- mpm_to_lx(lx, ...)
  }

  # remove ages at/beyond which lx is 0 or NA
  lx <- lx[1:max(which(lx > 0))]

  # Calculate Keyfitz's entropy
  if (trapeze == TRUE) {
    x <- seq_along(lx)
    H <- -area_under_curve(x, lx * log(lx)) / area_under_curve(x, lx)
  } else {
    H <- -sum(lx * log(lx)) / sum(lx)
  }

  return(H)
}

#' Collapse a matrix population model to a smaller number of stages
#'
#' Collapse a matrix population model to a smaller number of stages. For
#' instance, to compare properties of multiple projection matrices with
#' different numbers of stages, one might first collapse those matrices to a
#' standardized set of stages (e.g., propagule, pre-reproductive, reproductive,
#' and post-reproductive). The transition rates in the collapsed matrix are a
#' weighted average of the transition rates from the relevant stages of the
#' original matrix, weighted by the relative proportion of each stage class
#' expected at the stable distribution.
#'
#' @param matU The survival component of a matrix population model (i.e., a
#'   square projection matrix reflecting survival-related transitions; e.g.,
#'   progression, stasis, and retrogression)
#' @param matF The sexual component of a matrix population model (i.e., a square
#'   projection matrix reflecting transitions due to sexual reproduction)
#' @param matC The clonal component of a matrix population model (i.e., a square
#'   projection matrix reflecting transitions due to clonal reproduction).
#'   Defaults to \code{NULL}, indicating no clonal reproduction (i.e.,
#'   \code{matC} is a matrix of zeros).
#' @param collapse A list giving the mapping between stages of the original
#'   matrix and the desired stages of the collapsed matrix (e.g., \code{list(1,
#'   2:3, 4)}). Original stages may be passed as either indices or stage names
#'   corresponding to stage index or name in \code{matU}, \code{matF} and
#'   \code{matC}). Names given to the elements of \code{collapse} are used as
#'   stage names in the new, collapsed matrix.
#'
#'   See \emph{Missing Stages} for handling of \code{NA} within \code{collapse}.
#' @return A list with four elements:
#'   \item{matA}{Collapsed projection matrix}
#'   \item{matU}{Survival component of the collapsed projection matrix}
#'   \item{matF}{Sexual reproduction component of the collapsed projection
#'   matrix}
#'   \item{matC}{Clonal reproduction component of the collapsed projection
#'   matrix}
#'
#' @section Missing Stages:
#' The collapsed matrix will always be of dimension \code{length(collapse)},
#' even if one or more elements of the \code{collapse} argument is \code{NA}
#' (corresponding to a desired stage of the collapsed matrix that is not present
#' in the original matrix). In the collapsed matrix, any row/column
#' corresponding to a missing stage will be coerced to \code{NA}.
#'
#' If all stages within a collapsed group have zero weight in the stable stage
#' distribution, equal weights are used for that group to avoid undefined
#' values.
#'
#' @author Rob Salguero-Gómez <rob.salguero@@zoo.ox.ac.uk>
#' @author William K. Petry <wpetry@@ncsu.edu>
#'
#' @family transformation
#'
#' @references Salguero-Gomez, R. & Plotkin, J. B. 2010. Matrix dimensions bias
#'   demographic inferences: implications for comparative plant demography. The
#'   American Naturalist 176, 710-722. <doi:10.1086/657044>
#'
#' @note This method of collapsing a matrix population model preserves the
#'   equilibrium population growth rate (\eqn{lambda}) and relative stable
#'   distribution, but is not expected to preserve other traits such as relative
#'   reproductive values, sensitivities, net reproductive rates, life
#'   expectancy, etc.
#'
#' @seealso \code{\link{mpm_standardize}}
#' @examples
#' data(mpm1)
#'
#' # check which stages reproductive
#' repro_stages(matR = mpm1$matF)
#'
#' # collapse reproductive stages (3 and 4) into single stage
#' mpm_collapse(
#'   matU = mpm1$matU, matF = mpm1$matF,
#'   collapse = list(1, 2, 3:4, 5)
#' )
#'
#' # use stage names instead, and name stages in the collapsed matrix
#' mpm_collapse(
#'   matU = mpm1$matU, matF = mpm1$matF,
#'   collapse = list(
#'     seed = "seed", vegetative = "small",
#'     flowering = c("medium", "large"),
#'     dormant = "dormant"
#'   )
#' )
#'
#' @export mpm_collapse
mpm_collapse <- function(matU, matF, matC = NULL, collapse) {
  # validate arguments
  checkValidMat(matU)
  checkValidMat(matF)
  if (!is.null(matC)) {
    checkValidMat(matC, warn_all_zero = FALSE)
  }
  checkValidStages(matU, stages = collapse)

  # populate matC with zeros, if NULL
  if (is.null(matC)) {
    outC <- FALSE
    matC <- matrix(0, nrow = nrow(matU), ncol = ncol(matU))
    if (!is.null(dimnames(matU))) {
      dimnames(matC) <- dimnames(matU)
    }
  }

  # sum components to matA
  matA <- matU + matF + matC

  # dimensions of original and collapse matrices
  originalDim <- nrow(matA)
  collapseDim <- length(collapse)

  P <- matrix(0, nrow = collapseDim, ncol = originalDim)
  if (!is.null(names(collapse))) {
    rownames(P) <- names(collapse)
  }

  # convert `collapse` names to corresponding row/col numbers if needed
  if (all(vapply(collapse, is.character, logical(1)))) {
    collapse <- lapply(collapse, function(x) which(colnames(matU) %in% x))
  }

  for (i in 1:collapseDim) {
    columns <- as.numeric(collapse[[i]])
    if (!is.na(columns[1])) {
      P[i, columns] <- 1
    }
  }

  Q <- t(P)
  w <- stable.stage(matA)

  columns <- which(colSums(Q) > 1)

  for (j in columns) {
    rows <- which(Q[, j] == 1)
    rows_sum <- sum(w[rows])
    for (i in rows) {
      if (rows_sum == 0) {
        Q[i, j] <- 1 / length(rows)
      } else {
        Q[i, j] <- w[i] / rows_sum
      }
    }
  }

  # replace missing rows/cols with NA
  if (anyNA(collapse)) {
    i <- which(is.na(collapse))
    P[i, ] <- rep(NA_real_, originalDim)
    Q[, i] <- rep(NA_real_, originalDim)
  }

  # collapse
  collapseA <- P %*% matA %*% Q
  collapseU <- P %*% matU %*% Q
  collapseF <- P %*% matF %*% Q
  collapseC <- P %*% matC %*% Q

  return(list(
    matA = collapseA,
    matU = collapseU,
    matF = collapseF,
    matC = collapseC
  ))
}

#' Calculate net reproductive rate (R0) from a matrix population model
#'
#' @description
#' Calculate net reproductive rate (R0) from a matrix population model. The net
#' reproduction rate (R0) is the mean number of recruits produced during the
#' mean life expectancy of an individual. See section 5.3.5 of Caswell (2001).
#'
#' @param matU The survival component of a matrix population model (i.e., a
#'   square projection matrix reflecting survival-related transitions; e.g.
#'   progression, stasis, and retrogression). Optionally with named rows and
#'   columns indicating the corresponding life stage names.
#' @param matR The reproductive component of a matrix population model (i.e., a
#'   square projection matrix only reflecting transitions due to reproduction;
#'   either sexual, clonal, or both). If \code{matR} is supplied, \code{matF}
#'   and \code{matC} are ignored. Otherwise, \code{matR} is constructed by
#'   summing \code{matF} and \code{matC}.
#' @param matF The matrix reflecting sexual reproduction. If provided
#'   without \code{matC}, \code{matC} is assumed to be a zero matrix. If
#'   \code{matR} is provided, this argument is ignored.
#' @param matC The matrix reflecting clonal (asexual) reproduction.
#'   If provided without \code{matF}, \code{matF} is assumed to be a zero
#'   matrix. If \code{matR} is provided, this argument is ignored.
#' @param start Index (or stage name) of the first stage at which the author
#'   considers the beginning of life. Only used if \code{method = "start"}.
#'   Defaults to \code{1}.
#' @param method The method used to calculate net reproductive rate, either
#'   \code{"generation"} or \code{"start"}. Defaults to \code{"generation"}.
#'   See Details.
#'
#' @details
#' The \code{method} argument controls how net reproductive rate is calculated.
#'
#' If \code{method = "generation"}, net reproductive rate is calculated as the
#' per-generation population growth rate (i.e., the dominant eigenvalue of
#' \code{matR \%*\% N}, where \code{N} is the fundamental matrix). See Caswell
#' (2001) Section 5.3.4.
#'
#' If \code{method = "start"}, net reproductive rate is calculated as the
#' expected lifetime production of offspring that start life in stage
#' \code{start}, by an individual also starting life in stage \code{start}
#' (i.e., \code{(matR \%*\% N)[start,start]}).
#'
#' If offspring only arise in stage \code{start}, the two methods give the
#' same result.
#' @return Returns the net reproductive rate. If \code{matU} is singular (often
#'   indicating infinite life expectancy), returns \code{NA}.
#'
#' @author Roberto Salguero-Gomez <rob.salguero@@zoo.ox.ac.uk>
#' @author Hal Caswell <h.caswell@@uva.nl>
#'
#' @family life history traits
#'
#' @references Caswell, H. 2001. Matrix Population Models: Construction,
#'   Analysis, and Interpretation. Sinauer Associates; 2nd edition. ISBN:
#'   978-0878930968
#'
#' @examples
#' data(mpm1)
#'
#' net_repro_rate(mpm1$matU, mpm1$matF)
#'
#' # calculate R0 using the start method, specifying either the life stage index
#' # or name
#' net_repro_rate(mpm1$matU, mpm1$matF, method = "start", start = 1)
#' net_repro_rate(mpm1$matU, mpm1$matF, method = "start", start = "seed")
#'
#' # It is usually better to explicitly name the arguments, rather than relying
#' # on order.
#' net_repro_rate(
#'   matU = mpm1$matU, matF = mpm1$matF,
#'   method = "start", start = 1
#' )
#'
#' net_repro_rate(
#'   matU = mpm1$matU, matR = mpm1$matF,
#'   method = "start", start = "seed"
#' )
#'
#' @export net_repro_rate
net_repro_rate <- function(matU, matR = NULL, matF = NULL,
                           matC = NULL, start = 1, method = "generation") {
  # Call the helper function to construct matR if not provided
  matR <- process_fertility_inputs(matR, matF, matC)

  # validate arguments
  checkValidMat(matU, warn_surv_issue = TRUE)
  checkValidMat(matR)
  checkValidStartLife(start, matU)
  method <- match.arg(method, c("generation", "start"))
  if (!is.numeric(start)) {
    checkMatchingStageNames(M = matU, N = matR)
  }

  # matrix dimensions
  matDim <- nrow(matU)

  # try calculating fundamental matrix (will fail if matrix singular)
  N <- try(solve(diag(matDim) - matU), silent = TRUE)

  # calculate R0
  # first check for singular matU (if singular, R0 = NA)
  if (inherits(N, "try-error") && grepl("singular", N[1], fixed = TRUE)) {
    R0 <- NA_real_
  } else {
    R <- matR %*% N

    R0 <- switch(method,
      "generation" = lambda(R),
      "start" = R[start, start]
    )
  }

  return(R0)
}

test_that("net_repro_rate works correctly", {
  x <- net_repro_rate(mat_u, mat_f)
  x_r <- net_repro_rate(mat_u, matR = mat_f)
  x_c <- net_repro_rate(mat_u, matC = mat_c)
  x_fc <- net_repro_rate(mat_u, matF = mat_f, matC = mat_c)
  x_rfc <- net_repro_rate(mat_u, matR = mat_f + mat_c)
  x_precedence <- net_repro_rate(mat_u, matR = mat_f, matF = mat_c, matC = mat_c)
  x_start <- net_repro_rate(mat_u, mat_f, method = "start")
  x_zero <- suppressWarnings(net_repro_rate(mat_u_zero, mat_f))
  x_singular <- net_repro_rate(mat_u_singular, mat_f)
  x_named <- net_repro_rate(mat_u_named, mat_f_named, start = "sm")

  expect_length(x, 1L)
  expect_gt(x, 0)
  expect_identical(x, x_r)
  expect_gt(x_c, 0)
  expect_identical(x_fc, x_rfc)
  expect_identical(x, x_precedence)
  expect_length(x_start, 1)
  expect_gt(x_start, 0)
  expect_identical(x_zero, 0)
  expect_identical(x_singular, NA_real_)
  expect_identical(x, x_named)
})

test_that("net_repro_rate warns and fails gracefully", {
  expect_warning(net_repro_rate(mat_u_survissue, mat_f))
  expect_error(net_repro_rate(mat_u_na, mat_f))
  expect_error(net_repro_rate(mat_u, mat_f_na))
  expect_error(net_repro_rate(mat_u, matR = mat_f_na))
  expect_error(net_repro_rate(mat_u, mat_f, method = "eigen"))
  expect_error(net_repro_rate(mat_u_named, mat_f, start = "sm"))
  expect_error(net_repro_rate(mat_u, mat_f_named, start = "sm"))
  expect_error(net_repro_rate(mat_u_named, mat_f_named,
    start = "invalid stage"
  ))
})

#' Add stage names to matrices
#'
#' Adds user-supplied or automatically-generated stage names to a matrix
#' population model (MPM).
#'
#' @param mat An MPM, either as a single matrix or list of matrices.
#' @param names A character vector specifying the name of each life stage, in
#'   order. If provided, \code{prefix} and \code{left_pad} arguments are
#'   ignored.
#' @param prefix A string to be pre-pended to the stage number when
#'   automatically naming stages. Defaults to \code{stage_}.
#' @param left_pad Logical, whether to pre-pend \code{0} to stage names such
#'   that all stage numbers have equal length, enabling lexicographic sorting.
#'   For example, stage \code{1} becomes \code{01} for matrices with 10-99
#'   stages, \code{001} for matrices with 100-999 stages, and so on. Defaults to
#'   \code{TRUE}.
#'
#' @author William K. Petry <wpetry@@ncsu.edu>
#'
#' @family transformation
#'
#' @return The input matrix or matrices with named rows and columns.
#' @export name_stages
#'
#' @examples
#' matU <- rbind(
#'   c(0.0, 0.0, 0.0),
#'   c(0.3, 0.1, 0.0),
#'   c(0.0, 0.5, 0.8)
#' )
#' # (semi)automated naming
#' name_stages(matU)
#' name_stages(matU, prefix = "s")
#' # custom stage names
#' name_stages(matU, names = c("small", "medium", "large"))
#' # overwrite existing stage names
#' data(mpm1)
#' name_stages(mpm1)
#'
name_stages <- function(mat, names = NULL, prefix = "stage_", left_pad = TRUE) {
  # check argument inputs
  if (!is.matrix(mat) && !is.list(mat)) {
    stop("Argument `mat` must be either a matrix or list of matrices.\n")
  }
  if (is.matrix(mat)) {
    if (length(unique(dim(mat))) != 1L) {
      stop("When `mat` is supplied as a matrix, it must be square.\n")
    } else {
      mdim <- nrow(mat)
    }
  } else {
    if (length(unique(unlist(lapply(mat, dim), use.names = FALSE))) != 1L) {
      stop("Each matrix in `mat` must be square with the same dimensions.\n")
    } else {
      mdim <- nrow(mat[[1]])
    }
  }
  if (is.null(names) && !is.character(prefix)) {
    stop("Either stage `names` or a naming `prefix` must be supplied.\n")
  }
  if (!is.null(names) && is.character(prefix)) {
    warning("Naming `prefix` ignored, using stage `names` instead.\n")
  }
  # construct stage names
  if (is.null(names)) {
    if (isTRUE(left_pad)) {
      fmt <- paste0("%0", nchar(mdim), "d")
      names <- paste0(prefix, sprintf(fmt, 1:mdim))
    } else if (isFALSE(left_pad)) {
      names <- 1:mdim
    } else {
      stop("Argument `left_pad` must be of type logical (TRUE/FALSE).\n")
    }
  }
  if (mdim != length(names)) {
    stop(
      "Incorrect number of stage names supplied: ", length(names),
      " names supplied for ", mdim, " life stages.\n"
    )
  }
  # warn if overwriting existing stage names
  if (is.list(mat) && !is.null(unlist(lapply(mat, dimnames),
    use.names = FALSE
  )) ||
    is.matrix(mat) && !is.null(unlist(dimnames(mat), use.names = FALSE))) {
    warning("Existing stage names have been overwritten!\n")
  }
  # add stage names to matrix/matrices
  if (is.matrix(mat)) {
    dimnames(mat) <- list(names, names)
  } else {
    mat <- lapply(mat, `dimnames<-`, list(names, names))
  }
  return(mat)
}

test_that("name_stages works correctly", {
  s <- as.integer(10) # matrix dimension

  # generate lists of matrices
  x1 <- replicate(2, matrix(runif(s^2), s, s), simplify = FALSE)

  y1 <- name_stages(x1)
  expect_true(inherits(y1, "list"))
  expect_true(inherits(y1[[1]], "matrix"))
  expect_identical(nrow(y1[[1]]), s)

  y2 <- name_stages(x1[[1]])
  expect_true(inherits(y2, "matrix"))
  expect_identical(nrow(y2), s)

  y3 <- name_stages(x1[[1]], prefix = "", left_pad = FALSE)
  expect_identical(rownames(y3), as.character(1:s))

  y4 <- name_stages(x1[[1]], names = LETTERS[1:s], prefix = NULL)
  expect_identical(rownames(y4), LETTERS[1:s])
})

test_that("name_stages warns and fails gracefully", {
  s <- 10 # matrix dimension
  sp1 <- s + 1

  # generate lists of matrices
  x1 <- x2 <- replicate(2, matrix(runif(s^2), s, s), simplify = FALSE)
  x2[[3]] <- matrix(runif(sp1^2), sp1, sp1)

  # passing wrong object types
  expect_error(name_stages("a"))
  expect_error(name_stages(matrix(1:3)))
  expect_error(name_stages(list("a", "b", "c")))

  # passing list of matrices with different dimensions
  expect_error(name_stages(x2))

  # overwriting existing stage names
  expect_warning(name_stages(mat_u_named))

  # supplying both prefix and names arguments
  expect_warning(name_stages(x1, names = LETTERS[1:s]))

  # incorrect number of stage names
  expect_error(name_stages(x1, names = letters[1:(s - 1)], prefix = NULL))
})

#' Calculate Keyfitz entropy for a stage-based matrix population model
#'
#' Computes the discrete-time Keyfitz entropy for a stage-based (Lefkovitch) matrix
#' population model using the **U** submatrix. This implementation follows the
#' life-disparity / life-expectancy formulation (H_N) adapted to stage models (H_N^+):
#' it projects survivorship directly from \code{Umat} and quantifies the dispersion of
#' **ages at death** (in projection steps) relative to mean life expectancy.
#'
#' \strong{Interpretation.} Values reflect the \emph{shape} of mortality across age:
#' \itemize{
#'   \item \eqn{H < 1}: increasing mortality with age (senescence)
#'   \item \eqn{H = 1}: approximately age-constant mortality
#'   \item \eqn{H > 1}: decreasing mortality with age (negative senescence)
#' }
#'
#' @param Umat A square numeric matrix representing the U submatrix of a
#'   stage-based (Lefkovitch) matrix population model.
#' @param init_distrib The initial cohort distribution across
#'   stages. This should sum to 1. If it does not sum to 1, the function
#'   rescales it to 1. Defaults to an equal distribution across stages.
#' @param max_age The upper age, in units of the projection interval. Defaults
#'   to 1000 if no information is provided.
#' @param n_is_maxage If TRUE, survival p_n is set to zero. Defaults to FALSE.
#'
#' @return Returns a single numeric value representing the Keyfitz entropy for
#'   the given matrix. This value quantifies the dispersion of age at death (not
#'   stage at death).
#'
#' @details
#' The function avoids age-from-stage preprocessing by obtaining survivorship
#' from successive powers of \eqn{U} up to \code{max_age}. In pure age-class
#' (Leslie) models with \eqn{k} age classes and no persistence in the terminal
#' class, using \code{max_age = k} recovers the exact discrete-time value.
#'
#' The result depends on the starting cohort \code{init_distrib} (e.g., all
#' recruits, a single stage, or recruits at equilibrium). Choose \code{init_distrib}
#' to match the biological question.
#'
#' @author Stefano Giaimo <giaimo@@evolbio.mpg.de>
#' @author Owen Jones <jones@@biology.sdu.dk>
#'
#' @references
#' Keyfitz, N. 1977. Applied Mathematical Demography. New York: Wiley.
#' Giaimo S. 2024. Discretising and validating Keyfitz entropy for any demographic
#' classification. Methods in Ecology and Evolution. 15, 1278–1285.
#'
#' @family life history traits
#' @examples
#' data(mpm1)
#'
#' entropy_k_stage(mpm1$matU)
#' @importFrom utils head
#' @export

entropy_k_stage <- function(Umat, init_distrib = NULL, max_age = NULL, n_is_maxage = FALSE) {
  if (!inherits(Umat, "matrix")) {
    stop("m must be a matrix")
  }

  if (!is.numeric(Umat)) {
    stop("m must be a numeric matrix")
  }
  if (nrow(Umat) != ncol(Umat)) {
    stop("m must be a square matrix")
  }

  # Function from Stefano Giaimo
  matDim <- ncol(Umat)
  if (is.null(init_distrib)) {
    init_distrib <- rep(1 / matDim, matDim)
  }
  if (length(init_distrib) != nrow(Umat)) {
    stop("init_distrib must have a length of nrow(Umat)")
  }
  if (is.null(max_age)) {
    max_age <- 1000
  }

  # Rescale the initial stage distribution to sum to 1.
  init_distrib <- init_distrib / sum(init_distrib)

  Idmat <- diag(ncol(Umat))
  Nmat <- solve(Idmat - Umat)
  powUmat <- Reduce("%*%",
    replicate(max_age, Umat, simplify = FALSE),
    init = Idmat,
    accumulate = TRUE
  )
  lastpowUmat <- powUmat[[max_age + 1]]
  fac1 <- sapply(powUmat, function(x) sum(x %*% init_distrib))
  fac1 <- fac1[-1] / head(fac1, -1)
  if (n_is_maxage == TRUE) {
    fac1[max_age] <- 0
  }
  fac1 <- 1 - fac1
  fac2 <- sapply(head(powUmat, -1), function(x) {
    sum((Nmat %*% (x - lastpowUmat)) %*% init_distrib)
  })
  entr <- sum(fac1 * fac2) / sum((Nmat %*% (Idmat - lastpowUmat)) %*% init_distrib)
  return(entr)
}

#' Calculate generation time from a matrix population model
#'
#' @description
#' Calculate generation time from a matrix population model. Multiple
#' definitions of the generation time are supported: the time required for a
#' population to increase by a factor of R0 (the net reproductive rate; Caswell
#' (2001), section 5.3.5), the average parent-offspring age difference (Bienvenu
#' & Legendre (2015)), or the expected age at reproduction for a cohort (Coale
#' (1972), p. 18-19).
#'
#' @param matU The survival component of a matrix population model (i.e., a
#'   square projection matrix reflecting survival-related transitions; e.g.,
#'   progression, stasis, and retrogression).
#' @param matR The reproductive component of a matrix population model (i.e., a
#'   square projection matrix only reflecting transitions due to reproduction;
#'   either sexual, clonal, or both). If \code{matR} is supplied, \code{matF}
#'   and \code{matC} are ignored. Otherwise, \code{matR} is constructed by
#'   summing \code{matF} and \code{matC}.
#' @param matF The matrix reflecting sexual reproduction. If provided
#'   without \code{matC}, \code{matC} is assumed to be a zero matrix. If
#'   \code{matR} is provided, this argument is ignored.
#' @param matC The matrix reflecting clonal (asexual) reproduction.
#'   If provided without \code{matF}, \code{matF} is assumed to be a zero
#'   matrix. If \code{matR} is provided, this argument is ignored.
#' @param method The method used to calculate generation time. Defaults to "R0".
#'   See Details for explanation of calculations.
#' @param ... Additional arguments passed to \code{net_repro_rate} when
#'   \code{method = "R0"} or \code{mpm_to_*} when \code{method = "cohort"}.
#'   Ignored when \code{method = "age_diff"}.

#' @details
#' There are multiple definitions of generation time, three of which are
#' implemented by this function:
#'
#' 1. \code{"R0"} (default): This is the number of time steps required for the
#' population to grow by a factor of its net reproductive rate, equal to
#' \code{log(R0) / log(lambda)}. Here, \code{R0} is the net reproductive rate
#' (the per-generation population growth rate; Caswell 2001, Sec. 5.3.4), and
#' \code{lambda} is the population growth rate per unit time (the dominant
#' eigenvalue of \code{matU + matR}).
#'
#' 2. \code{"age_diff"}: This is the average age difference between parents and
#' offspring, equal to \code{(lambda v w) / (v matR w)} (Bienvenu & Legendre
#' (2015)). Here, \code{lambda} is the population growth rate per unit time (the
#' dominant eigenvalue of \code{matU + matR}), \code{v} is a row vector of
#' stage-specific reproductive values (the left eigenvector corresponding to
#' \code{lambda}), and \code{w} is a column vector of the stable stage
#' distribution (the right eigenvector corresponding to \code{lambda}).
#'
#' 3. \code{"cohort"}: This is the age at which members of a cohort are expected
#' to reproduce, equal to \code{sum(x lx mx) / sum(lx mx)} (Coale (1972), p.
#' 18-19). Here, \code{x} is age, \code{lx} is age-specific survivorship, and
#' \code{mx} is age-specific fertility. See functions \code{mpm_to_lx} and
#' \code{mpm_to_mx} for details about the conversion of matrix population models
#' to life tables.
#'
#' @return Returns generation time. If \code{matU} is singular (often indicating
#'   infinite life expectancy), returns \code{NA}.
#'
#' @note Note that the units of time in returned values are the same as the
#'   projection interval (`ProjectionInterval`) of the MPM.
#'
#' @author Patrick Barks <patrick.barks@@gmail.com>
#' @author William Petry <wpetry@@ncsu.edu>
#'
#' @family life history traits
#'
#' @references Bienvenu, F. & Legendre, S. 2015. A New Approach to the
#'   Generation Time in Matrix Population Models. The American Naturalist 185
#'   (6): 834–843. <doi:10.1086/681104>.
#'
#' Caswell, H. 2001. Matrix Population Models: Construction,
#'   Analysis, and Interpretation. Sinauer Associates; 2nd edition. ISBN:
#'   978-0878930968
#'
#' Coale, A.J. 1972. The Growth and Structure of Human Populations. Princeton
#' University Press. ISBN: 978-0691093574
#'
#' @examples
#' data(mpm1)
#'
#' # calculate generation time
#' gen_time(matU = mpm1$matU, matR = mpm1$matF) # defaults to "R0" method
#' gen_time(matU = mpm1$matU, matF = mpm1$matF) # defaults to "R0" method
#' gen_time(matU = mpm1$matU, matR = mpm1$matF, method = "age_diff")
#' gen_time(
#'   matU = mpm1$matU, matR = mpm1$matF, method = "cohort", lx_crit =
#'     0.001
#' )
#'
#' @export gen_time
gen_time <- function(matU, matR = NULL, matF = NULL, matC = NULL,
                     method = c("R0", "age_diff", "cohort"), ...) {
  # Call the helper function to construct matR if not provided
  matR <- process_fertility_inputs(matR, matF, matC)

  method <- match.arg(method)
  # leave remaining arg validation to functions that depend on them

  if (method == "R0") {
    R0 <- net_repro_rate(matU, matR, ...)
    lam <- lambda(matU + matR)
    out <- log(R0) / log(lam)
  } else if (method == "age_diff") {
    ignore <- list(...)
    # check for singularity
    matDim <- nrow(matU)
    N <- try(solve(diag(matDim) - matU), silent = TRUE)
    if (inherits(N, "try-error") && grepl("singular", N[1], fixed = TRUE)) {
      out <- NA_real_
    } else {
      A <- matU + matR
      lam <- lambda(A)
      v <- reproductive.value(A)
      w <- stable.stage(A)
      out <- as.numeric((lam * v %*% w) / (v %*% matR %*% w))
    }
  } else if (method == "cohort") {
    # check for singularity
    matDim <- nrow(matU)
    N <- try(solve(diag(matDim) - matU), silent = TRUE)
    if (inherits(N, "try-error") && grepl("singular", N[1], fixed = TRUE)) {
      out <- NA_real_
    } else {
      lt <- mpm_to_table(matU = matU, matF = matR, ...) # passes start/xmax/lx_crit via ...
      out <- sum(lt$x * lt$lx * lt$mx) / sum(lt$lx * lt$mx)
    }
  } else {
    stop("Unsupported method. Must be either 'R0', 'age_diff', or 'cohort'.")
  }
  return(out)
}

test_that("gen_time works correctly", {
  x <- gen_time(mat_u, mat_f)
  x_r <- gen_time(mat_u, matR = mat_f)
  x_c <- gen_time(mat_u, matC = mat_c)
  x_fc <- gen_time(mat_u, matF = mat_f, matC = mat_c)
  x_rfc <- gen_time(mat_u, matR = mat_f + mat_c)
  x_precedence <- gen_time(mat_u, matR = mat_f, matF = mat_c, matC = mat_c)
  x_start <- gen_time(mat_u, mat_f)
  x_singular <- gen_time(mat_u_singular, mat_f)

  expect_length(x, 1L)
  expect_gt(x, 0)
  expect_identical(x, x_r)
  expect_gt(x_c, 0)
  expect_identical(x_fc, x_rfc)
  expect_identical(x, x_precedence)
  expect_length(x_start, 1L)
  expect_gt(x_start, 0)
  expect_identical(x_singular, NA_real_)

  x_r0 <- gen_time(mat_u, mat_f, method = "R0")

  expect_identical(x, x_r0)

  x_po <- gen_time(mat_u, mat_f, method = "age_diff")
  x_singular_po <- gen_time(mat_u_singular, mat_f, method = "age_diff")
  expect_length(x_po, 1L)
  expect_gt(x_po, 0)
  expect_identical(x_singular_po, NA_real_)

  x_co <- gen_time(mat_u, mat_f, method = "cohort")
  x_singular_co <- gen_time(mat_u_singular, mat_f, method = "cohort")
  expect_length(x_co, 1L)
  expect_gt(x_co, 0)
  expect_identical(x_singular_co, NA_real_)
})

test_that("gen_time warns and fails gracefully", {
  expect_warning(gen_time(mat_u_survissue, mat_f))
  expect_error(gen_time(mat_u_na, mat_f))
  expect_error(gen_time(mat_u, mat_f_na))
  expect_error(gen_time(mat_u, matR = mat_f_na))

  expect_error(gen_time(mat_u_na, mat_f, method = "age_diff"))
  expect_error(gen_time(mat_u, mat_f_na, method = "age_diff"))
  expect_error(gen_time(mat_u, matR = mat_f_na, method = "age_diff"))

  expect_warning(gen_time(mat_u_survissue, mat_f))
  expect_error(gen_time(mat_u_na, mat_f, method = "cohort"))
  expect_error(gen_time(mat_u, mat_f_na, method = "cohort"))
  expect_error(gen_time(mat_u, matR = mat_f_na, method = "cohort"))
})

#' @noRd
checkValidMat <- function(M,
                          fail_all_na = TRUE,
                          fail_any_na = TRUE,
                          warn_all_zero = TRUE,
                          warn_surv_issue = FALSE) {
  mn <- deparse(substitute(M)) # name of object passed to M

  if (!is.matrix(M) || !is.numeric(M) || (nrow(M) != ncol(M))) {
    stop("Argument ", mn, " must be a square numeric matrix.\n", call. = FALSE)
  }
  if (fail_all_na && all(is.na(M))) {
    stop("Argument ", mn, " contains only missing values (i.e. all <NA>).\n",
      call. = FALSE
    )
  }
  if (fail_any_na && anyNA(M)) {
    stop("Argument ", mn, " contains missing values (i.e. <NA>).\n",
      call. = FALSE
    )
  }
  if (warn_all_zero && all(M == 0)) {
    warning("All elements of ", mn, " are zero.\n", call. = FALSE)
  }
  if (warn_surv_issue && any(colSums(M) > 1)) {
    warning("Argument ", mn, " has at least one stage-specific survival.\n",
      " probability > 1",
      call. = FALSE
    )
  }
}

#' @noRd
checkMatchingStageNames <- function(M, N = NULL) {
  if (is.null(N)) {
    if (!identical(rownames(M), colnames(M))) {
      stop(strwrap(prefix = " ", initial = "", "When naming lifestages, both
      rows and columns must be named and their names must be identical.\n"))
    }
  } else {
    if (!identical(rownames(M), colnames(M)) || !identical(
      rownames(N),
      colnames(N)
    ) ||
      !identical(M * 0L, N * 0L)) {
      stop(strwrap(prefix = " ", initial = "", "When naming lifestages, both
      rows and columns must be named their names must be identical, and they
      must be the same between both matrices passed to the function.\n"))
    }
  }
}

#' @noRd
checkValidStartLife <- function(s, M, start_vec = FALSE) {
  if (start_vec) {
    # check that abundances of all stages have been set if passing
    # a population vector
    if ((length(s) > 1 && length(s) != ncol(M)) ||
      (is.numeric(s) && length(s) == 1 && !(s %in% seq_len(nrow(M)))) ||
      (is.character(s) && length(s) == 1 &&
        !(s %in% unique(unlist(dimnames(M)))))) {
      stop(strwrap(prefix = " ", initial = "", "Argument 'start' must be an
      integer within 1:nrow(matU), a character matching a stage class name in
      dimnames(matU), or an integer vector of starting abundances of length
      ncol(matU).\n"),
        call. = FALSE
      )
    }
  } else {
    # check that start is a single value, either an index or named life stage
    if (length(s) != 1 || !(s %in% seq_len(nrow(M))) &&
      !(s %in% unique(unlist(dimnames(M))))) {
      stop(strwrap(prefix = " ", initial = "", "Argument 'start' must be an
      integer within 1:nrow(matU), or a character matching a stage class name
      in dimnames(matU).\n"),
        call. = FALSE
      )
    }
  }
}

#' @importFrom stats na.omit
#' @noRd
checkValidStages <- function(M, stages) {
  # checks that the stages vector supplied has valid indices or names
  arg <- deparse(substitute(stages))
  if (is.logical(stages) && length(stages) != ncol(M)) {
    stop("Length of `", arg, "` (logical vector) must equal the dimension of ",
      "the matrix (", ncol(M), ").\n",
      call. = FALSE
    )
  } else if (is.numeric(stages) &&
    !all(stages[!is.na(stages)] %in% seq_len(ncol(M)))) {
    stop("Some stage indices ",
      ifelse(is.null(arg), "", paste0("in `", arg, "` ")),
      "exceeded matrix dimension (", nrow(M), "): ",
      paste(stages[stages > ncol(M) | stages <= 0L], collapse = ", "),
      call. = FALSE
    )
  } else if (is.character(stages) &&
    !all(stages[!is.na(stages)] %in% colnames(M))) {
    stop("Some stage names ",
      ifelse(is.null(arg), "", paste0("in `", arg, "` ")),
      "were not found in matrix: ",
      paste(stages[!stages %in% colnames(M)], collapse = ", "),
      call. = FALSE
    )
  } else if (is.list(stages) && any(vapply(stages, is.numeric, logical(1))) &&
    !all(na.omit(unique(unlist(stages))) %in%
      seq_len(ncol(M)))) {
    stop("Some stage indices ",
      ifelse(is.null(arg), "", paste0("in `", arg, "` ")),
      "exceeded matrix dimension (", nrow(M), "): ",
      paste(
        unique(unlist(stages))[!unique(unlist(stages)) %in%
          seq_len(ncol(M))],
        collapse = ", "
      ),
      call. = FALSE
    )
  } else if (is.list(stages) && any(vapply(stages, is.character, logical(1))) &&
    !all(na.omit(unique(unlist(stages))) %in%
      colnames(M))
  ) {
    stop("Some stage names ",
      ifelse(is.null(arg), "", paste0("in `", arg, "` ")),
      "were not found in matrix: ",
      paste(unique(unlist(stages))[!unique(unlist(stages)) %in% colnames(M)],
        collapse = ", "
      ),
      call. = FALSE
    )
  }
}

#' @noRd
colSums2 <- function(mat) {
  # like colSums(x, na.rm = TRUE), except that column with only NAs will return
  # NA rather than 0
  apply(mat, 2, function(x) {
    ifelse(all(is.na(x)), NA_real_, sum(x, na.rm = TRUE))
  })
}


### utilities to calculate mean matrix from list of matrices
#' @noRd
meanMat <- function(x, na.rm = FALSE) {
  n_row <- vapply(x, nrow, numeric(1))
  n_col <- vapply(x, ncol, numeric(1))
  if (length(unique(n_row)) != 1 || length(unique(n_col)) != 1) {
    stop("All matrices in list must be of same dimension.\n")
  }
  if (na.rm) x <- lapply(x, zero_NA)
  n <- length(x)
  return(Reduce("+", x) / n)
}


#' @noRd
zero_NA <- function(m) {
  m[is.na(m)] <- 0
  return(m)
}


#' @noRd
area_under_curve <- function(x, y) {
  delta_x <- diff(x)
  if (any(delta_x <= 0)) stop("area_under_curve: x should be ascending.\n")
  rect_heights <- (y[-1] + y[-length(y)]) / 2
  return(sum(delta_x * rect_heights))
}


#' @noRd
#' @importFrom MASS ginv
.matrix_inverse <- function(mat) {
  mat_inv <- try(solve(mat), silent = TRUE)
  if (inherits(mat_inv, "try-error")) {
    mat_inv <- ginv(mat)
  }
  return(mat_inv)
}


#' @noRd
lambda <- function(x) {
  return(popdemo::eigs(x, what = "lambda"))
}


#' @noRd
stable.stage <- function(x) {
  return(popdemo::eigs(x, what = "ss"))
}


#' @noRd
reproductive.value <- function(x) {
  return(popdemo::eigs(x, what = "rv"))
}


#' @noRd
process_fertility_inputs <- function(matR = NULL, matF = NULL, matC = NULL) {
  # If matR is provided, return it as is
  if (!is.null(matR)) {
    return(matR)
  }

  # If matR is NULL, process matF and matC
  if (!is.null(matF) && is.null(matC)) {
    # If matF is provided but matC is NULL, assume matC is a zero matrix
    matC <- matrix(0, nrow = nrow(matF), ncol = ncol(matF))
  } else if (is.null(matF) && !is.null(matC)) {
    # If matC is provided but matF is NULL, assume matF is a zero matrix
    matF <- matrix(0, nrow = nrow(matC), ncol = ncol(matC))
  } else if (is.null(matF) && is.null(matC)) {
    # If both matF and matC are NULL, return NULL
    return(NULL)
  }

  # Combine matF and matC to create matR
  matR <- matF + matC

  # Return the constructed matR
  return(matR)
}

#' @noRd
get.leslie <- function(lx, mx) {
  nage <- length(lx)
  A <- matrix(0, nrow = nage, ncol = nage)
  A[1, ] <- lx * mx
  A[2:nage, 1:(nage - 1)] <- diag(1, nage - 1)
  return(A)
}

test_that("checkValidStartLife works correctly", {
  M <- matrix(1:4, nrow = 2)

  startLife <- 1
  expect_silent(checkValidStartLife(startLife, M))

  startLife <- c(3, 4)
  expect_error(checkValidStartLife(startLife, M))

  startLife <- 5
  expect_error(checkValidStartLife(startLife, M))

  startLife <- c(0.4, 0.6)
  expect_silent(checkValidStartLife(startLife, M, start_vec = TRUE))

  startLife <- c(0, 0.4, 0.6)
  expect_error(checkValidStartLife(startLife, M, start_vec = TRUE))
})

test_that("meanMat works correctly", {
  s <- 3 # matrix dimension

  # generate lists of matrices
  x1 <- x2 <- replicate(5, matrix(runif(s^2), s, s), simplify = FALSE)
  x2[[1]][1, 1] <- NA

  y1 <- meanMat(x1)
  expect_true(inherits(y1, "matrix"))
  expect_identical(nrow(y1), as.integer(s))

  # na handling
  y2 <- meanMat(x2)
  expect_true(inherits(y2, "matrix"))
  expect_true(is.na(y2[1, 1]))

  y3 <- meanMat(x2, na.rm = TRUE)
  expect_false(is.na(y3[1, 1]))
})

test_that("meanMat warns and fails gracefully", {
  x3 <- list(matrix(0:3, nrow = 2), matrix(1:9, nrow = 3))
  expect_error(meanMat(x3))
})

#' Compute distribution of stage at death from a stage-structured MPM
#'
#' Calculates the expected distribution of deaths across stages for a cohort
#' governed by the survival/transition submatrix \code{U} of a matrix population
#' model. The result can be interpreted as the probability of dying in each
#' stage.
#'
#' @section Note:
#' This function uses the fundamental matrix of the Markov chain implied by
#' \code{U} to estimate the total expected time spent in each stage, then
#' combines this with per-stage death probabilities. The output describes where
#' in the life cycle deaths occur, given a specified starting stage
#' distribution.
#'
#' @section Warning:
#' Results may be misleading if \code{U} includes reproduction or if any column
#' sums exceed 1. Use only the survival/transition submatrix. The calculation
#' also assumes that all stages are transient (i.e. eventual death is certain).
#'
#' @param matU A square numeric matrix giving the survival/transition (\eqn{U})
#'   submatrix of a stage-based (Lefkovitch) matrix population model.
#' @param start Optional numeric vector of length \eqn{n} giving the initial
#'   cohort distribution across stages. It will be rescaled to sum to 1.
#'   Defaults to a uniform distribution if \code{NULL}.
#'
#' @return A numeric vector giving the expected contribution of each stage to
#'   deaths. Names are taken from \code{rownames(U)} when available.
#'
#' @author Owen Jones <jones@@biology.sdu.dk>
#'
#' @family life history traits
#'
#' @examples
#' data(mpm1)
#'
#' # Uniform starting cohort (default)
#' stage_at_death_dist(mpm1$matU)
#'
#' # Starting entirely in stage 1
#' stage_at_death_dist(mpm1$matU, start = c(1, 0, 0, 0, 0))
#'
#' # Starting with the SSD (requires the full A matrix)
#' matA <- mpm1$matU + mpm1$matF
#' ssd <- popdemo::eigs(matA, "ss")
#' stage_at_death_dist(mpm1$matU, start = ssd)
#'
#' @export
stage_at_death_dist <- function(matU, start = NULL) {
  if (!is.matrix(matU) || !is.numeric(matU) || nrow(matU) != ncol(matU)) {
    stop("matU must be a square numeric matrix")
  }

  n <- ncol(matU) # define n before using it

  if (!is.null(start)) {
    if (!is.numeric(start)) {
      stop("'start' must be numeric")
    }
    if (length(start) != n) {
      stop(paste0("'start' must have length ", n, " (number of stages)"))
    }
    if (any(start < 0)) {
      stop("'start' must not contain negative values")
    }
    if (sum(start) == 0) {
      stop("'start' must contain at least one non-zero value")
    }
  }

  w <- if (is.null(start)) rep(1 / n, n) else as.numeric(start)
  w <- w / sum(w)

  # per-stage death probability (assumes U excludes reproduction)
  d <- 1 - colSums(matU)

  # basic sanity check (optional)
  if (any(d < -1e-12)) {
    stop("Some column sums of matU exceed 1; matU should be survival/transition only.")
  }

  N <- solve(diag(n) - matU) # fundamental matrix for cohort occupancy
  occ <- as.vector(N %*% w)
  deaths <- as.vector(d * occ)

  names(deaths) <- rownames(matU)
  return(deaths)
}

test_that("stage_at_death_dist works correctly", {
  x_default <- stage_at_death_dist(mat_u)
  x_start <- stage_at_death_dist(mat_u, start = c(1, 0, 0, 0))
  x_named <- stage_at_death_dist(mat_u_named)

  expect_type(x_default, "double")
  expect_length(x_default, ncol(mat_u))
  expect_equal(sum(x_default), 1)

  expect_type(x_start, "double")
  expect_length(x_start, ncol(mat_u))
  expect_equal(sum(x_start), 1)
  expect_false(isTRUE(all.equal(x_default, x_start)))

  expect_identical(names(x_named), rownames(mat_u_named))
})

test_that("stage_at_death_dist warns and fails gracefully", {
  expect_error(stage_at_death_dist(mat_u_notsq))
  expect_error(stage_at_death_dist(mat_u, start = "stage_1"))
  expect_error(stage_at_death_dist(mat_u, start = c(1, 0, 0)))
  expect_error(stage_at_death_dist(mat_u, start = c(1, -1, 0, 0)))
  expect_error(stage_at_death_dist(mat_u, start = c(0, 0, 0, 0)))
  expect_error(stage_at_death_dist(mat_u_survissue))
})

#' Calculate Demetrius' entropy from trajectories of age-specific survivorship
#' and fecundity
#'
#' This function calculates Demetrius' entropy from vectors of age-specific
#' survivorship (\code{lx}) and fecundity (\code{mx}). Users can choose between
#' the scaled (Caswell, 2001 eqns. 4.94-4.97) or unscaled (from Demetrius 1978)
#' method.
#'
#' The scaled version accounts for population growth or shrinkage by adjusting
#' the contributions of survivorship and fecundity using the dominant eigenvalue
#' (lambda). Specifically, each contribution is weighted by lambda raised
#' to the negative power of age. Conversely, the unscaled version does not
#' account for population growth. It calculates entropy directly from the
#' proportional contributions of survivorship and fecundity without adjustment
#' for population dynamics.
#'
#' @section Warning: Note that this function may produce unexpected results if
#'   used on partial survivorship and fecundity trajectories. In addition, it is
#'   sensitive to the length of the these vectors. We direct users to the
#'   functions `\code{\link{shape_surv}}` and `\code{\link{shape_rep}}` which
#'   are relatively robust to these issues.
#'
#' @param lx Either a survivorship trajectory (a vector of
#'   monotonically-declining values in the interval [0,1]), or submatrix U from
#'   a matrix population model.
#' @param mx Either an age-specific fecundity trajectory (a vector of
#'   non-negative values), or submatrix F from a matrix population model.
#' @param type Calculation type, either `scaled` (default) or `unscaled`.
#' @param ... Additional variables passed to `mpm_to_lx` and `mpm_to_mx` if the
#'   data are supplied as matrices. This could include the `start` argument to
#'   select a starting stage.
#'
#' @return Demetrius' entropy.
#'
#' @author Roberto Salguero-Gomez <rob.salguero@@zoo.ox.ac.uk>
#' @author Patrick Barks <patrick.barks@@gmail.com>
#' @author Richard Hinrichsen <rich@@hinrichsenenvironmental.com>
#' @author Owen Jones <jones@@biology.sdu.dk>
#'
#' @family life history traits
#'
#' @references Demetrius, L. 1978. Adaptive value, entropy and survivorship
#'   curves. Nature, 275(5677), 213–214.
#'
#'   Caswell, H. 2001. Matrix Population Models: Construction, Analysis, and
#'   Interpretation. Sinauer Associates.
#'
#' @examples
#' data(mpm1)
#'
#' # derive trajectories of lx and mx, starting from stage 2
#' lx <- mpm_to_lx(mpm1$matU, start = 2)
#' mx <- mpm_to_mx(mpm1$matU, mpm1$matF, start = 2)
#'
#' entropy_d(lx, mx, type = "unscaled")
#' entropy_d(lx, mx, type = "scaled")
#'
#'
#' # calculate entropy directly from MPM
#' entropy_d(lx = mpm1$matU, mx = mpm1$matF, start = 2)
#'
#' @export
entropy_d <- function(lx, mx, type = "scaled", ...) {
  if (inherits(lx, "matrix") && inherits(mx, "matrix")) {
    mx <- mpm_to_mx(lx, mx, ...)
  }

  if (inherits(lx, "matrix")) {
    lx <- mpm_to_lx(lx, ...)
  }

  # Validate arguments
  if (any(lx < 0 | lx > 1)) {
    stop("All values of lx must be within the interval [0, 1].\n")
  }
  if (any(diff(lx) > 1e-7)) {
    stop("Values of lx must be monotonically declining.\n")
  }
  if (any(mx < 0)) {
    stop("All values of mx must be >= 0.\n")
  }

  lxmx <- lx * mx

  if (type == "scaled") {
    # Create Leslie matrix and calculate lambda
    A <- get.leslie(lx, mx)
    nage <- dim(A)[1]
    lambda <- max(Mod(eigen(A)$values))
    expon <- 1:nage
    LAM <- lambda^(-expon)
    px <- lxmx * LAM
  } else if (type == "unscaled") {
    px <- lxmx / sum(lxmx)
  } else {
    stop("Invalid type. Please choose either 'scaled' or 'unscaled'.")
  }

  log_px <- log(px)
  log_px[px == 0] <- 0
  H <- -sum(px * log_px)

  return(H)
}

test_that("entropy_d works correctly", {
  lx <- 0.8^(0:8)
  mx <- c(0.00, 0.00, 1.10, 1.50, 1.60, 1.70, 1.50, 1.20, 0.70)

  x <- entropy_d(lx, mx)

  expect_length(x, 1L)
  expect_gt(x, 0)
})

test_that("entropy_d warns and fails gracefully", {
  lx1 <- c(1.1, 0.6, 0.5, 0.4) # lx is greater than 1
  mx1 <- c(4.4, 3.3, 2.2, 1.1)
  expect_error(entropy_d(lx1, mx1))

  lx2 <- c(1.0, 0.6, 0.61, 0.5) # lx not monotonically declining
  mx2 <- c(4.4, 3.3, 2.2, 1.1)
  expect_error(entropy_d(lx2, mx2))

  lx3 <- c(1.0, 0.6, 0.6, 0.5) # negative mx
  mx3 <- c(4.4, 3.3, 2.2, -0.1)
  expect_error(entropy_d(lx3, mx3))
})

#' Convert between age-specific survivorship, survival, or mortality hazard
#'
#' Convert between vectors of age-specific survivorship (\code{lx}), survival
#' probability (\code{px}), or mortality hazard (\code{hx}). Input vectors must
#' be arranged in order of increasing age, starting with age 0.
#'
#' @param lx Vector of age-specific survivorship.
#' @param px Vector of age-specific survival probabilities.
#' @param hx Vector of age-specific mortality hazards.
#'
#' @return A vector.
#'
#' @details \code{lx} gives the proportional survivorship to the start of age
#'   class \code{x} (where survivorship at first age class is defined as 1),
#'   \code{px} gives the probability of survival between age \code{x} and
#'   \code{x+1}, and \code{hx} gives the time-averaged mortality hazard (also
#'   called force of mortality) between age \code{x} and \code{x+1}.
#'
#' @note Note that the units of time for the returned vectors (i.e., \code{x})
#'   are the same as the (\code{ProjectionInterval}) of the MPM.
#'
#' @author Patrick Barks <patrick.barks@@gmail.com>
#'
#' @family life tables
#'
#' @references
#' Caswell, H. 2001. Matrix Population Models: Construction, Analysis, and
#' Interpretation. Sinauer Associates; 2nd edition. ISBN: 978-0878930968
#'
#' Caswell, H. 2006. Applications of Markov chains in demography. pp. 319-334 in
#' A.N. Langville and W.J. Stewart (editors) MAM2006: Markov Anniversary
#' Meeting. Boson Books, Raleigh, North Caroline, USA
#'
#' Ergon, T., Borgan, Ø., Nater, C. R., & Vindenes, Y. 2018. The utility of
#' mortality hazard rates in population analyses. Methods in Ecology and
#' Evolution, 9, 2046-2056. <doi:10.1111/2041-210X.13059>
#'
#' Horvitz, C. & Tuljapurkar, S. 2008. Stage dynamics, period survival, and
#' mortality plateaus. The American Naturalist 172: 203-2015.
#' <doi:10.1086/589453>
#'
#' Jones, O. R., Scheuerlein, A., Salguero-Gomez, R., Camarda, C. G., Schaible,
#' R., Casper, B. B., Dahlgren, J. P., Ehrlén, J., García, M. B., Menges, E.,
#' Quintana-Ascencio, P. F., Caswell, H., Baudisch, A. & Vaupel, J. 2014.
#' Diversity of ageing across the tree of life. Nature 505, 169-173.
#' <doi:10.1038/nature12789>
#'
#' Jones O. R. 2021. Life tables: Construction and interpretation In:
#' Demographic Methods Across the Tree of Life. Edited by Salguero-Gomez R &
#' Gamelon M. Oxford University Press. Oxford, UK. ISBN: 9780198838609
#'
#' Preston, S., Heuveline, P., & Guillot, M. 2000. Demography: Measuring and
#' Modeling Population Processes. Wiley. ISBN: 9781557864512
#'
#' @examples
#' lx <- c(1, 0.8, 0.7, 0.5, 0.3, 0.1)
#'
#' # convert from lx
#' px <- lx_to_px(lx)
#' hx <- lx_to_hx(lx)
#'
#' # convert from px
#' lx <- px_to_lx(px)
#' hx <- px_to_hx(px)
#'
#' # convert from hx
#' lx <- hx_to_lx(hx)
#' px <- hx_to_px(hx)
#'
#' @name lifetable_convert
NULL



#' @rdname lifetable_convert
#' @export lx_to_px
lx_to_px <- function(lx) {
  if (length(lx) == 1) {
    NA_real_
  } else {
    c(lx[-1] / lx[-length(lx)], NA)
  }
}


#' @rdname lifetable_convert
#' @export lx_to_hx
lx_to_hx <- function(lx) {
  if (length(lx) == 1) {
    NA_real_
  } else {
    px <- lx_to_px(lx)
    px_to_hx(px)
  }
}


#' @rdname lifetable_convert
#' @export px_to_lx
px_to_lx <- function(px) {
  c(1, cumprod(px[-length(px)]))
}


#' @rdname lifetable_convert
#' @export px_to_hx
px_to_hx <- function(px) {
  -log(px)
}


#' @rdname lifetable_convert
#' @export hx_to_lx
hx_to_lx <- function(hx) {
  px <- hx_to_px(hx)
  px_to_lx(px)
}


#' @rdname lifetable_convert
#' @export hx_to_px
hx_to_px <- function(hx) {
  exp(-hx)
}

test_that("shape_surv works correctly", {
  # constant hazard
  lx1 <- 0.5^(0:20)
  s1a <- shape_surv(lx1, trunc = TRUE)
  expect_identical(s1a, 0)

  # constant hazard, custom xlim
  s1b <- shape_surv(lx1, xmin = 2, xmax = 10, trunc = TRUE)
  expect_identical(s1b, 0)

  # increasing hazard
  x2 <- seq(0, 1, 0.1)
  hx2 <- 0.5 * x2
  lx2 <- hx_to_lx(hx2)
  s2 <- shape_surv(lx2, trunc = TRUE)
  expect_true(s2 > 0 & s2 < 0.5)

  # declining hazard
  x3 <- seq(0, 1, 0.1)
  hx3 <- 0.2 - 0.2 * x3
  lx3 <- hx_to_lx(hx3)
  s3 <- shape_surv(lx3, trunc = TRUE)
  expect_true(s3 < 0 & s3 > -0.5)

  # check works with data frame
  lt <- data.frame(x = x3, lx = lx3)
  s4 <- shape_surv(lt, trunc = TRUE)
  expect_equal(s4, s3, tolerance = 1e-09)
})


test_that("shape_surv warns and fails gracefully", {
  # first element of lx is not 1
  expect_error(shape_surv(c(0.8, 0.7, 0.6), trunc = TRUE))

  # Zero not dealt with
  expect_error(shape_surv(c(1, 0.5, 0.25, 0)))

  # zombies
  expect_error(shape_surv(c(1, 0.7, 0.8, 0.3), trunc = TRUE))

  # < 3 nozero values of lx
  expect_error(shape_surv(c(1, 0.5, 0), trunc = TRUE))

  #' surv' doesn't contain both x and lx
  surv1 <- list(years = 0:3, lx = c(1, 0.8, 0.7, 0.6))
  expect_error(shape_surv(surv1))

  # x and lx must be the same length
  surv2 <- list(x = 0:2, lx = c(1, 0.8, 0.7, 0.6))
  expect_error(shape_surv(surv2))

  # lx must start with 1 where x[1] is 0
  surv3 <- list(x = 0:3, lx = c(0.9, 0.8, 0.7, 0.6))
  expect_error(shape_surv(surv3))

  # much as we'd like to reverse ageing, x must all be ascending
  surv3 <- list(x = c(0, 1, 2, 1), lx = c(1, 0.8, 0.7, 0.6))
  expect_error(shape_surv(surv3))
})

test_that("surv_conversion functions work correctly", {
  # convert from lx
  lx1 <- c(1, 0.8, 0.7, 0.5, 0.3, 0.1)
  px1 <- lx_to_px(lx1)
  hx1 <- lx_to_hx(lx1)

  # convert from px
  lx2 <- px_to_lx(px1)
  hx2 <- px_to_hx(px1)

  expect_true(all(lx2 == lx1))
  expect_true(all(hx2 == hx1 | (is.na(hx2) & is.na(hx1))))

  # convert from hx
  lx3 <- hx_to_lx(hx2)
  px3 <- hx_to_px(hx2)

  expect_true(all(lx3 == lx1))
  expect_true(all(px3 == px1 | (is.na(px3) & is.na(px3))))
})


test_that("surv_conversion functions warn and fail gracefully", {
  expect_true(is.na(lx_to_px(1)))
  expect_true(is.na(lx_to_hx(1)))
})

#' @title Age of reproductive maturity
#'
#' @description
#' Apply Markov chain approaches to compute age-specific
#' trajectory of reproduction for individuals in a matrix population model.
#' Includes functions to calculate the probability of achieving reproductive
#' maturity (\code{mature_prob}), mean age at first reproduction
#' (\code{mature_age}), and distribution of individuals first achieving
#' reproductive maturity among stage class (\code{mature_distrib}).
#'
#' @param matU The survival component of a matrix population model (i.e., a
#'   square projection matrix reflecting survival-related transitions; e.g.,
#'   progression, stasis, and retrogression).
#' @param matR The reproductive component of a matrix population model (i.e., a
#'   square projection matrix only reflecting transitions due to reproduction;
#'   either sexual, clonal, or both). If \code{matR} is not provided, it will be
#'   constructed by summing \code{matF} and \code{matC}.
#' @param matF (Optional) The matrix reflecting sexual reproduction. If provided
#'   without \code{matC}, \code{matC} is assumed to be a zero matrix. If
#'   \code{matR} is provided, this argument is ignored.
#' @param matC (Optional) The matrix reflecting clonal (asexual) reproduction.
#'   If provided without \code{matF}, \code{matF} is assumed to be a zero
#'   matrix. If \code{matR} is provided, this argument is ignored.
#' @param start The index (or stage name) of the first stage at which the author
#'   considers the beginning of life. Defaults to \code{1}.
#' @param repro_stages A vector of stage names or indices indicating which
#'   stages are reproductive. Alternatively, a logical vector of length
#'   \code{ncol(matU)} indicating whether each stage is reproductive
#'   (\code{TRUE}) or not (\code{FALSE}).
#'
#' @return For \code{mature_distrib}, a vector giving the proportion of
#'   individuals that first reproduce within each stage class. For all others, a
#'   scalar trait value.
#'
#' @note Note that the units of time in returned values are the same as the
#'   \code{ProjectionInterval} of the MPM.
#'
#' @author Roberto Salguero-Gomez <rob.salguero@@zoo.ox.ac.uk>
#' @author Hal Caswell <hcaswell@@whoi.edu>
#' @author Owen R. Jones <jones@@biology.sdu.dk>
#' @author Patrick Barks <patrick.barks@@gmail.com>
#'
#' @family life history traits
#'
#' @references Caswell, H. 2001. Matrix Population Models: Construction,
#'   Analysis, and Interpretation. Sinauer Associates; 2nd edition. ISBN:
#'   978-0878930968
#'
#' @examples
#' data(mpm1)
#'
#' mature_prob(matU = mpm1$matU, matR = mpm1$matF, start = 2)
#' mature_prob(mpm1$matU, mpm1$matF, start = 2)
#' mature_age(matU = mpm1$matU, matR = mpm1$matF, start = 2)
#' mature_age(matU = mpm1$matU, matF = mpm1$matF, start = 2)
#'
#' ### distribution of first reproductive maturity among stage classes
#' repstage <- repro_stages(mpm1$matF)
#' mature_distrib(mpm1$matU, start = 2, repro_stages = repstage)
#'
#' @name repro_maturity
NULL


#' @rdname repro_maturity
#' @export mature_prob
mature_prob <- function(matU, matR = NULL, matF = NULL,
                        matC = NULL, start = 1L) {
  # Call the helper function to construct matR if not provided
  matR <- process_fertility_inputs(matR, matF, matC)

  # validate arguments
  checkValidMat(matU, warn_surv_issue = TRUE)
  checkValidMat(matR)
  checkValidStartLife(start, matU)
  if (!is.numeric(start)) {
    checkMatchingStageNames(M = matU, N = matR)
  }

  fec_stages <- apply(matR, 2, function(x) any(x > 0))
  Bprime <- calc_Bprime(matU, fec_stages)

  return(unname(Bprime[2, start]))
}


#' @rdname repro_maturity
#' @export mature_age
mature_age <- function(matU, matR = NULL, matF = NULL,
                       matC = NULL, start = 1L) {
  # Call the helper function to construct matR if not provided
  matR <- process_fertility_inputs(matR, matF, matC)

  # validate arguments
  checkValidMat(matU, warn_surv_issue = TRUE)
  checkValidMat(matR)
  checkValidStartLife(start, matU)
  if (!is.numeric(start)) {
    checkMatchingStageNames(M = matU, N = matR)
  }

  m <- nrow(matU)
  fec_stages <- apply(matR, 2, function(x) any(x > 0))

  Uprime <- matU
  Uprime[, fec_stages] <- 0

  Bprime <- calc_Bprime(matU, fec_stages)

  if (Bprime[2, start] == 0) {
    # if Pr[maturity] == 0
    return(NA_real_)
  } else {
    # mean age at first reproduction ('L_a' in Caswell 2001, p 124)
    D <- diag(c(Bprime[2, ]))
    Uprimecond <- D %*% Uprime %*% .matrix_inverse(D)
    dimnames(Uprimecond) <- dimnames(matU)
    expTimeReprod <- colSums(.matrix_inverse(diag(m) - Uprimecond))
    return(expTimeReprod[start])
  }
}


#' @rdname repro_maturity
#' @export mature_distrib
mature_distrib <- function(matU, start = 1L, repro_stages) {
  # validate arguments
  checkValidMat(matU, warn_surv_issue = TRUE)
  checkValidStartLife(start, matU)
  if (length(repro_stages) > ncol(matU)) {
    stop("More repro_stages were supplied, (", length(repro_stages),
      ") than there are life stages (", ncol(matU), ").\n",
      call. = FALSE
    )
  }
  if (is.logical(repro_stages) && ncol(matU) != length(repro_stages)) {
    stop("length(repro_stages) must equal ncol(matU).\n", call. = FALSE)
  }
  if (!is.numeric(start)) {
    checkMatchingStageNames(M = matU)
  }
  if (is.numeric(repro_stages) || is.character(repro_stages)) {
    checkValidStages(matU, repro_stages)
  }

  primeU <- matU
  primeU[, repro_stages] <- 0

  N <- .matrix_inverse(diag(nrow(primeU)) - primeU)

  n1 <- rep(0, nrow(matU))
  names(n1) <- rownames(matU)
  n1[repro_stages] <- N[repro_stages, start] / sum(N[repro_stages, start])
  n1[is.nan(n1)] <- 0.0 # coerce NaN to 0

  return(n1)
}


#' @noRd
calc_Bprime <- function(matU, fec_stages) {
  # Note: internal function; doesn't need stage name support in
  # current use context
  m <- ncol(matU)

  # Probability of survival to first sexual reproductive event
  # Note: U matrices are called 'T' in Caswell (2001)
  Uprime <- matU
  Uprime[, fec_stages] <- 0

  Mprime <- matrix(0, nrow = 2, ncol = m)

  for (p in 1:m) {
    if (fec_stages[p]) {
      Mprime[2, p] <- 1
    } else {
      Mprime[1, p] <- 1 - colSums(matU)[p]
    }
  }

  Bprime <- Mprime %*% .matrix_inverse(diag(m) - Uprime)
  return(Bprime)
}

test_that("mature_ functions work correctly", {
  x_prob1 <- mature_prob(mat_u, mat_f)
  expect_type(x_prob1, "double")
  expect_length(x_prob1, 1L)

  x_prob_matf <- mature_prob(mat_u, matF = mat_f)
  x_prob_matc <- mature_prob(mat_u, matC = mat_c)
  expect_identical(x_prob_matf, x_prob1)
  expect_identical(x_prob_matc, mature_prob(mat_u, mat_c))

  x_prob2 <- mature_prob(mat_u, mat_f, start = 2)
  expect_gt(x_prob2, x_prob1)

  x_age1 <- mature_age(mat_u, mat_f)
  expect_type(x_age1, "double")
  expect_length(x_age1, 1L)

  x_age_matf <- mature_age(mat_u, matF = mat_f)
  x_age_matc <- mature_age(mat_u, matC = mat_c)
  expect_identical(x_age_matf, x_age1)
  expect_identical(x_age_matc, mature_age(mat_u, mat_c))

  x_age2 <- mature_age(mat_u, mat_f, start = 2)
  expect_lt(x_age2, x_age1)

  # test life cycles with no connection from 'start' to repro stage(s)
  matU <- rbind(
    c(0.0, 0.0, 0.0),
    c(0.1, 0.2, 0.0),
    c(0.0, 0.0, 0.0)
  )

  matF <- rbind(
    c(0.0, 0.0, 5.3),
    c(0.0, 0.0, 0.0),
    c(0.0, 0.0, 0.0)
  )

  x_age3 <- mature_age(matU, matF, start = 1)
  expect_true(is.na(x_age3))
})


test_that("mature_ functions warn and fail gracefully", {
  expect_warning(mature_prob(mat_u_survissue, mat_f))
  expect_error(mature_age(mat_u_na, mat_f))
  expect_error(mature_prob(mat_u, mat_f, start = "stage name"))
  expect_error(mature_age(mat_u_named, mat_f_named, start = "invalid stage"))
  expect_error(mature_prob(mat_u))
  expect_error(mature_age(mat_u))
})

#' Calculate shape of reproduction over age
#'
#' Calculates a 'shape' value of distribution of reproduction over age by
#' comparing the area under a cumulative reproduction curve (over age) with the
#' area under a cumulative function describing constant reproduction.
#'
#' @param rep Either 1) a numeric vector describing reproduction over age (mx),
#'   2) a \code{data.frame} / \code{list} with one column / element titled 'mx'
#'   describing a reproduction over age, optionally a column / element 'x'
#'   containing age classes (each element a number representing the age at the
#'   start of the class), or 3) a matrix, specifically the reproduction
#'   submatrix (e.g. F matrix) of a matrix population model. If \code{rep} is
#'   provided as a matrix, then \code{surv} must be provided as the U submatrix
#'   of the matrix population model.
#'
#'   In case (2), if x is not supplied, the function will assume age classes
#'   starting at 0 with time steps of unit. If x ends at maximum longevity,
#'   \code{mx[which.max(x)]} should equal 0; however it is possible to supply
#'   partial reproduction schedules.
#' @param surv An optional argument to be used if rep is provided as a matrix
#'   (the reproduction submatrix of the matrix population model.) If \code{rep}
#'   is provided as a matrix, then \code{surv} should also be provided as a the
#'   U submatrix of the matrix population model.
#' @param xmin,xmax The minimum and maximum age respectively over which to
#'   evaluate shape. If not given, these default to \code{min(x)} and
#'   \code{max(x)} respectively.
#' @param ... Additional variables passed to `mpm_to_mx` when the data are
#'   provided as matrices.
#'
#' @return a shape value describing symmetry of reproduction over age by
#'   comparing the area under a cumulative reproduction curve over age with the
#'   area under constant reproduction. May take any real value between -0.5 and
#'   +0.5. A value of 0 indicates negligible ageing (neither generally
#'   increasing nor generally decreasing reproduction with age); positive values
#'   indicate senescence (generally decreasing reproduction with age); negative
#'   values indicate negative senescence (generally increasing reproduction with
#'   age). A value of +0.5 indicates that (hypothetically) all individuals are
#'   born to individuals of age 0; a value of -0.5 indicates that all
#'   individuals are born at the age of maximum longevity.
#'
#' @author Iain Stott <iainmstott@@gmail.com>
#'
#' @references Wrycza, T.F. and Baudisch, A., 2014. The pace of aging: Intrinsic
#'   time scales in demography. Demographic Research, 30, pp.1571-1590.
#'   <doi:10.4054/DemRes.2014.30.57>
#'
#'   Baudisch, A. 2011, The pace and shape of ageing. Methods in Ecology and
#'   Evolution, 2: 375-382. <doi:10.1111/j.2041-210X.2010.00087.x>
#'
#'   Baudisch, A, Stott, I. 2019. A pace and shape perspective on fertility.
#'   Methods Ecol Evol. 10: 1941– 1951. <doi:10.1111/2041-210X.13289>
#'
#' @family life history traits
#'
#' @examples
#' # increasing mx yields negative shape
#' mx <- c(0, 0, 0.3, 0.4, 0.5, 0.6)
#' shape_rep(mx)
#'
#' # decreasing mx yields positive shape
#' mx <- c(1.1, 1.0, 0.9, 0.8, 0.7, 0.6, 0.5, 0.4)
#' shape_rep(mx)
#'
#' # constant mx yields shape = 0
#' mx <- c(0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1)
#' shape_rep(mx)
#'
#' # calculate mx trajectory first
#' mpm_to_mx(matU = mpm1$matU, matR = mpm1$matF)
#'
#'
#' # providing the matrices directly
#' data(mpm1)
#' shape_rep(rep = mpm1$matF, surv = mpm1$matU)
#'
#' @export shape_rep
shape_rep <- function(rep, surv = NULL, xmin = NULL, xmax = NULL, ...) {
  if (inherits(surv, "matrix")) {
    if (is.null(surv)) {
      stop("'surv' must be provided as the U submatrix of the model.")
    }
    rep <- mpm_to_mx(matU = surv, matR = rep, ...)
  }

  if (inherits(rep, "numeric")) {
    mx <- rep
    x <- seq_along(mx) - 1
  }
  if (inherits(rep, c("list", "data.frame"))) {
    if (!all(c("x", "mx") %in% names(rep))) {
      stop("`rep` doesn't contain both `x` and `mx`.\n")
    }
    x <- rep$x
    mx <- rep$mx
    if (length(x) != length(mx)) {
      stop("`x` and `mx` must be the same length")
    }
  }
  if (is.null(xmin)) xmin <- x[min(which(mx > 0))]
  if (is.null(xmax)) xmax <- max(x)
  if (any(diff(x) <= 0)) stop("much as we'd like to reverse ageing, `x`
                              must all be ascending.\n")
  if (any(mx < 0)) stop("You appear to have minus-babies (check `mx`
                        for negative values).\n")
  x_sub <- x[x >= xmin & x <= xmax]
  if (length(x_sub) <= 2) {
    stop("must have > 2 nonzero values of `mx` to calculate shape.\n")
  }
  ltdim <- length(x)
  Bx <- c(0, cumsum(mx[1:(ltdim - 1)]))
  Bx_sub <- Bx[x >= xmin & x <= xmax]
  xStd <- (x_sub - xmin) / (xmax - xmin)
  Bxmin <- Bx_sub[which.min(xStd)]
  Bxmax <- Bx_sub[which.max(xStd)]
  BxStd <- (Bx_sub - Bxmin) / (Bxmax - Bxmin)
  aucStd <- area_under_curve(xStd, BxStd)
  aucFlat <- 0.5
  shape <- aucStd - aucFlat
  shape
}

test_that("shape_rep works correctly", {
  # constant mx
  mx1 <- c(0, 1, 1, 1, 1, 1, 1, 1, 1)
  s1a <- shape_rep(mx1)
  expect_identical(s1a, 0)

  # constant hazard, custom xlim
  s1b <- shape_rep(mx1, xmin = 2, xmax = 5)
  expect_identical(s1b, 0)

  # decreasing mx
  mx2 <- seq(1, 0, -0.1)
  s2 <- shape_rep(mx2)
  expect_true(s2 > 0 & s2 < 0.5)

  # increasing mx
  mx3 <- seq(0, 1, 0.1)
  s3 <- shape_rep(mx3)
  expect_true(s3 < 0 & s3 > -0.5)

  # check works with data frame
  lt <- data.frame(x = seq_along(mx3) - 1, mx = mx3)
  s4 <- shape_rep(lt)
  expect_identical(s4, s3)
})


test_that("shape_rep warns and fails gracefully", {
  # negative reproduction
  # You appear to have minus-babies (check mx)
  expect_error(shape_rep(c(0, 1, 1, 1, -0.1, 0)))

  # < 3 nozero values of mx
  # must have > 2 nonzero values of mx to calculate shape
  expect_error(shape_rep(c(0, 0.5, 0.5)))

  # "'rep' doesn't contain both x and mx"
  rep1 <- list(mx = c(0, 0, 0.3, 0.4, 0.5, 0.6))
  expect_error(shape_rep(rep1))

  rep1 <- list(years = 0:5, mx = c(0, 0, 0.3, 0.4, 0.5, 0.6))
  expect_error(shape_rep(rep1))

  # x and mx must be the same length
  rep1 <- list(x = 0:4, mx = c(0, 0, 0.3, 0.4, 0.5, 0.6))
  expect_error(shape_rep(rep1))

  # much as we'd like to reverse ageing, x must all be ascending
  rep1 <- list(x = c(0, 1, 2, 3, 4, 3), mx = c(0, 0, 0.3, 0.4, 0.5, 0.6))
  expect_error(shape_rep(rep1))
})

#' Derive mean vital rates from a matrix population model
#'
#' @description
#' Derive mean vital rates of survival, growth (or development), shrinkage (or
#' de-development), stasis, dormancy, or reproduction from a matrix population
#' model, by averaging across stage classes. These functions include optional
#' arguments for custom weighting of different stage classes (see
#' \emph{Weighting stages}), excluding certain stage classes from the
#' calculation (see \emph{Excluding stages}), and defining the set of
#' biologically-possible transitions (see \emph{Possible transitions}).
#'
#' These decompositions assume that all transition rates are products of a
#' stage-specific survival term (column sums of \code{matU}) and a lower level
#' vital rate that is conditional on survival (growth/development,
#' shrinkage/de-development, stasis, dormancy, or a/sexual reproduction).
#' Reproductive vital rates that are not conditional on survival (i.e., within a
#' stage class from which there is no survival) are also allowed.
#'
#' @param matU The survival component of a matrix population model (i.e., a
#'   square projection matrix reflecting survival-related transitions; e.g.,
#'   progression, stasis, and retrogression)
#' @param matR The reproductive component of a matrix population model (i.e., a
#'   square projection matrix reflecting transitions due to reproduction; either
#'   sexual, clonal, or both)
#' @param posU A logical matrix of the same dimension as \code{matU}, with
#'   elements indicating whether a given \code{matU} transition is possible
#'   (\code{TRUE}) or not (\code{FALSE}). Defaults to \code{matU > 0} (see
#'   \emph{Possible transitions}).
#' @param posR A logical matrix of the same dimension as \code{matR}, with
#'   elements indicating whether a given \code{matR} transition is possible
#'   (\code{TRUE}) or not (\code{FALSE}). Defaults to \code{matR > 0} (see
#'   \emph{Possible transitions}).
#' @param exclude Integer, character or logical vector indicating stages for
#'   which transitions both \emph{to} and \emph{from} the stage should be
#'   excluded from the calculation of vital rates. See section
#'   \emph{Excluding stages}.
#' @param exclude_row Integer, character or logical vector indicating stages for
#'   which transitions both \emph{to} and \emph{from} the stage should be
#'   excluded from the calculation of vital rates. See section
#'   \emph{Excluding stages}.
#' @param exclude_col Integer, character or logical vector indicating stages for
#'   which transitions both \emph{to} and \emph{from} the stage should be
#'   excluded from the calculation of vital rates. See section
#'   \emph{Excluding stages}.
#' @param dorm_stages Integer or character vector indicating dormant stage
#'   classes.
#' @param weights_row Vector of stage-specific weights to apply while summing
#'   vital rates across rows within columns. See section \emph{Weighting
#'   stages}.
#' @param weights_col Vector of stage-specific weights to apply while averaging
#'   vital rates across columns. See section \emph{Weighting stages}.
#' @param surv_only_na If there is only one possible \code{matU} transition in a
#'   given column, should that transition be attributed exclusively to survival?
#'   If \code{TRUE}, the vital rate of growth/stasis/shrinkage in that column
#'   will be coerced to \code{NA}. If \code{FALSE}, dividing the single
#'   transition by the stage-specific survival probability will always yield a
#'   value of \code{1}. Defaults to \code{TRUE}.
#'
#' @return Vector of vital rates. Vital rates corresponding to impossible
#'   transitions are coerced to \code{NA} (see \emph{Possible transitions}).
#'
#' @section Possible transitions:
#' A transition rate of \code{0} within a matrix population model may indicate
#' that the transition is not possible in the given life cycle (e.g., tadpoles
#' never revert to eggs), or that the transition rate is possible but was
#' estimated to be \code{0} in the relevant population and time period. If vital
#' rates are to be averaged across multiple stage classes, or compared across
#' populations, it may be important to distinguish between these two types of
#' zeros.
#'
#' By default, the \code{vr_} functions assume that a transition rate of
#' \code{0} indicates an impossible transition, in which case a value of
#' \code{NA} will be used in relevant calculations. Specifically, the arguments
#' \code{posU} and \code{posR} are specified by the logical expressions
#' \code{(matU > 0)} and \code{(matR > 0)}, respectively. If the matrix
#' population model includes transitions that are estimated to be \code{0} but
#' still in fact possible, one should specify the \code{posU} and/or \code{posR}
#' arguments manually.
#'
#' @section Weighting stages:
#' In averaging vital rates across stages, it may be desirable to weight stage
#' classes differently (e.g., based on reproductive values or stable
#' distributions). Weights are generally applied when averaging across columns,
#' i.e., across transitions \emph{from} a set of stage classes (e.g., averaging
#' stage-specific survival probabilities across multiple stages). All \code{vr_}
#' functions therefore include an optional argument \code{weights_from}.
#'
#' In principle, particularly for vital rates of reproduction, the user can also
#' apply weights when summing across rows within columns, i.e., across
#' reproductive transitions \emph{to} a set of stage classes (e.g., summing the
#' production of different types of offspring, such as seeds vs. seedlings). The
#' function \code{vr_fecundity} therefore also includes an optional
#' argument \code{weights_to}.
#'
#' If supplied, \code{weights_from} will automatically be scaled to sum to 1
#' over the set of possible transitions, whereas \code{weights_to} will not be
#' rescaled because we wish to enable the use of reproductive values here, which
#' do not naturally sum to 1.
#'
#' @section Excluding stages:
#' It may be desirable to exclude one or more stages from the calculation of
#' certain vital rates. For instance, we might not believe that 'growth' to a
#' dormant stage class really reflects biological growth, in which case we could
#' exclude transitions \emph{to} the dormant stage class using the argument
#' \code{exclude_row}. We may or may not want to ignore 'growth' transitions
#' \emph{from} the dormant stage class, which can be done using
#' \code{exclude_col}. To exclude transitions both \emph{to and from} a given
#' set of stages, use argument \code{exclude}.
#'
#' @note Fecundity-related outputs assume a post-breeding census
#'   interpretation. In pre-breeding models, birth rate and newborn survival are
#'   generally confounded within fecundity transitions. See Kendall et al.
#'   (2019).
#'
#' @author Patrick Barks <patrick.barks@@gmail.com>
#'
#' @family vital rates
#'
#' @examples
#' # create example MPM (stage 4 is dormant)
#' matU <- rbind(
#'   c(0.1, 0, 0, 0),
#'   c(0.5, 0.2, 0.1, 0.1),
#'   c(0, 0.3, 0.3, 0.1),
#'   c(0, 0, 0.5, 0.4)
#' )
#'
#' matF <- rbind(
#'   c(0, 0.7, 1.1, 0),
#'   c(0, 0.3, 0.8, 0),
#'   c(0, 0, 0, 0),
#'   c(0, 0, 0, 0)
#' )
#'
#' vr_survival(matU, exclude_col = 4)
#' vr_growth(matU, exclude = 4)
#' vr_shrinkage(matU, exclude = 4)
#' vr_stasis(matU, exclude = 4)
#'
#' # `exclude*` and `*_stages` arguments can accept stage names
#' matU <- name_stages(matU)
#' matF <- name_stages(matF)
#' vr_dorm_enter(matU, dorm_stages = "stage_4")
#' vr_dorm_exit(matU, dorm_stages = 4)
#'
#' vr_fecundity(matU, matF, exclude_col = 4)
#'
#' @name vr
NULL


#' @rdname vr
#' @export vr_survival
vr_survival <- function(matU,
                        posU = matU > 0,
                        exclude_col = NULL,
                        weights_col = NULL) {
  vr_vec <- vr_vec_survival(
    matU = matU,
    posU = posU,
    exclude_col = exclude_col
  )

  return(column_weight(vr_vec, weights_col))
}


#' @rdname vr
#' @export vr_growth
vr_growth <- function(matU,
                      posU = matU > 0,
                      exclude = NULL,
                      exclude_row = NULL,
                      exclude_col = NULL,
                      weights_col = NULL,
                      surv_only_na = TRUE) {
  vr_vec <- vr_vec_growth(
    matU = matU,
    posU = posU,
    exclude = exclude,
    exclude_row = exclude_row,
    exclude_col = exclude_col,
    surv_only_na = surv_only_na
  )

  return(column_weight(vr_vec, weights_col))
}


#' @rdname vr
#' @export vr_shrinkage
vr_shrinkage <- function(matU,
                         posU = matU > 0,
                         exclude = NULL,
                         exclude_row = NULL,
                         exclude_col = NULL,
                         weights_col = NULL,
                         surv_only_na = TRUE) {
  vr_vec <- vr_vec_shrinkage(
    matU = matU,
    posU = posU,
    exclude = exclude,
    exclude_row = exclude_row,
    exclude_col = exclude_col,
    surv_only_na = surv_only_na
  )

  return(column_weight(vr_vec, weights_col))
}


#' @rdname vr
#' @export vr_stasis
vr_stasis <- function(matU,
                      posU = matU > 0,
                      exclude = NULL,
                      weights_col = NULL,
                      surv_only_na = TRUE) {
  vr_vec <- vr_vec_stasis(
    matU = matU,
    posU = posU,
    exclude = exclude,
    surv_only_na = surv_only_na
  )

  return(column_weight(vr_vec, weights_col))
}


#' @rdname vr
#' @export vr_dorm_enter
vr_dorm_enter <- function(matU,
                          posU = matU > 0,
                          dorm_stages,
                          weights_col = NULL) {
  vr_vec <- vr_vec_dorm_enter(
    matU = matU,
    posU = posU,
    dorm_stages = dorm_stages
  )

  return(column_weight(vr_vec, weights_col))
}


#' @rdname vr
#' @export vr_dorm_exit
vr_dorm_exit <- function(matU,
                         posU = matU > 0,
                         dorm_stages,
                         weights_col = NULL) {
  vr_vec <- vr_vec_dorm_exit(
    matU = matU,
    posU = posU,
    dorm_stages = dorm_stages
  )

  return(column_weight(vr_vec, weights_col))
}


#' @rdname vr
#' @export vr_fecundity
vr_fecundity <- function(matU,
                         matR,
                         posR = matR > 0,
                         exclude_col = NULL,
                         weights_row = NULL,
                         weights_col = NULL) {
  vr_vec <- vr_vec_reproduction(
    matU = matU,
    matR = matR,
    posR = posR,
    exclude_col = exclude_col,
    weights_row = weights_row
  )

  return(column_weight(vr_vec, weights_col))
}


#' @noRd
column_weight <- function(x, weights) {
  if (is.null(weights)) {
    out <- mean(x, na.rm = TRUE)
  } else {
    weights[is.na(x)] <- NA_real_
    weights <- weights / sum(weights, na.rm = TRUE)
    out <- sum(weights * x, na.rm = TRUE)
  }
  if (is.nan(out)) out <- NA_real_
  return(out)
}

test_that("vr functions work correctly", {
  ### test vr_U
  U1 <- rbind(
    c(0, 0, 0, 0),
    c(0.5, 0.2, 0.1, 0),
    c(0, 0.3, 0.3, 0.1),
    c(0, 0, 0.5, 0.6)
  )

  surv1 <- vr_survival(U1)
  expect_type(surv1, "double")
  expect_length(surv1, 1)

  surv2 <- vr_survival(U1, exclude_col = 4)
  expect_true(surv2 != surv1)

  grow1 <- vr_growth(U1)
  expect_type(grow1, "double")
  expect_length(grow1, 1)

  grow2 <- vr_growth(U1, posU = matrix(TRUE, nrow = 4, ncol = 4))
  expect_true(grow2 != grow1)

  shri1 <- vr_shrinkage(U1)
  expect_type(shri1, "double")
  expect_length(shri1, 1)

  shri2 <- vr_shrinkage(U1, exclude_row = 2:3)
  expect_true(is.na(shri2))

  stas1 <- vr_stasis(U1)
  expect_type(stas1, "double")
  expect_length(stas1, 1)

  stas2 <- vr_stasis(U1, posU = matrix(TRUE, nrow = 4, ncol = 4))
  expect_true(stas2 != stas1)

  dent1 <- vr_dorm_enter(U1, dorm_stages = 4)
  expect_type(dent1, "double")
  expect_length(dent1, 1)

  dexi1 <- vr_dorm_exit(U1, dorm_stages = 4)
  expect_type(dexi1, "double")
  expect_length(dexi1, 1)

  # reproduction
  R1 <- rbind(
    c(0, 0, 1.1, 1.6),
    c(0, 0, 0.8, 0.4),
    c(0, 0, 0, 0),
    c(0, 0, 0, 0)
  )

  fecu1 <- vr_fecundity(U1, R1)
  expect_type(fecu1, "double")
  expect_length(fecu1, 1)

  # weighting
  v <- popdemo::eigs(U1 + R1, what = "rv") # reproductive value
  fecu2 <- vr_fecundity(U1, R1, weights_row = v)
  expect_true(fecu2 != fecu1)

  # no survival in a stage with reproduction
  U2 <- rbind(
    c(0, 0, 0, 0),
    c(0.5, 0.2, 0.1, 0),
    c(0, 0.3, 0.3, 0),
    c(0, 0, 0.5, 0)
  )

  fecu3 <- vr_fecundity(U2, R1)
  expect_true(fecu3 != fecu1)
})


test_that("vr functions warn and fail gracefully", {
  expect_error(vr_survival(mat_u_na))
  expect_error(vr_growth(mat_u_na))
  expect_error(vr_shrinkage(mat_u_na))
  expect_error(vr_stasis(mat_u_na))
  expect_error(vr_fecundity(mat_u_na))
})

#' Calculate time to reach quasi-stationary stage distribution
#'
#' @description
#' Calculates the time for a cohort projected with a matrix population
#' model to reach a defined quasi-stationary stage distribution.
#'
#' @param mat A matrix population model, or component thereof (i.e., a square
#'   projection matrix). Optionally with named rows and columns indicating the
#'   corresponding life stage names.
#' @param start The index (or stage name) of the first stage at which the author
#'   considers the beginning of life. Defaults to \code{1}. Alternately, a
#'   numeric vector giving the starting population vector (in which case
#'   \code{length(start)} must match \code{ncol(matU))}. See section
#'   \emph{Starting from multiple stages}.
#' @param conv Proportional distance threshold from the stationary stage
#'   distribution indicating convergence. For example, this value should be
#'   \code{0.01} if the user wants to obtain the time step when the stage
#'   distribution is within a distance of 1\% of the stationary stage
#'   distribution.
#' @param N Maximum number of time steps over which the population will be
#'   projected. Time steps are in the same units as the matrix population model
#'   (see \code{AnnualPeriodicity} column in COM(P)ADRE metadata). Defaults to
#'   \code{100,000}.
#'
#' @details
#' Some matrix population models are parameterised with a stasis loop at the
#' largest/most-developed stage class, which can lead to artefactual plateaus
#' in the mortality or fertility trajectories derived from such models. These
#' plateaus occur as a projected cohort approaches its stationary stage
#' distribution (SSD). Though there is generally no single time point at which
#' the SSD is reached, we can define a quasi-stationary stage distribution (QSD)
#' based on a given distance threshold from the SSD, and calculate the number of
#' time steps required for a cohort to reach the QSD. This quantity can then be
#' used to subset age trajectories of mortality or fertility to periods earlier
#' than the QSD, so as to avoid artefactual plateaus in mortality or fertility.
#'
#' \strong{Starting from multiple stages}
#'
#' Rather than specifying argument \code{start} as a single stage class from
#' which all individuals start life, it may sometimes be desirable to allow for
#' multiple starting stage classes. For example, if we want to start our
#' calculation of QSD from reproductive maturity (i.e., first reproduction), we
#' should account for the possibility that there may be multiple stage classes
#' in which an individual could first reproduce.
#'
#' To specify multiple starting stage classes, specify argument \code{start} as
#' the desired starting population vector, giving the proportion
#' of individuals starting in each stage class (the length of \code{start}
#' should match the number of columns in the relevant MPM).
#'
#' @seealso
#' \code{\link{mature_distrib}} for calculating the proportion of
#' individuals achieving reproductive maturity in each stage class.
#'
#'
#' @note The time required for a cohort to reach its QSD depends on the initial
#'   population vector of the cohort (for our purposes, the starting stage
#'   class), and so does not fundamentally require an ergodic matrix (where the
#'   long-term equilibrium traits are independent of the initial population
#'   vector). However, methods for efficiently calculating the stationary stage
#'   distribution (SSD) generally do require ergodicity.
#'
#'   If the supplied matrix (\code{mat}) is non-ergodic, \code{qsd_converge}
#'   first checks for stage classes with no connection (of any degree) from the
#'   starting stage class specified by argument \code{start}, and strips such
#'   stages from the matrix. These unconnected stages have no impact on
#'   age-specific traits that we might derive from the matrix (given the
#'   specified starting stage), but often lead to non-ergodicity and therefore
#'   prevent the reliable calculation of SSD. If the reduced matrix is ergodic,
#'   the function internally updates the starting stage class and continues with
#'   the regular calculation. Otherwise, if the matrix cannot be made ergodic,
#'   the function will return \code{NA} with a warning.
#'
#' @return An integer indicating the first time step at which the
#'   quasi-stationary stage distribution is reached (or an \code{NA} and a
#'   warning if the quasi-stationary distribution is not reached).
#'
#' @author Hal Caswell <h.caswell@@uva.nl>
#' @author Owen Jones <jones@@biology.sdu.dk>
#' @author Roberto Salguero-Gomez <rob.salguero@@zoo.ox.ac.uk>
#' @author Patrick Barks <patrick.barks@@gmail.com>
#'
#' @family life tables
#'
#' @references
#' Caswell, H. 2001. Matrix Population Models: Construction, Analysis, and
#' Interpretation. Sinauer Associates; 2nd edition. ISBN: 978-0878930968
#'
#' Horvitz, C. C., & Tuljapurkar, S. 2008. Stage dynamics, period survival, and
#' mortality plateaus. The American Naturalist, 172(2), 203–215.
#'
#' Jones, O. R., Scheuerlein, A., Salguero-Gomez, R., Camarda, C. G., Schaible,
#' R., Casper, B. B., Dahlgren, J. P., Ehrlén, J., García, M. B., Menges, E.,
#' Quintana-Ascencio, P. F., Caswell, H., Baudisch, A. & Vaupel, J. 2014.
#' Diversity of ageing across the tree of life. Nature 505, 169-173.
#' <doi:10.1038/nature12789>
#'
#' Salguero-Gomez R. 2018. Implications of clonality for ageing research.
#' Evolutionary Ecology, 32, 9-28. <doi:10.1007/s10682-017-9923-2>
#'
#' @examples
#' data(mpm1)
#'
#' # starting stage = 2 (i.e., "small")
#' qsd_converge(mpm1$matU, start = 2)
#' qsd_converge(mpm1$matU, start = "small") # equivalent using named life stages
#'
#' # convergence threshold = 0.001
#' qsd_converge(mpm1$matU, start = 2, conv = 0.001)
#'
#' # starting from first reproduction
#' repstages <- repro_stages(mpm1$matF)
#' n1 <- mature_distrib(mpm1$matU, start = 2, repro_stages = repstages)
#' qsd_converge(mpm1$matU, start = n1)
#'
#' @importFrom popdemo isErgodic project
#' @export qsd_converge
qsd_converge <- function(mat, start = 1L, conv = 0.01, N = 1e5L) {
  # validate arguments
  checkValidMat(mat)
  checkValidStartLife(start, mat, start_vec = TRUE)

  if (length(start) == 1) {
    start_vec <- rep(0.0, nrow(mat))
    if (!is.null(dimnames(mat))) {
      checkMatchingStageNames(mat)
      names(start_vec) <- colnames(mat)
    }
    start_vec[start] <- 1.0
  } else {
    start_vec <- start
  }

  # if not ergodic, remove stages not connected from start
  if (!popdemo::isErgodic(mat)) {
    nonzero <- rep(FALSE, nrow(mat))
    nonzero[start_vec > 0] <- TRUE

    n <- start_vec
    t <- 1L

    while (!all(nonzero) && t < (nrow(mat) * 2)) {
      n <- mat %*% n
      nonzero[n > 0] <- TRUE
      t <- t + 1L
    }

    mat <- as.matrix(mat[nonzero, nonzero])
    start_vec <- start_vec[nonzero]
  }

  # if still not ergodic, check whether observed dist at t = N matches stable
  #  dist
  if (!popdemo::isErgodic(mat)) {
    # check whether stable dist is __0__
    check_stable_zero <- stable_zero(mat, n1 = start_vec)
    if (check_stable_zero) {
      w <- rep(0, nrow(mat))
    } else {
      w <- popdemo::eigs(mat, what = "ss")

      n <- start_vec
      dist <- 1
      t <- 0L

      while (dist > 0.001 && t < N) {
        n <- mat %*% n
        n <- n / sum(n)
        dist <- 0.5 * (sum(abs(n - w)))
        t <- t + 1L
      }

      if (dist > 0.001) {
        warning(strwrap(prefix = " ", initial = "", "Matrix is still non-ergodic
        after removing stages not connected from stage 'start', and stable
        distribution doesn't match observed distribution after N iterations."),
          call. = FALSE
        )
        cat("\n")
        return(NA_integer_)
      }
    }
  } else {
    w <- stable.stage(mat)
  }

  # set up a cohort with 1 individ in first stage class, and 0 in all others
  n <- start_vec

  # iterate cohort (n = cohort population vector, p = proportional structure)
  dist <- conv + 1
  t <- 0L

  while (!is.na(dist) && dist > conv && t < N) {
    dist <- 0.5 * (sum(abs(n - w)))
    n <- mat %*% n
    if (sum(n) > 0) n <- n / sum(n)
    t <- t + 1L
  }

  return(ifelse(is.na(dist) | dist > conv, NA_integer_, t))
}


#' @noRd
stable_zero <- function(mat, n1) {
  n <- n1
  for (i in seq_len(nrow(mat))) {
    n <- mat %*% n
  }
  return(sum(n) == 0)
}

test_that("qsd_converge works correctly", {
  x1 <- qsd_converge(mat_u)
  expect_length(x1, 1L)
  expect_gt(x1, 0)

  # matU all zero
  x2 <- suppressWarnings(qsd_converge(mat_u_zero))
  expect_identical(x2, 1L)

  # multiple starting stages
  x3 <- qsd_converge(mat_u, start = c(0.8, 0.2, 0, 0))
  expect_length(x3, 1L)
  expect_gt(x3, 0)

  # named life stages
  x4 <- qsd_converge(mat_u_named, start = "sm")
  expect_identical(x1, x4)
})

test_that("qsd_converge warns and fails gracefully", {
  expect_error(qsd_converge(mat_u_na))
  expect_error(qsd_converge(mat_u, start = 10))
  expect_error(qsd_converge(mat_u, start = "stage name"))
  expect_error(qsd_converge(mat_u_named, start = "invalid name"))
})

test_that("qsd_converge works w/ non-ergodic matrix", {
  mat_no_ergo <- matrix(
    c(
      0, 2.2, 5.8, 0, 0,
      0.8, 0, 0, 0, 0,
      0, 0.89, 0, 0, 0,
      0, 0, 0.93, 0, 0,
      0, 0, 0, 0.75, 0.5
    ),
    nrow = 5, ncol = 5, byrow = TRUE
  )

  # fast convergence
  t_qsd <- qsd_converge(mat_no_ergo)

  expect_length(t_qsd, 1L)

  # multi-state w/ non-ergodic also works.
  f_qsd <- qsd_converge(mat_no_ergo,
    start = c(1, 2, 3, 2, 0)
  )

  expect_length(f_qsd, 1L)
  # This depends on default value of conv in qsd_converge
})

#' Calculate Keyfitz entropy for an age-based matrix population model
#'
#' Computes Keyfitz entropy from the U submatrix of an age-based matrix
#' population model.
#'
#' @param Umat A square numeric matrix representing the U submatrix of a matrix
#'   population model. The dimension corresponds to the number of age classes
#'
#' @return Returns a single numeric value representing the Keyfitz entropy
#' for the given matrix. This value quantifies the dispersion of age at death.
#'
#' @author Lotte de Vries <c.devries@@uva.nl>
#' @author Owen Jones <jones@@biology.sdu.dk>
#'
#' @references Keyfitz, N. 1977. Applied Mathematical Demography. New York:
#'   Wiley.
#'
#'   de Vries, C., Bernard, C., & Salguero-Gómez, R. 2023. Discretising
#'   Keyfitz' entropy for studies of actuarial senescence and comparative
#'   demography. Methods in Ecology and Evolution, 14, 1312–1319.
#'   <doi:10.1111/2041-210X.14083>
#'
#' @family life history traits
#' @examples
#' data(leslie_mpm1)
#'
#' entropy_k_age(leslie_mpm1$matU)
#' @export

entropy_k_age <- function(Umat) {
  if (!inherits(Umat, "matrix")) {
    stop("m must be a matrix")
  }

  if (!is.numeric(Umat)) {
    stop("m must be a numeric matrix")
  }
  if (nrow(Umat) != ncol(Umat)) {
    stop("m must be a square matrix")
  }

  # Function from Lotte de Vries
  maxage <- dim(Umat)[1]
  e0 <- rep(0, maxage)
  e0[1] <- 1
  l <- rep(0, 2 * maxage)
  temp <- rep(1, maxage) - colSums(Umat)
  M <- diag(temp)
  N <- solve(diag(maxage) - Umat)
  eta1 <- rep(1, maxage) %*% N
  B <- M %*% N
  etadagger <- eta1 %*% B
  eta0 <- eta1 %*% e0
  H_new <- etadagger[1] / eta0

  return(as.vector(H_new))
}

test_that("entropy_k_age works correctly with Leslie matrices", {
  data(leslie_mpm1)
  testMatrix <- leslie_mpm1$matU

  x <- entropy_k_age(testMatrix)
  expect_true(inherits(x, "numeric"))
  expect_length(x, 1L)
  expect_gte(x, 0)

  # Does function return NA when the matrix contains an NA?
  testMatrix_NA <- testMatrix
  testMatrix_NA[1, 1] <- NA
  x <- entropy_k_age(testMatrix_NA)

  expect_true(is.na(x))
})

test_that("entropy_k_age warns and fails gracefully with Leslie matrices", {
  data(leslie_mpm1)
  testMatrix <- leslie_mpm1$matU


  # Matrix not square
  testMatrix_ns <- testMatrix[-1]
  expect_error(entropy_k_age(testMatrix_ns))

  # Matrix not numeric
  testMatrix_char <- apply(testMatrix, c(1, 2), as.character)
  expect_error(entropy_k_age(testMatrix_char))
})

#' Calculate mean and variance of life expectancy from a matrix population model
#'
#' Applies Markov chain approaches to obtain mean and variance of life
#' expectancy from a matrix population model (MPM).
#'
#' @param matU The survival component of a MPM (i.e., a square projection matrix
#'   reflecting survival-related transitions; e.g., progression, stasis, and
#'   retrogression). Optionally with named rows and columns indicating the
#'   corresponding life stage names.
#' @param start The index (or stage name) of the first stage of the life cycle
#'   which the user considers to be the beginning of life. Defaults to \code{1}.
#'   If set to `NULL` the function returns mean life expectancy from each of the
#'   stages of the MPM.
#' @param mixdist A vector with a length equal to the dimension of the MPM
#'   defining how the function should average the output over the. possible
#'   starting states. See section \emph{Starting from multiple stages}. If this
#'   argument is used, `start` must be set to `NULL`.
#'
#' @section Starting from multiple stages:
#' Sometimes, it is necessary to calculate life expectancy considering multiple
#' starting stage classes instead of just a single stage from which all
#' individuals begin their lives. This scenario arises when there are several
#' possible stages at which an individual can start a particular life event,
#' such as reproductive maturity.
#' To handle such cases, the function provides support for multiple starting
#' stage classes. When calculating life expectancy in this context, the outputs
#' should be averaged using weights determined by the distribution of
#' individuals across these stages. To achieve this, the `start` argument should
#' be set to `NULL`, indicating that the starting stage is not specified, and
#' the `mixdist` argument should be utilized.
#' In the context described, The `mixdist` argument expects a vector that
#' represents the proportion of individuals with their first reproduction in
#' each stage of the MPM. By providing this distribution, the function
#' calculates the mean lifespan by appropriately weighting the life expectancies
#' corresponding to each starting stage.
#' For a practical example that demonstrates this usage, please refer to the
#' code example below.
#'
#' @return Returns life expectancy in the units of the projection interval
#'   (`ProjectionInterval`) of the MPM. If \code{matU} is singular (often
#'   indicating infinite life expectancy), returns \code{NA}.
#'
#' @author Christine M. Hernández <cmh352@@cornell.edu>
#' @author Owen R. Jones <jones@@biology.sdu.dk>
#'
#' @family life history traits
#'
#' @references Caswell, H. 2001. Matrix Population Models: Construction,
#'   Analysis, and Interpretation. Sinauer Associates; 2nd edition. ISBN:
#'   978-0878930968
#'
#' @examples
#' data(mpm1)
#'
#' # mean life expectancy starting from stage class 2
#' life_expect_mean(mpm1$matU, start = 2)
#'
#' # equivalent using named life stages
#' life_expect_mean(mpm1$matU, start = "small")
#'
#' # mean life expectancies starting from each of the stages
#' life_expect_mean(mpm1$matU, start = NULL)
#'
#' # mean life expectancy starting from first reproduction, where this varies
#' # across individuals
#' rep_stages <- repro_stages(mpm1$matF)
#' (n1 <- mature_distrib(mpm1$matU, start = 2, repro_stages = rep_stages))
#' life_expect_mean(mpm1$matU, mixdist = n1, start = NULL)
#'
#' # variance of life expectancy from stage class 1
#' life_expect_var(mpm1$matU, start = 1)
#'
#' # variance of life expectancy from stage class 1
#' life_expect_var(mpm1$matU, start = "seed")
#'
#' # variance of life expectancy from each stage class
#' life_expect_var(mpm1$matU, start = NULL)
#'
#' # variance of life expectancies with a set mixing distribution
#' life_expect_var(mpm1$matU, mixdist = c(0.0, 0.1, 0.3, 0.1, 0.5), start = NULL)
#'
#' # setting mixdist to ignore all but one stage should produce the same result
#' # as setting the start argument to that stage
#' life_expect_mean(mpm1$matU, start = 3)
#' life_expect_mean(mpm1$matU, mixdist = c(0, 0, 1, 0, 0), start = NULL)
#'
#' @rdname life_expect
#' @export life_expect_mean
life_expect_mean <- function(matU, mixdist = NULL, start = 1L) {
  # validate arguments
  # You cannot use both mixdist and start.
  if (!is.null(mixdist) && !is.null(start)) {
    stop("You cannot apply the mixing distribution and also specify a starting
         state. The mixing distribution defines how you want the function to
         average over all possible starting states.")
  }

  # check that the MPM is valid
  checkValidMat(matU, warn_surv_issue = TRUE)

  # check that, if it is not NULL, start is valid (i.e. that is it an integer or
  # stage name that matches the MPM)
  if (!is.null(start)) {
    checkValidStartLife(start, matU, start_vec = TRUE)
  }

  # if start is a character string (e.g. a stage name) turn it into a numeric.
  if (inherits(start, "character")) {
    startNumeric <- match(start, colnames(matU))
  } else {
    startNumeric <- start
  }

  # if start is not feasible, throw an error
  if (!is.null(start)) {
    if (!startNumeric %in% 1:nrow(matU)) {
      stop("The start value must match a stage name of matU, or be an integer
         between 1 and the number of stages.")
    }
  }

  matDim <- dim(matU)[1]

  ## Calculate Ex(R | current state)
  # This is an alternative to exactLTRE::fundamental_matrix, to avoid dependency
  # try calculating fundamental matrix (will fail if matrix singular)
  N <- try(solve(diag(matDim) - matU), silent = TRUE)

  # If the calculation of fundamental matrix produces an error, then make the
  # output NA. Otherwise, calculate the life expectancy.
  if (inherits(N, "try-error")) {
    expLCond_z <- NA_real_
  } else {
    expLCond_z <- rep(1, matDim) %*% N
  }


  # If mixdist is NOT null
  if (!is.null(mixdist)) {
    expL <- expLCond_z %*% mixdist
    expL_out <- as.vector(expL)
    names(expL_out) <- colnames(expL)
    return(expL_out)
  }

  # If start is NOT null
  if (!is.null(start)) {
    return(expLCond_z[startNumeric])
  }

  if (is.null(start)) {
    expLCond_z_out <- as.vector(expLCond_z)
    names(expLCond_z_out) <- colnames(expLCond_z)
    return(expLCond_z_out)
  }
}

# Calculate the variance in lifespan:
# note: this calculates the variance in the number of time steps!
#' @rdname life_expect
#' @export life_expect_var
life_expect_var <- function(matU, mixdist = NULL, start = 1L) {
  # validate arguments
  # You cannot use both mixdist and start.
  if (!is.null(mixdist) && !is.null(start)) {
    stop("You cannot apply the mixing distribution and also specify a starting
         state. The mixing distribution defines how you want the function to
         average over all possible starting states.")
  }

  # check that the MPM is valid
  checkValidMat(matU, warn_surv_issue = TRUE)

  # check that, if it is not NULL, start is valid (i.e. that is it an integer or
  # stage name that matches the MPM)
  if (!is.null(start)) {
    checkValidStartLife(start, matU, start_vec = TRUE)
  }

  # if start is a character string (e.g. a stage name) turn it into a numeric.
  if (inherits(start, "character")) {
    startNumeric <- match(start, colnames(matU))
  } else {
    startNumeric <- start
  }

  matDim <- dim(matU)[1]

  # calculate the fundamental matrix
  # This is an alternative to exactLTRE::fundamental_matrix, to avoid dependency
  # try calculating fundamental matrix (will fail if matrix singular)
  N <- try(solve(diag(matDim) - matU), silent = TRUE)

  if (inherits(N, "try-error")) {
    return(NA_real_)
  } else {
    expLCond_z_prelim <- suppressWarnings(life_expect_mean(matU, mixdist = NULL, start = NULL))

    # Convert to a matrix, required for the calculations below
    expLCond_z <- matrix(expLCond_z_prelim, nrow = 1, ncol = nrow(matU))
    colnames(expLCond_z) <- names(expLCond_z_prelim)

    ## Calculate Ex(R | current state)

    ## Var(L | current state) using eqn. 5.12 from Hal Caswell (2001)
    eT <- matrix(data = 1, ncol = matDim, nrow = 1) # column vector of 1's
    varLCond_z <- eT %*% (2 * N %*% N - N) - (expLCond_z)^2

    if (is.null(mixdist)) {
      outputVar <- varLCond_z
    } else {
      # variance in LRO due to differences along trajectories:
      varL_within <- varLCond_z %*% mixdist
      # variance in LRO due to differences among starting states:
      varL_between <- t(mixdist) %*% t(expLCond_z^2) -
        (t(mixdist) %*% t(expLCond_z))^2
      # total variance in lifespan, given the mixing distribution:
      outputVar <- varL_within + varL_between
    }
    if (!is.null(start)) {
      return(as.vector(outputVar[startNumeric]))
    }
    if (is.null(start)) {
      outputVar_prelim <- outputVar
      outputVar <- as.vector(outputVar_prelim)
      names(outputVar) <- colnames(outputVar_prelim)
      return(outputVar)
    }
  }
}

test_that("life_expect_mean works correctly", {
  l0 <- life_expect_mean(mat_u)
  l0_singular <- life_expect_mean(mat_u_singular)
  l1 <- life_expect_mean(mat_u_named, start = "sm")

  l0_1 <- life_expect_mean(mat_u, start = NULL)
  l0_2 <- life_expect_mean(mat_u, start = NULL, mixdist = c(1, 2, 3, 4))
  l0_3 <- life_expect_mean(mat_u, start = 3)

  expect_gt(l0, 0)
  expect_true(is.na(l0_singular))
  expect_identical(l0, l1)
  expect_vector(l0_1)
  expect_vector(l0_2)
  expect_vector(l0_3)

  l0_s2 <- life_expect_mean(mat_u, start = 2)
  l0_s2_mix <- life_expect_mean(mat_u, start = NULL, mixdist = c(0, 1, 0, 0))
  expect_identical(l0_s2, l0_s2_mix)
})

test_that("life_expect_mean warns and fails gracefully", {
  expect_warning(life_expect_mean(mat_u_zero))
  expect_warning(life_expect_mean(mat_u_survissue))
  expect_error(life_expect_mean(mat_u, start = 10))
  expect_error(life_expect_mean(mat_u, start = "stage name"))
  expect_error(life_expect_mean(mat_u_na))
  expect_error(life_expect_mean(mat_u_named, start = "invalid stage"))
  expect_error(life_expect_mean(mat_u, start = 2, mixdist = c(1, 2, 3, 4)))
  expect_error(life_expect_mean(mat_u, start = 10))
  expect_error(life_expect_mean(mat_u, start = 0))
})


test_that("life_expect_var works correctly", {
  l0 <- life_expect_var(mat_u)
  l0_singular <- life_expect_var(mat_u_singular)
  l1 <- life_expect_var(mat_u_named, start = "sm")

  expect_gt(l0, 0)
  expect_true(is.na(l0_singular))
  expect_identical(l0, l1)

  l0_1 <- life_expect_var(mat_u, start = NULL)
  l0_2 <- life_expect_mean(mat_u, start = NULL, mixdist = c(1, 2, 3, 4))
  l0_3 <- life_expect_mean(mat_u, start = 3)
  expect_vector(l0_1)
  expect_vector(l0_2)
  expect_vector(l0_3)
})

test_that("life_expect_var warns and fails gracefully", {
  expect_warning(life_expect_var(mat_u_zero))
  expect_warning(life_expect_var(mat_u_survissue))
  expect_error(life_expect_var(mat_u, start = 10))
  expect_error(life_expect_var(mat_u, start = "stage name"))
  expect_error(life_expect_var(mat_u_na))
  expect_error(life_expect_var(mat_u_named, start = "invalid stage"))
})

#' Rearrange stages of a matrix population model to segregate reproductive and
#' non-reproductive stages
#'
#' Rearrange stages of a matrix population model so that all inter-reproductive
#' stages fall in the final rows/columns of the matrix. This is a preparatory
#' step to collapsing the matrix model into a standardized set of stages (e.g.,
#' propagule, pre-reproductive, reproductive, and post-reproductive).
#'
#' @param matU The survival component of a matrix population model (i.e., a
#'   square projection matrix reflecting survival-related transitions; e.g.,
#'   progression, stasis, and retrogression)
#' @param matF The sexual component of a matrix population model (i.e., a square
#'   projection matrix reflecting transitions due to sexual reproduction)
#' @param matC The clonal component of a matrix population model (i.e., a square
#'   projection matrix reflecting transitions due to clonal reproduction).
#'   Defaults to \code{NULL}, indicating no clonal reproduction (i.e.,
#'   \code{matC} is a matrix of zeros).
#' @param repro_stages Logical vector of length \code{ncol(matU)} indicating
#'   which stages are reproductive. Alternatively, a vector of stage indices or
#'   stage names of the reproductive classes.
#' @param matrix_stages A character vector identifying organized matrix stages.
#' @return Returns a list with 6 elements:
#' \item{matU}{Rearranged survival matrix}
#' \item{matF}{Rearranged sexual reproduction matrix}
#' \item{matC}{Rearranged clonal reproduction matrix}
#' \item{matrix_stages}{Rearranged vector of organized matrix stages}
#' \item{repro_stages}{Rearranged logical vector of reproductive stages}
#' \item{nonRepInterRep}{Numeric index for any rearranged inter-reproductive
#'  stages}
#'
#' @author Rob Salguero-Gómez <rob.salguero@@zoo.ox.ac.uk>
#'
#' @family transformation
#'
#' @seealso \code{\link{mpm_standardize}}
#' @examples
#' matU <- rbind(
#'   c(0.1, 0, 0, 0, 0),
#'   c(0.5, 0.2, 0.1, 0, 0),
#'   c(0, 0.3, 0.3, 0.1, 0),
#'   c(0, 0, 0.4, 0.4, 0.1),
#'   c(0, 0, 0, 0.1, 0.4)
#' )
#'
#' matF <- rbind(
#'   c(0, 1.1, 0, 1.6, 0),
#'   c(0, 0.8, 0, 0.4, 0),
#'   c(0, 0, 0, 0, 0),
#'   c(0, 0, 0, 0, 0),
#'   c(0, 0, 0, 0, 0)
#' )
#'
#' repro_stages <- c(2, 4)
#' matrix_stages <- c("prop", "active", "active", "active", "active")
#'
#' mpm_rearrange(matU, matF,
#'   repro_stages = repro_stages,
#'   matrix_stages = matrix_stages
#' )
#'
#' @export mpm_rearrange
#'
mpm_rearrange <- function(matU, matF, matC = NULL, repro_stages,
                          matrix_stages) {
  # validate arguments
  checkValidMat(matU)
  checkValidMat(matF)
  if (!is.null(matC)) checkValidMat(matC, warn_all_zero = FALSE)
  # convert repro_stages indices and names to logical vector, if needed
  if (any(is.numeric(repro_stages[!is.na(repro_stages)]))) {
    temprs <- vector(mode = "logical", length = ncol(matU))
    temprs[repro_stages] <- TRUE
    repro_stages <- temprs
  } else if (any(is.character(repro_stages[!is.na(repro_stages)]))) {
    repro_stages <- colnames(matU) %in% repro_stages
  }
  if (ncol(matU) != ncol(matF) ||
    ncol(matU) != length(repro_stages) ||
    length(repro_stages) != length(matrix_stages)) {
    stop("Arguments do not correspond to MPM of single dimension.\n",
      call. = FALSE
    )
  }
  checkMatchingStageNames(matU, matF)
  if (!is.null(matC)) {
    checkValidMat(matC, warn_all_zero = FALSE)
    checkMatchingStageNames(matU, matC)
  }
  checkValidStages(matU, repro_stages)

  # populate matC with zeros, if NULL
  if (is.null(matC)) {
    matC <- matrix(0, nrow = ncol(matF), ncol = ncol(matF))
  }

  # preliminaries
  matDim <- ncol(matF) # matrix dim
  Rep <- which(repro_stages == TRUE) # reproductive stage indices
  out <- NULL # initalize output list

  # which stages reproductive, incl. inter-reproductive
  if (length(Rep) > 0) {
    allRep <- Rep[1]:Rep[length(Rep)]
  } else {
    allRep <- integer(0)
  }

  # which stages non-reproductive inter-reproductive
  nonRepInterRep <- allRep[which(!allRep %in% Rep)]

  if (length(nonRepInterRep) > 0) {
    # if there are non-repro inter-repro stages, rearrange
    allElseStages <- which(!(1:matDim %in% nonRepInterRep))
    rearrange <- c(allElseStages, nonRepInterRep)
    out$matU <- matU[rearrange, rearrange]
    out$matF <- matF[rearrange, rearrange]
    out$matC <- matC[rearrange, rearrange]
    out$matrix_stages <- matrix_stages[rearrange]
    out$repro_stages <- repro_stages[rearrange]
  } else {
    # else, no need to rearrange
    out$matU <- matU
    out$matF <- matF
    out$matC <- matC
    out$matrix_stages <- matrix_stages
    out$repro_stages <- repro_stages
  }

  # inter-repro stages that were moved to the last column(s)
  out$nonRepInterRep <- ifelse(length(nonRepInterRep) > 0, nonRepInterRep, NA)

  return(out)
}

test_that("mpm_rearrange works correctly", {
  # mpm with inter-reproductive stage
  repro_stages <- apply(mat_f_inter, 2, function(x) any(x > 0))
  matrix_stages <- c("prop", "active", "active", "active", "active")

  x <- mpm_rearrange(mat_u_inter, mat_f_inter,
    repro_stages = repro_stages,
    matrix_stages = matrix_stages
  )

  expect_type(x, "list")
  expect_length(x, 6)

  expect_equal(sum(mat_u_inter), sum(x$matU), tolerance = 1e-09)
  expect_equal(sum(mat_f_inter), sum(x$matF), tolerance = 1e-09)
  expect_identical(sort(matrix_stages), sort(x$matrix_stages))
  expect_true(x$nonRepInterRep %in% seq_along(repro_stages))


  # mpm without inter-reproductive stage
  repro_stages <- apply(mat_f, 2, function(x) any(x > 0))
  matrix_stages <- c("prop", "active", "active", "active")

  x <- mpm_rearrange(mat_u, mat_f, matC = NULL, repro_stages, matrix_stages)

  expect_identical(mat_u, x$matU)
  expect_identical(mat_f, x$matF)
  expect_identical(matrix_stages, x$matrix_stages)
  expect_true(is.na(x$nonRepInterRep))

  # mpm with no reproduction
  repro_stages <- apply(mat_f_zero, 2, function(x) any(x > 0))
  matrix_stages <- c("prop", "active", "active", "active")

  x <- suppressWarnings(
    mpm_rearrange(mat_u, mat_f_zero, matC = NULL, repro_stages, matrix_stages)
  )

  expect_identical(mat_f_zero, x$matF)
  expect_identical(mat_u, x$matU)
})


test_that("mpm_rearrange warns and fails gracefully", {
  repro_stages <- apply(mat_f, 2, function(x) any(x > 0))
  matrix_stages <- c("prop", "active", "active")

  expect_error(mpm_rearrange(mat_u, mat_f,
    repro_stages = repro_stages,
    matrix_stages = matrix_stages
  ))
})

#' Example matrix population model (MPM)
#'
#' @description
#' An example matrix population model (MPM) used for demonstration and testing
#' purposes. The MPM consists of five stage classes: 'seed', 'small', 'medium',
#' 'large', and 'dormant'.
#'
#' @format
#' A list with two elements:
#' \describe{
#'   \item{matU}{The survival-related component of the MPM.}
#'   \item{matF}{The sexual reproduction component of the MPM.}
#' }
"mpm1"

#' Example Leslie matrix population model (MPM)
#'
#' @description
#' An example Leslie matrix population model (MPM) used for demonstration and
#' testing purposes.
#'
#' @format
#' A list with two elements:
#' \describe{
#'   \item{matU}{The survival-related component of the MPM.}
#'   \item{matF}{The sexual reproduction component of the MPM.}
#' }
"leslie_mpm1"

#' Derive survival-independent vital rates for growth, stasis, shrinkage, and
#' reproduction
#'
#' @description
#' Divides columns of a matrix population model by the corresponding
#' stage-specific survival probability, to obtain lower-level vital rates for
#' growth, stasis, shrinkage, and reproduction. Vital rates corresponding to
#' biologically impossible transitions are coerced to \code{NA}.
#'
#' These decompositions assume that all transition rates are products of a
#' stage-specific survival term (column sums of \code{matU}) and a lower level
#' vital rate that is conditional on survival (growth, shrinkage, stasis, or
#' reproduction). Reproductive vital rates that are not conditional on survival
#' (i.e., within a stage class from which there is no survival) are also
#' allowed.
#'
#' @param matU The survival component of a matrix population model (i.e., a
#'   square projection matrix reflecting survival-related transitions; e.g.
#'   progression, stasis, and retrogression)
#' @param matR The reproductive component of a matrix population model (i.e., a
#'   square projection matrix reflecting transitions due to reproduction; either
#'   sexual, clonal, or both)
#' @param posU A logical matrix of the same dimension as \code{matU}, with
#'   elements indicating whether a given \code{matU} transition is possible
#'   (\code{TRUE}) or not (\code{FALSE}). Defaults to \code{matU > 0} (see
#'   Details).
#' @param posR A logical matrix of the same dimension as \code{matR}, with
#'   elements indicating whether a given \code{matR} transition is possible
#'   (\code{TRUE}) or not (\code{FALSE}). Defaults to \code{matR > 0} (see
#'   Details).
#' @param surv_only_na If there is only one possible \code{matU} transition in a
#'   given column, should that transition be attributed exclusively to survival?
#'   If \code{TRUE}, the vital rate of growth/stasis/shrinkage in that column
#'   will be coerced to \code{NA}. If \code{FALSE}, dividing the single
#'   transition by the stage-specific survival probability will always yield a
#'   value of \code{1}. Defaults to \code{TRUE}.
#'
#' @details
#' A transition rate of \code{0} within a matrix population model may indicate
#' that the transition is not possible in the given life cycle (e.g., tadpoles
#' never revert to eggs), or that the transition is possible but was estimated
#' to be \code{0} in the relevant population and time period. If vital rates are
#' to be averaged across multiple stage classes, or compared across populations,
#' it may be important to distinguish between these two types of zeros.
#'
#' By default, \code{vr_mat} assumes that a transition rate of
#' \code{0} indicates an impossible transition, in which case a value of
#' \code{NA} will be returned in the relevant matrix cell. Specifically, the
#' arguments \code{posU} and \code{posR} are specified by the logical
#' expressions \code{(matU > 0)} and \code{(matR > 0)}, respectively. If the
#' matrix population model includes transitions that are possible but estimated
#' to be \code{0}, one should specify the \code{posU} and/or \code{posR}
#' arguments manually.
#'
#' @return A matrix of vital rates. Vital rates corresponding to impossible
#'   transitions will be coerced to \code{NA} (see Details).
#'
#' @author Patrick Barks <patrick.barks@@gmail.com>
#'
#' @family vital rates
#'
#' @references Caswell, H. 2001. Matrix Population Models: Construction,
#'   Analysis, and Interpretation. Sinauer Associates; 2nd edition. ISBN:
#'   978-0878930968
#'
#' @examples
#' matU <- rbind(
#'   c(0.1, 0, 0, 0),
#'   c(0.5, 0.2, 0.1, 0),
#'   c(0, 0.3, 0.3, 0.1),
#'   c(0, 0, 0.5, 0.6)
#' )
#'
#' matR <- rbind(
#'   c(0, 0, 1.1, 1.6),
#'   c(0, 0, 0.8, 0.4),
#'   c(0, 0, 0, 0),
#'   c(0, 0, 0, 0)
#' )
#'
#' # extract vital rates of survival from matU
#' vr_mat_U(matU)
#'
#' # extract vital rates of reproduction from matR
#' vr_mat_R(matU, matR)
#'
#' @name vr_mat

#' @rdname vr_mat
#' @export vr_mat_U
vr_mat_U <- function(matU, posU = matU > 0, surv_only_na = TRUE) {
  checkValidMat(matU)
  sigma <- colSums(matU, na.rm = TRUE)
  sigma[sigma == 0] <- NA_real_ # can't calculate lower-vr if no survival

  vmat <- t(t(matU) / sigma)
  vmat[!posU] <- NA_real_

  if (surv_only_na) {
    surv_only <- colSums(posU) == 1
    vmat[, surv_only] <- NA_real_
  }

  return(vmat)
}

#' @rdname vr_mat
#' @export vr_mat_R
vr_mat_R <- function(matU, matR, posR = matR > 0) {
  checkValidMat(matU)
  checkValidMat(matR)
  sigma <- colSums(matU)

  sigma[sigma == 0] <- 1 # avoid NaN if no survival

  vmat <- t(t(matR) / sigma)
  vmat[!posR] <- NA_real_
  return(vmat)
}

test_that("vr_mat functions work correctly", {
  ### test vr_mat_U
  U1 <- rbind(
    c(0, 0, 0, 0),
    c(0.5, 0.2, 0.1, 0),
    c(0, 0.3, 0.3, 0.1),
    c(0, 0, 0.5, 0.6)
  )

  x1 <- vr_mat_U(U1)
  expect_true(inherits(U1, "matrix"))
  expect_true(all(is.na(x1[, 1])))

  x2 <- vr_mat_U(U1, surv_only_na = FALSE)
  expect_false(all(is.na(x2[, 1])))

  # all transitions possible
  x3 <- vr_mat_U(U1, posU = matrix(TRUE, nrow = 4, ncol = 4))
  expect_false(anyNA(x3))


  ### test vr_mat_R
  R1 <- rbind(
    c(0, 0, 1.1, 1.6),
    c(0, 0, 0.8, 0.4),
    c(0, 0, 0, 0),
    c(0, 0, 0, 0)
  )

  x4 <- vr_mat_R(U1, R1)
  expect_true(inherits(x4, "matrix"))
  expect_true(all(is.na(x4[, 1:2])))

  # all transitions possible
  x5 <- vr_mat_R(U1, R1, posR = matrix(TRUE, nrow = 4, ncol = 4))
  expect_false(anyNA(x5))
})


test_that("vr_mat functions warn and fail gracefully", {
  expect_error(vr_mat_U(mat_u_na))
  expect_error(vr_mat_R(mat_u, mat_f_na))
})

#' Calculate longevity from a matrix population model
#'
#' Calculate longevity (the age \emph{x} at which survivorship for a synthetic
#' cohort falls below some critical proportion) from a matrix population model
#'
#' @param matU The survival component of a matrix population model (i.e., a
#'   square projection matrix reflecting survival-related transitions; e.g.,
#'   progression, stasis, and retrogression). Optionally with named rows and
#'   columns indicating the corresponding life stage names.
#' @param start The index (or stage name) of the first stage at which the author
#'   considers the beginning of life. Defaults to \code{1}. Alternately, a
#'   numeric vector giving the starting population vector (in which case
#'   \code{length(start)} must match \code{ncol(matU))}. See section
#'   \emph{Starting from multiple stages}.
#' @param x_max The maximum age, in units of the MPM projection interval, to
#'   which survivorship will be calculated. Defaults to \code{1000}.
#' @param lx_crit Proportion of initial cohort remaining before all are
#'   considered dead (a value between 0 and 1). Defaults to \code{0.01}.
#'
#' @return Returns longevity, the integer age at which expected survivorship
#'   falls below \code{lx_crit}. If survivorship doesn't reach \code{lx_crit} by
#'   \code{x_max}, returns \code{NA} and prints a warning message.
#'
#' @section Starting from multiple stages:
#' Rather than specifying argument \code{start} as a single stage class from
#' which all individuals start life, it may sometimes be desirable to allow for
#' multiple starting stage classes. For example, if we want to start our
#' calculation of longevity from reproductive maturity (i.e., first
#' reproduction), we should account for the possibility that there may be
#' multiple stage classes in which an individual could first reproduce.
#'
#' To specify multiple starting stage classes, specify argument \code{start} as
#' the desired starting population vector, giving the proportion
#' of individuals starting in each stage class (the length of \code{start}
#' should match the number of columns in the relevant MPM).
#'
#' @note Note that the units of time in returned values are the same as the
#'   (\code{ProjectionInterval}) of the MPM.
#'
#' @seealso
#' \code{\link{mature_distrib}} for calculating the proportion of
#' individuals achieving reproductive maturity in each stage class.
#'
#' @author Roberto Salguero-Gomez <rob.salguero@@zoo.ox.ac.uk>
#' @author Hal Caswell <hcaswell@@whoi.edu>
#'
#' @family life history traits
#'
#' @references Caswell, H. 2001. Matrix Population Models: Construction,
#'   Analysis, and Interpretation. Sinauer Associates; 2nd edition. ISBN:
#'   978-0878930968
#'
#'   Morris, W. F. & Doak, D. F. 2003. Quantitative Conservation Biology:
#'   Theory and Practice of Population Viability Analysis. Sinauer Associates,
#'   Sunderland, Massachusetts, USA
#'
#' @examples
#' data(mpm1)
#'
#' longevity(mpm1$matU, start = 2)
#' longevity(mpm1$matU, start = "small") # equivalent using named life stages
#' longevity(mpm1$matU, start = 2, lx_crit = 0.05)
#'
#' # starting from first reproduction
#' repstages <- repro_stages(mpm1$matF)
#' n1 <- mature_distrib(mpm1$matU, start = 2, repro_stages = repstages)
#' longevity(mpm1$matU, start = n1)
#'
#' @export longevity
longevity <- function(matU, start = 1L, x_max = 1000, lx_crit = 0.01) {
  # validate arguments
  checkValidMat(matU, warn_surv_issue = TRUE)
  checkValidStartLife(start, matU, start_vec = TRUE)
  if (lx_crit < 0 || lx_crit > 1) {
    stop("lx_crit must be a proportion between 0 and 1.\n", call. = FALSE)
  }

  if (length(start) == 1) {
    start_vec <- rep(0.0, nrow(matU))
    if (!is.null(dimnames(matU))) {
      checkMatchingStageNames(matU)
      names(start_vec) <- colnames(matU)
    }
    start_vec[start] <- 1.0
  } else {
    start_vec <- start
  }

  lx <- sum(start_vec)
  t <- 0L

  while (lx > lx_crit && t < x_max) {
    start_vec <- matU %*% start_vec
    lx <- sum(start_vec)
    t <- t + 1L
  }

  if (lx <= lx_crit) {
    longevity <- t
  } else {
    longevity <- NA_integer_
    warning("survivorship did not reach 'lx_crit' by 'x_max': returning NA.\n")
  }

  return(longevity)
}

test_that("longevity works correctly", {
  x <- longevity(mat_u)
  x_zero <- suppressWarnings(longevity(mat_u_zero))
  x_noconverg <- suppressWarnings(longevity(mat_u_singular, start = 3))
  x_named <- longevity(mat_u_named, start = "sm")

  expect_length(x, 1L)
  expect_gt(x, 0)
  expect_identical(x_zero, 1L)
  expect_identical(as.integer(x_noconverg), as.integer(NA_real_))
  expect_identical(x, x_named)
})

test_that("longevity warns and fails gracefully", {
  expect_warning(longevity(mat_u_zero))
  expect_warning(longevity(mat_u_survissue))
  expect_warning(longevity(mat_u_singular, start = 3))
  expect_error(longevity(mat_u_na))
  expect_error(longevity(mat_u, start = 10))
  expect_error(longevity(mat_u, start = "stage name"))
  expect_error(longevity(mat_u_named, start = "invalid stage"))
  expect_error(longevity(mat_u, lx_crit = 10))
})

#' Derive stage-specific vital rates from a matrix population model
#'
#' @description
#' Derive a vector of stage-specific vital rates of survival, growth, shrinkage,
#' stasis, dormancy, or reproduction from a matrix population model. These
#' functions include optional arguments for excluding certain stage classes from
#' the calculation (see \emph{Excluding stages}), and defining the set of
#' biologically-possible transitions (see \emph{Possible transitions}).
#'
#' This decomposition assume that all transition rates are products of a
#' stage-specific survival term (column sums of \code{matU}) and a lower level
#' vital rate that is conditional on survival (growth, shrinkage, stasis,
#' dormancy, or reproduction). Reproductive vital rates that are not conditional
#' on survival (i.e., within a stage class from which there is no survival) are
#' also allowed.
#'
#' @param matU The survival component of a matrix population model (i.e., a
#'   square projection matrix only containing survival-related transitions;
#'   progression, stasis, and retrogression).
#' @param matR The reproductive component of a matrix population model (i.e., a
#'   square projection matrix only reflecting transitions due to reproduction;
#'   either sexual, clonal, or both).
#' @param posU A logical matrix of the same dimension as \code{matU}, with
#'   elements indicating whether a given \code{matU} transition is possible
#'   (\code{TRUE}) or not (\code{FALSE}). Defaults to \code{matU > 0} (see
#'   Details).
#' @param posR A logical matrix of the same dimension as \code{matR}, with
#'   elements indicating whether a given \code{matR} transition is possible
#'   (\code{TRUE}) or not (\code{FALSE}). Defaults to \code{matR > 0} (see
#'   Details).
#' @param exclude Integer, character or logical vector indicating stages for
#'   which transitions both \emph{to} and \emph{from} the stage should be
#'   excluded from the calculation of vital rates. See section
#'   \emph{Excluding stages}.
#' @param exclude_row Integer, character or logical vector indicating stages for
#'   which transitions both \emph{to} and \emph{from} the stage should be
#'   excluded from the calculation of vital rates. See section
#'   \emph{Excluding stages}.
#' @param exclude_col Integer, character or logical vector indicating stages for
#'   which transitions both \emph{to} and \emph{from} the stage should be
#'   excluded from the calculation of vital rates. See section
#'   \emph{Excluding stages}.
#' @param dorm_stages Integer or character vector indicating dormant stage
#'   classes.
#' @param weights_row Vector of stage-specific weights to apply while summing
#'   vital rates across rows within columns (e.g., reproductive value vector).
#' @param surv_only_na If there is only one possible \code{matU} transition in a
#'   given column, should that transition be attributed exclusively to survival?
#'   If \code{TRUE}, the vital rate of growth/stasis/shrinkage in that column
#'   will be coerced to \code{NA}. If \code{FALSE}, dividing the single
#'   transition by the stage-specific survival probability will always yield a
#'   value of \code{1}. Defaults to \code{TRUE}.
#'
#' @return Vector of vital rates. Vital rates corresponding to impossible
#'   transitions are coerced to \code{NA} (see \emph{Possible transitions}).
#'
#' @section Possible transitions:
#' A transition rate of \code{0} within a matrix population model may indicate
#' that the transition is not possible in the given life cycle (e.g., tadpoles
#' never revert to eggs), or that the transition rate is possible but was
#' estimated to be \code{0} in the relevant population and time period. If vital
#' rates are to be averaged across multiple stage classes, or compared across
#' populations, it may be important to distinguish between these two types of
#' zeros.
#'
#' By default, the \code{vitals_} functions assume that a transition rate of
#' \code{0} indicates an impossible transition, in which case a value of
#' \code{NA} will be used in relevant calculations. Specifically, the arguments
#' \code{posU} and \code{posR} are specified by the logical expressions
#' \code{(matU > 0)} and \code{(matR > 0)}, respectively. If the matrix
#' population model includes transitions that are estimated to be \code{0} but
#' still in fact possible, one should specify the \code{posU} and/or \code{posR}
#' arguments manually.
#'
#' @section Excluding stages:
#' It may be desirable to exclude one or more stages from the calculation of
#' certain vital rates. For instance, a user might not believe that 'growth' to
#' a dormant stage class really reflects biological growth, in which case the
#' user could exclude transitions \emph{to} the dormant stage class using the
#' argument \code{exclude_row}. The user may or may not want to ignore 'growth'
#' transitions \emph{from} the dormant stage class, which can be done using
#' \code{exclude_col}. The argument \code{exclude_col} effectively just coerces
#' the respective vital rate to \code{NA}, to prevent it from getting used in
#' subsequent calculations. To exclude transitions both \emph{to and from} a
#' given set of stages, use argument \code{exclude}.
#'
#' @note Reproduction-related outputs assume a post-breeding census
#'   interpretation. In pre-breeding models, birth rate and newborn survival are
#'   generally confounded within fecundity transitions. See Kendall et al.
#'   (2019).
#'
#' @author Patrick Barks <patrick.barks@@gmail.com>
#'
#' @family vital rates
#'
#' @examples
#' # create example MPM (stage 4 is dormant)
#' matU <- rbind(
#'   c(0.1, 0, 0, 0),
#'   c(0.5, 0.2, 0.1, 0.1),
#'   c(0, 0.3, 0.3, 0.1),
#'   c(0, 0, 0.5, 0.4)
#' )
#'
#' matR <- rbind(
#'   c(0, 0.7, 1.1, 0),
#'   c(0, 0.3, 0.8, 0),
#'   c(0, 0, 0, 0),
#'   c(0, 0, 0, 0)
#' )
#'
#' vr_vec_survival(matU, exclude_col = 4)
#' vr_vec_growth(matU, exclude = 4)
#'
#' # `exclude*` and `*_stages` arguments can accept stage names
#' matU <- name_stages(matU)
#' matR <- name_stages(matR)
#' vr_vec_shrinkage(matU, exclude = 4)
#' vr_vec_stasis(matU, exclude = "stage_4")
#'
#' vr_vec_dorm_enter(matU, dorm_stages = 4)
#' vr_vec_dorm_exit(name_stages(matU), dorm_stages = "stage_4")
#'
#' vr_vec_reproduction(matU, matR, exclude_col = "stage_4")
#'
#' @name vr_vec
NULL


#' @rdname vr_vec
#' @export vr_vec_survival
vr_vec_survival <- function(matU,
                            posU = matU > 0,
                            exclude_col = NULL) {
  checkValidMat(matU)
  checkValidStages(matU, exclude_col)
  pos_vital <- apply(posU, 2, any)

  v <- colSums(matU)
  v[!pos_vital] <- NA_real_
  v[exclude_col] <- NA_real_
  return(v)
}


#' @rdname vr_vec
#' @export vr_vec_growth
vr_vec_growth <- function(matU,
                          posU = matU > 0,
                          exclude = NULL,
                          exclude_row = NULL,
                          exclude_col = NULL,
                          surv_only_na = TRUE) {
  checkValidMat(matU)
  checkValidStages(matU, exclude)
  checkValidStages(matU, exclude_row)
  checkValidStages(matU, exclude_col)

  vmat <- vr_mat_U(
    matU = matU,
    posU = posU,
    surv_only_na = surv_only_na
  )

  tri_low <- lower.tri(vmat, diag = FALSE)
  vmat[!tri_low] <- NA_real_

  vmat[exclude, ] <- NA_real_
  vmat[exclude_row, ] <- NA_real_

  v <- colSums2(vmat)

  v[exclude] <- NA_real_
  v[exclude_col] <- NA_real_

  return(v)
}


#' @rdname vr_vec
#' @export vr_vec_shrinkage
vr_vec_shrinkage <- function(matU,
                             posU = matU > 0,
                             exclude = NULL,
                             exclude_row = NULL,
                             exclude_col = NULL,
                             surv_only_na = TRUE) {
  checkValidMat(matU)
  checkValidStages(matU, exclude)
  checkValidStages(matU, exclude_row)
  checkValidStages(matU, exclude_col)

  vmat <- vr_mat_U(
    matU = matU,
    posU = posU,
    surv_only_na = surv_only_na
  )

  tri_upp <- upper.tri(vmat, diag = FALSE)
  vmat[!tri_upp] <- NA_real_

  vmat[exclude, ] <- NA_real_
  vmat[exclude_row, ] <- NA_real_

  v <- colSums2(vmat)

  v[exclude] <- NA_real_
  v[exclude_col] <- NA_real_

  return(v)
}


#' @rdname vr_vec
#' @export vr_vec_stasis
vr_vec_stasis <- function(matU,
                          posU = matU > 0,
                          exclude = NULL,
                          surv_only_na = TRUE) {
  checkValidMat(matU)
  checkValidStages(matU, exclude)

  vmat <- vr_mat_U(
    matU = matU,
    posU = posU,
    surv_only_na = surv_only_na
  )

  v <- diag(vmat)
  v[exclude] <- NA_real_

  pos_vital <- diag(posU)
  v[!pos_vital] <- NA_real_

  return(v)
}


#' @rdname vr_vec
#' @export vr_vec_dorm_enter
vr_vec_dorm_enter <- function(matU,
                              posU = matU > 0,
                              dorm_stages) {
  checkValidMat(matU)
  checkValidStages(matU, dorm_stages)

  vmat <- vr_mat_U(
    matU = matU,
    posU = posU
  )

  v <- colSums2(vmat[dorm_stages, , drop = FALSE])
  v[dorm_stages] <- NA_real_

  pos_vital <- apply(posU[dorm_stages, , drop = FALSE], 2, any)
  v[!pos_vital] <- NA_real_

  return(v)
}


#' @rdname vr_vec
#' @export vr_vec_dorm_exit
vr_vec_dorm_exit <- function(matU,
                             posU = matU > 0,
                             dorm_stages) {
  checkValidMat(matU)
  checkValidStages(matU, dorm_stages)

  vmat <- vr_mat_U(
    matU = matU,
    posU = posU
  )

  vmat[dorm_stages, ] <- 0
  v <- colSums2(vmat)

  # convert dorm_stages to indices, if necessary
  if (is.character(dorm_stages)) {
    dorm_stages <- which(colnames(matU) == dorm_stages)
  }

  # possible transitions from dormant to non-dormant
  pos_exit <- matrix(FALSE, nrow = nrow(matU), ncol = nrow(matU))
  pos_exit[-dorm_stages, dorm_stages] <- TRUE
  pos_vital <- apply(posU & pos_exit, 2, any)
  v[!pos_vital] <- NA_real_

  return(v)
}


#' @rdname vr_vec
#' @export vr_vec_reproduction
vr_vec_reproduction <- function(matU,
                                matR,
                                posR = matR > 0,
                                exclude_col = NULL,
                                weights_row = NULL) {
  checkValidMat(matU)
  checkValidMat(matR)
  checkMatchingStageNames(matU, matR)
  checkValidStages(matU, exclude_col)

  vmat <- vr_mat_R(
    matU = matU,
    matR = matR,
    posR = posR
  )

  if (is.null(weights_row)) weights_row <- rep(1, nrow(matU))
  v <- colSums2(vmat * weights_row)

  v[exclude_col] <- NA_real_

  pos_vital <- apply(posR, 2, any)
  v[!pos_vital] <- NA_real_

  return(v)
}

test_that("vr_vec functions work correctly", {
  ### test vr_vec_U
  U1 <- rbind(
    c(0, 0, 0, 0),
    c(0.5, 0.2, 0.1, 0),
    c(0, 0.3, 0.3, 0.1),
    c(0, 0, 0.5, 0.6)
  )

  surv1 <- vr_vec_survival(U1)
  expect_type(surv1, "double")
  expect_length(surv1, 4)

  surv2 <- vr_vec_survival(U1, exclude_col = 4)
  expect_true(is.na(surv2[4]))

  surv3 <- vr_vec_survival(U1, exclude_col = c(FALSE, FALSE, FALSE, TRUE))
  expect_true(is.na(surv3[4]))

  grow1 <- vr_vec_growth(U1)
  expect_type(grow1, "double")
  expect_length(grow1, 4)

  grow2 <- vr_vec_growth(U1, posU = matrix(TRUE, nrow = 4, ncol = 4))
  expect_false(is.na(grow2[1]))

  grow3 <- vr_vec_growth(U1, surv_only_na = FALSE)
  expect_false(is.na(grow3[1]))

  shri1 <- vr_vec_shrinkage(U1)
  expect_type(shri1, "double")
  expect_length(shri1, 4)

  shri2 <- vr_vec_shrinkage(U1, exclude_row = 2)
  expect_true(is.na(shri2[3]))

  stas1 <- vr_vec_stasis(U1)
  expect_type(stas1, "double")
  expect_length(stas1, 4)

  stas2 <- vr_vec_stasis(U1, posU = matrix(TRUE, nrow = 4, ncol = 4))
  expect_identical(stas2[1], 0)

  dent1 <- vr_vec_dorm_enter(U1, dorm_stages = 4)
  expect_type(dent1, "double")
  expect_length(dent1, 4)
  expect_false(is.na(dent1[3]))

  dexi1 <- vr_vec_dorm_exit(U1, dorm_stages = 4)
  expect_type(dexi1, "double")
  expect_length(dexi1, 4)
  expect_false(is.na(dexi1[4]))


  R1 <- rbind(
    c(0, 0, 1.1, 1.6),
    c(0, 0, 0.8, 0.4),
    c(0, 0, 0, 0),
    c(0, 0, 0, 0)
  )

  fecu1 <- vr_vec_reproduction(U1, R1)
  expect_type(fecu1, "double")
  expect_length(fecu1, 4)
  expect_true(all(is.na(fecu1[1:2])))

  # weighting
  v <- popdemo::eigs(U1 + R1, what = "rv") # reproductive value
  fecu2 <- vr_vec_reproduction(U1, R1, weights_row = v)
  expect_true(fecu2[3] != fecu1[3])

  # no survival in a stage with reproduction
  U2 <- rbind(
    c(0, 0, 0, 0),
    c(0.5, 0.2, 0.1, 0),
    c(0, 0.3, 0.3, 0),
    c(0, 0, 0.5, 0)
  )

  fecu3 <- vr_vec_reproduction(U2, R1)
  expect_identical(fecu3[4], sum(R1[, 4]))
})


test_that("vr_vec functions warn and fail gracefully", {
  expect_error(vr_vec_survival(mat_u_na))
  expect_error(vr_vec_growth(mat_u_na))
  expect_error(vr_vec_shrinkage(mat_u_na))
  expect_error(vr_vec_stasis(mat_u_na))
  expect_error(vr_vec_reproduction(mat_u_na))
})

#' Generate a life table from a matrix population model
#'
#' This function uses age-from-stage decomposition methods to generate a life
#' table from a matrix population model. A detailed description of these methods
#' can be found in section 5.3 "Age-specific traits from stage-specific models"
#' of Caswell (2001).
#'
#' @param matU The survival component of a matrix population model (i.e., a
#'   square projection matrix reflecting survival-related transitions; e.g.,
#'   progression, stasis, and/or retrogression). Optionally with named rows and
#'   columns indicating the corresponding life stage names.
#' @param matR The reproductive component of a matrix population model (i.e., a
#'   square projection matrix only reflecting transitions due to reproduction;
#'   either sexual, clonal, or both). If \code{matR} is supplied, \code{matF}
#'   and \code{matC} are ignored. Otherwise, \code{matR} is constructed by
#'   summing \code{matF} and \code{matC}.
#' @param matF The matrix reflecting sexual reproduction. If provided
#'   without \code{matC}, \code{matC} is assumed to be a zero matrix. If
#'   \code{matR} is provided, this argument is ignored.
#' @param matC The matrix reflecting clonal (asexual) reproduction.
#'   If provided without \code{matF}, \code{matF} is assumed to be a zero
#'   matrix. If \code{matR} is provided, this argument is ignored.
#' @param start The index (or stage name) of the first stage at which the author
#'   considers the beginning of life. Defaults to \code{1}. Alternately, a
#'   numeric vector giving the starting population vector (in which case
#'   \code{length(start)} must match \code{ncol(matU))}. See section
#'   \emph{Starting from multiple stages}.
#' @param xmax Maximum age to which the life table will be calculated (defaults
#'   to \code{1000}). Time steps are in the same units as the matrix population
#'   model (see MatrixPeriodicity metadata variable COM(P)ADRE).
#' @param lx_crit Minimum value of lx to which age-specific traits will be
#'   calculated (defaults to \code{0.01}).
#' @param radix The starting number of individuals in the synthetic life table
#'   (defaults to \code{1}). If \code{radix} is set to 1, a simplified life
#'   table is produced.
#' @param remove_final Life table calculations typically assume that the final
#'   age class is closed and that all individuals die in that age class. This
#'   can mean that mortality/hazard is artificially inflated for this age class.
#'   Users can prevent this by setting `remove_final` to `TRUE` (the default is
#'   `FALSE`).
#'
#' @return A \code{data.frame} containing a variable number columns, depending
#'   on input variables. Columns include:
#'
#'   \item{x}{age at the start of the age interval \code{[x, x+1)}}
#'   \item{Nx}{The number of individuals alive at age x. The initial number is
#'   set with \code{radix}}
#'   \item{Dx}{proportion of original cohort dying during the age interval
#'   \code{[x, x+1)}}
#'   \item{lx}{survivorship, defined as the proportion of initial cohort
#'   surviving to the start of age interval \code{[x, x+1)}}
#'   \item{dx}{proportion of original cohort dying in the age interval \code{[x,
#'   x+1)}}
#'   \item{ax}{The average time survived within the interval by those that die
#'   during the age interval \code{[x, x+1)}. Assumed to be 0.5}
#'   \item{hx}{force of mortality (hazard) during the age interval \code{[x,
#'   x+1)}}
#'   \item{qx}{probability of death during the interval \code{[x, x+1)} for
#'   those entering the interval}
#'   \item{px}{probability of survival for the interval \code{[x, x+1)} for
#'   those entering the interval}
#'   \item{Lx}{total person-years lived during the interval \code{[x, x+1)}}
#'   \item{Tx}{total person years lived beyond age x}
#'   \item{ex}{remaining life expectancy at age x}
#'
#' If \code{matF} is provided, also includes:
#'   \item{mx}{per-capita rate of sexual reproduction during the interval
#'    \code{[x, x+1)} }
#'   \item{lxmx}{expected number of sexual offspring per original
#'   cohort member produced during the interval \code{[x, x+1)}}
#'
#' If \code{matC} is provided, also includes:
#'   \item{cx}{per-capita rate of clonal reproduction  during the interval
#'   \code{[x, x+1)}}
#'   \item{lxcx}{expected number of clonal offspring per original
#'   cohort member produced during the interval \code{[x, x+1)}}
#'
#' If only \code{matR} is provided, the calculations proceed as with \code{matF}.
#'
#' If both \code{matF} and \code{matC} are provided, also includes:
#'   \item{mxcx}{per-capita rate of total reproduction (sexual + clonal) during
#'   the interval \code{[x, x+1)}}
#'   \item{lxmxcx}{expected number of total offspring (sexual + clonal) per
#'   original cohort member produced during the interval \code{[x, x+1)}}
#'
#' @section Starting from multiple stages: Rather than specifying argument
#'   \code{start} as a single stage class from which all individuals start life,
#'   it may sometimes be desirable to allow for multiple starting stage classes.
#'   For example, if the user wants to start the calculation of age-specific
#'   traits from reproductive maturity (i.e., first reproduction), the user
#'   should account for the possibility that there may be multiple stage classes
#'   in which an individual could first reproduce.
#'
#' To specify multiple starting stage classes, specify argument \code{start} as
#' the desired starting population vector (\strong{n1}), giving the proportion
#' of individuals starting in each stage class (the length of \code{start}
#' should match the number of columns in the relevant MPM).
#'
#' See function \code{\link{mature_distrib}} for calculating the proportion of
#' individuals achieving reproductive maturity in each stage class.
#'
#' @note The life table is calculated recursively until the age class (x)
#'   reaches \code{xmax} or survivorship (lx) falls below \code{lx_crit} —
#'   whichever comes first. To force calculation to \code{xmax}, set
#'   \code{lx_crit = 0}. Conversely, to force calculation to \code{lx_crit}, set
#'   \code{xmax = Inf}.
#'
#' @note The life table calculations assume that the final age interval is
#'   closed and that all remaining individuals die in this interval. Therefore,
#'   for this interval, the probability of death \code{qx} is 1, the probability
#'   of survival \code{px} is 0 and, because we assume that deaths are evenly
#'   distributed during the interval, the remaining life expectancy for
#'   individuals at the start of the interval is 0.5. Depending on analyses, it
#'   may be a good idea to remove the final row of the table.
#'
#'   If \code{lx_crit} is sufficiently small that only a very small proportion
#'   of the cohort reach this age (i.e., < 0.05), this should have minimal
#'   impact on results. Nevertheless, for many analyses, the final row of the
#'   life table should be treated with caution and perhaps removed from
#'   subsequent analyses.
#'
#' @note If reproduction columns are included, the resulting \code{mx},
#'   \code{cx}, and related trajectories reflect reproductive output as
#'   represented at the census boundary. As a result, reproductive trajectories
#'   from pre-breeding and post-breeding models are not necessarily directly
#'   comparable. See Kendall et al. (2019).
#'
#' @note Note that the units of time (e.g.. `x` and `ex`) in the returned life
#'   table are the same as the projection interval (`ProjectionInterval`) of the
#'   MPM.
#'
#' @author Owen R. Jones <jones@@biology.sdu.dk>
#' @author Roberto Salguero-Gómez <rob.salguero@@zoo.ox.ac.uk>
#' @author Hal Caswell <h.caswell@@uva.nl>
#'
#' @family life tables
#'
#' @references
#' Caswell, H. 2001. Matrix Population Models: Construction, Analysis, and
#' Interpretation. Sinauer Associates; 2nd edition. ISBN: 978-0878930968
#'
#' Caswell, H. 2006. Applications of Markov chains in demography. pp. 319-334 in
#' A.N. Langville and W.J. Stewart (editors) MAM2006: Markov Anniversary
#' Meeting. Boson Books, Raleigh, North Caroline, USA
#'
#' Horvitz, C. & Tuljapurkar, S. 2008. Stage dynamics, period survival, and
#' mortality plateaus. The American Naturalist 172: 203-2015.
#' <doi:10.1086/589453>
#'
#' Jones, O. R., Scheuerlein, A., Salguero-Gomez, R., Camarda, C. G., Schaible,
#' R., Casper, B. B., Dahlgren, J. P., Ehrlén, J., García, M. B., Menges, E.,
#' Quintana-Ascencio, P. F., Caswell, H., Baudisch, A. & Vaupel, J. 2014.
#' Diversity of ageing across the tree of life. Nature 505, 169-173.
#' <doi:10.1038/nature12789>
#'
#' Jones O. R. 2021. Life tables: Construction and interpretation In:
#' Demographic Methods Across the Tree of Life. Edited by Salguero-Gomez R &
#' Gamelon M. Oxford University Press. Oxford, UK. ISBN: 9780198838609
#'
#' Preston, S., Heuveline, P., & Guillot, M. 2000. Demography: Measuring and
#' Modeling Population Processes. Wiley. ISBN: 9781557864512
#'
#' @examples
#' data(mpm1)
#'
#' mpm_to_table(matU = mpm1$matU, start = 2, xmax = 15)
#'
#' # equivalent using named life stages
#' mpm_to_table(matU = mpm1$matU, start = "small", xmax = 15)
#' mpm_to_table(matU = mpm1$matU, matR = mpm1$matF, start = 2, xmax = 15)
#'
#' ### starting from first reproduction
#' repStages <- repro_stages(mpm1$matF)
#' n1 <- mature_distrib(matU = mpm1$matU, start = 2, repro_stages = repStages)
#' mpm_to_table(matU = mpm1$matU, start = n1)
#' @export mpm_to_table
mpm_to_table <- function(matU, matR = NULL, matF = NULL, matC = NULL, start = 1L,
                         xmax = 1000, lx_crit = 0.01, radix = 1,
                         remove_final = FALSE) {
  use_matR <- !is.null(matR)
  matR_total <- process_fertility_inputs(matR = matR, matF = matF, matC = matC)

  # validate arguments
  checkValidMat(matU, warn_surv_issue = TRUE)
  if (!is.null(matR_total)) checkValidMat(matR_total)
  if (!use_matR && !is.null(matF)) checkValidMat(matF)
  if (!use_matR && !is.null(matC)) checkValidMat(matC)
  checkValidStartLife(start, matU, start_vec = TRUE)

  # remove_final
  if (!remove_final %in% c(TRUE, FALSE)) {
    stop("remove_final must be either TRUE or FALSE")
  }

  # Age-specific survivorship (lx)
  lx <- mpm_to_lx(matU, start, xmax, lx_crit)
  if (lx[length(lx)] > 0.05) {
    warning(strwrap(
      prefix = " ", initial = "",
      "There are still a large proportion of the synthetic
  cohort remaining alive in the final row of the life table.
    Consider changing values for `lx_crit` or `xmax`.\n"
    ))
  }
  # Number left alive at age x
  Nx <- lx * radix

  # Number of age groups
  N <- length(lx)

  # Proportion of ORIGINAL cohort dying during each age interval
  dx <- c(lx[1:(N - 1)] - lx[2:N], lx[length(lx)])

  # Number of deaths occuring during the interval (assuming an initial
  # population size defined by the radix)
  # If radix = 1 (the default) then dx = Dx
  Dx <- dx * radix

  # Probability of dying between x and x+1 (qx)
  qx <- dx / lx

  # Probability of surviving between x and x+1 (px)
  px <- 1 - qx

  # Person-years lived between x and x+1
  # We use an ax value of 0.5, which means that we implicitly assume that deaths
  # are distributed uniformly across the interval. ax is defined as the average
  # number of years lived in the interval by those dying in the interval

  ax <- 0.5
  Lx <- (px * lx) + (ax * dx)

  # Person=years lived above age x
  Tx <- rev(cumsum(rev(Lx)))

  # Death rate (hazard) in the cohort between ages x and x + 1
  hx <- dx / Lx

  # Life expectancy from age x
  ex <- Tx / lx

  # Force of mortality (hazard)
  qx <- dx / lx

  # Start to assemble output object
  # If radix is set to be a value other than 1 a full life table is produced,
  # otherwise a streamlined version is produced. In most use cases the
  # streamlined version is sufficient.
  if (radix != 1) {
    out <- data.frame(
      x = 0:(N - 1),
      Nx = Nx,
      Dx = Dx,
      lx = lx,
      dx = dx,
      ax = ax,
      hx = hx,
      qx = qx,
      px = px,
      Lx = Lx,
      Tx = Tx,
      ex = ex
    )
  } else {
    out <- data.frame(
      x = 0:(N - 1),
      lx = lx,
      dx = dx,
      hx = hx,
      qx = qx,
      px = px,
      ex = ex
    )
  }

  # Reproduction
  if (use_matR) {
    out$mx <- mpm_to_mx(
      matU = matU, matR = matR_total, start = start,
      xmax = N - 1, lx_crit = lx_crit
    )
    out$lxmx <- out$lx * out$mx
  } else if (!is.null(matF)) {
    out$mx <- mpm_to_mx(matU = matU, matF = matF, start = start, xmax = N - 1, lx_crit = lx_crit)
    out$lxmx <- out$lx * out$mx
  }

  if (!use_matR && !is.null(matC)) {
    out$cx <- mpm_to_mx(matU = matU, matC = matC, start = start, xmax = N - 1, lx_crit = lx_crit)
    out$lxcx <- out$lx * out$cx
  }

  if (!use_matR && !is.null(matF) && !is.null(matC)) {
    out$mxcx <- out$mx + out$cx
    out$lxmxcx <- out$lx * out$mxcx
  }

  if (remove_final == TRUE) {
    return(out[-nrow(out), ])
  }
  if (remove_final == FALSE) {
    return(out)
  }
}

test_that("mpm_to_table works correctly", {
  xmax <- 20

  x_u <- mpm_to_table(matU = mat_u, xmax = xmax, lx_crit = 0)
  x_uf <- mpm_to_table(matU = mat_u, matF = mat_f, xmax = xmax, lx_crit = 0)
  x_uc <- mpm_to_table(matU = mat_u, matC = mat_c, xmax = xmax, lx_crit = 0)
  x_ufc <- mpm_to_table(matU = mat_u, matF = mat_f, matC = mat_c, xmax = xmax, lx_crit = 0)
  x_r <- mpm_to_table(matU = mat_u, matR = mat_f, xmax = xmax, lx_crit = 0)
  x_rfc <- mpm_to_table(matU = mat_u, matR = mat_f + mat_c, xmax = xmax, lx_crit = 0)
  x_precedence <- mpm_to_table(
    matU = mat_u, matR = mat_f, matF = mat_c, matC = mat_c,
    xmax = xmax, lx_crit = 0
  )
  x_u_named <- mpm_to_table(matU = mat_u_named, start = "sm", xmax = xmax, lx_crit = 0)
  x_uf_named <- mpm_to_table(
    matU = mat_u_named, mat_f_named,
    start = "sm", xmax = xmax,
    lx_crit = 0
  )

  expect_s3_class(x_u, "data.frame")
  expect_identical(nrow(x_u), as.integer(xmax + 1))
  expect_identical(ncol(x_u), 7L)
  expect_identical(ncol(x_uf), 9L)
  expect_identical(ncol(x_uc), 9L)
  expect_identical(ncol(x_ufc), 13L)
  expect_identical(x_uf, x_r)
  expect_identical(ncol(x_rfc), 9L)
  expect_identical(x_r$mx, x_precedence$mx)
  expect_false("cx" %in% names(x_rfc))
  expect_equal(x_rfc$mx, x_ufc$mxcx)
  expect_identical(x_u, x_u_named)
  expect_identical(x_uf, x_uf_named)
})

test_that("mpm_to_table warns and fails gracefully", {
  xmax <- 20

  expect_error(mpm_to_table(matU = mat_u_na, matF = mat_f, xmax = xmax))
  expect_error(mpm_to_table(matU = mat_u, matF = mat_f_na, xmax = xmax))
  expect_error(mpm_to_table(matU = mat_u, matC = mat_c_na, xmax = xmax))
  expect_error(mpm_to_table(matU = mat_u, matR = mat_f_na, xmax = xmax))
  expect_error(mpm_to_table(
    matU = mat_u,
    matF = mat_f_named, start = "sm",
    xmax = xmax
  ))
})


mu_1 <- matrix(c(0.08, 0.9, 0.08, 0.9), ncol = 2)
mu_1
mpm_to_table(mu_1)
expect_s3_class(mpm_to_table(matU = mu_1), "data.frame")

expect_error(mpm_to_table(matU = mu_1, remove_final = "Yes"))
expect_warning(mpm_to_table(matU = mu_1, lx_crit = 0.1))

expect_s3_class(mpm_to_table(matU = mu_1, radix = 1000), "data.frame")

expect_s3_class(mpm_to_table(
  matU = mu_1,
  radix = 1000,
  remove_final = TRUE
), "data.frame")

#' Scale a matrix population model so that lambda equals 1
#'
#' Scales a matrix population model (MPM) so that the dominant eigenvalue
#' (\eqn{\lambda}) of the full projection matrix equals 1. The function accepts
#' either a full projection matrix (\code{matA}) or a set of component matrices
#' consisting of the survival component (\code{matU}) plus at least one
#' reproductive component (\code{matF} and/or \code{matC}).
#'
#' @param matU The survival component of a matrix population model (i.e., a
#'   square projection matrix reflecting survival-related transitions; e.g.,
#'   progression, stasis, and retrogression). Required when using component
#'   inputs.
#' @param matF The sexual reproduction component of a matrix population model.
#'   Optional if \code{matC} is supplied. If omitted while \code{matC} is
#'   supplied, it is assumed to be a zero matrix.
#' @param matC The clonal reproduction component of a matrix population model.
#'   Optional if \code{matF} is supplied. If omitted while \code{matF} is
#'   supplied, it is assumed to be a zero matrix.
#' @param matA The full projection matrix of a matrix population model.
#'   Optional if component matrices are supplied. If supplied together with
#'   components, it must equal \code{matU + matF + matC}.
#'
#' @return A list with six elements:
#' \describe{
#'   \item{matA}{The scaled full projection matrix.}
#'   \item{matU}{The scaled survival matrix. Returned as \code{NULL} if only
#'   \code{matA} is supplied.}
#'   \item{matF}{The scaled sexual reproduction matrix. Returned as \code{NULL}
#'   if only \code{matA} is supplied.}
#'   \item{matC}{The scaled clonal reproduction matrix. Returned as \code{NULL}
#'   if only \code{matA} is supplied.}
#'   \item{lambda_original}{The dominant eigenvalue of the original projection
#'   matrix.}
#'   \item{lambda_scaled}{The dominant eigenvalue of the scaled projection
#'   matrix.}
#' }
#'
#' @details When component matrices are used, \code{matU} must be supplied
#' together with at least one of \code{matF} or \code{matC}. Missing
#' reproductive components are treated as zero matrices of the appropriate
#' dimension.
#'
#' @author Owen R. Jones <jones@@biology.sdu.dk>
#'
#' @family transformation
#'
#' @examples
#' data(mpm1)
#'
#' # scale from the full projection matrix
#' matA <- mpm1$matU + mpm1$matF
#' scale_mpm_to_lambda1(matA = matA)
#'
#' # scale from component matrices, assuming no clonal reproduction
#' scale_mpm_to_lambda1(matU = mpm1$matU, matF = mpm1$matF)
#'
#' @export
scale_mpm_to_lambda1 <- function(matU = NULL,
                                 matF = NULL,
                                 matC = NULL,
                                 matA = NULL) {
  has_A <- !is.null(matA)
  has_U <- !is.null(matU)
  has_F <- !is.null(matF)
  has_C <- !is.null(matC)
  has_components <- has_U || has_F || has_C

  if (!has_A && !(has_U && (has_F || has_C))) {
    stop(
      "Supply either 'matA' or 'matU' plus at least one of 'matF' or 'matC'.",
      call. = FALSE
    )
  }

  if (has_components && !has_U) {
    stop("Argument 'matU' is required when supplying component matrices.",
      call. = FALSE
    )
  }

  validate_scale_input <- function(M, arg) {
    if (!is.matrix(M) || !is.numeric(M) || nrow(M) != ncol(M)) {
      stop("Argument '", arg, "' must be a square numeric matrix.",
        call. = FALSE
      )
    }
    if (anyNA(M)) {
      stop("Argument '", arg, "' contains missing values (i.e. <NA>).",
        call. = FALSE
      )
    }
    if (!all(is.finite(M))) {
      stop("Argument '", arg, "' must contain only finite values.",
        call. = FALSE
      )
    }
  }

  if (has_components) {
    validate_scale_input(matU, "matU")

    if (!has_F && !has_C) {
      stop(
        "Supply at least one of 'matF' or 'matC' when using component inputs.",
        call. = FALSE
      )
    }

    if (has_F) {
      validate_scale_input(matF, "matF")
      if (!all(dim(matU) == dim(matF))) {
        stop(
          "Arguments 'matU' and 'matF' must have the same dimensions.",
          call. = FALSE
        )
      }
      checkMatchingStageNames(matU, matF)
    } else {
      matF <- matrix(0, nrow = nrow(matU), ncol = ncol(matU))
      dimnames(matF) <- dimnames(matU)
    }

    if (has_C) {
      validate_scale_input(matC, "matC")
      if (!all(dim(matU) == dim(matC))) {
        stop(
          "Arguments 'matU' and 'matC' must have the same dimensions.",
          call. = FALSE
        )
      }
      checkMatchingStageNames(matU, matC)
    } else {
      matC <- matrix(0, nrow = nrow(matU), ncol = ncol(matU))
      dimnames(matC) <- dimnames(matU)
    }

    matA_from_components <- matU + matF + matC
  }

  if (has_A) {
    validate_scale_input(matA, "matA")
    if (has_components) {
      if (!all(dim(matA) == dim(matU))) {
        stop(
          "Arguments 'matA' and 'matU' must have the same dimensions.",
          call. = FALSE
        )
      }
      checkMatchingStageNames(matU, matA)
      if (!isTRUE(all.equal(matA, matA_from_components,
        tolerance = sqrt(.Machine$double.eps)
      ))) {
        stop(
          "Argument 'matA' must equal 'matU + matF + matC'.",
          call. = FALSE
        )
      }
    }
  } else {
    matA <- matA_from_components
  }

  lambda_original <- suppressWarnings(as.numeric(Re(lambda(matA))))

  if (!is.finite(lambda_original) || lambda_original <= 0) {
    stop(
      "Argument 'matA' must have a finite positive dominant eigenvalue.",
      call. = FALSE
    )
  }

  matA_scaled <- matA / lambda_original

  out <- list(
    matA = matA_scaled,
    matU = NULL,
    matF = NULL,
    matC = NULL,
    lambda_original = lambda_original,
    lambda_scaled = suppressWarnings(as.numeric(Re(lambda(matA_scaled))))
  )

  if (has_components) {
    out$matU <- matU / lambda_original
    out$matF <- matF / lambda_original
    out$matC <- matC / lambda_original
  }

  return(out)
}

test_that("scale_mpm_to_lambda1 works correctly with matA only", {
  matA <- mat_u + mat_f

  x <- scale_mpm_to_lambda1(matA = matA)

  expect_type(x, "list")
  expect_length(x, 6L)
  expect_null(x$matU)
  expect_null(x$matF)
  expect_null(x$matC)
  expect_true(is.numeric(x$lambda_original))
  expect_equal(x$lambda_scaled, 1, tolerance = 1e-8)
  expect_equal(as.numeric(Re(popdemo::eigs(x$matA, what = "lambda"))),
    1,
    tolerance = 1e-8
  )
})

test_that("scale_mpm_to_lambda1 works correctly with component matrices", {
  matA <- mat_u + mat_f + mat_c

  x1 <- scale_mpm_to_lambda1(matU = mat_u, matF = mat_f)
  expect_type(x1, "list")
  expect_length(x1, 6L)
  expect_true(all(x1$matC == 0))
  expect_equal(x1$lambda_scaled, 1, tolerance = 1e-8)
  expect_equal(x1$matA, x1$matU + x1$matF + x1$matC, tolerance = 1e-8)

  x2 <- scale_mpm_to_lambda1(matU = mat_u, matC = mat_c)
  expect_true(all(x2$matF == 0))
  expect_equal(x2$lambda_scaled, 1, tolerance = 1e-8)
  expect_equal(x2$matA, x2$matU + x2$matF + x2$matC, tolerance = 1e-8)

  x3 <- scale_mpm_to_lambda1(matU = mat_u, matF = mat_f, matC = mat_c)
  expect_equal(x3$lambda_scaled, 1, tolerance = 1e-8)
  expect_equal(x3$matA, x3$matU + x3$matF + x3$matC, tolerance = 1e-8)

  x4 <- scale_mpm_to_lambda1(matU = mat_u, matF = mat_f, matC = mat_c, matA = matA)
  expect_equal(x4$lambda_scaled, 1, tolerance = 1e-8)
  expect_equal(x4$matA, x4$matU + x4$matF + x4$matC, tolerance = 1e-8)
})

test_that("scale_mpm_to_lambda1 preserves stage names", {
  matA_named <- mat_u_named + mat_f_named

  x <- scale_mpm_to_lambda1(matU = mat_u_named, matF = mat_f_named, matA = matA_named)

  expect_identical(dimnames(x$matA), dimnames(matA_named))
  expect_identical(dimnames(x$matU), dimnames(mat_u_named))
  expect_identical(dimnames(x$matF), dimnames(mat_f_named))
  expect_identical(dimnames(x$matC), dimnames(mat_u_named))
})

test_that("scale_mpm_to_lambda1 warns and fails gracefully", {
  expect_error(scale_mpm_to_lambda1())
  expect_error(scale_mpm_to_lambda1(matU = mat_u))
  expect_error(scale_mpm_to_lambda1(matF = mat_f))
  expect_error(scale_mpm_to_lambda1(matC = mat_c))
  expect_error(scale_mpm_to_lambda1(matU = mat_u, matF = mat_f_notsq))
  expect_error(scale_mpm_to_lambda1(matU = mat_u, matF = mat_f, matC = mat_c, matA = mat_u + mat_f))
  expect_error(scale_mpm_to_lambda1(matU = mat_u_na, matF = mat_f))
  expect_error(scale_mpm_to_lambda1(matA = matrix(0, nrow = 4, ncol = 4)))
  expect_error(scale_mpm_to_lambda1(matU = mat_u_named, matF = mat_f))
})

#' Derive mean vital rates from a matrix population model
#'
#' @description
#' Derive mean vital rates corresponding to separate demographic processes from
#' a matrix population model. Specifically, this function decomposes vital rates
#' of survival, progression, retrogression, sexual reproduction and clonal
#' reproduction, with various options for weighting and grouping stages of the
#' life cycle.
#'
#' @param matU The survival component of a matrix population model (i.e., a
#'   square projection matrix reflecting survival-related transitions; e.g.
#'   progression, stasis, and retrogression).
#' @param matF The sexual component of a matrix population model (i.e., a square
#'   projection matrix reflecting transitions due to sexual reproduction)
#' @param matC The clonal component of a matrix population model (i.e., a square
#'   projection matrix reflecting transitions due to clonal reproduction).
#'   Defaults to \code{NULL}, indicating no clonal reproduction (i.e.
#'   \code{matC} is a matrix of zeros).
#' @param weights Vector of stage-specific weights to apply while averaging
#'   vital rates. Default is \code{NULL} reflecting equal weighting for all
#'   stages. May also be \code{"SSD"} to weight vital rates by the stable
#'   distribution of \code{matA}.
#' @param splitStages What groups should vital rates be averaged over. Either:
#'
#' \code{"all"}: all stages grouped.
#'
#' \code{"ontogeny"}: group juvenile stages (all stages prior to the first stage
#' with sexual reproduction) and adult stages.
#'
#' \code{"matrixStages"}: group according to a standardized set of stage classes
#' (propagule, active, and dormant). If \code{splitStages = "matrixStages"},
#' must also specify separate argument \code{matrixStages}.
#'
#' @param matrixStages Vector of stage-specific standardized matrix classes
#'   ("prop" for propagule, "active", and/or "dorm" for dormant). Only used if
#'   \code{splitStages = "matrixClass"}.
#'
#' @return A list of averaged vital rates.
#'
#' @author Roberto Salguero-Gomez <rob.salguero@@zoo.ox.ac.uk>
#'
#' @family vital rates
#'
#' @references Caswell, H. 2001. Matrix Population Models: Construction,
#'   Analysis, and Interpretation. Sinauer Associates; 2nd edition. ISBN:
#'   978-0878930968
#'
#' @examples
#' matU <- rbind(
#'   c(0.1, 0, 0, 0),
#'   c(0.5, 0.2, 0.1, 0),
#'   c(0, 0.3, 0.3, 0.1),
#'   c(0, 0, 0.5, 0.6)
#' )
#'
#' matF <- rbind(
#'   c(0, 0, 1.1, 1.6),
#'   c(0, 0, 0.8, 0.4),
#'   c(0, 0, 0, 0),
#'   c(0, 0, 0, 0)
#' )
#'
#' matC <- rbind(
#'   c(0, 0, 0.4, 0.5),
#'   c(0, 0, 0.3, 0.1),
#'   c(0, 0, 0, 0),
#'   c(0, 0, 0, 0)
#' )
#'
#' # Vital rate outputs without weights
#' vital_rates(matU, matF, matC, splitStages = "all")
#' vital_rates(matU, matF, matC, splitStages = "ontogeny")
#'
#' # Group vital rates according to specified matrixStages
#' ms <- c("prop", "active", "active", "active")
#' vital_rates(matU, matF, matC,
#'   splitStages = "matrixStages",
#'   matrixStages = ms
#' )
#'
#' # Vital rate outputs weighted by the stable stage distribution of 'matA'
#' vital_rates(matU, matF, matC, splitStages = "all", weights = "SSD")
#'
#' @export vital_rates
vital_rates <- function(matU, matF, matC = NULL, weights = NULL,
                        splitStages = "all", matrixStages = NULL) {
  # validate arguments
  checkValidMat(matU)
  checkValidMat(matF)
  if (!is.null(matC)) checkValidMat(matC, warn_all_zero = FALSE)

  if (all(
    !is.null(weights), weights != "SSD",
    length(weights) != nrow(matU)
  )) {
    stop(strwrap(prefix = " ", initial = "", "If `weights` are provided,
    `length(weights)` should be of the same dimension as `matU`.\n"),
      call. = FALSE
    )
  }

  if (!splitStages %in% c("all", "ontogeny", "matrixStages")) {
    stop(strwrap(prefix = " ", initial = "", "Argument `splitStages` must be
    one of `all`, `ontogeny`, or `matrixStages`.\n"), call. = FALSE)
  }
  if (splitStages == "matrixStages") {
    if (is.null(matrixStages)) {
      stop(strwrap(
        prefix = " ", initial = "",
        "If `splitStages` = `matrixStages`, argument `matrixStages`
                   must be provided.\n"
      ), call. = FALSE)
    }
    if (length(matrixStages) != nrow(matU)) {
      stop("`length(matrixStages)` should be of the same dimension
           as `matU`.\n", call. = FALSE)
    }
  }

  if (!is.null(matrixStages)) {
    if (!all(matrixStages %in% c("prop", "active", "dorm"))) {
      stop("matrixStage names must be 'prop','active' or 'dorm'.\n",
        call. = FALSE
      )
    }
  }

  if (is.null(matC)) {
    matC <- matrix(0, nrow = nrow(matU), ncol = ncol(matU))
  }

  matDim <- dim(matU)[1]
  matA <- matU + matF + matC

  surv <- colSums(matU)
  fec <- colSums(matF)
  clo <- colSums(matC)

  matUIndep <- matrix(NA_real_, matDim, matDim)
  for (i in 1:matDim) {
    matUIndep[, i] <- matU[, i] / surv[i]
  }
  prog <- retr <- matUIndep
  prog[is.nan(prog)] <- 0
  retr[is.nan(retr)] <- 0

  prog[which(upper.tri(matUIndep, diag = TRUE))] <- 0
  retr[which(lower.tri(matUIndep, diag = TRUE))] <- 0

  prog <- colSums(prog)
  retr <- colSums(retr)

  # possible transitions
  i_surv <- surv > 0
  i_prog <- prog > 0
  i_retr <- retr > 0
  i_fec <- fec > 0
  i_clo <- clo > 0

  if (is.null(weights)) {
    weights <- rep(1.0, matDim)
  } else if (weights[1] == "SSD") {
    weights <- stable.stage(matA)
  }

  weights <- weights / sum(weights)

  surv1 <- surv * weights
  prog1 <- prog * weights
  retr1 <- retr * weights
  fec1 <- fec * weights
  clo1 <- clo * weights

  out <- NULL

  if (splitStages == "all") {
    out$surv <- sum(surv1[i_surv]) / sum(weights[i_surv])
    out$retr <- sum(retr1[i_retr]) / sum(weights[i_retr])
    out$prog <- sum(prog1[i_prog]) / sum(weights[i_prog])
    out$fec <- sum(fec1[i_fec]) / sum(weights[i_fec])
    out$clo <- sum(clo1[i_clo]) / sum(weights[i_clo])
  }

  if (splitStages == "ontogeny") {
    # This adu classification does not account for non- and post-reproductive
    adu <- colSums(matF) > 0
    juv <- colSums(matF) == 0

    out$survJuv <- sum(surv1[juv & i_surv]) / sum(weights[juv & i_surv])
    out$retrJuv <- sum(retr1[juv & i_retr]) / sum(weights[juv & i_retr])
    out$progJuv <- sum(prog1[juv & i_prog]) / sum(weights[juv & i_prog])
    out$cloJuv <- sum(clo1[juv & i_clo]) / sum(weights[juv & i_clo])

    out$survAdu <- sum(surv1[adu & i_surv]) / sum(weights[adu & i_surv])
    out$retrAdu <- sum(retr1[adu & i_retr]) / sum(weights[adu & i_retr])
    out$progAdu <- sum(prog1[adu & i_prog]) / sum(weights[adu & i_prog])
    out$fecAdu <- sum(fec1[adu & i_fec]) / sum(weights[adu & i_fec])
    out$cloAdu <- sum(clo1[adu & i_clo]) / sum(weights[adu & i_clo])
  }

  if (splitStages == "matrixStages") {
    prop <- matrixStages == "prop"
    acti <- matrixStages == "active"
    dorm <- matrixStages == "dorm"

    out$survProp <- sum(surv1[prop & i_clo]) / sum(weights[prop & i_clo])
    out$progProp <- sum(prog1[prop & i_clo]) / sum(weights[prop & i_clo])

    out$survActive <- sum(surv1[acti & i_surv]) / sum(weights[acti & i_surv])
    out$retrActive <- sum(retr1[acti & i_retr]) / sum(weights[acti & i_retr])
    out$progActive <- sum(prog1[acti & i_prog]) / sum(weights[acti & i_prog])
    out$fecActive <- sum(fec1[acti & i_fec]) / sum(weights[acti & i_fec])
    out$cloActive <- sum(clo1[acti & i_clo]) / sum(weights[acti & i_clo])

    out$survDorm <- sum(surv1[dorm & i_surv]) / sum(weights[dorm & i_surv])
    out$retrDorm <- sum(retr1[dorm & i_retr]) / sum(weights[dorm & i_retr])
    out$progDorm <- sum(prog1[dorm & i_prog]) / sum(weights[dorm & i_prog])
  }

  # convert NaN to 0 (non-existent vital rate within group will yield NaN)
  out <- lapply(out, function(x) ifelse(is.nan(x), 0.0, x))

  return(out)
}

test_that("vital_rates functions work correctly", {
  vrOut <- vital_rates(matU = mat_u, matF = mat_f)

  expect_type(vrOut, "list")
  expect_length(vrOut, 5)
  expect_identical(vrOut$clo, 0)

  vrOut_c <- vital_rates(
    matU = mat_u,
    matF = mat_f,
    matC = mat_c
  )

  expect_type(vrOut_c, "list")
  expect_length(vrOut_c, 5)


  vrOut_ss <-
    vital_rates(
      matU = mat_u,
      matF = mat_f,
      splitStages = "all"
    )

  expect_type(vrOut_ss, "list")
  expect_length(vrOut_ss, 5)
  expect_identical(vrOut_ss$clo, 0)

  vrOut_ss2 <-
    vital_rates(
      matU = mat_u,
      matF = mat_f,
      splitStages = "ontogeny"
    )

  expect_type(vrOut_ss2, "list")
  expect_length(vrOut_ss2, 9)
  expect_identical(vrOut_ss2$cloAdu, 0)

  vrOut_ss3 <-
    vital_rates(
      matU = mat_u,
      matF = mat_f,
      splitStages = "matrixStages",
      matrixStages = c("active", "active", "active", "dorm")
    )

  expect_type(vrOut_ss3, "list")
  expect_length(vrOut_ss3, 10)
  expect_identical(vrOut_ss3$clo, 0)

  vrOut_wt <- vital_rates(
    matU = mat_u,
    matF = mat_f,
    weights = "SSD"
  )

  expect_type(vrOut_wt, "list")
  expect_length(vrOut_wt, 5)
  expect_identical(vrOut$clo, 0)

  vrOut_wt2 <-
    vital_rates(
      matU = mat_u,
      matF = mat_f,
      weights = c(1, 2, 3, 4)
    )

  expect_type(vrOut_wt2, "list")
  expect_length(vrOut_wt2, 5)
  expect_identical(vrOut_wt2$clo, 0)
})


test_that("vital_rates functions warn and fail gracefully", {
  expect_error(vital_rates(mat_u_na, mat_f_na))

  expect_error(vital_rates(mat_u, mat_f_na))

  expect_error(vital_rates(mat_u_na, mat_f))

  expect_error(vital_rates(mat_u, mat_f, weights = 3))

  expect_error(vital_rates(mat_u, mat_f, splitStages = "wrongName"))

  expect_error(vital_rates(
    mat_u,
    mat_f,
    splitStages = "matrixStages",
    matrixStages = c("active", "active", "active", "x")
  ))

  expect_error(vital_rates(mat_u, mat_f, splitStages = "matrixStages"))

  expect_error(vital_rates(
    mat_u,
    mat_f,
    splitStages = "matrixStages",
    matrixStages = c("active", "active", "active")
  ))
})

#' Calculate shape of survival over age
#'
#' Calculates a 'shape' value of survival lifespan inequality by comparing the
#' area under a survival curve (over age) with the area under a constant
#' survival function.
#'
#' @param surv Either 1) a numeric vector describing a survival curve (lx), 2) a
#'   \code{data.frame} / \code{list} with one column / element titled 'lx'
#'   describing a survival curve, optionally a column / element 'x' containing
#'   age classes (each element a number representing the age at the start of the
#'   class), or 3), a \code{matrix}, specifically the U submatrix of a matrix
#'   population model (A).
#'
#'   In case (2) If \code{x} is not supplied, the function will assume age
#'   classes starting at \code{0} with time steps of \code{1} unit of the
#'   \code{ProjectionInterval}. If \code{x} begins at \code{0} then \code{lx[1]}
#'   should equal \code{1}. If \code{x} ends at maximum longevity, then
#'   \code{lx[which.max(x)]} should equal \code{0}; however it is possible to
#'   supply partial survivorship curves.
#'
#' @param xmin,xmax The minimum and maximum age respectively over which to
#'   evaluate shape. If not given, these default to \code{min(x)} and
#'   \code{max(x)} respectively.
#' @param trunc logical determining whether to truncate life tables or not when
#'   any \code{lx == 0}. Usually this is the case only for the final value of
#'   \code{lx}. As the function calculates \code{log(lx)}, these value(s) cannot
#'   be handled. \code{trunc == TRUE} strips out the zero value(s). An
#'   alternative to this is to transform the zeroes to something approximating
#'   zero (e.g., 1e-7).
#' @param ... Additional variables passed to `mpm_to_lx`, if data are supplied
#'   as a matrix.
#' @return a shape value describing lifespan inequality by comparing the area
#'   under a survival (\code{lx}) curve over age with the area under a constant
#'   (Type II) survival function. The shape value may take any real value
#'   between -0.5 and +0.5. A value of 0 indicates negligible ageing (neither
#'   generally increasing nor generally decreasing survival with age); negative
#'   values indicate negative senescence (generally increasing survival with
#'   age); positive values indicate senescence (generally decreasing survival
#'   with age). A value of +0.5 indicates that all individuals die at age of
#'   maximum longevity; a value of -0.5 indicates that (hypothetically) all
#'   individuals die at birth.
#'
#' @author Iain Stott <iainmstott@@gmail.com>
#'
#' @references Wrycza, T.F. and Baudisch, A., 2014. The pace of aging: Intrinsic
#'   time scales in demography. Demographic Research, 30, pp.1571-1590.
#'   <doi:10.4054/DemRes.2014.30.57>
#'
#'   Baudisch, A. 2011, The pace and shape of ageing. Methods in Ecology and
#'   Evolution, 2: 375-382. <doi:10.1111/j.2041-210X.2010.00087.x>
#'
#'   Baudisch, A, Stott, I. 2019. A pace and shape perspective on fertility.
#'   Methods Ecol Evol. 10: 1941– 1951.
#'   <doi:10.1111/2041-210X.13289>
#'
#' @family life history traits
#'
#' @examples
#' # exponential decline in lx yields shape = 0
#' lx <- 0.7^(0:20)
#' shape_surv(lx)
#'
#' data(mpm1)
#' shape_surv(mpm1$matU)
#'
#' lx <- mpm_to_lx(mpm1$matU, start = 1)
#' shape_surv(lx)
#'
#' @export shape_surv
shape_surv <- function(surv, xmin = NULL, xmax = NULL, trunc = FALSE, ...) {
  if (inherits(surv, "matrix")) {
    surv <- mpm_to_lx(surv, ...)
  }

  if (inherits(surv, "numeric")) {
    lx <- surv
    x <- seq_along(lx) - 1
    if (lx[1] != 1) {
      stop("if `x` isn't given, `lx` must start with 1 as `x[1]` is assumed to
           be 0.\n")
    }
  }
  if (inherits(surv, c("list", "data.frame"))) {
    if (!all(c("x", "lx") %in% names(surv))) {
      stop("`surv` doesn't contain both `x` and `lx`.\n")
    }
    x <- surv$x
    lx <- surv$lx
    if (length(x) != length(lx)) {
      stop("`x` and `lx` must be the same length")
    }
    if ((x[1] %in% 0) && !(lx[1] %in% 1)) {
      stop("`lx` must start with `1` where `x[1]` is `0`.\n")
    }
  }
  if (!trunc) {
    if (any(lx %in% 0)) {
      stop(strwrap(prefix = " ", initial = "", "`lx` cannot be zero (we
      calculate the `log`).
      Consider `trunc = TRUE`, or transforming zero values. See `?shape_surv`
      for more details.\n"))
    }
  }
  if (trunc) {
    x <- x[lx > 0]
    lx <- lx[lx > 0]
  }
  if (is.null(xmin)) xmin <- min(x)
  if (is.null(xmax)) xmax <- max(x)
  if (any(diff(x) <= 0)) stop("much as we'd like to reverse ageing,
                              x must all be ascending.\n")
  if (any(diff(lx) > 1e-7)) stop("please don't bring people back
                                 from the dead (check lx).\n")
  x_sub <- x[x >= xmin & x <= xmax]
  if (length(x_sub) <= 2) {
    stop("must have > 2 nonzero values of lx to calculate shape.\n")
  }
  lx_sub <- lx[x >= xmin & x <= xmax]
  lx_log <- log(lx_sub)
  xStd <- (x_sub - xmin) / (xmax - xmin)
  lxmin <- lx_log[which.min(xStd)]
  lxmax <- lx_log[which.max(xStd)]
  lxStd <- (lx_log - lxmin) / (lxmax - lxmin)
  aucStd <- area_under_curve(xStd, lxStd)
  aucFlat <- 0.5
  shape <- aucFlat - aucStd
  shape
}

test_that("entropy_k works correctly", {
  lx <- 0.8^(0:20)

  x <- entropy_k(lx)
  x_trap <- entropy_k(lx, trapeze = TRUE)

  expect_length(x, 1L)
  expect_length(x_trap, 1L)
  expect_gte(x, 0)
  expect_gte(x_trap, 0)
})

test_that("entropy_k warns and fails gracefully", {
  lx1 <- c(1.1, 0.6, 0.5, 0.4) # lx is greater than 1
  expect_error(entropy_k(lx1))

  lx2 <- c(1.0, 0.6, 0.61, 0.5) # lx not monotonically declining
  expect_error(entropy_k(lx2))
})

complete -c rage -l max-work-factor -d 'Maximum work factor to allow for passphrase decryption.' -r
complete -c rage -s r -l recipient -d 'Encrypt to the specified RECIPIENT. May be repeated.' -r
complete -c rage -s R -l recipients-file -d 'Encrypt to the recipients listed at PATH. May be repeated.' -r -F
complete -c rage -s i -l identity -d 'Use the identity file at IDENTITY. May be repeated.' -r -F
complete -c rage -s j -d 'Use age-plugin-PLUGIN-NAME in its default mode as an identity.' -r
complete -c rage -s o -l output -d 'Write the result to the file at path OUTPUT.' -r -F
complete -c rage -s h -l help -d 'Print this help message and exit.'
complete -c rage -s V -l version -d 'Print version info and exit.'
complete -c rage -s e -l encrypt -d 'Encrypt the input (the default).'
complete -c rage -s d -l decrypt -d 'Decrypt the input.'
complete -c rage -s p -l passphrase -d 'Encrypt with a passphrase instead of recipients.'
complete -c rage -s a -l armor -d 'Encrypt to a PEM encoded format.'

complete -c rage-keygen -s o -l output -d 'Write the result to the file at path OUTPUT. Defaults to standard output.' -r -f -a "(__fish_complete_directories)"
complete -c rage-keygen -s h -l help -d 'Print this help message and exit.'
complete -c rage-keygen -s V -l version -d 'Print version info and exit.'
complete -c rage-keygen -s y -d 'Convert an identity file to a recipients file.'

---
title: "Deriving life history traits from an MPM"
author: 
 - Patrick Barks
 - Judy Che-Castaldo
 - Rob Salguero-Gomez
 - Others
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Deriving life history traits from an MPM}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
editor_options: 
  chunk_output_type: console
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```



## Introduction

The aim of this vignette is to provide further details on the functions in `Rage` for estimating life history traits and on the traits themselves. Life history describes the sequence and pattern of the key events in an organism's life cycle, pertaining to the schedule of survival, development, and reproduction. By aggregating individual-level demographic rates into an MPM, we can calculate a set of life history traits to describe the expected patterns for individuals within the population. These calculations follow methods from Caswell (2001) and Morris & Doak (2003).



## Loading MPMs and basic anatomy

We will start by loading the `Rage` package and an example MPM included in the package called `mpm1`, which we'll be using throughout this vignette.

```{r}
library(Rage) # load Rage
data(mpm1) # load data object 'mpm1'
mpm1 # display the contents
```

Although we demonstrate these trait calculations using a single MPM, each function can be applied to a set of MPMs for comparative analysis of life histories across species and populations. 



## Survival and lifespan traits

Two life history traits pertaining to survival that can be estimated using functions in `Rage` are mean life expectancy and longevity. Assuming that there is no trade-off between survival and reproduction, these functions do not require information about reproduction. Instead, they only require a __U__ matrix (supplied to the functions' `matU` argument).

The function `life_expect_mean()` estimates mean life expectancy, or the mean time to death, and the variance in time to death can be obtained using `life_expect_var()`.

```{r}
# mean life expectancy from "seed" stage
life_expect_mean(matU = mpm1$matU, start = 1)

# variance in life expectancy from "seed" stage
life_expect_var(matU = mpm1$matU, start = 1)
```

Life expectancy is dependent on a starting stage, which is specified using the `start` argument. That is, the expected time to death will be different for an individual that is currently in the first stage (i.e., `start = 1`) than if it is in a later stage along its life cycle. This is because the latter assumes the individual has survived to that stage, and it has already lived a portion of its lifespan. In the example MPM, life expectancy from the first stage ("seed") is shorter than that from the second stage ("small") due to the relatively low probability of survival in the seed stage, reducing the expected time to death.   

```{r}
# mean life expectancy from "small" stage
life_expect_mean(matU = mpm1$matU, start = 2)
```


In certain situations, it is useful to compute life expectancy while taking into account that there can be multiple initial stage classes rather than a single stage from which every individual commences their life cycle. This need occurs in scenarios where individuals can enter a life stage, such as reaching reproductive maturity, from more than one possible stage. To accommodate these instances, the `life_expect_mean` function supports multiple initial stage classes. In these scenarios, results should be averaged, with weights based on the distribution of individuals among the potential starting stages. This is achieved by setting the `start` parameter to `NULL`, which signifies an unspecified starting stage, and employing the `mixdist` parameter.

The `mixdist` parameter is expected to be a vector reflecting the proportion of individuals initiating reproduction at each stage within the Matrix Population Model (MPM). By inputting this distribution, the function computes the life expectancy by weighting the contribution from for each initial stage accordingly.

```{r}
life_expect_mean(matU = mpm1$matU, start = NULL, mixdist = c(0, 0.4, 0.6, 0, 0))
```


The function `longevity()` estimates the time to which the survivorship of a cohort falls below a user-defined critical threshold, specified as a value (between 0 and 1) supplied to the `lx_crit` argument. The specifications regarding the `start` argument for `life_expect()` also applies to this function. Using the example MPM, the post-germination years until survivorship falls to below 5% would be calculated as:

```{r}
longevity(matU = mpm1$matU, start = 2, lx_crit = 0.05)
```


Similarly to the mean life expectancy function, here one too can examine how longevity differs depending on the starting stage. With the example MPM, longevity increases from 2 years to 7 years after an individual germinates. Longevity is highest for the "medium" and "large" stages, but is the same for individuals starting in the "small" stage as those starting in the "dormant" stage.  

```{r}
longval <- NULL
startvec <- seq_len(nrow(mpm1$matU)) # vector of starting stages

for (i in startvec) {
  longval[i] <- longevity(matU = mpm1$matU, start = startvec[i], lx_crit = 0.05)
}

plot(longval,
  type = "l", xlab = "Starting stage",
  ylab = "Longevity to 5% survivorship"
)
longval
```

## Reproduction and maturation traits

In addition how long individuals can be expected to live, species and populations also differ in how quickly and often they reproduce, and how many offspring are produced and recruited into the population. Several related traits measure these various aspects of reproduction, and they require information contained in the __F__ submatrix as well as the __U__ submatrix.

The net reproductive rate, *R<sub>0</sub>* is the cumulative number of offspring that a newborn individual will produce over its lifetime. 

```{r}
net_repro_rate(matU = mpm1$matU, matR = mpm1$matF)
```

This life history trait is also called the the per-generation growth rate, because if an individual produces more than one offspring, it more than replaces itself and so the population will grow. Note that this growth rate is different than the population growth rate $\lambda because the units of *R<sub>0</sub>* are in generations rather than absolute time (i.e., the time step of the transition matrix, often 1 year). The generation time (*T*) can thus be defined as the amount of time it takes for a population to grow by a factor of *R<sub>0</sub>*, which can be obtained by:

```{r}
gen_time(matU = mpm1$matU, matR = mpm1$matF)
```

Generations may be bounded using different criteria, leading to estimates of *T* that are identical only for organisms that reproduce once at a uniform age. Alternative methods for calculating the generation time can be selected using the `method` argument. Currently supported are (*i*) the average age difference between parents and offspring (`method = "age_diff"`) and (*ii*) the expected age at which members of a cohort reproduce (`method = "cohort"`).

The age at reproductive maturity is the average amount of time an individual will take to enter a reproductive stage for the first time in the population. Similar to life expectancy, mean time to reproduction will depend on the starting stage, again specified using the `start` argument. In the case of the example MPM where the first stage is "seed", one may be more interested in estimating the time to reproduction for individuals that have already germinated (i.e., `start = 2`).

```{r}
mature_age(matU = mpm1$matU, matR = mpm1$matF, start = 2)
```


The probability of reaching reproductive maturity before death, for individuals that have already become established (e.g. germinated in the case of plants in this particular example), is:

```{r}
mature_prob(matU = mpm1$matU, matR = mpm1$matF, start = 2)
```

As discussed above, there may be multiple stages in which an individual can first reproduce. One can also use an MPM to estimate the proportion of individuals that would be expected to first reproduce in each of the reproductive stage classes. For this, we need to specify which stages are reproductive in the `repro_stages` argument. This can be determined from the __F__ matrix based on the columns that have non-zero values, or a non-zero probability of producing offspring.

```{r}
mpm1$matF # We see that the "medium" and "large" stages are reproductive

maturedist <- mature_distrib(
  matU = mpm1$matU, start = 1L,
  repro_stages = c(FALSE, FALSE, TRUE, TRUE, FALSE)
)
maturedist
```

One can then use this distribution to update the estimate of mean life expectancy from reproductive maturity, using the data-based distribution rather than the hypothetical one (40% in "small" stage and 60% in "medium" stage) used above:

```{r}
# mean life expectancy from maturity
life_expect_mean(matU = mpm1$matU, start = NULL, mixdist = c(maturedist))

# mean life expectancy from "small" stage
life_expect_mean(matU = mpm1$matU, start = 2)
```

In doing so, mean life expectancy from maturity is 0.67 years longer than that after germination (from "small" stage). In this case, this difference is due to the higher average survival once an individual reaches reproductive maturity.

## Life table component traits

Other life history traits are calculated from a life table rather than an MPM, in which case one can first use the `mpm_to_` group of functions to derive the necessary life table components: age-specific survivorship (*lx*), survival probability (*px*), mortality hazard (*hx*), and reproduction (*mx*). As above, start stage can too be specified as a specific stage (by number or by name), as well as a vector of proportions in each stage. One can calculate the age-specific rates for post-germination individuals using the example MPM:


```{r}
lx <- mpm_to_lx(matU = mpm1$matU, start = "small")
px <- mpm_to_px(matU = mpm1$matU, start = "small")
hx <- mpm_to_hx(matU = mpm1$matU, start = "small")
mx <- mpm_to_mx(matU = mpm1$matU, matR = mpm1$matF, start = "small")
```


There are multiple applications for these life table estimates. For example, one can visualize the difference in survival curves for individuals starting from the seed stage compared to survival post-germination. 

```{r}
lx_seed <- mpm_to_lx(matU = mpm1$matU, start = "seed")

plot(lx,
  xlab = "Survival time (years)", ylab = "Survivorship",
  type = "s", col = "black"
)
lines(lx_seed, type = "s", col = "orange")
legend("topright",
  inset = c(0.05, 0.05), c("From seed stage", "From small stage"),
  lty = c(1, 1), col = c("orange", "black")
)
```

In this case, there is a large drop in survival (and thus high mortality) in the first year for seeds. Specifically, more than 80% of seeds are not expected to survive their first year, compared to germinated individuals that have a 50% chance of surviving their first year.

Finally, one can use these life table components to calculate additional life history traits related to the types of survival and reproduction trajectories.

Demetrius' entropy is a measure of how reproductive episodes are spread across the lifespan (Demetrius & Gundlach 2014). A value of 0 indicates only one reproduction event in the entire life time (also known as semelparity), and larger values indicate a more even distribution of reproductive events over the life cycle. 

Keyfitz' entropy describes the shape of the age-specific survivorship curve (Demetrius 1978). Values greater than 1 correspond to survival curves type I (high survival in early part of life and decreasing over time), a value of 1 to survival curve type II (constant survival), and values less than 1 to survival curve type III (low survival early and increasing over time) (Salguero-Gómez et al. 2016). 

These can be calculated from the age-trajectories, or from the matrices directly (in this case, the age-trajectories are calculated internally).

Recently, de Vries et al. (2023) highlighted a problem in the calculations of Keyfitz' entropy for discrete age trajectories. Following this revelation it is recommended to use the function `entropy_k_age` to calculate entropy for age-based (Leslie) matrices and `entropy_k_stage` for stage-based Lefkovitch matrices. 

```{r}
entropy_d(lx, mx) # Demetrius' entropy
entropy_d(lx = mpm1$matU, mx = mpm1$matF, start = "small") # Demetrius' entropy

life_elas(lx) # Keyfitz' entropy
life_elas(lx = mpm1$matU, start = "small") # Keyfitz' entropy

entropy_k_stage(mpm1$matU)

data(leslie_mpm1) # load leslie matrix
entropy_k_age(leslie_mpm1$matU)
```

It is important to note that both of these entropy measures may produce unexpected results if only partial survivorship and fecundity age-based trajectories are used in the calculations. Indeed, both metrics of demographic entropy are sensitive to the length of the those trajectory vectors. For example, the Demetrius' entropy calculated above has a negative value due to being calculated from age-specific survivorships and fecundities starting from the "small" stage, and therefore it does not include the full life cycle. 

A more robust way to describe these trajectories is using the area under the curve method. 

## Shape of mortality and fecundity

A life course can be described in terms of its pace (timing of events) and its shape (the distribution of events, either for an individual or for a population). There are several candidate metrics for the pace and shape of mortality, and of fertility. A good measure of the pace of mortality is life expectancy (the average age of death) and a good measure of the shape of mortality are the life-table entropy measures mentioned above, which describe both the survivorship curve and the mortality trajectory (see Baudisch 2011). Measures describing the pace and shape of fertility have also been developed (Baudisch & Stott 2019). In addition to the entropy measures described above, the `shape_` functions are also useful metrics. 

```{r}
# shape of survival/mortality trajectory
shape_surv(lx)

# shape of fecundity trajectory
shape_rep(mx)
```

## References
Baudisch, A. 2011. The pace and shape of ageing. Methods in Ecology and Evolution, 2(4), 375– 382. <doi:10.1111/j.2041-210X.2010.00087.x>

Baudisch, A, Stott, I. 2019. A pace and shape perspective on fertility. Methods Ecol Evol. 10: 1941– 1951. <doi:10.1111/2041-210X.13289>

Caswell, H. 2001. Matrix Population Models: Construction, Analysis, and Interpretation. 2nd edition. Sinauer Associates, Sunderland, MA. ISBN-10: 0878930965

Demetrius, L. 1978. Adaptive value, entropy and survivorship curves. Nature 275: 213-214.

Demetrius, L., & Gundlach, V. M. 2014. Directionality theory and the entropic principle of natural selection. Entropy 16: 5428-5522.

de Vries, C., Bernard, C., & Salguero-Gómez, R. 2023. Discretising Keyfitz' entropy for studies of actuarial senescence and comparative demography. Methods in Ecology and Evolution, 14, 1312–1319. <doi:10.1111/2041-210X.14083>

Morris, W. F. & Doak, D. F. 2003. Quantitative Conservation Biology: Theory and Practice of Population Viability Analysis. Sinauer Associates, Sunderland, MA. ISBN-10: 0878935460

Salguero-Gómez, R., Jones, O. R., Jongejans, E., Blomberg, S. P., Hodgson, D. J., Mbeau-Ache, C., Zuidema, P. A., de Kroon, H., & Buckley, Y. M. 2016. Fast–Slow continuum and reproductive strategies structure plant life-history variation worldwide. Proceedings of the National Academy of Sciences 113 (1): 230–35

---
title: "Age-from-stage analyses"
author: "Patrick Barks"
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    toc: true
    depth: 2
vignette: >
  %\VignetteIndexEntry{Age-from-stage analyses}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```



## Introduction

Regardless of whether the stage classes of a matrix population model (MPM) are based on age, size, and/or ontogeny, it's possible to obtain age-specific schedules of survivorship (lx) and reproduction (mx) using 'age-from-stage' methods, as described by Caswell (2001).

## Preliminaries

We'll start by loading a few packages and a dataset that we'll be using throughout this vignette. 

```{r}
library(Rage) # load Rage
data(mpm1) # load data object 'mpm1'
mpm1 # display the contents
```

## Age-from-stage methods with a single matrix population model

This MPM has 5 stage class, and it's apparent from the `dimnames` attribute that the stages are not based solely on age. Nonetheless, we can estimate age-schedules of survivorship and reproduction using the functions `mpm_to_lx()` and `mpm_to_mx()` from `Rage`.

```{r}
dimnames(mpm1$matU)

# extract U and F matrices
mat_U <- mpm1$matU
mat_F <- mpm1$matF

# calculate lx
lx <- mpm_to_lx(mat_U, start = 1, xmax = 30)

# calculate mx
mx <- mpm_to_mx(mat_U, mat_F, start = 1, xmax = 30)
```

In addition to the relevant matrix components, the `mpm_to_*` functions require two extra arguments. The first, `start`, is an integer indicating which stage reflects the 'start of life'. Usually this will be `1`, but sometimes we might want to skip over stages that are propagule (i.e. seed) or dormant. The MPM we selected begins with a seed stage, so we may want to start from the second stage, corresponding to `start = 2`. The second argument, `N` is the number of time steps to calculate over.

```{r}
# calculate lx
lx <- mpm_to_lx(mat_U, start = 1, xmax = 30)

# calculate mx
mx <- mpm_to_mx(mat_U, mat_F, start = 1, xmax = 30)
```

Let's take a look at the trajectories.

```{r, fig.width = 6, fig.height = 4}
plot(lx, ylim = c(0, 1), type = "l", xlab = "Age")
plot(mx, type = "l", xlab = "Age")
```

## Extending to many matrix population models

Now we'll extend the basic approach above to many models. Specifically, we'll examine trajectories of survivorship for all of the tree species in `Compadre`. `Compadre` is a database of matrix population models. You can read more about it [here](https://compadre-db.org/), and the associated _R_ package [here](https://jonesor.github.io/Rcompadre/). The `Rcompadre` package contains a subset of the database that we can use to demonstrate computations on many models simultaneously.

First, we'll subset `Compadre` to our group of interest (`OrganismType == "Tree"`). We'll also remove matrices with missing values, and limit our selection to matrices with a periodicity (i.e. transition interval) of 1 year.

```{r}
library(Rcompadre)
data(Compadre)

# In older versions of Com(p)adre the ProjectionInterval column was called
# AnnualPeriodicity.
if ("AnnualPeriodicity" %in% names(Compadre)) {
  Compadre$ProjectionInterval <- Compadre$AnnualPeriodicity
}

comp_flag <- cdb_flag(Compadre, "check_NA_U")

comp_use <- subset(comp_flag, OrganismType == "Tree" &
  check_NA_U == FALSE &
  ProjectionInterval == 1)
```

Let's take a look at the species/populations that made the cut.

```{r}
CompadreData(comp_use)[, c(
  "SpeciesAccepted", "MatrixPopulation",
  "MatrixTreatment"
)]
```

Notice that there are 3 matrices for the species _Phyllanthus indofischeri_, reflecting different treatment groups. Let's collapse these replicates down to a single matrix per species, by averaging the relevant MPMs using `cdb_collapse()`. We'll also use the function `cdb_id_stages()`, to make sure we're only collapsing matrices that have the same stage class definitions.

```{r}
# add column ID-ing matrices with same MatrixClassAuthor vector
comp_use$stage_id <- cdb_id_stages(comp_use)

# collapse database to single matrix per species * MatrixClassAuthor
comp_collapse <- cdb_collapse(comp_use, "stage_id")

# check species/populations again
CompadreData(comp_collapse)[, c(
  "SpeciesAccepted", "MatrixPopulation",
  "MatrixTreatment"
)]
```

Next, let's look at the organized stage classes for each MPM. If any of our MPMs include propagule or dormant stage classes, we may want to account for them when calculating lx.

```{r}
MatrixClassOrganized(comp_collapse)
```

Indeed, 1 MPM includes a propagule stage. So let's use the function `mpm_first_active()` to determine the first 'active' stage class for each MPM, which we'll use to define the start of life.

```{r}
comp_collapse$start_life <- mpm_first_active(comp_collapse)
```

Finally, we'll use `lapply()` to apply the function `mpm_to_lx` to each row of `comp_collapse`. By default, `lapply()` will return a vector for each row, and the length of which is `xmax`. We can convert the output to an matrix, with columns representing each row from `comp_collapse` using `do.call`. After that, we'll use the function `matplot()` to plot age-trajectories of survivorship for each species.

```{r, fig.width = 6, fig.height = 4}
lx_list <- lapply(seq_len(nrow(comp_collapse)),
  function(x, comp_collapse) {
    U <- matU(comp_collapse$mat[[x]])

    rownames(U) <- colnames(U) # ensure row and col names are present

    mpm_to_lx(
      matU = U,
      start = comp_collapse$start_life[x],
      xmax = 40
    )
  },
  comp_collapse = comp_collapse
)

lx_array <- do.call(cbind, lx_list)

matplot(lx_array,
  type = "l", lty = 1, log = "y", ylim = c(0.0001, 1),
  lwd = 1.5, xlab = "Age (years)", ylab = "lx"
)
```

## References

Caswell, H. (2001). Matrix Population Models: Construction, Analysis, and Interpretation. 2nd edition. Sinauer Associates, Sunderland, MA. ISBN-10: 0878930965

---
title: "Suggested quality control"
author: "Owen Jones"
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    toc: true
    depth: 2
vignette: >
  %\VignetteIndexEntry{Suggested quality control}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```



# Introduction

The specific requirements and assumptions for each function in `Rage` varies. Here we provide notes giving an overview of these requirements, which are also applicable to other functions/calculations in packages such as `popbio` and `popdemo`. We make some suggestions of how users should filter their dataset before analysis. To assist with this, the function `Rcompadre::cdb_flag` conducts a series of checks on the matrices in a `compadreDB` object and adds "flags" to facilitate the filtering out of problematic matrices. This function (`cdb_flag`) can automatically be run by `Rcompadre::cdb_fetch` using the argument `flag = TRUE`.

# Types of issue

## Missing data

The most obvious requirement for most of the `Rage` methods is that missing (`NA`) values in matrices prevent calculations using those matrices. Sometimes these `NA` values are in one of the submatrices (i.e., **U**, **F** or **C**) of the matrix model, but other submatrices are complete. For example, there may be `NA` entries in the **F** submatrix, while the **U** matrix remains complete. These issues are flagged with the columns `check_NA_A`,  `check_NA_U`,  `check_NA_F` and  `check_NA_C`.


## Excessive zeros

Submatrices composed entirely of zero values can also be problematic. There may be good biological reasons for this phenomenon. Species that do not reproduce clonally will have zero-value **C** matrices. Another biologically reasonable explanation could be that in the particular focal population in the particular focal year, there was no sexual reproduction recorded, so the **F** matrix was composed entirely of zeros. Nevertheless, zero-value submatrices can cause some calculations to fail and it may be necessary to exclude them. These issues are flagged with the columns `check_zero_F`, `check_zero_C`, `check_zero_U`. 


## Excessive survival

In a biologically reasonable matrix population model, the set of survival and growth transitions (i.e., in the **U** matrix) from a particular stage cannot exceed 1. However, in some cases, errors in the original matrices (including data entry and rounding errors) cause this situation to occur and may persist in the data set. We can check for this error using column sums of the **U** matrix, and may wish to exclude matrices with any column sum greater than 1.  
This issue is examined using the column `SurvivalIssue` which gives the maximum value of the column sums for `matU` . An additional column `check_surv_gte_1` (produced with `cdb_flag`) reports whether any **single** value is greater than or equal to 1.

## Excessive mortality

At the opposite end of the survival spectrum, there may be some matrices where some of the column sums of the **U** matrix are zero, implying that there is no survival from that particular stage. This may be a perfectly valid parameterisation for a particular year/place but is biologically unreasonable in the longer term and users may wish to exclude problematic matrices from their analysis. This issue is indicated by the column `check_zero_U_colsum`.


## Irreducibility and ergodicity 

Several matrix manipulations or calculations require that the MPM (`matA`) be irreducible and ergodic (Stott et al. 2018). Irreducible MPMs are those where parameterised transition rates facilitate pathways from all stages to all other stages. Conversely, reducible MPMs depict incomplete life cycles where pathways from all stages to every other stage are not possible. Ergodic MPMs are those where there is a *single* asymptotic stable state that does not depend on initial stage structure. Conversely, non-ergodic MPMs are those where there are multiple asymptotic stable states, which depend on initial stage structure. MPMs that are reducible and/or non-ergodic are usually biologically unreasonable, both in terms of their life cycle description and their projected dynamics. They cause some calculations in `Rage` (and elsewhere) to fail. Irreducibility is necessary but not sufficient for ergodicity.  These issues are flagged with `check_irreducible` and `check_ergodic`. Even if  `Rage` functions do not fail due to these issues, the fact that they can indicate biologically unreasonable life cycles may mean that users nevertheless wish to exclude reducible, non-ergodic matrices from their analyses.

## Singularity of the U matrix

Matrices are said to be singular if they cannot be inverted. Inversion is required for many matrix calculations and, therefore, singularity can cause some calculations to fail. This issue is flagged with `check_singular_U`. Calculations for `longevity`, `life_expect_mean`, `life_expect_var` and `net_repro_rate` fail with singular matrices, so users may wish to exclude singular matrices when conducting analyses using these functions.

## Matrix split errors

A complete MPM (**A**) can be split into its component submatrices (i.e. **U**, **F** and **C**). The sum of these submatrices should equal the complete MPM (i.e. **A** = **U** + **F** + **C**). Sometimes, however, errors occur so that the submatrices do NOT sum to **A**. Normally, this is caused by rounding errors, but more significant errors are possible. This problem is flagged with `check_component_sum` (only relevant for divided (split) matrices). We recommend that users carefully check their matrices for these errors and correct or exclude them as appropriate.


# Function requirement summaries

It is a general requirement for almost all `Rage` functions that the matrices used as arguments do not include `NA` values. With divided (split) matrices, `NA` values may be present in some submatrices but not others. For example, the **U** matrix may be complete, but the **F** matrix may have `NA` values. In this case, functions that require an **F** matrix will fail, while those that only require a **U** matrix will work.  Users should filter the data to exclude entries with `NA` values in the matrices required for their analysis. The functions `mpm_split`, `mpm_rearrange` and `mpm_standardise` do not require complete `NA`-free matrices.

For functions that use the **U** matrix, we further suggest filtering the data to exclude the biologically unreasonable entries where one or more of the `matU` columns sum to zero, or to greater than 1 (see *Excessive Survival*, above). Alternatively, users could examine the offending matrices and make sensible corrections (e.g. to correct rounding errors).

For functions that use the **F** matrix, and where sexual reproduction is known to occur in the species, we suggest that users consider filtering the data to exclude entries where **F** is entirely zero. This is not always desirable because there are some situations where zero recorded reproduction is biologically reasonable. We suggest a similar approach for the **C** matrix.

# Other issues

When using age-from-stage methods, users should be aware of the issue of convergence to quasi-stationary distribution (see \code{\link{qsd_converge}}). Briefly, All age-from-stage calculations produce age-trajectories that inevitably asymptote as a mathematical consequence of describing the vital rates as functions of discrete stages (Horvitz & Tuljapurkar, 2008). This mathematical artefact can introduce bias into measures obtained using age-from-stage methods. `Rage` provides a convenient and principled way of correcting for this artefact by imposing a lower probability threshold defined by the degree of convergence to the quasi-stationary distribution (see \code{\link{qsd_converge}}). We suggest that users filter out from their analyses matrices that do not pass this threshold criterion.

Users should also be aware of the issue of census type. For populations that reproduce in a pulse once per year. The demographic census may be carried out before or after the reproduction event. There are thus two types of census: Pre- and post-reproductive census. This distinction has potentially important implications for demographic measures because of its effects on measured population structure. For example, the fraction of individuals in the first age class will tend to be larger larger in a post-reproductive census than a pre-reproductive census. There is a column in the com(p)adre metadata (`CensusType`) that is intended to record this information but, because authors of source publications have rarely clearly stated this information, it is very incomplete. For serious analyses we therefore recommend that users carefully collect this information themselves from the source papers.

# Finally...

Although we highlight here a range of issues that could cause problems for MPM analyses we have likely inadvertently omitted some issues. We therefore urge users to carefully consider issues that may pertain to their particular analyses. 

# References

Horvitz, C. C., & Tuljapurkar, S. (2008). Stage dynamics, period survival, and mortality plateaus. The American Naturalist, 172(2), 203–215.

Stott, I., Townley, S., & Carslake, D. (2010). On reducibility and ergodicity of population projection matrix models. Methods in Ecology and Evolution. 1 (3), 242-252

---
title: "Getting started with Rage"
author:
  - Patrick Barks
  - William K. Petry
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    toc: true
    depth: 2
vignette: >
  %\VignetteIndexEntry{Getting started with Rage}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```


# Overview

`Rage` provides tools for manipulating and analysing matrix population models (MPMs). This vignette introduces the structure of the input and highlights key analysis functions across the package.

# Recap: Assumed knowledge
The functions in `Rage` assume that the reader is familiar with the basic structure of MPMs and how they are used to project population change and basic demographic statistics (e.g., the equilibrium population growth rate, $\lambda$). This section is intended as a recap of these concepts--readers ready to begin calculating more advanced life history metrics may want to skip ahead to [the next section](#mpms_in_r). We refer readers to [Caswell (2001)](#references) for a comprehensive introduction to MPMs.

## R.1. Basic anatomy of life cycles and MPMs
Matrix population models are a mathematical tool that integrate population dynamics and population structure to model the dynamics of populations with different 'types' of individuals, whether due to age, life stage, sex, genotype or any other attribute that causes differences in demographic rates. Consider a plant species with three different types of individuals: **seedlings** that have newly recruited into the population from seeds, non-reproductive **rosettes**, and **flowering** adults that make seeds and also reproduce asexually by vegetative budding. We can represent the biology of this life cycle graphically (Fig. 1) where each life stage is a node and the life stages are connected with arrows showing the paths (and probabilities) that individuals move between stages.

```{r,echo=FALSE,fig.align='center',fig.height=2,fig.width=6}
# hidden code to produce figures
library(DiagrammeR)
matA <- rbind(
  c(0.0, 0.0, 3.2),
  c(0.5, 0.3, 0.8),
  c(0.0, 0.4, 0.9)
)
stages <- c("seedling", "rosette", "flowering")
title <- NULL
graph <- expand.grid(to = stages, from = stages)
graph$trans <- round(c(matA), 3)
graph <- graph[graph$trans > 0, ]
nodes <- paste(paste0("'", stages, "'"), collapse = "; ")
graph$min_len <- (as.numeric(graph$to) - as.numeric(graph$from)) * 3
graph$col <- c(
  "PaleGreen4", "PaleGreen4", "PaleGreen4", "Goldenrod1",
  "MediumOrchid4", "PaleGreen4"
)
edges <- paste0("'", graph$from, "'", " -> ", "'", graph$to, "'",
  "[minlen=", graph$min_len,
  ",fontsize=", 10,
  ",color=", graph$col,
  ",xlabel=", paste("\"", graph$trans),
  "\"]\n",
  collapse = ""
)
grViz(
  paste(
    "
digraph {
  {
    graph[overlap=false];
    rank=same;
    node [shape=", "egg", ", fontsize=", 12, "];",
    nodes, "
  }",
    "ordering=out
  x [style=invis]
  x -> {", nodes, "} [style=invis]", edges,
    "labelloc=\"t\";
  label=\"", title, "\"
}"
  )
)
```
**Figure 1.** Life cycle diagram for a hypothetical plant with three life stages (nodes). Arrows are coloured by conventional groupings of demographic processes: growth and survival (green), sexual reproduction via seeds (gold) and asexual reproduction via budding (purple). The numbers along each arrow indicate the transition probability--how many individuals will there be in the stage at the arrow's end at the next time step for each individual at the arrow's start?

The **transition** or **projection matrix** (Fig. 2; called __A__ in matrix notation) for this life cycle is a square matrix where the rows and columns correspond to the life stages (nodes) and the elements correspond to transition probabilities (arrows). By convention, the columns reflect the current life stage (i.e., the arrow's start) and the rows reflect the arrow's end.

```{r,echo=FALSE,fig.align='center',fig.height=4,fig.width=4}
library(ggplot2)
ggdat <- merge(graph,
  expand.grid(to = stages, from = stages),
  by = c("to", "from"),
  all.x = TRUE, all.y = TRUE
)
ggdat$trans[is.na(ggdat$trans)] <- 0
ggdat$col[is.na(ggdat$col)] <- "transparent"
ggdat$to <- factor(ggdat$to, levels = c("flowering", "rosette", "seedling"))
ggdat$from <- factor(ggdat$from, levels = c("seedling", "rosette", "flowering"))
ggplot(ggdat, aes(x = from, y = to, label = trans)) +
  geom_tile(color = "black", fill = "white", linewidth = 0.25, show.legend = FALSE) +
  geom_text(size = 6) +
  scale_x_discrete(position = "top") +
  labs(x = "current life stage", y = "life stage at time t+1") +
  coord_equal(expand = FALSE) +
  theme_bw(base_size = 18) +
  theme(panel.border = element_blank())
```
**Figure 2.** Transition (projection) matrix corresponding to the life cycle in Fig. 1.

For basic MPM projection and equilibrium analyses, this single matrix is all that is needed. However, more advanced analyses like many of those in `Rage` use decompositions of the __A__ matrix into its components of growth and survival (__U__) and reproduction (__R__). Reproduction can be further decomposed into offspring produced sexually (__F__) and asexually/clonally (__C__; Fig. 3). The submatrices combine additively to create the transition matrix,

$$
\begin{aligned}
\mathbf{A} &= \mathbf{U} + \mathbf{R} \\
&= \mathbf{U} + \left(\mathbf{F} + \mathbf{C}\right).
\end{aligned}
$$
In our example, all of the elements of __A__ are found in only one of the submatrices. However, other life cycles may have multiple pathways from one life stage to another.

```{r,echo=FALSE,fig.align='center',fig.height=4,fig.width=9,out.width='100%'}
blankdat <- expand.grid(to = stages, from = stages, trans = 0)
blankdat$to <- factor(blankdat$to, levels = c(
  "flowering", "rosette",
  "seedling"
))
blankdat$from <- factor(blankdat$from,
  levels = c("seedling", "rosette", "flowering")
)
ggdat$col <- factor(ggdat$col,
  levels = c(
    "PaleGreen4", "Goldenrod1",
    "MediumOrchid4", "transparent"
  ),
  labels = c("U", "F", "C", "t")
)
ggplot(
  ggdat[ggdat$col != "t", ],
  aes(x = from, y = to, fill = col, label = trans)
) +
  geom_tile(
    data = blankdat,
    aes(fill = NULL), fill = "white", color = "black", linewidth = 0.25
  ) +
  geom_text(data = blankdat, aes(fill = NULL), size = 6) +
  geom_tile(color = "black", linewidth = 0.25, show.legend = FALSE) +
  geom_text(size = 6) +
  scale_x_discrete(position = "top") +
  scale_fill_manual(values = c(
    "F" = "goldenrod1",
    "C" = "mediumorchid4",
    "U" = "palegreen4",
    "t" = "white"
  )) +
  labs(x = "current life stage", y = "life stage at time t+1") +
  coord_equal(expand = FALSE) +
  facet_wrap(~col, nrow = 1) +
  theme_bw(base_size = 18) +
  theme(
    panel.border = element_blank(),
    strip.text = element_text(face = "bold"),
    strip.placement = "outside"
  )
```
**Figure 3.** Decomposition of the transition matrix, __A__, into __U__, __F__ and __C__ submatrices. The coloured elements correspond to the coloured arrows in Fig. 1.

## R.2. Projecting population change
Simple matrix multiplication allows the full transition matrix, __A__, to be projected forward in time for any starting population. We initiate the model with a *population vector*, $\mathbf{n}_{t=0}$, that contains the number of individuals in each life stage. Right multiplying the initial population vector to the transition matrix, __A__, yields the population vector at the next time step via the projection equation,

$$
\mathbf{n}_{t+1} = \mathbf{A}\mathbf{n}_{t}.
$$
Suppose we initialize the example plant population, $\mathbf{n}_{t=0}$, with five seedlings, 10 rosettes, and 15 flowering individuals. We expect that the population in the next time step will be,

$$
\begin{aligned}
\mathbf{n}_{t=1} &= \mathbf{A}\mathbf{n}_{t=0}, \\
\left[
\begin{matrix}
48.0 \\
17.5 \\
17.5
\end{matrix}
\right]
&=
\left[
\begin{matrix}
0 & 0 & 3.2 \\
0.5 & 0.3 & 0.8 \\
0 & 0.4 & 0.9
\end{matrix}
\right]
\left[
\begin{matrix}
5 \\
10 \\
15
\end{matrix}
\right].
\end{aligned}
$$
Summing the population vectors gives us the total population size ($N_{t=0} =\sum\mathbf{n}_{t=0} = 30$ and $N_{t=1} = 83$). Likewise, we can measure the instantaneous per-capita population growth rate as $N_{t+1}/N_{t} = 83/30 = 2.7\overline{6}$). Recursively substituting the new $\mathbf{n}_{t+1}$ into the projection equation in place of $\mathbf{n}_{t}$ allows us to continue projecting the population dynamics and life stage composition forward in time.

## R.3. Analysis of population equilibrium (eigenanalysis)
Provided that the transition matrix, __A__, meets certain conditions, iterative projections from any starting population vector $\mathbf{n}_{t=0}$ will eventually converge to an equilibrium per-capita population growth rate ($\lambda$) and stable stage distribution ($\mathbf{w}$). We can determine $\lambda$ and $\mathbf{w}$ by finding the dominant eigenvalue and its associated (right) eigenvector, respectively. The `popdemo` package provide functions to do these calculations (the package `popbio` also provides extensive functionality for matrix model analysis):

```{r,message=FALSE}
library(popdemo)

# define the transition matrix, A
A <- rbind(
  c(0.0, 0.0, 3.2),
  c(0.5, 0.3, 0.8),
  c(0.0, 0.4, 0.9)
)

# lambda: equilibrium per-capita population growth rate
popdemo::eigs(A = A, what = "lambda")

# w: stable stage distribution (relative frequencies)
popdemo::eigs(A = A, what = "ss")
```
From $\lambda$, we infer that the population would eventually grow at an equilibrium rate of 51% per time step. Likewise, from $\mathbf{w}$ we infer that nearly half of individuals will be seedlings, about a third will be rosettes and a fifth will be flowering at equilibrium.

Finally, we are reminded that MPM *projections* rely on consistency of the demographic vital rates (i.e., transition matrix elements) over time. Projections tell us what would happen if this strong assumption were to be met. Projections differ radically from *forecasts*, which are designed to predict what will happen. Forecasting models may use MPMs to drive population change, but will also typically require a component that predicts changes in the underlying demographic vital rates. [Keyfitz (1972)](#references) unpacks these differences further, with special emphasis on the value of projection as a window into the fundamental processes that drive the behaviour of populations. Much of the functionality of `Rage` rests on this ethos.

# Representing and loading MPMs in `Rage` {#mpms_in_r}

We'll begin by loading one of the example MPMs included in the `Rage` package, `mpm1`, which can be retrieved using the base 'data' function.

```{r}
library(Rage) # load Rage
data(mpm1) # load data object 'mpm1'
mpm1 # display the contents
```

`mpm1` is a list containing the decomposition of an MPM projection matrix into
its components of growth and survival (__U__) and reproduction (__R__ = __F__ + __C__). Many
`Rage` functions accept one or more of these components in analyses. The
biological meaning of each list element is indicated by its name:

- `matU` is the __U__ matrix, which is the __growth/survival__ component of an MPM,
containing transitions related to progression, stasis and retrogression. This is how individuals move among ages or life stages.
- `matR` is the __R__ matrix, which is the __reproductive__ component of an MPM,
containing transitions due to reproduction; either sexual, clonal, or both.
- `matF` is the __F__ matrix, which is the __sexual reproduction__ or fecundity 
component of the MPM, containing transitions due to sexual reproduction.
- `matC` is the __C__ matrix, which is the __clonal__ component of an MPM,
containing transitions due to clonal reproduction.

When the mode of reproduction is known (e.g., a species only reproduces sexually or the sexual and clonal offspring have been counted separately), `matR` can be substituted with `matF` or `matC`.

For any life history, we can reconstruct the full projection matrix, __A__, by adding together the components. For `mpm1`,  __A__ = __U__ + __F__. A population with separately counted sexual and clonal offspring production would be __A__ = __U__ + __F__ + __C__.

`Rage` functions currently accept MPM components as arguments; therefore, it is not necessary to group the matrix components together in a list. We anticipate that future package releases will define methods that accept objects of classes `matrix`, `CompadreMat`, and `CompadreDB` for compatibility with the [COM(P)ADRE databases](https://compadre-db.org/) and its accessor package [`Rcompadre`](https://github.com/jonesor/Rcompadre).

# Families of `Rage` functions for life history analysis
The functions in `Rage` fall into five broad categories, and are detailed in the subsections below:

```{r, echo=FALSE, message=FALSE, warnings=FALSE, results='asis'}
tabl <- "
| Function category                       | Stand-alone vignette             |
|-----------------------------------------|----------------------------------|
| 1. [Vital rates](#vitalrates)  | [VitalRates](a02_VitalRates.html)     |
| 2. [Life tables](#lifetable) | [AgeFromStage](a04_AgeFromStage.html) |
| 3. [Perturbation analysis](#perturb)    | n/a |
| 4. [Deriving life history traits](#lifehist) | [LifeHistoryTraits](a03_LifeHistoryTraits.html) |
| 5. [Transformation of matrices](#maniptransform) | n/a                                   |
"
cat(tabl)
```

A full list of functions by category is available at the [documentation webpage](https://jonesor.github.io/Rage/reference/index.html).

## 1. Standardized vital rates {#vitalrates}

The elements of an MPM transition matrix (__A__) generally are composites of two
or more vital rates (sometimes called ‘lower-level vital rates’). For example,
the transition from stage 1 to stage 2 (element $a_{21}$) may colloquially be thought
of as 'growth,' but importantly this transition is growth *conditional* on the 
individual surviving. Assuming a post-breeding census design, we can retroactively
partition each transition in MPM submatrices into survival (using the column sums 
of __U__) and one of the following: growth (using the lower triangle of __U__), 
shrinkage (upper triangle of __U__), stasis (diagonal of __U__), dormancy (from 
__U__ with user-indicated dormant stages), fecundity (from __F__) or clonality 
(from __C__).

The `vr_vec_*` family of functions provide a means to calculate lower-level vital 
rates for each life stage in the input matrix or matrices. For example, we can
calculate stage-specific survival and stasis from our example __U__ matrix:

```{r}
vr_vec_survival(matU = mpm1$matU)
vr_vec_stasis(matU = mpm1$matU)
```

Multiplying these two together yields the probability of stasis conditional on
survival, in other words the diagonal of our example __U__ matrix.

```{r}
# product of Pr(survival) and Pr(stasis) yields Pr(stasis|survived)
vr_vec_survival(matU = mpm1$matU) * vr_vec_stasis(matU = mpm1$matU)
diag(mpm1$matU) # equivalent to the diagonal of U matrix
```

`Rage` also provides functions to summarize these vital rates _across_ stage 
classes using the `vr_*` family of functions. These functions return a single value that
is the mean of the corresponding stage-specific vital rate vector from `vr_vec_*`.
Life stages may be excluded, allowing the user to tailor these calculations to 
the life history of the organism and their working definition of each vital rate.
Similarly, custom weights are allowed to control the contributions of life stages 
to the average. This functionality is more fully described in the 
[Vital Rates vignette](a02_VitalRates.html).

```{r}
vr_survival(matU = mpm1$matU, exclude_col = 1) # exclude 'seed' stage
mean(vr_vec_survival(mpm1$matU)[-1]) # equivalent to the mean without 'seed'
```

## 2. Deriving life tables from MPMs for age-from-stage analyses {#lifetable}

Classical demographic analyses were based on age-structured populations, in which 
surviving individuals always progress to the next age class. However, MPMs allow 
for demographic rates to be structured by any number of biological criteria, 
whether stage, sex, size, or condition. In these alternatively-structured models, 
individuals may remain in the same state or even 'regress' to states they had 
previously. Here there is no one-to-one mapping between state and age. Each 
individual still has an age, and understanding their age-specific demographic 
trajectories nevertheless offers a useful window into their life history. `Rage`
provides functions that decompose MPMs into life tables, thereby allowing for a 
wide array of demographic analyses based on age-specific trajectories (described 
below in [section 4](#lifehist)).

Following the age-from-stage methods of [Caswell (2001, ch. 5.3)](#references), 
we can convert an MPM to a full, age-structured life table or any of its component
columns.

```{r}
lt <- mpm_to_table(matU = mpm1$matU, matF = mpm1$matF) # full life table
lt
lx <- mpm_to_lx(matU = mpm1$matU) # survivorship to start of each age class
lx
```

`Rage` also supports translation between the different ways of expressing
age-specific mortality/survival, whether proportional survivorship (`lx`),
survival probability (`px`) or mortality hazard (`hx`), following an `x_to_y`
function name pattern.

```{r}
lx_to_px(lx = lx) # survivorship to survival probability
lx_to_hx(lx = lx) # survivorship to mortality hazard
```

When individuals can remain in the last life stage class, age-specific trajectories
of mortality and fertility can appear to plateau as a cohort asymptotically 
approaches its stable stage distribution (SSD). For these MPMs, we can choose
a quasi-stationary distribution (QSD) of stages that is within an arbitrarily 
close threshold of the SSD. Subsetting age-specific demographic trajectories to 
time points before a cohort reaches the QSD will minimise these plateau artefacts. 
In the example MPM, `mpm1`, a cohort of germinated individuals quickly converges 
towards the SSD:

```{r, warning=FALSE, message=FALSE, fig.align='center', fig.height=5, fig.width=6}
# project a germinated cohort through the U matrix
cohort <- popdemo::project(A = mpm1$matU, vector = c(0, 1, 0, 0, 0), time = 10)
popStructure <- vec(cohort) / rowSums(vec(cohort))

matplot(popStructure,
  type = "l", xlab = "time",
  ylab = "proportion in stage class"
)
```

Using `qsd_converge`, we can find the time at which the cohort reaches the QSD
and subset the life table or any component age-specific trajectory accordingly.

```{r}
# calculate time to QSD from the U matrix of an MPM
(q <- qsd_converge(mat = mpm1$matU, start = "small"))

# subset the life table rows to ages prior to the QSD
lt_preQSD <- lt[1:q, ]

# plot mortality trajectory from the life table subset (blue),
# showing plateau effect if the trajectory (grey) was allowed to continue to the
# QSD (dashed vertical line) and beyond
plot(qx ~ x,
  data = lt, type = "l", col = "darkgrey", ylim = c(0, 1),
  xlab = "age"
)
lines(qx ~ x, data = lt_preQSD, type = "l", col = "blue", lwd = 4)
abline(v = q, lty = "dashed")
```

We refer readers to the [AgeFromStage vignette](a04_AgeFromStage.html) for further
detail on these methods.

## 3. Perturbation analyses {#perturb}

Perturbation analyses of MPMs measure the response of demographic statistics, such 
as the equilibrium per-capita population growth rate ($\lambda$), to perturbations 
typically applied at the level of individual matrix elements and which may be perturbations of
a fixed or proportionate amount (**sensitivities** and **elasticities**, 
respectively). By extension, the vital rates (survival, growth, etc.) or
transition type (stasis, retrogression, etc.) underlying multiple matrix elements 
may be similarly perturbed. `Rage` provides functions for each scope of
perturbation, allowing the user to specify the desired demographic statistic and 
type of perturbation.

```{r}
# construct the transition matrix A = U + F (+ C when present)
mpm1$matA <- with(mpm1, matU + matF)

# sensitivity of lambda to...
# ...matrix element perturbations
perturb_matrix(
  matA = mpm1$matA,
  type = "sensitivity", demog_stat = "lambda"
)
# ...vital rate perturbations
perturb_vr(
  matU = mpm1$matU, matF = mpm1$matF,
  type = "sensitivity", demog_stat = "lambda"
)
# ...transition type perturbations
perturb_trans(
  matU = mpm1$matU, matF = mpm1$matF,
  type = "sensitivity", demog_stat = "lambda"
)
```

## 4. Deriving life history traits {#lifehist}

What is the life expectancy of an individual? At what age will it begin to 
reproduce? How likely is it to survive to reproduction? What is the generation 
time? These high-level questions address the population-level life history traits 
that emerge from aggregating individual-level demographic rates, and tracing 
trajectories through the life cycle.

`Rage` provides functions to calculate a wide range of life history traits from 
both MPMs and life tables. We illustrate the general pattern of these functions 
below, referring the reader to the [LifeHistoryTraits vignette](a03_LifeHistoryTraits.html)
and the [documentation webpage](https://jonesor.github.io/Rage/reference/index.html) 
for a complete list of functionality.

MPM-based analyses take the required submatrix (or submatrices), optionally allowing
the user to specify the life stage at which to start the calculation, choose among 
alternative methods and set cut-off thresholds.
```{r}
# post-germination time steps until post-germination survivorship falls below 5%
longevity(matU = mpm1$matU, start = "small", lx_crit = 0.05)
# expected lifetime production of 'small' offspring by a 'small' individual
net_repro_rate(
  matU = mpm1$matU, matR = mpm1$matF, start = "seed",
  method = "start"
)
```

It can sometimes be useful to calculate age-based trajectories from stage-based matrices. This can be achieved with functions like `mpm_to_lx`.

```{r}
# derive post-germination survivorship trajectory from U matrix
lx <- mpm_to_lx(matU = mpm1$matU, start = "small")
```

## 5. Transforming MPMs {#maniptransform}

`Rage` includes a variety of functions that can be used to manipulate or transform
MPMs. For example, we can collapse an MPM to a smaller number of stage classes
using `mpm_collapse()`.

```{r}
# collapse 'small', 'medium', and 'large' stages into single stage class
col1 <- mpm_collapse(
  matU = mpm1$matU, matF = mpm1$matF,
  collapse = list(1, 2:4, 5)
)
col1$matA
```

Notice that the stage names are lost during this operation. We can re-add new 
stage names using the function `name_stages`, electing to use generic or custom 
names.

```{r}
# automated stage naming
(col1_auto <- name_stages(mat = col1, prefix = "class_"))

# overwrite with custom stages
(col1_cust <- name_stages(
  mat = col1, names = c("seed", "active", "dormant"),
  prefix = NULL
))
```

The transition rates in the collapsed matrix are a weighted average of the
transition rates from the relevant stages of the original matrix, weighted by
the stable distribution at equilibrium. This process guarantees that the
collapsed MPM will retain the same population growth rate as the original.
However, other demographic and life history characteristics will not necessarily
be preserved.

```{r}
# compare population growth rate of original and collapsed MPM (preserved)
popdemo::eigs(A = mpm1$matA, what = "lambda")
popdemo::eigs(A = col1_cust$matA, what = "lambda")

# compare net reproductive rate of original and collapsed MPM (not preserved)
net_repro_rate(matU = mpm1$matU, matR = mpm1$matF)
net_repro_rate(matU = col1_cust$matU, matR = col1_cust$matF)
```

# References {#references}

Caswell, H. (2001). Matrix Population Models: Construction, Analysis, and Interpretation. 2nd edition. Sinauer Associates, Sunderland, MA. ISBN-10: 0878930965

Keyfitz, N. (1972). On future population. Journal of the American Statistical Association 67:347-363.

---
title: "Deriving vital rates from an MPM"
author: "Patrick Barks"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Deriving vital rates from an MPM}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
editor_options: 
  chunk_output_type: console
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```



## Introduction

The transition rates that make up matrix population models (MPMs) generally
reflect products of two or more vital rates (sometimes called 'lower-level vital rates'). Assuming a post-breeding census design, we can retroactively break apart each transition rate into at least two vital rate components: survival, and 'something' conditional on survival. That 'something' might be growth, shrinkage, stasis, dormancy, fecundity, or clonality.

The `vr_` group of functions can be used to derive vital rates from any MPM that was initially estimated using a post-breeding census design. The vital rates may be summarized across the entire MPM (`vr_` functions), summarized within stage classes (`vr_vec_` functions), or not summarized (`vr_mat_` functions).

## Preliminaries: Constructing an MPM from lower-level vital rates

```{r}
library(Rage)
```

We'll start by creating an MPM based on a set of underlying vital rates. Our MPM will be separated into a growth/survival component (`matU`) and a sexual reproduction component (`matF`), and consist of 4 stage classes: 'seed', 'small', 'large', and 'dormant'. First we'll specify the vital rates.

```{r}
## survival probabilities
s1 <- 0.20 # seed
s2 <- 0.40 # small
s3 <- 0.50 # large
s4 <- 0.30 # dormant

## matU vital rates conditional on survival (growth, shrinkage, stasis, etc.)
g12 <- 0.15 # seed to small      (growth)
g13 <- 0.05 # seed to large      (growth)
g11 <- 1 - g12 - g13 # seed to seed       (stasis)
g23 <- 0.45 # small to large     (growth)
g24 <- 0.10 # small to dormant   (enter dormancy)
g22 <- 1 - g23 - g24 # small to small     (stasis)
g34 <- 0.22 # large to dormant   (enter dormancy)
g33 <- 1 - g34 # large to large     (stasis)
g43 <- 0.50 # dormant to large   (exit dormancy)
g44 <- 1 - g43 # dormant to dormant (stasis)

## matF vital rates conditional on survival (i.e. fecundity)
f2 <- 0.4 # small
f3 <- 1.1 # large
```

Next we'll use the vital rates to construct an MPM, based on a post-breeding census design.

```{r}
# growth/survival component
matU <- rbind(
  c(s1 * g11, 0, 0, 0),
  c(s1 * g12, s2 * g22, 0, 0),
  c(s1 * g13, s2 * g23, s3 * g33, s4 * g43),
  c(0, s2 * g24, s3 * g34, s4 * g44)
)

# sexual reproduction component
matF <- rbind(
  c(0, s2 * f2, s3 * f3, 0),
  c(0, 0, 0, 0),
  c(0, 0, 0, 0),
  c(0, 0, 0, 0)
)
```

## Deriving vital rates of survival

Given a post-breeding census design, stage-specific vital rates of survival can be obtained by taking column sums of the __U__ matrix (`matU` in our code). This works out algebraically because the other vital rates within `matU`, which are conditional on survival, must sum to 1 within a given stage class. We can obtain a vector of stage-specific vital rates of survival using the `vr_vec_survival()` function, which, for simple cases, is equivalent to `colSums()`.

```{r}
(surv <- vr_vec_survival(matU))
# equivalent to...
(surv <- colSums(matU))
```

As expected, these values match the survival probabilities that we initially specified.

## Deriving vital rates conditional on survival

To obtain the other set of vital rates — vital rates conditional on survival (sometimes called 'survival-independent vital rates') — we simply need to divide each column of `matU` or `matF` by the corresponding survival probability. This procedure is implemented by the `vr_mat_` functions.

```{r}
# matU vital rates conditional on survival
vr_mat_U(matU)

# matF vital rates conditional on survival
vr_mat_R(matU, matF)
```

We've now recovered all the vital rates that we initially specified when we constructed our MPM.

## Summarizing vital rates within stages

The vital rates produced by `vr_mat_U()` represent a variety of processes, including growth, shrinkage, stasis, and dormancy. To summarize these processes within stage classes we can use the `vr_vec_` group of functions, which simply sum the vital rates corresponding to a given process within stage classes.

#### Growth, shrinkage, stasis, and dormancy

We'll start by examining vital rates of growth, which will be found in the bottom-left triangle of the __U__ matrix. Given the results from `vr_mat_U()` above, our baseline expectation for the stage-specific vital rates of growth would be 0.20 (0.15 + 0.05), 0.55 (0.45 + 0.10), 0.22, and `NA`.

```{r}
vr_vec_growth(matU)
```

However, the calculations above include transitions to the dormant stage class as a component of 'growth', because the dormant stage class is represented by the right-most column (and so transitions to the dormant stage are in the bottom-left triangle of `matU`). To exclude transitions _to_ the dormant stage from our calculation of growth, we can set the `exclude_row` argument to `4`, so that any survival-independent transition _to_ stage 4 is excluded from the calculation of growth.

```{r}
vr_vec_growth(matU, exclude_row = 4)
```

Next let's examine vital rates of shrinkage, which will occur in the top-right triangle of `matU`. Based again on the results from `vr_mat_U()`, our naive expectation for vital rates of shrinkage would be `NA`, `NA`, `NA`, 0.5.

```{r}
vr_vec_shrinkage(matU)
```

However, in this case, the transition from the dormant stage to the large stage is being mistakenly considered as shrinkage, simply because it's in the top-right triangle of `matU`. To exclude transitions _from_ the dormant stage from our calculations, we can set the `exclude_col` argument to `4`. Note that we use `exclude_row` to exclude transitions _to_ a given stage, and `exclude_col` to exclude transitions _from_ a given stage.

```{r}
vr_vec_shrinkage(matU, exclude_col = 4)
```

Next let's calculate vital rates of stasis, which occur along the diagonal of `matU`. Depending on our purpose, we may or may not consider the dormant to dormant transition as 'stasis'. If not, we could exclude it using one of the `exclude_` arguments, as above. But we'll leave it in for now.

```{r}
vr_vec_stasis(matU)
```

Finally, let's calculate vital rates for entering and exiting dormancy. For these, we'll need to explicitly specify which stage(s) are dormant using the `dorm_stages` argument.

```{r}
vr_vec_dorm_enter(matU, dorm_stages = 4)
vr_vec_dorm_exit(matU, dorm_stages = 4)
```

#### Fecundity

The survival-independent vital rates of fecundity produced by `vr_mat_R()` are more straightforward to summarize, as they only reflect a single process (i.e.
fecundity).

```{r}
vr_vec_reproduction(matU, matF)
```

In the example MPM we're using here, individuals produced by fecundity always begin life in the 'seed' stage. However, for some life cycles there are multiple stages in which an offspring could begin life. In these cases, it may be desirable to weight the different types of offspring when summing vital rates of fecundity within a stage class (e.g. by their reproductive value). This type of weighting can be accomplished with the `weights_row` argument.

## Summarizing vital rates across stages

In some cases, it may be desirable to summarize vital rates across stage classes to obtain a single mean vital rate for the entire MPM. This can be done using the `vr_` group of functions.

#### Simple average across stage classes

By default, the `vr_` functions take a simple average of the stage-specific vital rates returned by the corresponding `vr_vec_` function. Here's an example with growth.

```{r}
vr_growth(matU, exclude_row = 4)

# equivalent to
(vec_growth <- vr_vec_growth(matU, exclude_row = 4))
mean(vec_growth, na.rm = TRUE)
```

Note that the two stage classes from which there are no growth transitions do not contribute to the MPM-average vital rate of growth. In practice, this is because the `vr_vec_` functions return NAs rather than 0s for stages that do not exhibit the relevant transition type. This is an important issue that we'll return to in a later section.

#### Weighted average across stage classes

Rather than taking a simple average of the given vital rate across stage classes, we may wish to take a _weighted_ average across stage classes. For instance, we may wish to weight stage classes based on the stable distribution at equilibrium. Here's an example of how to do that using the `weights_col` argument.

```{r}
# calculate the stable distribution using popdemo::eigs
library(popdemo)
matA <- matU + matF
w <- popdemo::eigs(matA, what = "ss")

# calculate mean vital rate of growth weighted by the stable distribution
vr_growth(matU, exclude_row = 4, weights_col = w)

# equivalent to
pos <- !is.na(vec_growth) # possible transitions
sum(vec_growth[pos] * w[pos]) / sum(w[pos]) # weighted average
```

## Distinguishing between possible and impossible transitions

By default, all `vr_` functions assume that a transition rate of 0 indicates an impossible transition within the given life cycle (e.g. tadpoles never revert to eggs), in which case a value of `NA` will be used in any relevant calculations. However, a transition rate of 0 could alternatively indicate a transition that is generally possible, but was simply estimated to be 0 in the relevant population and time period.

This distinction between possible and impossible transitions can be important when calculating vital rates — particularly if we want to summarize vital rates across stages (i.e. summarize at the level of the MPM).

Let's consider the same `matU` we defined above, but imagine that it reflects a life cycle with a different set of stages: 'seed', 'small', 'medium', and 'large'. Now our MPM has a single shrinkage transition (large to medium). If we summarize the vital rate of shrinkage across stages using `vr_shrinkage()`, we'll simply recover the single shrinkage vital rate from the large to medium transition (i.e. because the other stages, with no shrinkage, do not contribute to the average).

```{r}
vr_vec_shrinkage(matU)
vr_shrinkage(matU)
```

However, imagine now that the medium to small transition, which is 0 within `matU`, is in fact a possible shrinkage transition in our life cycle of interest — its rate was simply estimated to be 0 in the relevant sample and time period. In this case, we'll need to manually specify the matrix of possible transitions using the `posU` argument.

```{r}
(pos <- matU > 0) # possible transitions based on matU
pos[2, 3] <- TRUE # set medium-to-small shrinkage tr to possible

vr_vec_shrinkage(matU, posU = pos)
vr_shrinkage(matU, posU = pos)
```

Now, instead of returning `NA` for the vital rate of shrinkage from the medium stage class, `vr_vec_shrinkage()` returns a value of 0. Correspondingly, `vr_shrinkage()` incorporates this 0 into its calculation and returns an MPM-average shrinkage rate of 0.25 (`mean(c(0, 0.5)`) rather than 0.5, as before.

## References

Caswell, H. (2001). Matrix Population Models: Construction, Analysis, and Interpretation. 2nd edition. Sinauer Associates, Sunderland, MA. ISBN-10: 0878930965

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/shape_rep.R
\name{shape_rep}
\alias{shape_rep}
\title{Calculate shape of reproduction over age}
\usage{
shape_rep(rep, surv = NULL, xmin = NULL, xmax = NULL, ...)
}
\arguments{
\item{rep}{Either 1) a numeric vector describing reproduction over age (mx),
  2) a \code{data.frame} / \code{list} with one column / element titled 'mx'
  describing a reproduction over age, optionally a column / element 'x'
  containing age classes (each element a number representing the age at the
  start of the class), or 3) a matrix, specifically the reproduction
  submatrix (e.g. F matrix) of a matrix population model. If \code{rep} is
  provided as a matrix, then \code{surv} must be provided as the U submatrix
  of the matrix population model.

  In case (2), if x is not supplied, the function will assume age classes
  starting at 0 with time steps of unit. If x ends at maximum longevity,
  \code{mx[which.max(x)]} should equal 0; however it is possible to supply
  partial reproduction schedules.}

\item{surv}{An optional argument to be used if rep is provided as a matrix
(the reproduction submatrix of the matrix population model.) If \code{rep}
is provided as a matrix, then \code{surv} should also be provided as a the
U submatrix of the matrix population model.}

\item{xmin, xmax}{The minimum and maximum age respectively over which to
evaluate shape. If not given, these default to \code{min(x)} and
\code{max(x)} respectively.}

\item{...}{Additional variables passed to `mpm_to_mx` when the data are
provided as matrices.}
}
\value{
a shape value describing symmetry of reproduction over age by
  comparing the area under a cumulative reproduction curve over age with the
  area under constant reproduction. May take any real value between -0.5 and
  +0.5. A value of 0 indicates negligible ageing (neither generally
  increasing nor generally decreasing reproduction with age); positive values
  indicate senescence (generally decreasing reproduction with age); negative
  values indicate negative senescence (generally increasing reproduction with
  age). A value of +0.5 indicates that (hypothetically) all individuals are
  born to individuals of age 0; a value of -0.5 indicates that all
  individuals are born at the age of maximum longevity.
}
\description{
Calculates a 'shape' value of distribution of reproduction over age by
comparing the area under a cumulative reproduction curve (over age) with the
area under a cumulative function describing constant reproduction.
}
\examples{
# increasing mx yields negative shape
mx <- c(0, 0, 0.3, 0.4, 0.5, 0.6)
shape_rep(mx)

# decreasing mx yields positive shape
mx <- c(1.1, 1.0, 0.9, 0.8, 0.7, 0.6, 0.5, 0.4)
shape_rep(mx)

# constant mx yields shape = 0
mx <- c(0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1)
shape_rep(mx)

# calculate mx trajectory first
mpm_to_mx(matU = mpm1$matU, matR = mpm1$matF)


# providing the matrices directly
data(mpm1)
shape_rep(rep = mpm1$matF, surv = mpm1$matU)

}
\references{
Wrycza, T.F. and Baudisch, A., 2014. The pace of aging: Intrinsic
  time scales in demography. Demographic Research, 30, pp.1571-1590.
  <doi:10.4054/DemRes.2014.30.57>

  Baudisch, A. 2011, The pace and shape of ageing. Methods in Ecology and
  Evolution, 2: 375-382. <doi:10.1111/j.2041-210X.2010.00087.x>

  Baudisch, A, Stott, I. 2019. A pace and shape perspective on fertility.
  Methods Ecol Evol. 10: 1941– 1951. <doi:10.1111/2041-210X.13289>
}
\seealso{
Other life history traits: 
\code{\link{entropy_d}()},
\code{\link{entropy_k}()},
\code{\link{entropy_k_age}()},
\code{\link{entropy_k_stage}()},
\code{\link{gen_time}()},
\code{\link{life_elas}()},
\code{\link{life_expect_mean}()},
\code{\link{longevity}()},
\code{\link{net_repro_rate}()},
\code{\link{repro_maturity}},
\code{\link{shape_surv}()},
\code{\link{stage_at_death_dist}()}
}
\author{
Iain Stott <iainmstott@gmail.com>
}
\concept{life history traits}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mpm_standardize.R
\name{mpm_standardize}
\alias{mpm_standardize}
\alias{mpm_standardise}
\title{Transform a matrix population model to a standardized form}
\usage{
mpm_standardize(matU, matF, matC = NULL, repro_stages, matrix_stages)

mpm_standardise(matU, matF, matC = NULL, repro_stages, matrix_stages)
}
\arguments{
\item{matU}{The survival component of a matrix population model (i.e., a
square projection matrix reflecting survival-related transitions; e.g.
progression, stasis, and retrogression).}

\item{matF}{The sexual component of a matrix population model (i.e., a square
projection matrix reflecting transitions due to sexual reproduction).}

\item{matC}{The clonal component of a matrix population model (i.e., a square
projection matrix reflecting transitions due to clonal reproduction).
Defaults to \code{NULL}, indicating no clonal reproduction (i.e.,
\code{matC} is a matrix of zeros).}

\item{repro_stages}{Logical vector of length \code{ncol(matU)} indicating
which stages are reproductive. Alternatively, a vector of stage indices or
stage names of the reproductive classes.}

\item{matrix_stages}{Character vector of matrix stage types (e.g.,
"propagule", "active", or "dormant").}
}
\value{
A list with four elements reflecting the standardized matrix and
  its components:
  \item{matA}{Standardized projection matrix}
  \item{matU}{Survival component of the standardized projection matrix}
  \item{matF}{Sexual reproduction component of the standardized projection
  matrix}
  \item{matC}{Clonal reproduction component of the standardized projection
  matrix}
}
\description{
Transform a matrix population model to a standardized set of stage classes
(e.g., propagule, pre-reproductive, reproductive, and post-reproductive). The
transition rates in the standardized matrix are a weighted mean of the
transition rates and per-capita reproductive values from the relevant stages
of the original matrix, weighted by the relative proportion of each stage
class expected at the stable distribution.
}
\details{
This function is a wrapper for the functions
  \code{\link{mpm_rearrange}}, \code{\link{standard_stages}} and
  \code{\link{mpm_collapse}}, which it calls in sequence.
}
\note{
The method used by this function to collapse a matrix population model
  preserves the equilibrium population growth rate (\eqn{\lambda}) and
  relative stable distribution, but is not expected to preserve other
  demographic characteristics such as relative reproductive value,
  sensitivities, net reproductive rate, life expectancy, etc.
}
\section{Missing Stages}{

The returned standardized matrix will always be of dimension \code{4}, even
if one or more standardized stages is missing from the original matrix
population model. If a standardized stage is missing, the corresponding
row/column of the standardized matrix will be coerced to \code{NA}.
}

\examples{
matU <- rbind(
  c(0.1, 0, 0, 0, 0),
  c(0.5, 0.2, 0.1, 0, 0),
  c(0, 0.3, 0.3, 0.1, 0),
  c(0, 0, 0.4, 0.4, 0.1),
  c(0, 0, 0, 0.1, 0.4)
)

matF <- rbind(
  c(0, 1.1, 0, 1.6, 0),
  c(0, 0.8, 0, 0.4, 0),
  c(0, 0, 0, 0, 0),
  c(0, 0, 0, 0, 0),
  c(0, 0, 0, 0, 0)
)

matC <- rbind(
  c(0, 0.6, 0, 0.5, 0),
  c(0, 0.1, 0, 0.3, 0),
  c(0, 0, 0, 0, 0),
  c(0, 0, 0, 0, 0),
  c(0, 0, 0, 0, 0)
)

repro_stages <- c(2, 4)
matrix_stages <- c("prop", "active", "active", "active", "active")

mpm_standardize(matU, matF, matC, repro_stages, matrix_stages)

}
\seealso{
Other transformation: 
\code{\link{is_leslie_matrix}()},
\code{\link{leslie_collapse}()},
\code{\link{mpm_collapse}()},
\code{\link{mpm_rearrange}()},
\code{\link{mpm_split}()},
\code{\link{name_stages}()},
\code{\link{repro_stages}()},
\code{\link{scale_mpm_to_lambda1}()},
\code{\link{standard_stages}()}
}
\author{
Rob Salguero-Gomez <rob.salguero@zoo.ox.ac.uk>
}
\concept{transformation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vr.R
\name{vr}
\alias{vr}
\alias{vr_survival}
\alias{vr_growth}
\alias{vr_shrinkage}
\alias{vr_stasis}
\alias{vr_dorm_enter}
\alias{vr_dorm_exit}
\alias{vr_fecundity}
\title{Derive mean vital rates from a matrix population model}
\usage{
vr_survival(matU, posU = matU > 0, exclude_col = NULL, weights_col = NULL)

vr_growth(
  matU,
  posU = matU > 0,
  exclude = NULL,
  exclude_row = NULL,
  exclude_col = NULL,
  weights_col = NULL,
  surv_only_na = TRUE
)

vr_shrinkage(
  matU,
  posU = matU > 0,
  exclude = NULL,
  exclude_row = NULL,
  exclude_col = NULL,
  weights_col = NULL,
  surv_only_na = TRUE
)

vr_stasis(
  matU,
  posU = matU > 0,
  exclude = NULL,
  weights_col = NULL,
  surv_only_na = TRUE
)

vr_dorm_enter(matU, posU = matU > 0, dorm_stages, weights_col = NULL)

vr_dorm_exit(matU, posU = matU > 0, dorm_stages, weights_col = NULL)

vr_fecundity(
  matU,
  matR,
  posR = matR > 0,
  exclude_col = NULL,
  weights_row = NULL,
  weights_col = NULL
)
}
\arguments{
\item{matU}{The survival component of a matrix population model (i.e., a
square projection matrix reflecting survival-related transitions; e.g.,
progression, stasis, and retrogression)}

\item{posU}{A logical matrix of the same dimension as \code{matU}, with
elements indicating whether a given \code{matU} transition is possible
(\code{TRUE}) or not (\code{FALSE}). Defaults to \code{matU > 0} (see
\emph{Possible transitions}).}

\item{exclude_col}{Integer, character or logical vector indicating stages for
which transitions both \emph{to} and \emph{from} the stage should be
excluded from the calculation of vital rates. See section
\emph{Excluding stages}.}

\item{weights_col}{Vector of stage-specific weights to apply while averaging
vital rates across columns. See section \emph{Weighting stages}.}

\item{exclude}{Integer, character or logical vector indicating stages for
which transitions both \emph{to} and \emph{from} the stage should be
excluded from the calculation of vital rates. See section
\emph{Excluding stages}.}

\item{exclude_row}{Integer, character or logical vector indicating stages for
which transitions both \emph{to} and \emph{from} the stage should be
excluded from the calculation of vital rates. See section
\emph{Excluding stages}.}

\item{surv_only_na}{If there is only one possible \code{matU} transition in a
given column, should that transition be attributed exclusively to survival?
If \code{TRUE}, the vital rate of growth/stasis/shrinkage in that column
will be coerced to \code{NA}. If \code{FALSE}, dividing the single
transition by the stage-specific survival probability will always yield a
value of \code{1}. Defaults to \code{TRUE}.}

\item{dorm_stages}{Integer or character vector indicating dormant stage
classes.}

\item{matR}{The reproductive component of a matrix population model (i.e., a
square projection matrix reflecting transitions due to reproduction; either
sexual, clonal, or both)}

\item{posR}{A logical matrix of the same dimension as \code{matR}, with
elements indicating whether a given \code{matR} transition is possible
(\code{TRUE}) or not (\code{FALSE}). Defaults to \code{matR > 0} (see
\emph{Possible transitions}).}

\item{weights_row}{Vector of stage-specific weights to apply while summing
vital rates across rows within columns. See section \emph{Weighting
stages}.}
}
\value{
Vector of vital rates. Vital rates corresponding to impossible
  transitions are coerced to \code{NA} (see \emph{Possible transitions}).
}
\description{
Derive mean vital rates of survival, growth (or development), shrinkage (or
de-development), stasis, dormancy, or reproduction from a matrix population
model, by averaging across stage classes. These functions include optional
arguments for custom weighting of different stage classes (see
\emph{Weighting stages}), excluding certain stage classes from the
calculation (see \emph{Excluding stages}), and defining the set of
biologically-possible transitions (see \emph{Possible transitions}).

These decompositions assume that all transition rates are products of a
stage-specific survival term (column sums of \code{matU}) and a lower level
vital rate that is conditional on survival (growth/development,
shrinkage/de-development, stasis, dormancy, or a/sexual reproduction).
Reproductive vital rates that are not conditional on survival (i.e., within a
stage class from which there is no survival) are also allowed.
}
\note{
Fecundity-related outputs assume a post-breeding census
  interpretation. In pre-breeding models, birth rate and newborn survival are
  generally confounded within fecundity transitions. See Kendall et al.
  (2019).
}
\section{Possible transitions}{

A transition rate of \code{0} within a matrix population model may indicate
that the transition is not possible in the given life cycle (e.g., tadpoles
never revert to eggs), or that the transition rate is possible but was
estimated to be \code{0} in the relevant population and time period. If vital
rates are to be averaged across multiple stage classes, or compared across
populations, it may be important to distinguish between these two types of
zeros.

By default, the \code{vr_} functions assume that a transition rate of
\code{0} indicates an impossible transition, in which case a value of
\code{NA} will be used in relevant calculations. Specifically, the arguments
\code{posU} and \code{posR} are specified by the logical expressions
\code{(matU > 0)} and \code{(matR > 0)}, respectively. If the matrix
population model includes transitions that are estimated to be \code{0} but
still in fact possible, one should specify the \code{posU} and/or \code{posR}
arguments manually.
}

\section{Weighting stages}{

In averaging vital rates across stages, it may be desirable to weight stage
classes differently (e.g., based on reproductive values or stable
distributions). Weights are generally applied when averaging across columns,
i.e., across transitions \emph{from} a set of stage classes (e.g., averaging
stage-specific survival probabilities across multiple stages). All \code{vr_}
functions therefore include an optional argument \code{weights_from}.

In principle, particularly for vital rates of reproduction, the user can also
apply weights when summing across rows within columns, i.e., across
reproductive transitions \emph{to} a set of stage classes (e.g., summing the
production of different types of offspring, such as seeds vs. seedlings). The
function \code{vr_fecundity} therefore also includes an optional
argument \code{weights_to}.

If supplied, \code{weights_from} will automatically be scaled to sum to 1
over the set of possible transitions, whereas \code{weights_to} will not be
rescaled because we wish to enable the use of reproductive values here, which
do not naturally sum to 1.
}

\section{Excluding stages}{

It may be desirable to exclude one or more stages from the calculation of
certain vital rates. For instance, we might not believe that 'growth' to a
dormant stage class really reflects biological growth, in which case we could
exclude transitions \emph{to} the dormant stage class using the argument
\code{exclude_row}. We may or may not want to ignore 'growth' transitions
\emph{from} the dormant stage class, which can be done using
\code{exclude_col}. To exclude transitions both \emph{to and from} a given
set of stages, use argument \code{exclude}.
}

\examples{
# create example MPM (stage 4 is dormant)
matU <- rbind(
  c(0.1, 0, 0, 0),
  c(0.5, 0.2, 0.1, 0.1),
  c(0, 0.3, 0.3, 0.1),
  c(0, 0, 0.5, 0.4)
)

matF <- rbind(
  c(0, 0.7, 1.1, 0),
  c(0, 0.3, 0.8, 0),
  c(0, 0, 0, 0),
  c(0, 0, 0, 0)
)

vr_survival(matU, exclude_col = 4)
vr_growth(matU, exclude = 4)
vr_shrinkage(matU, exclude = 4)
vr_stasis(matU, exclude = 4)

# `exclude*` and `*_stages` arguments can accept stage names
matU <- name_stages(matU)
matF <- name_stages(matF)
vr_dorm_enter(matU, dorm_stages = "stage_4")
vr_dorm_exit(matU, dorm_stages = 4)

vr_fecundity(matU, matF, exclude_col = 4)

}
\seealso{
Other vital rates: 
\code{\link{vital_rates}()},
\code{\link{vr_mat}},
\code{\link{vr_vec}}
}
\author{
Patrick Barks <patrick.barks@gmail.com>
}
\concept{vital rates}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Rage-package.R
\docType{package}
\name{Rage-package}
\alias{Rage}
\alias{Rage-package}
\title{Rage: Life History Metrics from Matrix Population Models}
\description{
Functions for calculating life history metrics using matrix population models ('MPMs'). Described in Jones et al. (2021) \doi{10.1101/2021.04.26.441330}.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/jonesor/Rage}
  \item \url{https://jonesor.github.io/Rage/}
  \item Report bugs at \url{https://github.com/jonesor/Rage/issues}
}

}
\author{
\strong{Maintainer}: Owen Jones \email{jones@biology.sdu.dk} (\href{https://orcid.org/0000-0001-5720-4686}{ORCID})

Authors:
\itemize{
  \item Patrick Barks \email{barks@biology.sdu.dk} (\href{https://orcid.org/0000-0002-5947-8151}{ORCID})
  \item Pol Capdevila \email{pcapdevila.pc@gmail.com} (\href{https://orcid.org/0000-0002-2842-4302}{ORCID})
  \item Hal Caswell \email{h.caswell@uva.nl} (\href{https://orcid.org/0000-0003-4394-6894}{ORCID})
  \item Judy P. Che-Castaldo \email{jchecastaldo@lpzoo.org} (\href{https://orcid.org/0000-0002-9118-9202}{ORCID})
  \item Richard A. Hinrichsen \email{rich@hinrichsenenvironmental.com} (\href{https://orcid.org/0000-0003-0761-3005}{ORCID})
  \item John Jackson \email{jjackson0308@gmail.com} (\href{https://orcid.org/0000-0002-4563-2840}{ORCID})
  \item Tamora James \email{tamoradjames@protonmail.com} (\href{https://orcid.org/0000-0003-1363-4742}{ORCID})
  \item Sam Levin \email{levisc8@gmail.com} (\href{https://orcid.org/0000-0002-3289-9925}{ORCID})
  \item William K. Petry \email{wpetry@ncsu.edu} (\href{https://orcid.org/0000-0002-5230-5987}{ORCID})
  \item Roberto Salguero-Gomez \email{rob.salguero@zoo.ox.ac.uk} (\href{https://orcid.org/0000-0002-6085-4433}{ORCID})
  \item Iain Stott \email{stott@biology.sdu.dk} (\href{https://orcid.org/0000-0003-2724-7436}{ORCID})
  \item Chelsea C. Thomas \email{esochels@gmail.com} (\href{https://orcid.org/0000-0002-8155-9353}{ORCID})
  \item Christina M. Hernández \email{cmh352@cornell.edu} (\href{https://orcid.org/0000-0002-7188-8217}{ORCID})
  \item Lotte de Vries \email{c.devries@uva.nl} (\href{https://orcid.org/0000-0001-8955-0479}{ORCID})
  \item Stefano Giaimo \email{giaimo@evolbio.mpg.de} (\href{https://orcid.org/0000-0003-0421-3065}{ORCID})
}

Other contributors:
\itemize{
  \item Danny Buss \email{dlb50@cam.ac.uk} [contributor]
  \item Caroline Schuette \email{cschuette17@gmail.com} (\href{https://orcid.org/0000-0002-2063-8736}{ORCID}) [contributor]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/evenness_shannon.R
\name{evenness_shannon}
\alias{evenness_shannon}
\title{Shannon Evenness (Pielou's J)}
\usage{
evenness_shannon(p)
}
\arguments{
\item{p}{A numeric vector of counts or probabilities. If counts are
provided, they are normalised to sum to 1 internally.}
}
\value{
A numeric value between 0 and 1 representing Shannon evenness. A
  value of 1 indicates an even distribution, while low values indicate uneven
  distributions. A value of 0 indicates that all deaths are lumped into one
  stage.
}
\description{
Computes the general Shannon evenness index (aka Pielou's J), defined as the
ratio of the observed Shannon entropy to the maximum possible entropy given
the number of categories. This function is included here because Shannon
evenness can be applied to stage-at-death distributions to quantify how
evenly deaths are distributed across stages.
}
\details{
Zero entries in \code{p} contribute nothing to entropy
  but do count towards the total number of categories when computing
  the maximum entropy (\eqn{H_{\max} = \log(K)}).
}
\examples{
# Even
evenness_shannon(c(0.2, 0.2, 0.2, 0.2, 0.2))

# Skewed
evenness_shannon(c(0.8, 0.1, 0.05, 0.03, 0.02))

evenness_shannon(c(1, 0, 0, 0, 0))

# From a matrix model's stage at death distribution
data(mpm1)
matA <- mpm1$matU + mpm1$matF
ssd <- popdemo::eigs(matA, "ss")
x <- stage_at_death_dist(mpm1$matU, start = ssd)
evenness_shannon(x)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/qsd_converge.R
\name{qsd_converge}
\alias{qsd_converge}
\title{Calculate time to reach quasi-stationary stage distribution}
\usage{
qsd_converge(mat, start = 1L, conv = 0.01, N = 100000L)
}
\arguments{
\item{mat}{A matrix population model, or component thereof (i.e., a square
projection matrix). Optionally with named rows and columns indicating the
corresponding life stage names.}

\item{start}{The index (or stage name) of the first stage at which the author
considers the beginning of life. Defaults to \code{1}. Alternately, a
numeric vector giving the starting population vector (in which case
\code{length(start)} must match \code{ncol(matU))}. See section
\emph{Starting from multiple stages}.}

\item{conv}{Proportional distance threshold from the stationary stage
distribution indicating convergence. For example, this value should be
\code{0.01} if the user wants to obtain the time step when the stage
distribution is within a distance of 1\% of the stationary stage
distribution.}

\item{N}{Maximum number of time steps over which the population will be
projected. Time steps are in the same units as the matrix population model
(see \code{AnnualPeriodicity} column in COM(P)ADRE metadata). Defaults to
\code{100,000}.}
}
\value{
An integer indicating the first time step at which the
  quasi-stationary stage distribution is reached (or an \code{NA} and a
  warning if the quasi-stationary distribution is not reached).
}
\description{
Calculates the time for a cohort projected with a matrix population
model to reach a defined quasi-stationary stage distribution.
}
\details{
Some matrix population models are parameterised with a stasis loop at the
largest/most-developed stage class, which can lead to artefactual plateaus
in the mortality or fertility trajectories derived from such models. These
plateaus occur as a projected cohort approaches its stationary stage
distribution (SSD). Though there is generally no single time point at which
the SSD is reached, we can define a quasi-stationary stage distribution (QSD)
based on a given distance threshold from the SSD, and calculate the number of
time steps required for a cohort to reach the QSD. This quantity can then be
used to subset age trajectories of mortality or fertility to periods earlier
than the QSD, so as to avoid artefactual plateaus in mortality or fertility.

\strong{Starting from multiple stages}

Rather than specifying argument \code{start} as a single stage class from
which all individuals start life, it may sometimes be desirable to allow for
multiple starting stage classes. For example, if we want to start our
calculation of QSD from reproductive maturity (i.e., first reproduction), we
should account for the possibility that there may be multiple stage classes
in which an individual could first reproduce.

To specify multiple starting stage classes, specify argument \code{start} as
the desired starting population vector, giving the proportion
of individuals starting in each stage class (the length of \code{start}
should match the number of columns in the relevant MPM).
}
\note{
The time required for a cohort to reach its QSD depends on the initial
  population vector of the cohort (for our purposes, the starting stage
  class), and so does not fundamentally require an ergodic matrix (where the
  long-term equilibrium traits are independent of the initial population
  vector). However, methods for efficiently calculating the stationary stage
  distribution (SSD) generally do require ergodicity.

  If the supplied matrix (\code{mat}) is non-ergodic, \code{qsd_converge}
  first checks for stage classes with no connection (of any degree) from the
  starting stage class specified by argument \code{start}, and strips such
  stages from the matrix. These unconnected stages have no impact on
  age-specific traits that we might derive from the matrix (given the
  specified starting stage), but often lead to non-ergodicity and therefore
  prevent the reliable calculation of SSD. If the reduced matrix is ergodic,
  the function internally updates the starting stage class and continues with
  the regular calculation. Otherwise, if the matrix cannot be made ergodic,
  the function will return \code{NA} with a warning.
}
\examples{
data(mpm1)

# starting stage = 2 (i.e., "small")
qsd_converge(mpm1$matU, start = 2)
qsd_converge(mpm1$matU, start = "small") # equivalent using named life stages

# convergence threshold = 0.001
qsd_converge(mpm1$matU, start = 2, conv = 0.001)

# starting from first reproduction
repstages <- repro_stages(mpm1$matF)
n1 <- mature_distrib(mpm1$matU, start = 2, repro_stages = repstages)
qsd_converge(mpm1$matU, start = n1)

}
\references{
Caswell, H. 2001. Matrix Population Models: Construction, Analysis, and
Interpretation. Sinauer Associates; 2nd edition. ISBN: 978-0878930968

Horvitz, C. C., & Tuljapurkar, S. 2008. Stage dynamics, period survival, and
mortality plateaus. The American Naturalist, 172(2), 203–215.

Jones, O. R., Scheuerlein, A., Salguero-Gomez, R., Camarda, C. G., Schaible,
R., Casper, B. B., Dahlgren, J. P., Ehrlén, J., García, M. B., Menges, E.,
Quintana-Ascencio, P. F., Caswell, H., Baudisch, A. & Vaupel, J. 2014.
Diversity of ageing across the tree of life. Nature 505, 169-173.
<doi:10.1038/nature12789>

Salguero-Gomez R. 2018. Implications of clonality for ageing research.
Evolutionary Ecology, 32, 9-28. <doi:10.1007/s10682-017-9923-2>
}
\seealso{
\code{\link{mature_distrib}} for calculating the proportion of
individuals achieving reproductive maturity in each stage class.

Other life tables: 
\code{\link{age_from_stage}},
\code{\link{lifetable_convert}},
\code{\link{mpm_to_table}()}
}
\author{
Hal Caswell <h.caswell@uva.nl>

Owen Jones <jones@biology.sdu.dk>

Roberto Salguero-Gomez <rob.salguero@zoo.ox.ac.uk>

Patrick Barks <patrick.barks@gmail.com>
}
\concept{life tables}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/is_leslie_matrix.R
\name{is_leslie_matrix}
\alias{is_leslie_matrix}
\title{Determine if a matrix is a Leslie matrix population model}
\usage{
is_leslie_matrix(A, includes_mat_F = TRUE)
}
\arguments{
\item{A}{Matrix to be tested}

\item{includes_mat_F}{A logical argument (default `TRUE`) indicating whether
A is expected to include fecundity. The idea here is that A may not include
fertility, but could still be a valid Leslie matrix if fertility was truly
measured to be 0, or if fertility was not measured at all. Thus, this
argument relaxes the test for the first row of A summing to a positive
value.}
}
\value{
A logical value indicating whether the matrix is a Leslie matrix or
not
}
\description{
Checks if a given matrix is a Leslie matrix.
}
\details{
A formal Leslie matrix satisfies the following conditions:
* All elements are non-negative.
* The subdiagonal elements (survivals), excluding the last column, are all
  between 0 and 1.
* The sum of the elements in the first row (fecundities) is positive (unless
  `includes_mat_F = FALSE`).
* The upper triangle, excluding the first row, contains only 0s.
* The diagonal, excluding the top-left corner (`A[1,1]`), contains only 0s.
* The lower triangle, excluding the subdiagonal, contains only 0s.

**Plus-group extension.** In some models the oldest age class is an open-
ended *plus group* representing individuals that have survived beyond the
maximum explicitly modelled age. These individuals remain in the last class
with probability `A[n,n] > 0`. This function accepts such matrices by also
exempting the bottom-right corner (`A[n,n]`) from the diagonal-zero
requirement. When a plus-group matrix is passed to `leslie_collapse`, the
`A[n,n]` term is correctly carried through the internal `leslie_expand` step.
}
\examples{
# Standard Leslie matrix
A <- matrix(c(
  0.1, 1.2, 1.1,
  0.1, 0.0, 0.0,
  0.0, 0.2, 0.0
), nrow = 3, byrow = TRUE)
is_leslie_matrix(A) # true

# Plus-group Leslie matrix: A[n,n] > 0 is permitted
A_plus <- matrix(c(
  0.1, 1.2, 1.1,
  0.1, 0.0, 0.0,
  0.0, 0.2, 0.3
), nrow = 3, byrow = TRUE)
is_leslie_matrix(A_plus) # true

# Not a Leslie matrix: non-zero off-diagonal elements
A <- matrix(c(
  0.1, 1.2, 1.1,
  0.1, 0.2, 0.1,
  0.2, 0.3, 0.3
), nrow = 3, byrow = TRUE)
is_leslie_matrix(A) # false

# leslie_mpm1 is a plus-group Leslie matrix (A[n,n] > 0)
data(leslie_mpm1)
A <- leslie_mpm1$matU + leslie_mpm1$matF
is_leslie_matrix(A) # true

}
\seealso{
Other transformation: 
\code{\link{leslie_collapse}()},
\code{\link{mpm_collapse}()},
\code{\link{mpm_rearrange}()},
\code{\link{mpm_split}()},
\code{\link{mpm_standardize}()},
\code{\link{name_stages}()},
\code{\link{repro_stages}()},
\code{\link{scale_mpm_to_lambda1}()},
\code{\link{standard_stages}()}
}
\author{
Owen Jones <jones@biology.sdu.dk>
}
\concept{transformation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mpm_split.R
\name{mpm_split}
\alias{mpm_split}
\title{Convert matrix population model into U, F and C matrices}
\usage{
mpm_split(matA)
}
\arguments{
\item{matA}{A matrix population model (i.e., a square projection matrix).}
}
\value{
A list of three matrices: \code{matU},\code{matF} and \code{matC}.
\code{matC} will always contain only zeros.
}
\description{
Splits a matrix population model into three constituent matrices, \bold{U}
(growth and survival processes), \bold{F} (sexual reproduction) and \bold{C}
(clonal reproduction). \strong{Warning!} The functionality is very basic: it
assumes that sexual reproduction is located in the top row of the matrix, and
that everything else is growth or survival (i.e. the \bold{U} matrix).
Clonality is assumed to be non-existent.
}
\examples{
matA <- rbind(
  c(0.1, 0, 5.3, 4.2),
  c(0.5, 0.2, 0.1, 0),
  c(0, 0.3, 0.3, 0.1),
  c(0, 0, 0.5, 0.6)
)

mpm_split(matA)

}
\seealso{
Other transformation: 
\code{\link{is_leslie_matrix}()},
\code{\link{leslie_collapse}()},
\code{\link{mpm_collapse}()},
\code{\link{mpm_rearrange}()},
\code{\link{mpm_standardize}()},
\code{\link{name_stages}()},
\code{\link{repro_stages}()},
\code{\link{scale_mpm_to_lambda1}()},
\code{\link{standard_stages}()}
}
\author{
Owen R. Jones <jones@biology.sdu.dk>
}
\concept{transformation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/entropy_d.R
\name{entropy_d}
\alias{entropy_d}
\title{Calculate Demetrius' entropy from trajectories of age-specific survivorship
and fecundity}
\usage{
entropy_d(lx, mx, type = "scaled", ...)
}
\arguments{
\item{lx}{Either a survivorship trajectory (a vector of
monotonically-declining values in the interval [0,1]), or submatrix U from
a matrix population model.}

\item{mx}{Either an age-specific fecundity trajectory (a vector of
non-negative values), or submatrix F from a matrix population model.}

\item{type}{Calculation type, either `scaled` (default) or `unscaled`.}

\item{...}{Additional variables passed to `mpm_to_lx` and `mpm_to_mx` if the
data are supplied as matrices. This could include the `start` argument to
select a starting stage.}
}
\value{
Demetrius' entropy.
}
\description{
This function calculates Demetrius' entropy from vectors of age-specific
survivorship (\code{lx}) and fecundity (\code{mx}). Users can choose between
the scaled (Caswell, 2001 eqns. 4.94-4.97) or unscaled (from Demetrius 1978)
method.
}
\details{
The scaled version accounts for population growth or shrinkage by adjusting
the contributions of survivorship and fecundity using the dominant eigenvalue
(lambda). Specifically, each contribution is weighted by lambda raised
to the negative power of age. Conversely, the unscaled version does not
account for population growth. It calculates entropy directly from the
proportional contributions of survivorship and fecundity without adjustment
for population dynamics.
}
\section{Warning}{
 Note that this function may produce unexpected results if
  used on partial survivorship and fecundity trajectories. In addition, it is
  sensitive to the length of the these vectors. We direct users to the
  functions `\code{\link{shape_surv}}` and `\code{\link{shape_rep}}` which
  are relatively robust to these issues.
}

\examples{
data(mpm1)

# derive trajectories of lx and mx, starting from stage 2
lx <- mpm_to_lx(mpm1$matU, start = 2)
mx <- mpm_to_mx(mpm1$matU, mpm1$matF, start = 2)

entropy_d(lx, mx, type = "unscaled")
entropy_d(lx, mx, type = "scaled")


# calculate entropy directly from MPM
entropy_d(lx = mpm1$matU, mx = mpm1$matF, start = 2)

}
\references{
Demetrius, L. 1978. Adaptive value, entropy and survivorship
  curves. Nature, 275(5677), 213–214.

  Caswell, H. 2001. Matrix Population Models: Construction, Analysis, and
  Interpretation. Sinauer Associates.
}
\seealso{
Other life history traits: 
\code{\link{entropy_k}()},
\code{\link{entropy_k_age}()},
\code{\link{entropy_k_stage}()},
\code{\link{gen_time}()},
\code{\link{life_elas}()},
\code{\link{life_expect_mean}()},
\code{\link{longevity}()},
\code{\link{net_repro_rate}()},
\code{\link{repro_maturity}},
\code{\link{shape_rep}()},
\code{\link{shape_surv}()},
\code{\link{stage_at_death_dist}()}
}
\author{
Roberto Salguero-Gomez <rob.salguero@zoo.ox.ac.uk>

Patrick Barks <patrick.barks@gmail.com>

Richard Hinrichsen <rich@hinrichsenenvironmental.com>

Owen Jones <jones@biology.sdu.dk>
}
\concept{life history traits}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/life_elas.R
\name{life_elas}
\alias{life_elas}
\title{Calculate Keyfitz's entropy from a trajectory of age-specific survivorship}
\usage{
life_elas(lx, trapeze = FALSE, ...)
}
\arguments{
\item{lx}{Either a survivorship trajectory (a vector of
monotonically-declining values in the interval [0,1]), or submatrix U from
a matrix population model.}

\item{trapeze}{A logical argument indicating whether the composite trapezoid
approximation should be used for approximating the definite integral.}

\item{...}{Additional variables passed to `mpm_to_lx` if data are supplied as
a matrix. This could include the `start` argument to select a starting
stage.}
}
\value{
Keyfitz's life table entropy.
}
\description{
Calculate Keyfitz's entropy from a vector of age-specific survivorship
(\code{lx}), or from the U submatrix of a matrix population model.
}
\section{Warning}{

Note that this function, which was formerly called `entropy_k` may produce
unexpected results if used on partial survivorship trajectories. In addition,
it is sensitive to the length of the survivorship vector. We direct users to
the function `\code{\link{shape_surv}}` which is relatively robust to these
issues.

Furthermore, de Vries et al. 2023 have shown that the way this function
calculates entropy is problematic for other reasons. We recommend to use
`\code{\link{entropy_k_age}}` or ``\code{\link{entropy_k_stage}}` as
alternatives, See de Vries et al. 2023 for details.
}

\examples{
data(mpm1)

# derive lx trajectory, starting from stage 2
lx <- mpm_to_lx(mpm1$matU, start = 2)

# calculate Keyfitz' entropy
life_elas(lx)

# use trapezoid approximation for definite integral
life_elas(lx, trapeze = TRUE)

# calculate directly from the matrix
life_elas(mpm1$matU)

}
\references{
Keyfitz, N. 1977. Applied Mathematical Demography. New York:
  Wiley.

  Demetrius, L., & Gundlach, V. M. 2014. Directionality theory and
  the entropic principle of natural selection. Entropy 16: 5428-5522.

  de Vries, C., Bernard, C., & Salguero-Gómez, R. 2023. Discretising
  Keyfitz' entropy for studies of actuarial senescence and comparative
  demography. Methods in Ecology and Evolution, 14, 1312–1319.
  <doi:10.1111/2041-210X.14083>
}
\seealso{
Other life history traits: 
\code{\link{entropy_d}()},
\code{\link{entropy_k}()},
\code{\link{entropy_k_age}()},
\code{\link{entropy_k_stage}()},
\code{\link{gen_time}()},
\code{\link{life_expect_mean}()},
\code{\link{longevity}()},
\code{\link{net_repro_rate}()},
\code{\link{repro_maturity}},
\code{\link{shape_rep}()},
\code{\link{shape_surv}()},
\code{\link{stage_at_death_dist}()}
}
\author{
Owen R. Jones <jones@biology.sdu.dk>

Roberto Salguero-Gomez <rob.salguero@zoo.ox.ac.uk>
}
\concept{life history traits}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mpm_to_table.R
\name{mpm_to_table}
\alias{mpm_to_table}
\title{Generate a life table from a matrix population model}
\usage{
mpm_to_table(
  matU,
  matR = NULL,
  matF = NULL,
  matC = NULL,
  start = 1L,
  xmax = 1000,
  lx_crit = 0.01,
  radix = 1,
  remove_final = FALSE
)
}
\arguments{
\item{matU}{The survival component of a matrix population model (i.e., a
square projection matrix reflecting survival-related transitions; e.g.,
progression, stasis, and/or retrogression). Optionally with named rows and
columns indicating the corresponding life stage names.}

\item{matR}{The reproductive component of a matrix population model (i.e., a
square projection matrix only reflecting transitions due to reproduction;
either sexual, clonal, or both). If \code{matR} is supplied, \code{matF}
and \code{matC} are ignored. Otherwise, \code{matR} is constructed by
summing \code{matF} and \code{matC}.}

\item{matF}{The matrix reflecting sexual reproduction. If provided
without \code{matC}, \code{matC} is assumed to be a zero matrix. If
\code{matR} is provided, this argument is ignored.}

\item{matC}{The matrix reflecting clonal (asexual) reproduction.
If provided without \code{matF}, \code{matF} is assumed to be a zero
matrix. If \code{matR} is provided, this argument is ignored.}

\item{start}{The index (or stage name) of the first stage at which the author
considers the beginning of life. Defaults to \code{1}. Alternately, a
numeric vector giving the starting population vector (in which case
\code{length(start)} must match \code{ncol(matU))}. See section
\emph{Starting from multiple stages}.}

\item{xmax}{Maximum age to which the life table will be calculated (defaults
to \code{1000}). Time steps are in the same units as the matrix population
model (see MatrixPeriodicity metadata variable COM(P)ADRE).}

\item{lx_crit}{Minimum value of lx to which age-specific traits will be
calculated (defaults to \code{0.01}).}

\item{radix}{The starting number of individuals in the synthetic life table
(defaults to \code{1}). If \code{radix} is set to 1, a simplified life
table is produced.}

\item{remove_final}{Life table calculations typically assume that the final
age class is closed and that all individuals die in that age class. This
can mean that mortality/hazard is artificially inflated for this age class.
Users can prevent this by setting `remove_final` to `TRUE` (the default is
`FALSE`).}
}
\value{
A \code{data.frame} containing a variable number columns, depending
  on input variables. Columns include:

  \item{x}{age at the start of the age interval \code{[x, x+1)}}
  \item{Nx}{The number of individuals alive at age x. The initial number is
  set with \code{radix}}
  \item{Dx}{proportion of original cohort dying during the age interval
  \code{[x, x+1)}}
  \item{lx}{survivorship, defined as the proportion of initial cohort
  surviving to the start of age interval \code{[x, x+1)}}
  \item{dx}{proportion of original cohort dying in the age interval \code{[x,
  x+1)}}
  \item{ax}{The average time survived within the interval by those that die
  during the age interval \code{[x, x+1)}. Assumed to be 0.5}
  \item{hx}{force of mortality (hazard) during the age interval \code{[x,
  x+1)}}
  \item{qx}{probability of death during the interval \code{[x, x+1)} for
  those entering the interval}
  \item{px}{probability of survival for the interval \code{[x, x+1)} for
  those entering the interval}
  \item{Lx}{total person-years lived during the interval \code{[x, x+1)}}
  \item{Tx}{total person years lived beyond age x}
  \item{ex}{remaining life expectancy at age x}

If \code{matF} is provided, also includes:
  \item{mx}{per-capita rate of sexual reproduction during the interval
   \code{[x, x+1)} }
  \item{lxmx}{expected number of sexual offspring per original
  cohort member produced during the interval \code{[x, x+1)}}

If \code{matC} is provided, also includes:
  \item{cx}{per-capita rate of clonal reproduction  during the interval
  \code{[x, x+1)}}
  \item{lxcx}{expected number of clonal offspring per original
  cohort member produced during the interval \code{[x, x+1)}}

If only \code{matR} is provided, the calculations proceed as with \code{matF}.

If both \code{matF} and \code{matC} are provided, also includes:
  \item{mxcx}{per-capita rate of total reproduction (sexual + clonal) during
  the interval \code{[x, x+1)}}
  \item{lxmxcx}{expected number of total offspring (sexual + clonal) per
  original cohort member produced during the interval \code{[x, x+1)}}
}
\description{
This function uses age-from-stage decomposition methods to generate a life
table from a matrix population model. A detailed description of these methods
can be found in section 5.3 "Age-specific traits from stage-specific models"
of Caswell (2001).
}
\note{
The life table is calculated recursively until the age class (x)
  reaches \code{xmax} or survivorship (lx) falls below \code{lx_crit} —
  whichever comes first. To force calculation to \code{xmax}, set
  \code{lx_crit = 0}. Conversely, to force calculation to \code{lx_crit}, set
  \code{xmax = Inf}.

The life table calculations assume that the final age interval is
  closed and that all remaining individuals die in this interval. Therefore,
  for this interval, the probability of death \code{qx} is 1, the probability
  of survival \code{px} is 0 and, because we assume that deaths are evenly
  distributed during the interval, the remaining life expectancy for
  individuals at the start of the interval is 0.5. Depending on analyses, it
  may be a good idea to remove the final row of the table.

  If \code{lx_crit} is sufficiently small that only a very small proportion
  of the cohort reach this age (i.e., < 0.05), this should have minimal
  impact on results. Nevertheless, for many analyses, the final row of the
  life table should be treated with caution and perhaps removed from
  subsequent analyses.

If reproduction columns are included, the resulting \code{mx},
  \code{cx}, and related trajectories reflect reproductive output as
  represented at the census boundary. As a result, reproductive trajectories
  from pre-breeding and post-breeding models are not necessarily directly
  comparable. See Kendall et al. (2019).

Note that the units of time (e.g.. `x` and `ex`) in the returned life
  table are the same as the projection interval (`ProjectionInterval`) of the
  MPM.
}
\section{Starting from multiple stages}{
 Rather than specifying argument
  \code{start} as a single stage class from which all individuals start life,
  it may sometimes be desirable to allow for multiple starting stage classes.
  For example, if the user wants to start the calculation of age-specific
  traits from reproductive maturity (i.e., first reproduction), the user
  should account for the possibility that there may be multiple stage classes
  in which an individual could first reproduce.

To specify multiple starting stage classes, specify argument \code{start} as
the desired starting population vector (\strong{n1}), giving the proportion
of individuals starting in each stage class (the length of \code{start}
should match the number of columns in the relevant MPM).

See function \code{\link{mature_distrib}} for calculating the proportion of
individuals achieving reproductive maturity in each stage class.
}

\examples{
data(mpm1)

mpm_to_table(matU = mpm1$matU, start = 2, xmax = 15)

# equivalent using named life stages
mpm_to_table(matU = mpm1$matU, start = "small", xmax = 15)
mpm_to_table(matU = mpm1$matU, matR = mpm1$matF, start = 2, xmax = 15)

### starting from first reproduction
repStages <- repro_stages(mpm1$matF)
n1 <- mature_distrib(matU = mpm1$matU, start = 2, repro_stages = repStages)
mpm_to_table(matU = mpm1$matU, start = n1)
}
\references{
Caswell, H. 2001. Matrix Population Models: Construction, Analysis, and
Interpretation. Sinauer Associates; 2nd edition. ISBN: 978-0878930968

Caswell, H. 2006. Applications of Markov chains in demography. pp. 319-334 in
A.N. Langville and W.J. Stewart (editors) MAM2006: Markov Anniversary
Meeting. Boson Books, Raleigh, North Caroline, USA

Horvitz, C. & Tuljapurkar, S. 2008. Stage dynamics, period survival, and
mortality plateaus. The American Naturalist 172: 203-2015.
<doi:10.1086/589453>

Jones, O. R., Scheuerlein, A., Salguero-Gomez, R., Camarda, C. G., Schaible,
R., Casper, B. B., Dahlgren, J. P., Ehrlén, J., García, M. B., Menges, E.,
Quintana-Ascencio, P. F., Caswell, H., Baudisch, A. & Vaupel, J. 2014.
Diversity of ageing across the tree of life. Nature 505, 169-173.
<doi:10.1038/nature12789>

Jones O. R. 2021. Life tables: Construction and interpretation In:
Demographic Methods Across the Tree of Life. Edited by Salguero-Gomez R &
Gamelon M. Oxford University Press. Oxford, UK. ISBN: 9780198838609

Preston, S., Heuveline, P., & Guillot, M. 2000. Demography: Measuring and
Modeling Population Processes. Wiley. ISBN: 9781557864512
}
\seealso{
Other life tables: 
\code{\link{age_from_stage}},
\code{\link{lifetable_convert}},
\code{\link{qsd_converge}()}
}
\author{
Owen R. Jones <jones@biology.sdu.dk>

Roberto Salguero-Gómez <rob.salguero@zoo.ox.ac.uk>

Hal Caswell <h.caswell@uva.nl>
}
\concept{life tables}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/entropy_k_stage.R
\name{entropy_k_stage}
\alias{entropy_k_stage}
\title{Calculate Keyfitz entropy for a stage-based matrix population model}
\usage{
entropy_k_stage(Umat, init_distrib = NULL, max_age = NULL, n_is_maxage = FALSE)
}
\arguments{
\item{Umat}{A square numeric matrix representing the U submatrix of a
stage-based (Lefkovitch) matrix population model.}

\item{init_distrib}{The initial cohort distribution across
stages. This should sum to 1. If it does not sum to 1, the function
rescales it to 1. Defaults to an equal distribution across stages.}

\item{max_age}{The upper age, in units of the projection interval. Defaults
to 1000 if no information is provided.}

\item{n_is_maxage}{If TRUE, survival p_n is set to zero. Defaults to FALSE.}
}
\value{
Returns a single numeric value representing the Keyfitz entropy for
  the given matrix. This value quantifies the dispersion of age at death (not
  stage at death).
}
\description{
Computes the discrete-time Keyfitz entropy for a stage-based (Lefkovitch) matrix
population model using the **U** submatrix. This implementation follows the
life-disparity / life-expectancy formulation (H_N) adapted to stage models (H_N^+):
it projects survivorship directly from \code{Umat} and quantifies the dispersion of
**ages at death** (in projection steps) relative to mean life expectancy.
}
\details{
\strong{Interpretation.} Values reflect the \emph{shape} of mortality across age:
\itemize{
  \item \eqn{H < 1}: increasing mortality with age (senescence)
  \item \eqn{H = 1}: approximately age-constant mortality
  \item \eqn{H > 1}: decreasing mortality with age (negative senescence)
}


The function avoids age-from-stage preprocessing by obtaining survivorship
from successive powers of \eqn{U} up to \code{max_age}. In pure age-class
(Leslie) models with \eqn{k} age classes and no persistence in the terminal
class, using \code{max_age = k} recovers the exact discrete-time value.

The result depends on the starting cohort \code{init_distrib} (e.g., all
recruits, a single stage, or recruits at equilibrium). Choose \code{init_distrib}
to match the biological question.
}
\examples{
data(mpm1)

entropy_k_stage(mpm1$matU)
}
\references{
Keyfitz, N. 1977. Applied Mathematical Demography. New York: Wiley.
Giaimo S. 2024. Discretising and validating Keyfitz entropy for any demographic
classification. Methods in Ecology and Evolution. 15, 1278–1285.
}
\seealso{
Other life history traits: 
\code{\link{entropy_d}()},
\code{\link{entropy_k}()},
\code{\link{entropy_k_age}()},
\code{\link{gen_time}()},
\code{\link{life_elas}()},
\code{\link{life_expect_mean}()},
\code{\link{longevity}()},
\code{\link{net_repro_rate}()},
\code{\link{repro_maturity}},
\code{\link{shape_rep}()},
\code{\link{shape_surv}()},
\code{\link{stage_at_death_dist}()}
}
\author{
Stefano Giaimo <giaimo@evolbio.mpg.de>

Owen Jones <jones@biology.sdu.dk>
}
\concept{life history traits}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/shape_surv.R
\name{shape_surv}
\alias{shape_surv}
\title{Calculate shape of survival over age}
\usage{
shape_surv(surv, xmin = NULL, xmax = NULL, trunc = FALSE, ...)
}
\arguments{
\item{surv}{Either 1) a numeric vector describing a survival curve (lx), 2) a
  \code{data.frame} / \code{list} with one column / element titled 'lx'
  describing a survival curve, optionally a column / element 'x' containing
  age classes (each element a number representing the age at the start of the
  class), or 3), a \code{matrix}, specifically the U submatrix of a matrix
  population model (A).

  In case (2) If \code{x} is not supplied, the function will assume age
  classes starting at \code{0} with time steps of \code{1} unit of the
  \code{ProjectionInterval}. If \code{x} begins at \code{0} then \code{lx[1]}
  should equal \code{1}. If \code{x} ends at maximum longevity, then
  \code{lx[which.max(x)]} should equal \code{0}; however it is possible to
  supply partial survivorship curves.}

\item{xmin, xmax}{The minimum and maximum age respectively over which to
evaluate shape. If not given, these default to \code{min(x)} and
\code{max(x)} respectively.}

\item{trunc}{logical determining whether to truncate life tables or not when
any \code{lx == 0}. Usually this is the case only for the final value of
\code{lx}. As the function calculates \code{log(lx)}, these value(s) cannot
be handled. \code{trunc == TRUE} strips out the zero value(s). An
alternative to this is to transform the zeroes to something approximating
zero (e.g., 1e-7).}

\item{...}{Additional variables passed to `mpm_to_lx`, if data are supplied
as a matrix.}
}
\value{
a shape value describing lifespan inequality by comparing the area
  under a survival (\code{lx}) curve over age with the area under a constant
  (Type II) survival function. The shape value may take any real value
  between -0.5 and +0.5. A value of 0 indicates negligible ageing (neither
  generally increasing nor generally decreasing survival with age); negative
  values indicate negative senescence (generally increasing survival with
  age); positive values indicate senescence (generally decreasing survival
  with age). A value of +0.5 indicates that all individuals die at age of
  maximum longevity; a value of -0.5 indicates that (hypothetically) all
  individuals die at birth.
}
\description{
Calculates a 'shape' value of survival lifespan inequality by comparing the
area under a survival curve (over age) with the area under a constant
survival function.
}
\examples{
# exponential decline in lx yields shape = 0
lx <- 0.7^(0:20)
shape_surv(lx)

data(mpm1)
shape_surv(mpm1$matU)

lx <- mpm_to_lx(mpm1$matU, start = 1)
shape_surv(lx)

}
\references{
Wrycza, T.F. and Baudisch, A., 2014. The pace of aging: Intrinsic
  time scales in demography. Demographic Research, 30, pp.1571-1590.
  <doi:10.4054/DemRes.2014.30.57>

  Baudisch, A. 2011, The pace and shape of ageing. Methods in Ecology and
  Evolution, 2: 375-382. <doi:10.1111/j.2041-210X.2010.00087.x>

  Baudisch, A, Stott, I. 2019. A pace and shape perspective on fertility.
  Methods Ecol Evol. 10: 1941– 1951.
  <doi:10.1111/2041-210X.13289>
}
\seealso{
Other life history traits: 
\code{\link{entropy_d}()},
\code{\link{entropy_k}()},
\code{\link{entropy_k_age}()},
\code{\link{entropy_k_stage}()},
\code{\link{gen_time}()},
\code{\link{life_elas}()},
\code{\link{life_expect_mean}()},
\code{\link{longevity}()},
\code{\link{net_repro_rate}()},
\code{\link{repro_maturity}},
\code{\link{shape_rep}()},
\code{\link{stage_at_death_dist}()}
}
\author{
Iain Stott <iainmstott@gmail.com>
}
\concept{life history traits}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/perturb_stochastic.R
\name{perturb_stochastic}
\alias{perturb_stochastic}
\title{Calculate stochastic elasticities from a time-series of matrix population
models and corresponding population vectors}
\usage{
perturb_stochastic(X_t, u_t)
}
\arguments{
\item{X_t}{A list of matrix population models}

\item{u_t}{A list of corresponding population vectors}
}
\value{
A list of three matrices:
  \item{E}{matrix of stochastic elasticities}
  \item{E_mu}{matrix of stochastic elasticities to mean transition rates}
  \item{E_sigma}{matrix of stochastic elasticities to the variance in
  transition rates}
}
\description{
Calculate stochastic elasticities given a time-series of matrix population
models and corresponding population vectors, using the method described in
Haridas et al. (2009).
}
\examples{
# generate list of random MPMs
N <- 20 # number of years
s <- 3 # matrix dimension
X <- list() # matrix population model at time t
u <- list() # population vector at time t

for (t in 1:N) {
  X[[t]] <- matrix(runif(s^2), nrow = s, ncol = s)
}

# derive corresponding series of population vectors
u <- pop_vectors(X)

# calculate stochastic elasticities
perturb_stochastic(X, u)

}
\references{
Haridas, C. V., Tuljapurkar, S., & Coulson, T. 2009. Estimating
  stochastic elasticities directly from longitudinal data. Ecology Letters,
  12, 806-812. <doi:10.1111/j.1461-0248.2009.01330.x>
}
\seealso{
Other perturbation analysis: 
\code{\link{perturb_matrix}()},
\code{\link{perturb_trans}()},
\code{\link{perturb_vr}()},
\code{\link{pop_vectors}()}
}
\author{
Patrick Barks <patrick.barks@gmail.com>
}
\concept{perturbation analysis}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mature_.R
\name{repro_maturity}
\alias{repro_maturity}
\alias{mature_prob}
\alias{mature_age}
\alias{mature_distrib}
\title{Age of reproductive maturity}
\usage{
mature_prob(matU, matR = NULL, matF = NULL, matC = NULL, start = 1L)

mature_age(matU, matR = NULL, matF = NULL, matC = NULL, start = 1L)

mature_distrib(matU, start = 1L, repro_stages)
}
\arguments{
\item{matU}{The survival component of a matrix population model (i.e., a
square projection matrix reflecting survival-related transitions; e.g.,
progression, stasis, and retrogression).}

\item{matR}{The reproductive component of a matrix population model (i.e., a
square projection matrix only reflecting transitions due to reproduction;
either sexual, clonal, or both). If \code{matR} is not provided, it will be
constructed by summing \code{matF} and \code{matC}.}

\item{matF}{(Optional) The matrix reflecting sexual reproduction. If provided
without \code{matC}, \code{matC} is assumed to be a zero matrix. If
\code{matR} is provided, this argument is ignored.}

\item{matC}{(Optional) The matrix reflecting clonal (asexual) reproduction.
If provided without \code{matF}, \code{matF} is assumed to be a zero
matrix. If \code{matR} is provided, this argument is ignored.}

\item{start}{The index (or stage name) of the first stage at which the author
considers the beginning of life. Defaults to \code{1}.}

\item{repro_stages}{A vector of stage names or indices indicating which
stages are reproductive. Alternatively, a logical vector of length
\code{ncol(matU)} indicating whether each stage is reproductive
(\code{TRUE}) or not (\code{FALSE}).}
}
\value{
For \code{mature_distrib}, a vector giving the proportion of
  individuals that first reproduce within each stage class. For all others, a
  scalar trait value.
}
\description{
Apply Markov chain approaches to compute age-specific
trajectory of reproduction for individuals in a matrix population model.
Includes functions to calculate the probability of achieving reproductive
maturity (\code{mature_prob}), mean age at first reproduction
(\code{mature_age}), and distribution of individuals first achieving
reproductive maturity among stage class (\code{mature_distrib}).
}
\note{
Note that the units of time in returned values are the same as the
  \code{ProjectionInterval} of the MPM.
}
\examples{
data(mpm1)

mature_prob(matU = mpm1$matU, matR = mpm1$matF, start = 2)
mature_prob(mpm1$matU, mpm1$matF, start = 2)
mature_age(matU = mpm1$matU, matR = mpm1$matF, start = 2)
mature_age(matU = mpm1$matU, matF = mpm1$matF, start = 2)

### distribution of first reproductive maturity among stage classes
repstage <- repro_stages(mpm1$matF)
mature_distrib(mpm1$matU, start = 2, repro_stages = repstage)

}
\references{
Caswell, H. 2001. Matrix Population Models: Construction,
  Analysis, and Interpretation. Sinauer Associates; 2nd edition. ISBN:
  978-0878930968
}
\seealso{
Other life history traits: 
\code{\link{entropy_d}()},
\code{\link{entropy_k}()},
\code{\link{entropy_k_age}()},
\code{\link{entropy_k_stage}()},
\code{\link{gen_time}()},
\code{\link{life_elas}()},
\code{\link{life_expect_mean}()},
\code{\link{longevity}()},
\code{\link{net_repro_rate}()},
\code{\link{shape_rep}()},
\code{\link{shape_surv}()},
\code{\link{stage_at_death_dist}()}
}
\author{
Roberto Salguero-Gomez <rob.salguero@zoo.ox.ac.uk>

Hal Caswell <hcaswell@whoi.edu>

Owen R. Jones <jones@biology.sdu.dk>

Patrick Barks <patrick.barks@gmail.com>
}
\concept{life history traits}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mpm_collapse.R
\name{mpm_collapse}
\alias{mpm_collapse}
\title{Collapse a matrix population model to a smaller number of stages}
\usage{
mpm_collapse(matU, matF, matC = NULL, collapse)
}
\arguments{
\item{matU}{The survival component of a matrix population model (i.e., a
square projection matrix reflecting survival-related transitions; e.g.,
progression, stasis, and retrogression)}

\item{matF}{The sexual component of a matrix population model (i.e., a square
projection matrix reflecting transitions due to sexual reproduction)}

\item{matC}{The clonal component of a matrix population model (i.e., a square
projection matrix reflecting transitions due to clonal reproduction).
Defaults to \code{NULL}, indicating no clonal reproduction (i.e.,
\code{matC} is a matrix of zeros).}

\item{collapse}{A list giving the mapping between stages of the original
  matrix and the desired stages of the collapsed matrix (e.g., \code{list(1,
  2:3, 4)}). Original stages may be passed as either indices or stage names
  corresponding to stage index or name in \code{matU}, \code{matF} and
  \code{matC}). Names given to the elements of \code{collapse} are used as
  stage names in the new, collapsed matrix.

  See \emph{Missing Stages} for handling of \code{NA} within \code{collapse}.}
}
\value{
A list with four elements:
  \item{matA}{Collapsed projection matrix}
  \item{matU}{Survival component of the collapsed projection matrix}
  \item{matF}{Sexual reproduction component of the collapsed projection
  matrix}
  \item{matC}{Clonal reproduction component of the collapsed projection
  matrix}
}
\description{
Collapse a matrix population model to a smaller number of stages. For
instance, to compare properties of multiple projection matrices with
different numbers of stages, one might first collapse those matrices to a
standardized set of stages (e.g., propagule, pre-reproductive, reproductive,
and post-reproductive). The transition rates in the collapsed matrix are a
weighted average of the transition rates from the relevant stages of the
original matrix, weighted by the relative proportion of each stage class
expected at the stable distribution.
}
\note{
This method of collapsing a matrix population model preserves the
  equilibrium population growth rate (\eqn{lambda}) and relative stable
  distribution, but is not expected to preserve other traits such as relative
  reproductive values, sensitivities, net reproductive rates, life
  expectancy, etc.
}
\section{Missing Stages}{

The collapsed matrix will always be of dimension \code{length(collapse)},
even if one or more elements of the \code{collapse} argument is \code{NA}
(corresponding to a desired stage of the collapsed matrix that is not present
in the original matrix). In the collapsed matrix, any row/column
corresponding to a missing stage will be coerced to \code{NA}.

If all stages within a collapsed group have zero weight in the stable stage
distribution, equal weights are used for that group to avoid undefined
values.
}

\examples{
data(mpm1)

# check which stages reproductive
repro_stages(matR = mpm1$matF)

# collapse reproductive stages (3 and 4) into single stage
mpm_collapse(
  matU = mpm1$matU, matF = mpm1$matF,
  collapse = list(1, 2, 3:4, 5)
)

# use stage names instead, and name stages in the collapsed matrix
mpm_collapse(
  matU = mpm1$matU, matF = mpm1$matF,
  collapse = list(
    seed = "seed", vegetative = "small",
    flowering = c("medium", "large"),
    dormant = "dormant"
  )
)

}
\references{
Salguero-Gomez, R. & Plotkin, J. B. 2010. Matrix dimensions bias
  demographic inferences: implications for comparative plant demography. The
  American Naturalist 176, 710-722. <doi:10.1086/657044>
}
\seealso{
\code{\link{mpm_standardize}}

Other transformation: 
\code{\link{is_leslie_matrix}()},
\code{\link{leslie_collapse}()},
\code{\link{mpm_rearrange}()},
\code{\link{mpm_split}()},
\code{\link{mpm_standardize}()},
\code{\link{name_stages}()},
\code{\link{repro_stages}()},
\code{\link{scale_mpm_to_lambda1}()},
\code{\link{standard_stages}()}
}
\author{
Rob Salguero-Gómez <rob.salguero@zoo.ox.ac.uk>

William K. Petry <wpetry@ncsu.edu>
}
\concept{transformation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/standard_stages.R
\name{standard_stages}
\alias{standard_stages}
\title{Identify stages corresponding to different parts of the reproductive life
cycle}
\usage{
standard_stages(matF, repro_stages, matrix_stages)
}
\arguments{
\item{matF}{The sexual component of a matrix population model (i.e., a square
projection matrix reflecting transitions only due to \emph{sexual}
reproduction). It assumes that it has been rearranged so that
non-reproductive stages are in the final rows/columns.}

\item{repro_stages}{Logical vector identifying which stages are reproductive.}

\item{matrix_stages}{(character) vector of stages, values are \code{prop}
(propagule), \code{active}, and \code{dorm} (dormant).}
}
\value{
A list with four elements:
  \item{propStages}{Position of the propagule stages}
  \item{preRepStages}{Position of the pre-reproductive stages}
  \item{repStages}{Position of the reproductive stages}
  \item{postRepStages}{Position of the post-reproductive stages}
}
\description{
Identify the stages of a matrix population model that correspond to
different parts of the reproductive life cycle, namely propagule,
pre-reproductive, reproductive and post-reproductive. These classifications
are used to standardise matrices to allow comparisons across species with
different life cycle structures, see \code{\link{mpm_standardize}}.
}
\details{
Assumes that fecundity and mean fecundity matrices have been
rearranged so that non-reproductive stages are in the final rows/columns.
Output indicates groupings to be used when collapsing the matrix model.
}
\note{
Dormant stages are not currently handled.
}
\examples{
matU <- rbind(
  c(0.1, 0, 0, 0, 0),
  c(0.5, 0.2, 0.1, 0, 0),
  c(0, 0.3, 0.3, 0.1, 0),
  c(0, 0, 0.4, 0.4, 0.1),
  c(0, 0, 0, 0.1, 0.4)
)

matF <- rbind(
  c(0, 1.1, 0, 1.6, 0),
  c(0, 0.8, 0, 0.4, 0),
  c(0, 0, 0, 0, 0),
  c(0, 0, 0, 0, 0),
  c(0, 0, 0, 0, 0)
)

repro_stages <- c(FALSE, TRUE, FALSE, TRUE, FALSE)
matrix_stages <- c("prop", "active", "active", "active", "active")

r <- mpm_rearrange(matU, matF,
  repro_stages = repro_stages,
  matrix_stages = matrix_stages
)

standard_stages(r$matF, r$repro_stages, r$matrix_stages)

}
\seealso{
\code{\link{mpm_standardize}}

Other transformation: 
\code{\link{is_leslie_matrix}()},
\code{\link{leslie_collapse}()},
\code{\link{mpm_collapse}()},
\code{\link{mpm_rearrange}()},
\code{\link{mpm_split}()},
\code{\link{mpm_standardize}()},
\code{\link{name_stages}()},
\code{\link{repro_stages}()},
\code{\link{scale_mpm_to_lambda1}()}
}
\author{
Rob Salguero-Gomez <rob.salguero@zoo.ox.ac.uk>
}
\concept{transformation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vr_mat.R
\name{vr_mat}
\alias{vr_mat}
\alias{vr_mat_U}
\alias{vr_mat_R}
\title{Derive survival-independent vital rates for growth, stasis, shrinkage, and
reproduction}
\usage{
vr_mat_U(matU, posU = matU > 0, surv_only_na = TRUE)

vr_mat_R(matU, matR, posR = matR > 0)
}
\arguments{
\item{matU}{The survival component of a matrix population model (i.e., a
square projection matrix reflecting survival-related transitions; e.g.
progression, stasis, and retrogression)}

\item{posU}{A logical matrix of the same dimension as \code{matU}, with
elements indicating whether a given \code{matU} transition is possible
(\code{TRUE}) or not (\code{FALSE}). Defaults to \code{matU > 0} (see
Details).}

\item{surv_only_na}{If there is only one possible \code{matU} transition in a
given column, should that transition be attributed exclusively to survival?
If \code{TRUE}, the vital rate of growth/stasis/shrinkage in that column
will be coerced to \code{NA}. If \code{FALSE}, dividing the single
transition by the stage-specific survival probability will always yield a
value of \code{1}. Defaults to \code{TRUE}.}

\item{matR}{The reproductive component of a matrix population model (i.e., a
square projection matrix reflecting transitions due to reproduction; either
sexual, clonal, or both)}

\item{posR}{A logical matrix of the same dimension as \code{matR}, with
elements indicating whether a given \code{matR} transition is possible
(\code{TRUE}) or not (\code{FALSE}). Defaults to \code{matR > 0} (see
Details).}
}
\value{
A matrix of vital rates. Vital rates corresponding to impossible
  transitions will be coerced to \code{NA} (see Details).
}
\description{
Divides columns of a matrix population model by the corresponding
stage-specific survival probability, to obtain lower-level vital rates for
growth, stasis, shrinkage, and reproduction. Vital rates corresponding to
biologically impossible transitions are coerced to \code{NA}.

These decompositions assume that all transition rates are products of a
stage-specific survival term (column sums of \code{matU}) and a lower level
vital rate that is conditional on survival (growth, shrinkage, stasis, or
reproduction). Reproductive vital rates that are not conditional on survival
(i.e., within a stage class from which there is no survival) are also
allowed.
}
\details{
A transition rate of \code{0} within a matrix population model may indicate
that the transition is not possible in the given life cycle (e.g., tadpoles
never revert to eggs), or that the transition is possible but was estimated
to be \code{0} in the relevant population and time period. If vital rates are
to be averaged across multiple stage classes, or compared across populations,
it may be important to distinguish between these two types of zeros.

By default, \code{vr_mat} assumes that a transition rate of
\code{0} indicates an impossible transition, in which case a value of
\code{NA} will be returned in the relevant matrix cell. Specifically, the
arguments \code{posU} and \code{posR} are specified by the logical
expressions \code{(matU > 0)} and \code{(matR > 0)}, respectively. If the
matrix population model includes transitions that are possible but estimated
to be \code{0}, one should specify the \code{posU} and/or \code{posR}
arguments manually.
}
\examples{
matU <- rbind(
  c(0.1, 0, 0, 0),
  c(0.5, 0.2, 0.1, 0),
  c(0, 0.3, 0.3, 0.1),
  c(0, 0, 0.5, 0.6)
)

matR <- rbind(
  c(0, 0, 1.1, 1.6),
  c(0, 0, 0.8, 0.4),
  c(0, 0, 0, 0),
  c(0, 0, 0, 0)
)

# extract vital rates of survival from matU
vr_mat_U(matU)

# extract vital rates of reproduction from matR
vr_mat_R(matU, matR)

}
\references{
Caswell, H. 2001. Matrix Population Models: Construction,
  Analysis, and Interpretation. Sinauer Associates; 2nd edition. ISBN:
  978-0878930968
}
\seealso{
Other vital rates: 
\code{\link{vital_rates}()},
\code{\link{vr}},
\code{\link{vr_vec}}
}
\author{
Patrick Barks <patrick.barks@gmail.com>
}
\concept{vital rates}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gen_time.R
\name{gen_time}
\alias{gen_time}
\title{Calculate generation time from a matrix population model}
\usage{
gen_time(
  matU,
  matR = NULL,
  matF = NULL,
  matC = NULL,
  method = c("R0", "age_diff", "cohort"),
  ...
)
}
\arguments{
\item{matU}{The survival component of a matrix population model (i.e., a
square projection matrix reflecting survival-related transitions; e.g.,
progression, stasis, and retrogression).}

\item{matR}{The reproductive component of a matrix population model (i.e., a
square projection matrix only reflecting transitions due to reproduction;
either sexual, clonal, or both). If \code{matR} is supplied, \code{matF}
and \code{matC} are ignored. Otherwise, \code{matR} is constructed by
summing \code{matF} and \code{matC}.}

\item{matF}{The matrix reflecting sexual reproduction. If provided
without \code{matC}, \code{matC} is assumed to be a zero matrix. If
\code{matR} is provided, this argument is ignored.}

\item{matC}{The matrix reflecting clonal (asexual) reproduction.
If provided without \code{matF}, \code{matF} is assumed to be a zero
matrix. If \code{matR} is provided, this argument is ignored.}

\item{method}{The method used to calculate generation time. Defaults to "R0".
See Details for explanation of calculations.}

\item{...}{Additional arguments passed to \code{net_repro_rate} when
\code{method = "R0"} or \code{mpm_to_*} when \code{method = "cohort"}.
Ignored when \code{method = "age_diff"}.}
}
\value{
Returns generation time. If \code{matU} is singular (often indicating
  infinite life expectancy), returns \code{NA}.
}
\description{
Calculate generation time from a matrix population model. Multiple
definitions of the generation time are supported: the time required for a
population to increase by a factor of R0 (the net reproductive rate; Caswell
(2001), section 5.3.5), the average parent-offspring age difference (Bienvenu
& Legendre (2015)), or the expected age at reproduction for a cohort (Coale
(1972), p. 18-19).
}
\details{
There are multiple definitions of generation time, three of which are
implemented by this function:

1. \code{"R0"} (default): This is the number of time steps required for the
population to grow by a factor of its net reproductive rate, equal to
\code{log(R0) / log(lambda)}. Here, \code{R0} is the net reproductive rate
(the per-generation population growth rate; Caswell 2001, Sec. 5.3.4), and
\code{lambda} is the population growth rate per unit time (the dominant
eigenvalue of \code{matU + matR}).

2. \code{"age_diff"}: This is the average age difference between parents and
offspring, equal to \code{(lambda v w) / (v matR w)} (Bienvenu & Legendre
(2015)). Here, \code{lambda} is the population growth rate per unit time (the
dominant eigenvalue of \code{matU + matR}), \code{v} is a row vector of
stage-specific reproductive values (the left eigenvector corresponding to
\code{lambda}), and \code{w} is a column vector of the stable stage
distribution (the right eigenvector corresponding to \code{lambda}).

3. \code{"cohort"}: This is the age at which members of a cohort are expected
to reproduce, equal to \code{sum(x lx mx) / sum(lx mx)} (Coale (1972), p.
18-19). Here, \code{x} is age, \code{lx} is age-specific survivorship, and
\code{mx} is age-specific fertility. See functions \code{mpm_to_lx} and
\code{mpm_to_mx} for details about the conversion of matrix population models
to life tables.
}
\note{
Note that the units of time in returned values are the same as the
  projection interval (`ProjectionInterval`) of the MPM.
}
\examples{
data(mpm1)

# calculate generation time
gen_time(matU = mpm1$matU, matR = mpm1$matF) # defaults to "R0" method
gen_time(matU = mpm1$matU, matF = mpm1$matF) # defaults to "R0" method
gen_time(matU = mpm1$matU, matR = mpm1$matF, method = "age_diff")
gen_time(
  matU = mpm1$matU, matR = mpm1$matF, method = "cohort", lx_crit =
    0.001
)

}
\references{
Bienvenu, F. & Legendre, S. 2015. A New Approach to the
  Generation Time in Matrix Population Models. The American Naturalist 185
  (6): 834–843. <doi:10.1086/681104>.

Caswell, H. 2001. Matrix Population Models: Construction,
  Analysis, and Interpretation. Sinauer Associates; 2nd edition. ISBN:
  978-0878930968

Coale, A.J. 1972. The Growth and Structure of Human Populations. Princeton
University Press. ISBN: 978-0691093574
}
\seealso{
Other life history traits: 
\code{\link{entropy_d}()},
\code{\link{entropy_k}()},
\code{\link{entropy_k_age}()},
\code{\link{entropy_k_stage}()},
\code{\link{life_elas}()},
\code{\link{life_expect_mean}()},
\code{\link{longevity}()},
\code{\link{net_repro_rate}()},
\code{\link{repro_maturity}},
\code{\link{shape_rep}()},
\code{\link{shape_surv}()},
\code{\link{stage_at_death_dist}()}
}
\author{
Patrick Barks <patrick.barks@gmail.com>

William Petry <wpetry@ncsu.edu>
}
\concept{life history traits}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/leslie_collapse.R
\name{leslie_collapse}
\alias{leslie_collapse}
\title{Aggregate a Leslie matrix}
\usage{
leslie_collapse(A, m)
}
\arguments{
\item{A}{a Leslie matrix}

\item{m}{the dimensionality of the desired aggregated matrix}
}
\value{
a list including the following elements:
\item{B}{The aggregated matrix}
\item{W}{The weight matrix}
\item{Ak}{The original (or expanded) Leslie matrix raised to the k power}
\item{S}{The partitioning matrix}
\item{n}{The size of the original (or expanded) Leslie matrix}
\item{m}{The size of the aggregated matrix}
\item{k}{The quotient of the original size divided by the aggregated size}
\item{EFF}{The effectiveness of aggregation}
}
\description{
Takes a Leslie matrix and aggregates it to a desired dimension m using least
squares weights equal to the stable age distribution. The output includes the
aggregated matrix, the weight matrix, the original (or expanded) Leslie
matrix raised to the k power, the partitioning matrix, the size of the
original (or expanded) Leslie matrix, the size of the aggregated matrix, the
quotient of the original size divided by the aggregated size, and the
effectiveness of aggregation.
}
\examples{
data(leslie_mpm1)
A <- leslie_mpm1$matU + leslie_mpm1$matF
leslie_collapse(A, 4)

}
\references{
Hinrichsen, R. A. (2023). Aggregation of Leslie matrix models
  with application to ten diverse animal species. Population Ecology, 1–21.
  <doi:10.1002/1438-390X.12149>
}
\seealso{
Other transformation: 
\code{\link{is_leslie_matrix}()},
\code{\link{mpm_collapse}()},
\code{\link{mpm_rearrange}()},
\code{\link{mpm_split}()},
\code{\link{mpm_standardize}()},
\code{\link{name_stages}()},
\code{\link{repro_stages}()},
\code{\link{scale_mpm_to_lambda1}()},
\code{\link{standard_stages}()}
}
\author{
Richard A. Hinrichsen <rich@hinrichsenenvironmental.com>
}
\concept{transformation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mpm_rearrange.R
\name{mpm_rearrange}
\alias{mpm_rearrange}
\title{Rearrange stages of a matrix population model to segregate reproductive and
non-reproductive stages}
\usage{
mpm_rearrange(matU, matF, matC = NULL, repro_stages, matrix_stages)
}
\arguments{
\item{matU}{The survival component of a matrix population model (i.e., a
square projection matrix reflecting survival-related transitions; e.g.,
progression, stasis, and retrogression)}

\item{matF}{The sexual component of a matrix population model (i.e., a square
projection matrix reflecting transitions due to sexual reproduction)}

\item{matC}{The clonal component of a matrix population model (i.e., a square
projection matrix reflecting transitions due to clonal reproduction).
Defaults to \code{NULL}, indicating no clonal reproduction (i.e.,
\code{matC} is a matrix of zeros).}

\item{repro_stages}{Logical vector of length \code{ncol(matU)} indicating
which stages are reproductive. Alternatively, a vector of stage indices or
stage names of the reproductive classes.}

\item{matrix_stages}{A character vector identifying organized matrix stages.}
}
\value{
Returns a list with 6 elements:
\item{matU}{Rearranged survival matrix}
\item{matF}{Rearranged sexual reproduction matrix}
\item{matC}{Rearranged clonal reproduction matrix}
\item{matrix_stages}{Rearranged vector of organized matrix stages}
\item{repro_stages}{Rearranged logical vector of reproductive stages}
\item{nonRepInterRep}{Numeric index for any rearranged inter-reproductive
 stages}
}
\description{
Rearrange stages of a matrix population model so that all inter-reproductive
stages fall in the final rows/columns of the matrix. This is a preparatory
step to collapsing the matrix model into a standardized set of stages (e.g.,
propagule, pre-reproductive, reproductive, and post-reproductive).
}
\examples{
matU <- rbind(
  c(0.1, 0, 0, 0, 0),
  c(0.5, 0.2, 0.1, 0, 0),
  c(0, 0.3, 0.3, 0.1, 0),
  c(0, 0, 0.4, 0.4, 0.1),
  c(0, 0, 0, 0.1, 0.4)
)

matF <- rbind(
  c(0, 1.1, 0, 1.6, 0),
  c(0, 0.8, 0, 0.4, 0),
  c(0, 0, 0, 0, 0),
  c(0, 0, 0, 0, 0),
  c(0, 0, 0, 0, 0)
)

repro_stages <- c(2, 4)
matrix_stages <- c("prop", "active", "active", "active", "active")

mpm_rearrange(matU, matF,
  repro_stages = repro_stages,
  matrix_stages = matrix_stages
)

}
\seealso{
\code{\link{mpm_standardize}}

Other transformation: 
\code{\link{is_leslie_matrix}()},
\code{\link{leslie_collapse}()},
\code{\link{mpm_collapse}()},
\code{\link{mpm_split}()},
\code{\link{mpm_standardize}()},
\code{\link{name_stages}()},
\code{\link{repro_stages}()},
\code{\link{scale_mpm_to_lambda1}()},
\code{\link{standard_stages}()}
}
\author{
Rob Salguero-Gómez <rob.salguero@zoo.ox.ac.uk>
}
\concept{transformation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/life_expect.R
\name{life_expect_mean}
\alias{life_expect_mean}
\alias{life_expect_var}
\title{Calculate mean and variance of life expectancy from a matrix population model}
\usage{
life_expect_mean(matU, mixdist = NULL, start = 1L)

life_expect_var(matU, mixdist = NULL, start = 1L)
}
\arguments{
\item{matU}{The survival component of a MPM (i.e., a square projection matrix
reflecting survival-related transitions; e.g., progression, stasis, and
retrogression). Optionally with named rows and columns indicating the
corresponding life stage names.}

\item{mixdist}{A vector with a length equal to the dimension of the MPM
defining how the function should average the output over the. possible
starting states. See section \emph{Starting from multiple stages}. If this
argument is used, `start` must be set to `NULL`.}

\item{start}{The index (or stage name) of the first stage of the life cycle
which the user considers to be the beginning of life. Defaults to \code{1}.
If set to `NULL` the function returns mean life expectancy from each of the
stages of the MPM.}
}
\value{
Returns life expectancy in the units of the projection interval
  (`ProjectionInterval`) of the MPM. If \code{matU} is singular (often
  indicating infinite life expectancy), returns \code{NA}.
}
\description{
Applies Markov chain approaches to obtain mean and variance of life
expectancy from a matrix population model (MPM).
}
\section{Starting from multiple stages}{

Sometimes, it is necessary to calculate life expectancy considering multiple
starting stage classes instead of just a single stage from which all
individuals begin their lives. This scenario arises when there are several
possible stages at which an individual can start a particular life event,
such as reproductive maturity.
To handle such cases, the function provides support for multiple starting
stage classes. When calculating life expectancy in this context, the outputs
should be averaged using weights determined by the distribution of
individuals across these stages. To achieve this, the `start` argument should
be set to `NULL`, indicating that the starting stage is not specified, and
the `mixdist` argument should be utilized.
In the context described, The `mixdist` argument expects a vector that
represents the proportion of individuals with their first reproduction in
each stage of the MPM. By providing this distribution, the function
calculates the mean lifespan by appropriately weighting the life expectancies
corresponding to each starting stage.
For a practical example that demonstrates this usage, please refer to the
code example below.
}

\examples{
data(mpm1)

# mean life expectancy starting from stage class 2
life_expect_mean(mpm1$matU, start = 2)

# equivalent using named life stages
life_expect_mean(mpm1$matU, start = "small")

# mean life expectancies starting from each of the stages
life_expect_mean(mpm1$matU, start = NULL)

# mean life expectancy starting from first reproduction, where this varies
# across individuals
rep_stages <- repro_stages(mpm1$matF)
(n1 <- mature_distrib(mpm1$matU, start = 2, repro_stages = rep_stages))
life_expect_mean(mpm1$matU, mixdist = n1, start = NULL)

# variance of life expectancy from stage class 1
life_expect_var(mpm1$matU, start = 1)

# variance of life expectancy from stage class 1
life_expect_var(mpm1$matU, start = "seed")

# variance of life expectancy from each stage class
life_expect_var(mpm1$matU, start = NULL)

# variance of life expectancies with a set mixing distribution
life_expect_var(mpm1$matU, mixdist = c(0.0, 0.1, 0.3, 0.1, 0.5), start = NULL)

# setting mixdist to ignore all but one stage should produce the same result
# as setting the start argument to that stage
life_expect_mean(mpm1$matU, start = 3)
life_expect_mean(mpm1$matU, mixdist = c(0, 0, 1, 0, 0), start = NULL)

}
\references{
Caswell, H. 2001. Matrix Population Models: Construction,
  Analysis, and Interpretation. Sinauer Associates; 2nd edition. ISBN:
  978-0878930968
}
\seealso{
Other life history traits: 
\code{\link{entropy_d}()},
\code{\link{entropy_k}()},
\code{\link{entropy_k_age}()},
\code{\link{entropy_k_stage}()},
\code{\link{gen_time}()},
\code{\link{life_elas}()},
\code{\link{longevity}()},
\code{\link{net_repro_rate}()},
\code{\link{repro_maturity}},
\code{\link{shape_rep}()},
\code{\link{shape_surv}()},
\code{\link{stage_at_death_dist}()}
}
\author{
Christine M. Hernández <cmh352@cornell.edu>

Owen R. Jones <jones@biology.sdu.dk>
}
\concept{life history traits}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scale_mpm_to_lambda1.R
\name{scale_mpm_to_lambda1}
\alias{scale_mpm_to_lambda1}
\title{Scale a matrix population model so that lambda equals 1}
\usage{
scale_mpm_to_lambda1(matU = NULL, matF = NULL, matC = NULL, matA = NULL)
}
\arguments{
\item{matU}{The survival component of a matrix population model (i.e., a
square projection matrix reflecting survival-related transitions; e.g.,
progression, stasis, and retrogression). Required when using component
inputs.}

\item{matF}{The sexual reproduction component of a matrix population model.
Optional if \code{matC} is supplied. If omitted while \code{matC} is
supplied, it is assumed to be a zero matrix.}

\item{matC}{The clonal reproduction component of a matrix population model.
Optional if \code{matF} is supplied. If omitted while \code{matF} is
supplied, it is assumed to be a zero matrix.}

\item{matA}{The full projection matrix of a matrix population model.
Optional if component matrices are supplied. If supplied together with
components, it must equal \code{matU + matF + matC}.}
}
\value{
A list with six elements:
\describe{
  \item{matA}{The scaled full projection matrix.}
  \item{matU}{The scaled survival matrix. Returned as \code{NULL} if only
  \code{matA} is supplied.}
  \item{matF}{The scaled sexual reproduction matrix. Returned as \code{NULL}
  if only \code{matA} is supplied.}
  \item{matC}{The scaled clonal reproduction matrix. Returned as \code{NULL}
  if only \code{matA} is supplied.}
  \item{lambda_original}{The dominant eigenvalue of the original projection
  matrix.}
  \item{lambda_scaled}{The dominant eigenvalue of the scaled projection
  matrix.}
}
}
\description{
Scales a matrix population model (MPM) so that the dominant eigenvalue
(\eqn{\lambda}) of the full projection matrix equals 1. The function accepts
either a full projection matrix (\code{matA}) or a set of component matrices
consisting of the survival component (\code{matU}) plus at least one
reproductive component (\code{matF} and/or \code{matC}).
}
\details{
When component matrices are used, \code{matU} must be supplied
together with at least one of \code{matF} or \code{matC}. Missing
reproductive components are treated as zero matrices of the appropriate
dimension.
}
\examples{
data(mpm1)

# scale from the full projection matrix
matA <- mpm1$matU + mpm1$matF
scale_mpm_to_lambda1(matA = matA)

# scale from component matrices, assuming no clonal reproduction
scale_mpm_to_lambda1(matU = mpm1$matU, matF = mpm1$matF)

}
\seealso{
Other transformation: 
\code{\link{is_leslie_matrix}()},
\code{\link{leslie_collapse}()},
\code{\link{mpm_collapse}()},
\code{\link{mpm_rearrange}()},
\code{\link{mpm_split}()},
\code{\link{mpm_standardize}()},
\code{\link{name_stages}()},
\code{\link{repro_stages}()},
\code{\link{standard_stages}()}
}
\author{
Owen R. Jones <jones@biology.sdu.dk>
}
\concept{transformation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/longevity.R
\name{longevity}
\alias{longevity}
\title{Calculate longevity from a matrix population model}
\usage{
longevity(matU, start = 1L, x_max = 1000, lx_crit = 0.01)
}
\arguments{
\item{matU}{The survival component of a matrix population model (i.e., a
square projection matrix reflecting survival-related transitions; e.g.,
progression, stasis, and retrogression). Optionally with named rows and
columns indicating the corresponding life stage names.}

\item{start}{The index (or stage name) of the first stage at which the author
considers the beginning of life. Defaults to \code{1}. Alternately, a
numeric vector giving the starting population vector (in which case
\code{length(start)} must match \code{ncol(matU))}. See section
\emph{Starting from multiple stages}.}

\item{x_max}{The maximum age, in units of the MPM projection interval, to
which survivorship will be calculated. Defaults to \code{1000}.}

\item{lx_crit}{Proportion of initial cohort remaining before all are
considered dead (a value between 0 and 1). Defaults to \code{0.01}.}
}
\value{
Returns longevity, the integer age at which expected survivorship
  falls below \code{lx_crit}. If survivorship doesn't reach \code{lx_crit} by
  \code{x_max}, returns \code{NA} and prints a warning message.
}
\description{
Calculate longevity (the age \emph{x} at which survivorship for a synthetic
cohort falls below some critical proportion) from a matrix population model
}
\note{
Note that the units of time in returned values are the same as the
  (\code{ProjectionInterval}) of the MPM.
}
\section{Starting from multiple stages}{

Rather than specifying argument \code{start} as a single stage class from
which all individuals start life, it may sometimes be desirable to allow for
multiple starting stage classes. For example, if we want to start our
calculation of longevity from reproductive maturity (i.e., first
reproduction), we should account for the possibility that there may be
multiple stage classes in which an individual could first reproduce.

To specify multiple starting stage classes, specify argument \code{start} as
the desired starting population vector, giving the proportion
of individuals starting in each stage class (the length of \code{start}
should match the number of columns in the relevant MPM).
}

\examples{
data(mpm1)

longevity(mpm1$matU, start = 2)
longevity(mpm1$matU, start = "small") # equivalent using named life stages
longevity(mpm1$matU, start = 2, lx_crit = 0.05)

# starting from first reproduction
repstages <- repro_stages(mpm1$matF)
n1 <- mature_distrib(mpm1$matU, start = 2, repro_stages = repstages)
longevity(mpm1$matU, start = n1)

}
\references{
Caswell, H. 2001. Matrix Population Models: Construction,
  Analysis, and Interpretation. Sinauer Associates; 2nd edition. ISBN:
  978-0878930968

  Morris, W. F. & Doak, D. F. 2003. Quantitative Conservation Biology:
  Theory and Practice of Population Viability Analysis. Sinauer Associates,
  Sunderland, Massachusetts, USA
}
\seealso{
\code{\link{mature_distrib}} for calculating the proportion of
individuals achieving reproductive maturity in each stage class.

Other life history traits: 
\code{\link{entropy_d}()},
\code{\link{entropy_k}()},
\code{\link{entropy_k_age}()},
\code{\link{entropy_k_stage}()},
\code{\link{gen_time}()},
\code{\link{life_elas}()},
\code{\link{life_expect_mean}()},
\code{\link{net_repro_rate}()},
\code{\link{repro_maturity}},
\code{\link{shape_rep}()},
\code{\link{shape_surv}()},
\code{\link{stage_at_death_dist}()}
}
\author{
Roberto Salguero-Gomez <rob.salguero@zoo.ox.ac.uk>

Hal Caswell <hcaswell@whoi.edu>
}
\concept{life history traits}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vital_rates.R
\name{vital_rates}
\alias{vital_rates}
\title{Derive mean vital rates from a matrix population model}
\usage{
vital_rates(
  matU,
  matF,
  matC = NULL,
  weights = NULL,
  splitStages = "all",
  matrixStages = NULL
)
}
\arguments{
\item{matU}{The survival component of a matrix population model (i.e., a
square projection matrix reflecting survival-related transitions; e.g.
progression, stasis, and retrogression).}

\item{matF}{The sexual component of a matrix population model (i.e., a square
projection matrix reflecting transitions due to sexual reproduction)}

\item{matC}{The clonal component of a matrix population model (i.e., a square
projection matrix reflecting transitions due to clonal reproduction).
Defaults to \code{NULL}, indicating no clonal reproduction (i.e.
\code{matC} is a matrix of zeros).}

\item{weights}{Vector of stage-specific weights to apply while averaging
vital rates. Default is \code{NULL} reflecting equal weighting for all
stages. May also be \code{"SSD"} to weight vital rates by the stable
distribution of \code{matA}.}

\item{splitStages}{What groups should vital rates be averaged over. Either:

\code{"all"}: all stages grouped.

\code{"ontogeny"}: group juvenile stages (all stages prior to the first stage
with sexual reproduction) and adult stages.

\code{"matrixStages"}: group according to a standardized set of stage classes
(propagule, active, and dormant). If \code{splitStages = "matrixStages"},
must also specify separate argument \code{matrixStages}.}

\item{matrixStages}{Vector of stage-specific standardized matrix classes
("prop" for propagule, "active", and/or "dorm" for dormant). Only used if
\code{splitStages = "matrixClass"}.}
}
\value{
A list of averaged vital rates.
}
\description{
Derive mean vital rates corresponding to separate demographic processes from
a matrix population model. Specifically, this function decomposes vital rates
of survival, progression, retrogression, sexual reproduction and clonal
reproduction, with various options for weighting and grouping stages of the
life cycle.
}
\examples{
matU <- rbind(
  c(0.1, 0, 0, 0),
  c(0.5, 0.2, 0.1, 0),
  c(0, 0.3, 0.3, 0.1),
  c(0, 0, 0.5, 0.6)
)

matF <- rbind(
  c(0, 0, 1.1, 1.6),
  c(0, 0, 0.8, 0.4),
  c(0, 0, 0, 0),
  c(0, 0, 0, 0)
)

matC <- rbind(
  c(0, 0, 0.4, 0.5),
  c(0, 0, 0.3, 0.1),
  c(0, 0, 0, 0),
  c(0, 0, 0, 0)
)

# Vital rate outputs without weights
vital_rates(matU, matF, matC, splitStages = "all")
vital_rates(matU, matF, matC, splitStages = "ontogeny")

# Group vital rates according to specified matrixStages
ms <- c("prop", "active", "active", "active")
vital_rates(matU, matF, matC,
  splitStages = "matrixStages",
  matrixStages = ms
)

# Vital rate outputs weighted by the stable stage distribution of 'matA'
vital_rates(matU, matF, matC, splitStages = "all", weights = "SSD")

}
\references{
Caswell, H. 2001. Matrix Population Models: Construction,
  Analysis, and Interpretation. Sinauer Associates; 2nd edition. ISBN:
  978-0878930968
}
\seealso{
Other vital rates: 
\code{\link{vr}},
\code{\link{vr_mat}},
\code{\link{vr_vec}}
}
\author{
Roberto Salguero-Gomez <rob.salguero@zoo.ox.ac.uk>
}
\concept{vital rates}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{leslie_mpm1}
\alias{leslie_mpm1}
\title{Example Leslie matrix population model (MPM)}
\format{
A list with two elements:
\describe{
  \item{matU}{The survival-related component of the MPM.}
  \item{matF}{The sexual reproduction component of the MPM.}
}
}
\usage{
leslie_mpm1
}
\description{
An example Leslie matrix population model (MPM) used for demonstration and
testing purposes.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/perturb_trans.R
\name{perturb_trans}
\alias{perturb_trans}
\title{Perturbation analysis of transition types within a matrix population model}
\usage{
perturb_trans(
  matU,
  matF,
  matC = NULL,
  posU = matU > 0,
  posF = matF > 0,
  posC = matC > 0,
  exclude_row = NULL,
  exclude_col = NULL,
  pert = 1e-06,
  type = "sensitivity",
  demog_stat = "lambda",
  ...
)
}
\arguments{
\item{matU}{The survival component submatrix of a MPM (i.e., a square
projection matrix reflecting survival-related transitions; e.g.,
progression, stasis, and retrogression).}

\item{matF}{The sexual component submatrix of a MPM (i.e., a square
projection matrix reflecting transitions due to sexual reproduction).}

\item{matC}{The clonal component submatrix of a MPM (i.e., a square
projection matrix reflecting transitions due to clonal reproduction).
Defaults to \code{NULL}, indicating no clonal reproduction possible.}

\item{posU}{A logical matrix of the same dimension as \code{matU}, with
elements indicating whether a given \code{matU} transition is possible
(\code{TRUE}) or not (\code{FALSE}). Defaults to \code{matU > 0} (see
Details).}

\item{posF}{A logical matrix of the same dimension as \code{matF}, with
elements indicating whether a given \code{matF} transition is possible
(\code{TRUE}) or not (\code{FALSE}). Defaults to \code{matF > 0} (see
Details).}

\item{posC}{A logical matrix of the same dimension as \code{matC}, with
elements indicating whether a given \code{matC} transition is possible
(\code{TRUE}) or not (\code{FALSE}). Defaults to \code{matC > 0} (see
Details).}

\item{exclude_row}{A vector of row indices or stages names indicating stages
for which transitions \emph{to} the stage should be excluded from
perturbation analysis. Alternatively, a logical vector of length
\code{nrow(matU)} indicating which stages to include \code{TRUE} or exclude
\code{FALSE} from the calculation. See section \emph{Excluding stages}.}

\item{exclude_col}{A vector of column indices or stages names indicating
stages for which transitions \emph{to} the stage should be excluded from
perturbation analysis. Alternatively, a logical vector of length
\code{ncol(matU)} indicating which stages to include \code{TRUE} or exclude
\code{FALSE} from the calculation. See section \emph{Excluding stages}.}

\item{pert}{The magnitude of the perturbation (defaults to \code{1e-6}).}

\item{type}{An argument defining whether to return `sensitivity` or
`elasticity` values. Defaults to `sensitivity`.}

\item{demog_stat}{An argument defining which demographic statistic should be
used, as in "the sensitivity/elasticity of \code{demog_stat} to matrix
element perturbations." Defaults to the per-capita population growth rate
at equilibrium (\eqn{lambda}). Also accepts a user-supplied function that
performs a calculation on a MPM and returns a single numeric value.}

\item{...}{Additional arguments passed to the function \code{demog_stat}.}
}
\value{
A list with 5 elements: \item{stasis}{The sensitivity or elasticity
  of \code{demog_stat} to stasis.} \item{retrogression}{The sensitivity or
  elasticity of \code{demog_stat} to retrogression.} \item{progression}{The
  sensitivity or elasticity of \code{demog_stat} to progression.}
  \item{fecundity}{The sensitivity or elasticity of \code{demog_stat} to
  sexual fecundity.} \item{clonality}{The sensitivity or elasticity of
  \code{demog_stat} to clonality.}
}
\description{
Calculates the summed sensitivities or elasticities for various
transition types within a matrix population model (MPM), including stasis,
retrogression, progression, fecundity, and clonality.

Sensitivities or elasticities are calculated by perturbing elements of the
MPM and measuring the response of the per-capita population growth rate at
equilibrium (\eqn{\lambda}), or, with a user-supplied function, any other
demographic statistic.
}
\details{
A transition rate of \code{0} within a matrix population model can
either indicate that the transition is not possible in the given life cycle
(e.g., tadpoles never revert to eggs), or that the transition is possible but
was estimated to be \code{0} in the relevant population and time period.
Because transition rates of zero \emph{do} generally yield non-zero
sensitivities, it is important to distinguish between structural (i.e.
impossible) zeros and sampled zeros when summing multiple sensitivities for a
given process (e.g., progression/growth).

By default, the \code{perturb_} functions assume that a transition rate of
\code{0} indicates an impossible transition, in which case the sensitivity
for that transition will not be included in any calculation. Specifically,
the arguments \code{posX} are specified by the logical expression \code{(matX
> 0)}. If the matrix population model includes transitions that are possible
but estimated to be \code{0}, users should specify the \code{posX}
argument(s) manually.

If there are no possible transitions for a given process (e.g., clonality, in
many species), the value of sensitivity or elasticity returned for that
process will be \code{NA}.
}
\section{Excluding stages}{
 It may be desirable to exclude one or more stages
  from the calculation. For instance, we might not believe that 'progression'
  to a dormant stage class truly reflects progression. In this case we could
  exclude transitions \emph{to} the dormant stage class using the argument
  \code{exclude_row}. We may or may not want to ignore progression
  transitions \emph{from} the dormant stage class, which can be done in a
  similar way using the argument \code{exclude_col}. The \code{exclude_}
  arguments simply set the relevant row or column of the \code{posX}
  arguments to \code{FALSE}, to prevent those transitions from being used in
  subsequent calculations.
}

\examples{
matU <- rbind(
  c(0.1, 0, 0, 0),
  c(0.5, 0.2, 0.1, 0),
  c(0, 0.3, 0.3, 0.1),
  c(0, 0, 0.5, 0.6)
)

matF <- rbind(
  c(0, 0, 1.1, 1.6),
  c(0, 0, 0.8, 0.4),
  c(0, 0, 0, 0),
  c(0, 0, 0, 0)
)


perturb_trans(matU, matF)

# Use a larger perturbation than the default of 1e-6.
perturb_trans(matU, matF, pert = 0.01)

# Calculate the sensitivity/elasticity of the damping ratio to perturbations.
# First, define function for damping ratio:
damping <- function(matA) {
  eig <- eigen(matA)$values
  dm <- rle(Mod(eig))$values
  return(dm[1] / dm[2])
}

# Second, run the perturbation analysis using demog_stat = "damping".
perturb_trans(matU, matF, demog_stat = "damping")

}
\seealso{
Other perturbation analysis: 
\code{\link{perturb_matrix}()},
\code{\link{perturb_stochastic}()},
\code{\link{perturb_vr}()},
\code{\link{pop_vectors}()}
}
\author{
Rob Salguero-Gómez <rob.salguero@zoo.ox.ac.uk>

Patrick Barks <patrick.barks@gmail.com>
}
\concept{perturbation analysis}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mpm_to_.R
\name{age_from_stage}
\alias{age_from_stage}
\alias{mpm_to_mx}
\alias{mpm_to_lx}
\alias{mpm_to_px}
\alias{mpm_to_hx}
\title{Calculate age-specific traits from a matrix population model}
\usage{
mpm_to_mx(
  matU,
  matR = NULL,
  matF = NULL,
  matC = NULL,
  start = 1L,
  xmax = 1000,
  lx_crit = 0.01,
  tol = 1e-04
)

mpm_to_lx(matU, start = 1L, xmax = 1000, lx_crit = 0.01, tol = 1e-04)

mpm_to_px(matU, start = 1L, xmax = 1000, lx_crit = 0.01, tol = 1e-04)

mpm_to_hx(matU, start = 1L, xmax = 1000, lx_crit = 0.01, tol = 1e-04)
}
\arguments{
\item{matU}{The survival component of a MPM (i.e., a
square projection matrix reflecting survival-related transitions; e.g.,
progression, stasis, and retrogression). Optionally with named rows and
columns indicating the corresponding life stage names.}

\item{matR}{The reproductive component of a matrix population model (i.e., a
square projection matrix only reflecting transitions due to reproduction;
either sexual, clonal, or both). If \code{matR} is supplied, \code{matF}
and \code{matC} are ignored. Otherwise, \code{matR} is constructed by
summing \code{matF} and \code{matC}.}

\item{matF}{The matrix reflecting sexual reproduction. If provided
without \code{matC}, \code{matC} is assumed to be a zero matrix. If
\code{matR} is provided, this argument is ignored.}

\item{matC}{The matrix reflecting clonal (asexual) reproduction.
If provided without \code{matF}, \code{matF} is assumed to be a zero
matrix. If \code{matR} is provided, this argument is ignored.}

\item{start}{The index (or stage name) of the first stage at which the author
considers the beginning of life. Defaults to \code{1}. Alternately, a
numeric vector giving the starting population vector (in which case
\code{length(start)} must match \code{ncol(matU))}. See section
\emph{Starting from multiple stages}.}

\item{xmax}{Maximum age to which age-specific traits will be calculated
(defaults to \code{1000}).}

\item{lx_crit}{Minimum value of \code{lx} to which age-specific traits will
be calculated (defaults to \code{0.01}).}

\item{tol}{To account for floating point errors that occasionally lead to
values of \code{lx} slightly greater than 1, values of \code{lx} within the
open interval (\code{1}, \code{1 + tol}) are coerced to 1. Defaults to
\code{0.0001}. To prevent coercion, set \code{tol} to \code{0}.}
}
\value{
A vector
}
\description{
These functions use age-from-stage decomposition methods to calculate
age-specific survivorship (\code{lx}), survival probability (\code{px}),
mortality hazard (\code{hx}), or reproduction (\code{mx}) from a matrix
population model (MPM). A detailed description of these methods can be found
in sections 5.3.1 and 5.3.2 of Caswell (2001). A separate function
\code{\link{mpm_to_table}} uses the same methods to calculate a full life
table.
}
\note{
Note that the units of time for the returned vectors (i.e., \code{x})
  are the same as the projection interval (\code{ProjectionInterval}) of the
  MPM.

The output vector is calculated recursively until the age class
  (\code{x}) reaches \code{xmax} or survivorship (\code{lx}) falls below
  \code{lx_crit}, whichever comes first. To force calculation to \code{xmax},
  set \code{lx_crit} to \code{0}. Conversely, to force calculation to
  \code{lx_crit}, set \code{xmax} to \code{Inf}.

For \code{mpm_to_mx()}, the output reflects reproductive output as
  represented at the census boundary. As a result, \code{mx} trajectories
  from pre-breeding and post-breeding models are not necessarily directly
  comparable. See Kendall et al. (2019).

Note that the units of time in returned values (i.e., \code{x}) are the
  same as the projection interval (`ProjectionInterval`) of the MPM.
}
\section{Starting from multiple stages}{

Rather than specifying argument \code{start} as a single stage class from
which all individuals start life, it may sometimes be desirable to allow for
multiple starting stage classes. For example, if users want to start their
calculation of age-specific traits from reproductive maturity (i.e., first
reproduction), they should account for the possibility that there may be
multiple stage classes in which an individual could first reproduce.

To specify multiple starting stage classes, users should specify argument
\code{start} as the desired starting population vector (\strong{n1}), giving
the proportion of individuals starting in each stage class (the length of
\code{start} should match the number of columns in the relevant MPM).

See function \code{\link{mature_distrib}} for calculating the proportion of
individuals achieving reproductive maturity in each stage class.
}

\examples{
data(mpm1)

# age-specific survivorship
mpm_to_lx(mpm1$matU)
mpm_to_lx(mpm1$matU, start = 2) # starting from stage 2
mpm_to_lx(mpm1$matU, start = "small") # equivalent using named life stages
mpm_to_lx(mpm1$matU, xmax = 10) # to a maximum age of 10
mpm_to_lx(mpm1$matU, lx_crit = 0.05) # to a minimum lx of 0.05

# age-specific survival probability
mpm_to_px(mpm1$matU)

# age-specific mortality hazard
mpm_to_hx(mpm1$matU)

# age-specific fecundity
mpm_to_mx(mpm1$matU, mpm1$matF)


### starting from first reproduction
repstages <- repro_stages(mpm1$matF)
n1 <- mature_distrib(mpm1$matU, start = 2, repro_stages = repstages)

mpm_to_lx(mpm1$matU, start = n1)
mpm_to_px(mpm1$matU, start = n1)
mpm_to_hx(mpm1$matU, start = n1)
mpm_to_mx(mpm1$matU, mpm1$matF, start = n1)

# specifying matrices explictly
mpm_to_mx(matU = mpm1$matU, matF = mpm1$matF, start = n1)
mpm_to_mx(matU = mpm1$matU, matR = mpm1$matF, start = n1)
mpm_to_mx(matU = mpm1$matU, matC = mpm1$matF, start = n1)

}
\references{
Caswell, H. 2001. Matrix Population Models: Construction, Analysis, and
Interpretation. Sinauer Associates; 2nd edition. ISBN: 978-0878930968

Jones O. R. 2021. Life tables: Construction and interpretation In:
Demographic Methods Across the Tree of Life. Edited by Salguero-Gomez R &
Gamelon M. Oxford University Press. Oxford, UK. ISBN: 9780198838609

Preston, S., Heuveline, P., & Guillot, M. 2000. Demography: Measuring and
Modeling Population Processes. Wiley. ISBN: 9781557864512
}
\seealso{
\code{\link{lifetable_convert}}

Other life tables: 
\code{\link{lifetable_convert}},
\code{\link{mpm_to_table}()},
\code{\link{qsd_converge}()}
}
\author{
Owen R. Jones <jones@biology.sdu.dk>

Roberto Salguero-Gómez <rob.salguero@zoo.ox.ac.uk>

Hal Caswell <h.caswell@uva.nl>

Patrick Barks <patrick.barks@gmail.com>
}
\concept{life tables}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/repro_stages.R
\name{repro_stages}
\alias{repro_stages}
\title{Identify which stages in a matrix population model are reproductive}
\usage{
repro_stages(matR, na_handling = "return.true")
}
\arguments{
\item{matR}{The reproductive component of a matrix population model (i.e., a
square projection matrix reflecting transitions due to reproduction; either
sexual (e.g., \code{matF}), clonal (e.g., \code{matC}), or both).}

\item{na_handling}{One of \code{"return.na"}, \code{"return.true"}, or
\code{"return.false"}. Determines how values of \code{NA} within
\code{matR} should be handled. See Value for more details.}
}
\value{
A logical vector of length \code{ncol(matR)}, with values of
  \code{FALSE} corresponding to non-reproductive stages and values of
  \code{TRUE} corresponding to reproductive stages.\cr\cr For a given matrix
  stage (i.e., column of \code{matR}), if there are any positive values of
  reproduction, the function will return \code{TRUE}. However, for a given
  stage, if there are no positive values of reproduction and one or more
  values of \code{NA}, the function will return \code{NA} if
  \code{na_handling == "return.na"}, \code{TRUE} if \code{na_handling ==
  "return.true"}, or \code{FALSE} if \code{na_handling == "return.false"}.
}
\description{
Takes a reproductive matrix and returns a vector of logical values
(\code{TRUE}/\code{FALSE}) indicating which stages are reproductive (i.e.,
exhibit any positive values for reproduction). This function is a preparatory
step to collapsing the matrix model into a standardized set of stage classes
using the function \code{\link{mpm_standardize}}.
}
\examples{
matR1 <- rbind(
  c(0, 0.2, 0, 0.5),
  c(0, 0.3, 0, 0.6),
  c(0, 0, 0, 0),
  c(0, 0, 0, 0)
)

matR2 <- rbind(
  c(NA, NA, NA, 1.1),
  c(0, 0, 0.3, 0.7),
  c(0, 0, 0, 0),
  c(0, 0, 0, 0)
)

repro_stages(matR1)

# compare different methods for handling NA
repro_stages(matR2, na_handling = "return.na")
repro_stages(matR2, na_handling = "return.true")
repro_stages(matR2, na_handling = "return.false")
}
\seealso{
Other transformation: 
\code{\link{is_leslie_matrix}()},
\code{\link{leslie_collapse}()},
\code{\link{mpm_collapse}()},
\code{\link{mpm_rearrange}()},
\code{\link{mpm_split}()},
\code{\link{mpm_standardize}()},
\code{\link{name_stages}()},
\code{\link{scale_mpm_to_lambda1}()},
\code{\link{standard_stages}()}
}
\author{
Rob Salguero-Gomez <rob.salguero@zoo.ox.ac.uk>

Patrick Barks <patrick.barks@gmail.com>
}
\concept{transformation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/perturb_matrix.R
\name{perturb_matrix}
\alias{perturb_matrix}
\title{Perturbation analysis of a matrix population model}
\usage{
perturb_matrix(
  matA,
  pert = 1e-06,
  type = "sensitivity",
  demog_stat = "lambda",
  ...
)
}
\arguments{
\item{matA}{A matrix population model (i.e., a square projection matrix).}

\item{pert}{Magnitude of the perturbation. Defaults to \code{1e-6}.}

\item{type}{Whether to return \code{sensitivity} or \code{elasticity} values.}

\item{demog_stat}{The demographic statistic to be used, as in "the
sensitivity/elasticity of \code{demog_stat} to matrix element
perturbations." Defaults to the per-capita population growth rate at
equilibrium (\eqn{\lambda}). Also accepts a user-supplied function that
performs a calculation on a projection matrix and returns a single numeric
value.}

\item{...}{Additional arguments passed to the function \code{demog_stat}}
}
\value{
A sensitivity or elasticity matrix.
}
\description{
Perturbs elements within a matrix population model and measures the response
(sensitivity or elasticity) of the per-capita population growth rate at
equilibrium (\eqn{\lambda}), or, with a user-supplied function, any other
demographic statistic.
}
\examples{
matA <- rbind(
  c(0.1, 0, 1.5, 4.6),
  c(0.5, 0.2, 0.1, 0),
  c(0, 0.3, 0.3, 0.1),
  c(0, 0, 0.5, 0.6)
)

perturb_matrix(matA)

# use a larger perturbation than the default
perturb_matrix(matA, pert = 0.01)

# calculate the sensitivity/elasticity of the damping ratio to perturbations
damping <- function(matA) { # define function for damping ratio
  eig <- eigen(matA)$values
  dm <- rle(Mod(eig))$values
  return(dm[1] / dm[2])
}

perturb_matrix(matA, demog_stat = "damping")

}
\references{
Caswell, H. 2001. Matrix Population Models: Construction,
  Analysis, and Interpretation. Sinauer Associates; 2nd edition. ISBN:
  978-0878930968
}
\seealso{
Other perturbation analysis: 
\code{\link{perturb_stochastic}()},
\code{\link{perturb_trans}()},
\code{\link{perturb_vr}()},
\code{\link{pop_vectors}()}
}
\author{
Rob Salguero-Gomez <rob.salguero@zoo.ox.ac.uk>
}
\concept{perturbation analysis}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/entropy_k_age.R
\name{entropy_k_age}
\alias{entropy_k_age}
\title{Calculate Keyfitz entropy for an age-based matrix population model}
\usage{
entropy_k_age(Umat)
}
\arguments{
\item{Umat}{A square numeric matrix representing the U submatrix of a matrix
population model. The dimension corresponds to the number of age classes}
}
\value{
Returns a single numeric value representing the Keyfitz entropy
for the given matrix. This value quantifies the dispersion of age at death.
}
\description{
Computes Keyfitz entropy from the U submatrix of an age-based matrix
population model.
}
\examples{
data(leslie_mpm1)

entropy_k_age(leslie_mpm1$matU)
}
\references{
Keyfitz, N. 1977. Applied Mathematical Demography. New York:
  Wiley.

  de Vries, C., Bernard, C., & Salguero-Gómez, R. 2023. Discretising
  Keyfitz' entropy for studies of actuarial senescence and comparative
  demography. Methods in Ecology and Evolution, 14, 1312–1319.
  <doi:10.1111/2041-210X.14083>
}
\seealso{
Other life history traits: 
\code{\link{entropy_d}()},
\code{\link{entropy_k}()},
\code{\link{entropy_k_stage}()},
\code{\link{gen_time}()},
\code{\link{life_elas}()},
\code{\link{life_expect_mean}()},
\code{\link{longevity}()},
\code{\link{net_repro_rate}()},
\code{\link{repro_maturity}},
\code{\link{shape_rep}()},
\code{\link{shape_surv}()},
\code{\link{stage_at_death_dist}()}
}
\author{
Lotte de Vries <c.devries@uva.nl>

Owen Jones <jones@biology.sdu.dk>
}
\concept{life history traits}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_life_cycle.R
\name{plot_life_cycle}
\alias{plot_life_cycle}
\title{Plot a life cycle diagram from a matrix population model}
\usage{
plot_life_cycle(
  matA,
  stages,
  title = NULL,
  shape = "egg",
  fontsize = 10,
  nodefontsize = 12,
  edgecol = "grey",
  node_order = NULL
)
}
\arguments{
\item{matA}{A matrix population model (i.e., a square projection matrix)}

\item{stages}{Optional vector of stage class labels. If missing, it first
attempts to infer them from \code{dimnames(matA)}. If these are also
\code{NULL}, then reverts to integers \code{1:ncol(A)}.}

\item{title}{Optional title for the plot. Defaults to \code{NULL}.}

\item{shape}{The shape to be used for the stages of the diagram. Any node
shape accepted by \code{graphViz} is acceptable.}

\item{fontsize}{Size of the font used in the diagram.}

\item{nodefontsize}{Size of the font used in the node part of the diagram.}

\item{edgecol}{Colour of the arrows in the diagram.}

\item{node_order}{An optional numeric vector giving the order that the nodes
should be presented in the plot. Default is `NULL` whereby the order is the
same as `stages`, or row/column names, of the matrix.}
}
\value{
An object of class \code{grViz} representing the life cycle diagram
}
\description{
Plots the life cycle diagram illustrated by a matrix population model. This
function processes the matrix model and passes the information to the
graphViz function in DiagrammeR. See
\url{https://rich-iannone.github.io/DiagrammeR/}. Transitions with value
\code{0} are assumed to be biologically impossible, and arrows are omitted
from the diagram. Transition rates of \code{NA} are shown as arrows, labelled
with \code{NA}.
}
\examples{
matA <- rbind(
  c(0.1, 0, 0, 0, 1.4),
  c(0.5, 0.2, 0, 0, 0),
  c(0, 0.3, 0.3, 0, 0),
  c(0, 0, 0.4, 0.4, 0.1),
  c(0, 0, 0, 0.1, 0.4)
)

plot_life_cycle(matA)

# Plot and label unknown transitions as NA
matA_na <- matA
matA_na[1, 5] <- NA
plot_life_cycle(matA_na)

# One could save the diagram as a PNG file using a combination of `export_svg`
# (from the `DiagrammeRsvg` package) and `rsvg_png` (from the `rsvg` package)
# like this:
\dontrun{
p1 <- plot_life_cycle(matA)
p1 \%>\%
  DiagrammeRsvg::export_svg \%>\%
  charToRaw() \%>\%
  rsvg::rsvg_png("my life cycle.png")
}

# Change the order of the nodes and give them names
plot_life_cycle(matA,
  stages = c("A", "B", "C", "D", "E"),
  node_order = 5:1
)

}
\author{
Owen R. Jones <jones@biology.sdu.dk>
}
\concept{visualisation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pop_vectors.R
\name{pop_vectors}
\alias{pop_vectors}
\title{Derive a hypothetical set of population vectors corresponding to a
time-series of matrix population models}
\usage{
pop_vectors(A, start = "stable.stage")
}
\arguments{
\item{A}{A list of MPMs (i.e., square population projection matrices).}

\item{start}{Method to derive the first population vector in the series.
Either \code{stable.stage} to use the stable stage distribution of the mean
matrix as the starting vector, \code{uniform} to use a uniform starting
vector (all elements equal), or \code{random} to use a randomly-generated
starting vector. Defaults to the stable stage distribution.}
}
\value{
A list of population vectors
}
\description{
Derive a hypothetical set of population vectors (i.e. population size
distributions across stages) given a time-series of matrix population models
(MPMs), by taking the stable stage distribution of the mean matrix as the
starting vector (or optionally, a uniform or random starting vector), and
deriving subsequent vectors through recursive population projection.
}
\details{
This function is useful for providing population vectors as input to the
\code{\link{perturb_stochastic}} function which calculates stochastic
elasticities given a time-series of matrix population models and
corresponding population vectors, using the method described in Haridas et
al. (2009).
}
\examples{
# generate list of matrices
matA_l <- replicate(5, matrix(runif(9), 3, 3), simplify = FALSE)

# calculate corresponding population vectors
pop_vectors(matA_l)
pop_vectors(matA_l, start = "uniform")
pop_vectors(matA_l, start = "random")

}
\references{
Haridas, C. V., Tuljapurkar, S., & Coulson, T. 2009. Estimating
  stochastic elasticities directly from longitudinal data. Ecology Letters,
  12, 806-812. <doi:10.1111/j.1461-0248.2009.01330.x>
}
\seealso{
Other perturbation analysis: 
\code{\link{perturb_matrix}()},
\code{\link{perturb_stochastic}()},
\code{\link{perturb_trans}()},
\code{\link{perturb_vr}()}
}
\author{
Patrick Barks <patrick.barks@gmail.com>
}
\concept{perturbation analysis}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/net_repro_rate.R
\name{net_repro_rate}
\alias{net_repro_rate}
\title{Calculate net reproductive rate (R0) from a matrix population model}
\usage{
net_repro_rate(
  matU,
  matR = NULL,
  matF = NULL,
  matC = NULL,
  start = 1,
  method = "generation"
)
}
\arguments{
\item{matU}{The survival component of a matrix population model (i.e., a
square projection matrix reflecting survival-related transitions; e.g.
progression, stasis, and retrogression). Optionally with named rows and
columns indicating the corresponding life stage names.}

\item{matR}{The reproductive component of a matrix population model (i.e., a
square projection matrix only reflecting transitions due to reproduction;
either sexual, clonal, or both). If \code{matR} is supplied, \code{matF}
and \code{matC} are ignored. Otherwise, \code{matR} is constructed by
summing \code{matF} and \code{matC}.}

\item{matF}{The matrix reflecting sexual reproduction. If provided
without \code{matC}, \code{matC} is assumed to be a zero matrix. If
\code{matR} is provided, this argument is ignored.}

\item{matC}{The matrix reflecting clonal (asexual) reproduction.
If provided without \code{matF}, \code{matF} is assumed to be a zero
matrix. If \code{matR} is provided, this argument is ignored.}

\item{start}{Index (or stage name) of the first stage at which the author
considers the beginning of life. Only used if \code{method = "start"}.
Defaults to \code{1}.}

\item{method}{The method used to calculate net reproductive rate, either
\code{"generation"} or \code{"start"}. Defaults to \code{"generation"}.
See Details.}
}
\value{
Returns the net reproductive rate. If \code{matU} is singular (often
  indicating infinite life expectancy), returns \code{NA}.
}
\description{
Calculate net reproductive rate (R0) from a matrix population model. The net
reproduction rate (R0) is the mean number of recruits produced during the
mean life expectancy of an individual. See section 5.3.5 of Caswell (2001).
}
\details{
The \code{method} argument controls how net reproductive rate is calculated.

If \code{method = "generation"}, net reproductive rate is calculated as the
per-generation population growth rate (i.e., the dominant eigenvalue of
\code{matR \%*\% N}, where \code{N} is the fundamental matrix). See Caswell
(2001) Section 5.3.4.

If \code{method = "start"}, net reproductive rate is calculated as the
expected lifetime production of offspring that start life in stage
\code{start}, by an individual also starting life in stage \code{start}
(i.e., \code{(matR \%*\% N)[start,start]}).

If offspring only arise in stage \code{start}, the two methods give the
same result.
}
\examples{
data(mpm1)

net_repro_rate(mpm1$matU, mpm1$matF)

# calculate R0 using the start method, specifying either the life stage index
# or name
net_repro_rate(mpm1$matU, mpm1$matF, method = "start", start = 1)
net_repro_rate(mpm1$matU, mpm1$matF, method = "start", start = "seed")

# It is usually better to explicitly name the arguments, rather than relying
# on order.
net_repro_rate(
  matU = mpm1$matU, matF = mpm1$matF,
  method = "start", start = 1
)

net_repro_rate(
  matU = mpm1$matU, matR = mpm1$matF,
  method = "start", start = "seed"
)

}
\references{
Caswell, H. 2001. Matrix Population Models: Construction,
  Analysis, and Interpretation. Sinauer Associates; 2nd edition. ISBN:
  978-0878930968
}
\seealso{
Other life history traits: 
\code{\link{entropy_d}()},
\code{\link{entropy_k}()},
\code{\link{entropy_k_age}()},
\code{\link{entropy_k_stage}()},
\code{\link{gen_time}()},
\code{\link{life_elas}()},
\code{\link{life_expect_mean}()},
\code{\link{longevity}()},
\code{\link{repro_maturity}},
\code{\link{shape_rep}()},
\code{\link{shape_surv}()},
\code{\link{stage_at_death_dist}()}
}
\author{
Roberto Salguero-Gomez <rob.salguero@zoo.ox.ac.uk>

Hal Caswell <h.caswell@uva.nl>
}
\concept{life history traits}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stage_at_death_dist.R
\name{stage_at_death_dist}
\alias{stage_at_death_dist}
\title{Compute distribution of stage at death from a stage-structured MPM}
\usage{
stage_at_death_dist(matU, start = NULL)
}
\arguments{
\item{matU}{A square numeric matrix giving the survival/transition (\eqn{U})
submatrix of a stage-based (Lefkovitch) matrix population model.}

\item{start}{Optional numeric vector of length \eqn{n} giving the initial
cohort distribution across stages. It will be rescaled to sum to 1.
Defaults to a uniform distribution if \code{NULL}.}
}
\value{
A numeric vector giving the expected contribution of each stage to
  deaths. Names are taken from \code{rownames(U)} when available.
}
\description{
Calculates the expected distribution of deaths across stages for a cohort
governed by the survival/transition submatrix \code{U} of a matrix population
model. The result can be interpreted as the probability of dying in each
stage.
}
\section{Note}{

This function uses the fundamental matrix of the Markov chain implied by
\code{U} to estimate the total expected time spent in each stage, then
combines this with per-stage death probabilities. The output describes where
in the life cycle deaths occur, given a specified starting stage
distribution.
}

\section{Warning}{

Results may be misleading if \code{U} includes reproduction or if any column
sums exceed 1. Use only the survival/transition submatrix. The calculation
also assumes that all stages are transient (i.e. eventual death is certain).
}

\examples{
data(mpm1)

# Uniform starting cohort (default)
stage_at_death_dist(mpm1$matU)

# Starting entirely in stage 1
stage_at_death_dist(mpm1$matU, start = c(1, 0, 0, 0, 0))

# Starting with the SSD (requires the full A matrix)
matA <- mpm1$matU + mpm1$matF
ssd <- popdemo::eigs(matA, "ss")
stage_at_death_dist(mpm1$matU, start = ssd)

}
\seealso{
Other life history traits: 
\code{\link{entropy_d}()},
\code{\link{entropy_k}()},
\code{\link{entropy_k_age}()},
\code{\link{entropy_k_stage}()},
\code{\link{gen_time}()},
\code{\link{life_elas}()},
\code{\link{life_expect_mean}()},
\code{\link{longevity}()},
\code{\link{net_repro_rate}()},
\code{\link{repro_maturity}},
\code{\link{shape_rep}()},
\code{\link{shape_surv}()}
}
\author{
Owen Jones <jones@biology.sdu.dk>
}
\concept{life history traits}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lifetable_convert.R
\name{lifetable_convert}
\alias{lifetable_convert}
\alias{lx_to_px}
\alias{lx_to_hx}
\alias{px_to_lx}
\alias{px_to_hx}
\alias{hx_to_lx}
\alias{hx_to_px}
\title{Convert between age-specific survivorship, survival, or mortality hazard}
\usage{
lx_to_px(lx)

lx_to_hx(lx)

px_to_lx(px)

px_to_hx(px)

hx_to_lx(hx)

hx_to_px(hx)
}
\arguments{
\item{lx}{Vector of age-specific survivorship.}

\item{px}{Vector of age-specific survival probabilities.}

\item{hx}{Vector of age-specific mortality hazards.}
}
\value{
A vector.
}
\description{
Convert between vectors of age-specific survivorship (\code{lx}), survival
probability (\code{px}), or mortality hazard (\code{hx}). Input vectors must
be arranged in order of increasing age, starting with age 0.
}
\details{
\code{lx} gives the proportional survivorship to the start of age
  class \code{x} (where survivorship at first age class is defined as 1),
  \code{px} gives the probability of survival between age \code{x} and
  \code{x+1}, and \code{hx} gives the time-averaged mortality hazard (also
  called force of mortality) between age \code{x} and \code{x+1}.
}
\note{
Note that the units of time for the returned vectors (i.e., \code{x})
  are the same as the (\code{ProjectionInterval}) of the MPM.
}
\examples{
lx <- c(1, 0.8, 0.7, 0.5, 0.3, 0.1)

# convert from lx
px <- lx_to_px(lx)
hx <- lx_to_hx(lx)

# convert from px
lx <- px_to_lx(px)
hx <- px_to_hx(px)

# convert from hx
lx <- hx_to_lx(hx)
px <- hx_to_px(hx)

}
\references{
Caswell, H. 2001. Matrix Population Models: Construction, Analysis, and
Interpretation. Sinauer Associates; 2nd edition. ISBN: 978-0878930968

Caswell, H. 2006. Applications of Markov chains in demography. pp. 319-334 in
A.N. Langville and W.J. Stewart (editors) MAM2006: Markov Anniversary
Meeting. Boson Books, Raleigh, North Caroline, USA

Ergon, T., Borgan, Ø., Nater, C. R., & Vindenes, Y. 2018. The utility of
mortality hazard rates in population analyses. Methods in Ecology and
Evolution, 9, 2046-2056. <doi:10.1111/2041-210X.13059>

Horvitz, C. & Tuljapurkar, S. 2008. Stage dynamics, period survival, and
mortality plateaus. The American Naturalist 172: 203-2015.
<doi:10.1086/589453>

Jones, O. R., Scheuerlein, A., Salguero-Gomez, R., Camarda, C. G., Schaible,
R., Casper, B. B., Dahlgren, J. P., Ehrlén, J., García, M. B., Menges, E.,
Quintana-Ascencio, P. F., Caswell, H., Baudisch, A. & Vaupel, J. 2014.
Diversity of ageing across the tree of life. Nature 505, 169-173.
<doi:10.1038/nature12789>

Jones O. R. 2021. Life tables: Construction and interpretation In:
Demographic Methods Across the Tree of Life. Edited by Salguero-Gomez R &
Gamelon M. Oxford University Press. Oxford, UK. ISBN: 9780198838609

Preston, S., Heuveline, P., & Guillot, M. 2000. Demography: Measuring and
Modeling Population Processes. Wiley. ISBN: 9781557864512
}
\seealso{
Other life tables: 
\code{\link{age_from_stage}},
\code{\link{mpm_to_table}()},
\code{\link{qsd_converge}()}
}
\author{
Patrick Barks <patrick.barks@gmail.com>
}
\concept{life tables}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/name_stages.R
\name{name_stages}
\alias{name_stages}
\title{Add stage names to matrices}
\usage{
name_stages(mat, names = NULL, prefix = "stage_", left_pad = TRUE)
}
\arguments{
\item{mat}{An MPM, either as a single matrix or list of matrices.}

\item{names}{A character vector specifying the name of each life stage, in
order. If provided, \code{prefix} and \code{left_pad} arguments are
ignored.}

\item{prefix}{A string to be pre-pended to the stage number when
automatically naming stages. Defaults to \code{stage_}.}

\item{left_pad}{Logical, whether to pre-pend \code{0} to stage names such
that all stage numbers have equal length, enabling lexicographic sorting.
For example, stage \code{1} becomes \code{01} for matrices with 10-99
stages, \code{001} for matrices with 100-999 stages, and so on. Defaults to
\code{TRUE}.}
}
\value{
The input matrix or matrices with named rows and columns.
}
\description{
Adds user-supplied or automatically-generated stage names to a matrix
population model (MPM).
}
\examples{
matU <- rbind(
  c(0.0, 0.0, 0.0),
  c(0.3, 0.1, 0.0),
  c(0.0, 0.5, 0.8)
)
# (semi)automated naming
name_stages(matU)
name_stages(matU, prefix = "s")
# custom stage names
name_stages(matU, names = c("small", "medium", "large"))
# overwrite existing stage names
data(mpm1)
name_stages(mpm1)

}
\seealso{
Other transformation: 
\code{\link{is_leslie_matrix}()},
\code{\link{leslie_collapse}()},
\code{\link{mpm_collapse}()},
\code{\link{mpm_rearrange}()},
\code{\link{mpm_split}()},
\code{\link{mpm_standardize}()},
\code{\link{repro_stages}()},
\code{\link{scale_mpm_to_lambda1}()},
\code{\link{standard_stages}()}
}
\author{
William K. Petry <wpetry@ncsu.edu>
}
\concept{transformation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/perturb_vr.R
\name{perturb_vr}
\alias{perturb_vr}
\title{Perturbation analysis of vital rates in a matrix population model}
\usage{
perturb_vr(
  matU,
  matF,
  matC = NULL,
  pert = 1e-06,
  type = "sensitivity",
  demog_stat = "lambda",
  ...
)
}
\arguments{
\item{matU}{The survival component of a matrix population model (i.e., a
square projection matrix reflecting survival-related transitions; e.g.,
progression, stasis, and retrogression).}

\item{matF}{The sexual component of a matrix population model (i.e., a square
projection matrix reflecting transitions due to sexual reproduction).}

\item{matC}{The clonal component of a matrix population model (i.e., a square
projection matrix reflecting transitions due to clonal reproduction).
Defaults to \code{NULL}, indicating no clonal reproduction (i.e.,
\code{matC} is a matrix of zeros).}

\item{pert}{Magnitude of the perturbation. Defaults to \code{1e-6}.}

\item{type}{Whether to return \code{sensitivity} or \code{elasticity} values.
Defaults to \code{sensitivity}.}

\item{demog_stat}{The demographic statistic to be used, as in "the
sensitivity/elasticity of \code{demog_stat} to vital rate perturbations."
Defaults to the per-capita population growth rate at equilibrium
(\eqn{\lambda}). Also accepts a user-supplied function that performs a
calculation on a projection matrix and returns a single numeric value.}

\item{...}{Additional arguments passed to the function \code{demog_stat}.}
}
\value{
A list with 5 elements:
\item{survival}{sensitivity or elasticity of \code{demog_stat} to survival}
\item{growth}{sensitivity or elasticity of \code{demog_stat} to growth}
\item{shrinkage}{sensitivity or elasticity of \code{demog_stat} to shrinkage}
\item{fecundity}{sensitivity or elasticity of \code{demog_stat} to sexual
fecundity}
\item{clonality}{sensitivity or elasticity of \code{demog_stat} to clonality}
}
\description{
Perturbs lower-level vital rates within a matrix population model and
measures the response (sensitivity or elasticity) of the per-capita
population growth rate at equilibrium (\eqn{\lambda}), or, with a
user-supplied function, any other demographic statistic.

These decompositions assume that all transition rates are products of a
stage-specific survival term (column sums of \code{matU}) and a lower level
vital rate that is conditional on survival (growth, shrinkage, stasis,
dormancy, or reproduction). Reproductive vital rates that are not conditional
on survival (i.e., within a stage class from which there is no survival) are
also allowed.
}
\note{
Fecundity and clonality outputs assume a post-breeding census
  interpretation. In pre-breeding models, birth rate and newborn survival are
  generally confounded within fecundity transitions. See Kendall et al.
  (2019).
}
\examples{
matU <- rbind(
  c(0.1, 0, 0, 0),
  c(0.5, 0.2, 0.1, 0),
  c(0, 0.3, 0.3, 0.1),
  c(0, 0, 0.5, 0.6)
)

matF <- rbind(
  c(0, 0, 1.1, 1.6),
  c(0, 0, 0.8, 0.4),
  c(0, 0, 0, 0),
  c(0, 0, 0, 0)
)

perturb_vr(matU, matF)

# use elasticities rather than sensitivities
perturb_vr(matU, matF, type = "elasticity")

# use a larger perturbation than the default
perturb_vr(matU, matF, pert = 0.01)

# calculate the sensitivity/elasticity of the damping ratio to vital rate
#  perturbations
damping <- function(matA) { # define function for damping ratio
  eig <- eigen(matA)$values
  dm <- rle(Mod(eig))$values
  return(dm[1] / dm[2])
}

perturb_vr(matU, matF, demog_stat = "damping")

}
\seealso{
Other perturbation analysis: 
\code{\link{perturb_matrix}()},
\code{\link{perturb_stochastic}()},
\code{\link{perturb_trans}()},
\code{\link{pop_vectors}()}
}
\author{
Rob Salguero-Gomez <rob.salguero@zoo.ox.ac.uk>

Patrick Barks <patrick.barks@gmail.com>
}
\concept{perturbation analysis}
